unit MagMessageProp;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created:
   Site Name: Silver Spring, OIFO
   Developers: Richard Price
   Description: Main BP Queue Processor
   Note:
   }
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, Fmcmpnts, Diaccess, StdCtrls, Buttons, Fmcntrls;

type
  TfrmMagMessageProp = class(TForm)
    FMGets1: TFMGets;
    FMValidator1: TFMValidator;
    FMHelp1: TFMHelp;
    FMFiler1: TFMFiler;
    fmeMessageInterval: TFMEdit;
    lblInterval: TLabel;
    lblDTLastMessage: TLabel;
    bbtnOK: TBitBtn;
    bbtnCancel: TBitBtn;
    btnApply: TBitBtn;
    bbtnHelp: TBitBtn;
    FMLabel1: TFMLabel;
    procedure FormCreate(Sender: TObject);
    procedure btnApplyClick(Sender: TObject);
    procedure bbtnOKClick(Sender: TObject);
    procedure bbtnCancelClick(Sender: TObject);
    procedure fmeMessageIntervalExit(Sender: TObject);
    procedure fmeMessageIntervalKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure F1KeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormActivate(Sender: TObject);
    procedure bbtnHelpClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmMagMessageProp: TfrmMagMessageProp;
  frmIEN: string;

implementation

{$R *.dfm}
uses MagSiteParameters,unitDM, magWebHelp;

procedure TfrmMagMessageProp.FormCreate(Sender: TObject);
begin
  dmMain.FMComponentInit(self);
  btnApply.Enabled := false;
end;

procedure TfrmMagMessageProp.btnApplyClick(Sender: TObject);
begin
  if FMFiler1.AnythingToFile then begin
    if FMFiler1.DataProblemCheck then begin
      FMFiler1.DisplayErrors;
      end
    else if not FMFiler1.Update then FMFiler1.DisplayErrors;
    end;
  btnApply.Enabled := false;
end;


procedure TfrmMagMessageProp.bbtnOKClick(Sender: TObject);
begin
  if FMFiler1.AnythingToFile then begin
    if FMFiler1.DataProblemCheck then begin
      FMFiler1.DisplayErrors;
      end
    else if not FMFiler1.Update then FMFiler1.DisplayErrors;
    end;
  ModalResult := mrOK;
end;

procedure TfrmMagMessageProp.bbtnCancelClick(Sender: TObject);
begin
  ModalResult := mrOK;
end;

procedure TfrmMagMessageProp.fmeMessageIntervalExit(Sender: TObject);
begin
  with Sender as TFMEdit do begin
  if FMDBInternal <> Text then begin
    if Validate then begin
      FMFiler.AddFDA(FMFile,IENS,FMField,(sender as TFMEdit).FMCtrlInternal);
      btnApply.Enabled := true;
      end // if Validate
    else begin
      frmSiteParameters.SiteParametersDisplayHelp(Sender);
      Text := FMDBInternal; {reset value to previous}
      end;  // else
    end; // if FMDBInternal <> Text
end;  // with Sender as TFMEdit

end;

procedure TfrmMagMessageProp.fmeMessageIntervalKeyUp(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin

    SystemParametersInfo(SPI_SETBEEP, 1, nil, SPIF_SENDWININICHANGE);
  If Key = VK_Return Then Begin
    If HiWord(GetKeyState(VK_SHIFT)) <> 0 then
      SelectNext(Sender as TWinControl,False,True)
    else
      SelectNext(Sender as TWinControl,True,True) ;
      Key := VK_TAB
    end;
 SystemParametersInfo(SPI_SETBEEP, 0, nil, SPIF_SENDWININICHANGE);

end;

procedure TfrmMagMessageProp.F1KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if key = VK_F1 then
     frmSiteParameters.F1KeyDown(Sender,Key,Shift);
end;

procedure TfrmMagMessageProp.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  frmMagMessageProp.ModalResult := mrCancel;
  Action := caFree;
end;

procedure TfrmMagMessageProp.FormActivate(Sender: TObject);
begin
  Application.NormalizeTopMosts;
  SetWindowPos(frmMagMessageProp.Handle, HWND_TOPMOST, 0,0,0,0,
    SWP_NOACTIVATE+SWP_NOMOVE+SWP_NOSIZE);
end;

procedure TfrmMagMessageProp.bbtnHelpClick(Sender: TObject);
begin
  Application.CreateForm(TfrmMagBPHelp,frmMagBPHelp);
  frmMagBPHelp.WB.Navigate(ExtractFilePath(Application.ExeName)+
 'help\MAG_BP_User_Manual.htm#_Toc269903181');

end;

end.
