unit MagNewBP;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created:
   Site Name: Silver Spring, OIFO
   Developers: Richard Price
   Description: BP Add
   Note:
   }
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Trpcb, Diaccess, Fmcmpnts, Fmlookup, OleCtrls, { isp3,} Fmcntrls,MFunstr,
  ExtCtrls,{MagSecurityUnit,}  {MagBroker,} {maggut1,}{MagSec_C,} ComCtrls, Buttons,magbut;

type
  TfrmMagNewBP = class(TForm)
    FMLookUp1: TFMLookUp;
    FMFinder1: TFMFinder;
    GroupBox1: TGroupBox;
    fmeBPAddWSNetName: TFMEdit;
    lblWSName: TLabel;
    lblNetName: TLabel;
    sbFile: TBitBtn;
    sbCancel: TBitBtn;
    fmeBPAddWSName: TFMEdit;
    FMGets1: TFMGets;
    procedure FMEditChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormActivate(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure sbFileClick(Sender: TObject);
    procedure sbCancelClick(Sender: TObject);
    procedure F1KeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ProcList(EditList: TStringList;ChangedCtrlsList: TList);
    function PartialSearch(Search: string;SearchList: TStringList): integer;
    procedure FreeList(List: TList);
    procedure UpdateEditList(FMFile,FMField,IENs,Value:string;EditList: TStringList);
    procedure fmeBPAddWSNetNameExit(Sender: TObject);
    procedure fmeBPAddWSNameMouseDown(Sender: TObject;
      Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  private
    { Private declarations }
  public
    { Public declarations }
  end;


var
  frmMagNewBP: TfrmMagNewBP;
  AddRecord,ReleaseForm,Updates: boolean;
  iens: string;
  BPWSEditlist: TStringlist;
  ChangedCtrlsList: TList;

implementation

uses Magbtm1, UnitDM,MagSiteParameters;

{$R *.DFM}
procedure TfrmMagNewBP.UpdateEditList(FMFile,FMField,IENs,Value:string;EditList: TStringList);
var SearchListIndex:integer;
begin
 SearchListIndex := PartialSearch(FMFile+'^'+FMField+'^'+IENS+'^',Editlist);
 if SearchListIndex <> -1 then Editlist[SearchListIndex] := FMFile+'^'+FMField+'^'+IENS+'^'+Value
 else Editlist.Add(FMFile+'^'+FMField+'^'+IENS+'^'+Value);
end;

function TfrmMagNewBP.PartialSearch(Search: string;SearchList: TStringList): integer;
  var i: integer;
begin
  result:= -1;
  for i := 0 to Searchlist.Count - 1 do
   if pos(Search,SearchList[i]) <> 0 then result := i;
end;


procedure TfrmMagNewBP.FMEditChange(Sender: TObject);
var
 valid: string;
 SearchListIndex: integer;
begin
  with Sender as TFMEdit do begin
    if not FMModified then exit;
    if length(text) < 1 then begin
      Windows.MessageBox(frmMagNewBP.Handle,PChar('The BP Logical name is required! '),'',MB_SYSTEMMODAL);
      exit;
      end;
    valid :=  dmMain.MagFieldValidate(FMFile,IENS,FMField,'',text) ;
    if pos('-1',piece(Valid,'^',1)) <> 0 then begin
      Windows.MessageBox(frmMagNewBP.Handle,PChar(piece(Valid,'^',2)),'',MB_SYSTEMMODAL);
      Text := '';
      SetFocus;
      end
    else begin
      if FMField='.01' then begin
        valid :=dmMain.GetVariable('$S($D(^MAG(2006.8,"D","'+IntToStr(magbtm1.place)+'","'+
        text+'")):-1,1:1)');
        if valid <> '1' then begin
          Windows.MessageBox(frmMagNewBP.Handle,PChar('This BP Server name is already assigned!'),'',MB_SYSTEMMODAL);
          Text := '';
          SetFocus;
          end
        else UpdateEditList(FMFile,FMField,IENs,text,BPWSEditlist);
        end; //  if FMField='.01'
      end;  // else valid
    end; // with Sender as TFMEdit
end;

procedure TfrmMagNewBP.ProcList(EditList: TStringList;ChangedCtrlsList:Tlist);
var i: integer;
begin
 if (EditList.Count < 1) and (ChangedCtrlsList.Count < 1) then begin
    Windows.MessageBox(frmMagNewBP.Handle,PChar('Nothing to file!'),' ',MB_SYSTEMMODAL);
    exit;
    end;
  if not dmMain.FDAUpdate('',EditList,ChangedCtrlsList,iens) then
     Windows.MessageBox(frmMagNewBP.Handle,PChar('Changes not update!'),' ',MB_SYSTEMMODAL)
  else //Windows.MessageBox(frmMagNewBP.Handle,PChar('ien: '+iens),' ',MB_SYSTEMMODAL)
end;

procedure TfrmMagNewBP.FormClose(Sender: TObject;
  var Action: TCloseAction);

var i : integer;
begin
  FreeList(ChangedCtrlsList);
  FreeAndNil(ChangedCtrlsList);
  BPWSEditList.free;
  ModalResult := mrOK;
end;
procedure TfrmMagNewBP.sbCancelClick(Sender: TObject);
begin
  Close;
end;


procedure TfrmMagNewBP.FreeList(List: TList);
var
  index: integer;
  cnt: integer;
begin
  for index := List.Count - 1 downto 0 do
  begin
    dispose(List.Items[index]);
    List.Delete(index);
  end;

end;

procedure TfrmMagNewBP.FormActivate(Sender: TObject);
begin
  {$ifdef Win32}
          SetForegroundWindow(frmMagNewBP.Handle);
  {$else}
          BringWindowToTop(frmMagNewBP.Handle);
  {$endif}
//  Application.NormalizeTopMosts;
//  SetWindowPos(frmMagNewBP.Handle, HWND_TOPMOST, 0,0,0,0,
//   SWP_NOACTIVATE+SWP_NOMOVE+SWP_NOSIZE);
//  Application.HelpFile :=ExtractFilePath(Application.ExeName)+'help\BackProc.HLP';
  BPWSEditlist := TStringList.Create;
  ChangedCtrlsList := TList.Create;
  IENS := '+1,';
  BPWSEditlist.Add('2006.8^.04^'+IENS+'^'+IntToStr(magbtm1.place));
  BPWSEditlist.Add('2006.8^11^'+IENS+'^1');
  fmeBPAddWSName.IENS := IENS;
  fmeBPAddWSNetName.IENS := IENS;
  fmeBPAddWSNetName.Text := GetCompName(self);
  fmeBPAddWSName.Text := '';
  fmeBPAddWSName.SetFocus;
  application.ProcessMessages;
end;
procedure TfrmMagNewBP.FormCreate(Sender: TObject);
begin
  frmMagNewBP.Caption := format('%s - %s',[ ' '+SiteName+' ', 'New BP Server']);
  dmMain.FMComponentInit(self);
  end;

procedure TfrmMagNewBP.sbFileClick(Sender: TObject);
var
  i: integer;
  valid: string;
begin
  with fmeBPAddWSName do begin
    if length(text) < 1 then begin
      Windows.MessageBox(frmMagNewBP.Handle,PChar('The BP Logical name is required! '),'',MB_SYSTEMMODAL);
      exit;
      end;  // if length(text) < 1
    valid :=dmMain.GetVariable('$S($D(^MAG(2006.8,"D","'+IntToStr(magbtm1.place)+'","'+
      text+'")):-1,1:1)');
    if valid <> '1' then begin
      Windows.MessageBox(frmMagNewBP.Handle,PChar('This BP Server name is already assigned!'),'',MB_SYSTEMMODAL);
      exit;
      end
    else UpdateEditList(FMFile,FMField,IENs,text,BPWSEditlist);
    valid :=  dmMain.MagFieldValidate(FMFile,IENS,FMField,'',text) ;
    if pos('-1',piece(Valid,'^',1)) <> 0 then begin
      Windows.MessageBox(frmMagNewBP.Handle,PChar(piece(Valid,'^',2)),'',MB_SYSTEMMODAL);
      Text := '';
      exit;
      end; // if pos('-1',
    end; // with fmeBPAddWSName
  with fmeBPAddWSNetName do begin
    if length(text) < 1 then begin
      Windows.MessageBox(frmMagNewBP.Handle,PChar('Both the BP Server name and the Network name are required! '),'',MB_SYSTEMMODAL);
      exit;
      end;  // if length(text) < 1
    valid :=dmMain.GetVariable('$S($D(^MAG(2006.8,"C","'+IntToStr(magbtm1.place)+'","'+
     text+'")):-1,1:1)');
    if valid <> '1' then begin
      Windows.MessageBox(frmMagNewBP.Handle,PChar('This BP Server network name is already assigned!'),'',MB_SYSTEMMODAL);
      exit;
      end
    else UpdateEditList(FMFile,FMField,IENs,text,BPWSEditlist);
    valid :=  dmMain.MagFieldValidate(FMFile,IENS,FMField,'',text) ;
    if pos('-1',piece(Valid,'^',1)) <> 0 then begin
      Windows.MessageBox(frmMagNewBP.Handle,PChar(piece(Valid,'^',2)),'',MB_SYSTEMMODAL);
      Text := '';
      exit;
      end; // if pos('-1'
    end; // with fmeBPAddWSNetName
  ProcList(BPWSEditlist,ChangedCtrlsList);
  close;
end;


procedure TfrmMagNewBP.F1KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if key = VK_F1 then
     frmSiteParameters.F1KeyDown(Sender,Key,Shift);
end;

procedure TfrmMagNewBP.fmeBPAddWSNetNameExit(Sender: TObject);
var
 valid: string;
 SearchListIndex: integer;
begin
  with Sender as TFMEdit do begin
    if length(text) < 1 then begin
      Windows.MessageBox(frmMagNewBP.Handle,PChar('Both the BP Server name and the Network name are required! '),'',MB_SYSTEMMODAL);
      exit;
      end;
    valid :=  dmMain.MagFieldValidate(FMFile,IENS,FMField,'',text) ;
    if pos('-1',piece(Valid,'^',1)) <> 0 then begin
      Windows.MessageBox(frmMagNewBP.Handle,PChar(piece(Valid,'^',2)),'',MB_SYSTEMMODAL);
      Text := (Sender as TFMEdit).FMCtrlExternal;
      end
    else begin
      if FMField='50' then begin
        valid :=dmMain.GetVariable('$S($D(^MAG(2006.8,"C","'+IntToStr(magbtm1.place)+'","'+
        text+'")):-1,1:1)');
        if valid <> '1' then begin
          Windows.MessageBox(frmMagNewBP.Handle,PChar('This BP Server network name is already assigned!'),'',MB_SYSTEMMODAL);
          SetFocus;
          end
        else UpdateEditList(FMFile,FMField,IENs,text,BPWSEditlist);
        end; // if FMField='50'
      end;  // else valid
    end; // with Sender as TFMEdit
end;

procedure TfrmMagNewBP.fmeBPAddWSNameMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  FMEditChange(Sender);
end;

end.
