unit MagPurgeParam;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created:
   Site Name: Silver Spring, OIFO
   Developers: Richard Price
   Description: BP Purge Parameter editor
   Note:
   }
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Trpcb, Diaccess, Fmcmpnts, Fmlookup, OleCtrls, { isp3,} Fmcntrls,
  ExtCtrls, maggut1,MagSec_C, ComCtrls, Buttons;

type
  TfrmMagPurgePar = class(TForm)
    FMValidator1: TFMValidator;
    FMFiler1: TFMFiler;
    GroupBox1: TGroupBox;
    Splitter1: TSplitter;
    FMEdit1: TFMEdit;
    FMEdit2: TFMEdit;
    FMEdit4: TFMEdit;
    FMEdit5: TFMEdit;
    FMEdit6: TFMEdit;
    FMEdit3: TFMEdit;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    bbtnOK: TBitBtn;
    bbtnCancel: TBitBtn;
    FMHelp1: TFMHelp;
    gbAuorPurge: TGroupBox;
    lblSPT: TLabel;
    lbllpd: TLabel;
    lblSPFreq: TLabel;
    fmeSPT: TFMEdit;
    fmeLPD: TFMEdit;
    fmcbAPurge: TFMCheckBox;
    fmcbSchedPurge: TFMCheckBox;
    fmeSPF: TFMEdit;
    btnApply: TBitBtn;
    FMGets1: TFMGets;
    fmcbPOPPurge: TFMCheckBox;
    fmrgPurgeCriteria: TFMRadioGroup;
    procedure FormCreate(Sender: TObject);
    procedure FMEditChange(Sender: Tobject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure bbtnOKClick(Sender: TObject);
    procedure bbtnCancelClick(Sender: TObject);
    procedure bbtnHelpClick(Sender: TObject);
    procedure UpdateBoolean(sender: TObject;var Filer:TFMFiler);
    procedure btnApplyClick(Sender: TObject);
    procedure fmeFilePrefixChange(Sender: TObject);
    procedure fmcbInterfaceSwitchClick(Sender: TObject);
    procedure fmrgPurgeCriteriaClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmMagPurgePar: TfrmMagPurgePar;

implementation

uses Magbtm1, UnitDM;

{$R *.DFM}

procedure TfrmMagPurgePar.FormCreate(Sender: TObject);

begin
//  dmMain.FMComponentInit(self);
  Caption := format('%s - %s',[' '+Magbtm1.SiteName,'BP Purge Parameters']);
  application.ShowHint:=true;
  application.HintPause :=1000;
  application.HintHidePause := 8000;
  dmMain.fmgSiteParam.IENS := IntToStr(magbtm1.place) +',';
  FMValidator1.IENS := IntToStr(magbtm1.place) +',';
  label1.Caption:=FMEdit1.FMDisplayName;
  label2.Caption:=FMEdit2.FMDisplayName;
  label3.Caption:=FMEdit3.FMDisplayName;
  label4.Caption:=FMEdit4.FMDisplayName;
  label5.Caption:=FMEdit5.FMDisplayName;
  label6.Caption:=FMEdit6.FMDisplayName;
  dmMain.fmgSiteParam.GetAndFill;
  btnApply.Enabled := false;

end;

procedure TfrmMagPurgePar.FMEditChange(Sender: TObject);
begin
 with sender as TFMEdit do
   if Validate then begin
     FMFiler1.AddChgdControl(Sender);
     btnApply.Enabled := true;
     end
   else begin
     GetAndDisplayHelp;
     fmGets1.FileNumber := FMFile;
     fmGets1.FieldNumbers.Add(FMField);
     FMGets := fmGets1;
     fmGets1.IENS := iens;
     fmGets1.GetAndFill;
     end;
end;

procedure TfrmMagPurgePar.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  MAGBTMF1.mnuPurgeParameters.checked := false;
end;

procedure TfrmMagPurgePar.btnApplyClick(Sender: TObject);
begin
  if FMFiler1.AnythingToFile then FMFiler1.Update;
  btnApply.Enabled := false;
end;

procedure TfrmMagPurgePar.bbtnOKClick(Sender: TObject);
begin
  if FMFiler1.AnythingToFile then begin
    FMFiler1.Update;
    dmMain.fmgSiteParam.GetAndFill;
    btnApply.Enabled := false;
    end;
  frmMagPurgePar.Close;
end;

procedure TfrmMagPurgePar.bbtnCancelClick(Sender: TObject);
begin
  frmMagPurgePar.Close;
end;

procedure TfrmMagPurgePar.bbtnHelpClick(Sender: TObject);
begin
application.HelpContext(54);
end;

procedure TfrmMagPurgePar.UpdateBoolean(sender: TObject; var Filer:TFMFiler);
begin
  with sender as TFMCheckBox do begin
    checked := false;
    FMCtrlInternal := FMValueUnchecked;
    Filer.AddChgdControl(sender);
    btnApply.Enabled := true;
    end;
end;


procedure TfrmMagPurgePar.fmeFilePrefixChange(Sender: TObject);
begin
  with Sender as TFMEdit do
    if Validate then FMFiler1.AddChgdControl(Sender);
end;

{ with sender as TFMEdit do
   if Validate then begin
     FMFiler1.AddChgdControl(Sender);
     btnApply.Enabled := true;
     end
   else begin
     GetAndDisplayHelp;
     fmGets1.FileNumber := FMFile;
     fmGets1.FieldNumbers.Add(FMField);
     FMGets := fmGets1;
     fmGets1.IENS := iens;
     fmGets1.GetAndFill;
     end;
}





procedure TfrmMagPurgePar.fmcbInterfaceSwitchClick(Sender: TObject);
begin
 with sender as TFMCheckBox do begin
   if FMModified then
     if Validate then begin
       FMFiler1.AddChgdControl(Sender);
       btnApply.Enabled := true;
       end
     else begin
       GetAndDisplayHelp;
       Checked := FMTag;
       end;
   end; // with sender
end;

procedure TfrmMagPurgePar.fmrgPurgeCriteriaClick(Sender: TObject);
begin
 with sender as TFMRadioGroup do begin
   if FMModified then
     if Validate then begin
       FMFiler1.AddChgdControl(Sender);
       btnApply.Enabled := true;
       end
     else begin
       GetAndDisplayHelp;
 //      Checked := FMTag;
       end;
   end; // with sender

end;

end.
