unit MagQueMS;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created:
   Site Name: Silver Spring, OIFO
   Developers: Richard Price
   Description: BP Queue Manager Utility
   Note:
   }
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, StdCtrls, ExtCtrls;

type
  TfrmMagQueStatus = class(TForm)
    pnlStatusSelect: TPanel;
    lbQueueStatus: TListBox;
    lbQueueSelected: TListBox;
    bAddStatus: TButton;
    bRemoveStatus: TButton;
    bbtnCancel: TBitBtn;
    bbtnOK: TBitBtn;
    bbtnHelp: TBitBtn;
    Panel1: TPanel;
    function LimitListSize(Percent,RecordSize:integer;Sender: TObject):string;
    procedure bAddStatusClick(Sender: TObject);
    procedure bRemoveStatusClick(Sender: TObject);
    procedure bbtnOKClick(Sender: TObject);
    procedure bbtnCancelClick(Sender: TObject);
    procedure lbQueueSelectedClick(Sender: TObject);
    procedure lbQueueStatusClick(Sender: TObject);
    procedure Clear;
    procedure bbtnHelpClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmMagQueStatus: TfrmMagQueStatus;

implementation

uses Magbtm1,MagQueMan, UnitDM, magWebHelp;

{$R *.DFM}

procedure TfrmMagQueStatus.bAddStatusClick(Sender: TObject);

 var i,j: integer;
begin
bAddStatus.Enabled := false;
bRemoveStatus.Enabled := false;
for i := lbQueueStatus.Items.Count - 1 downto 0 do
  if lbQueueStatus.Selected[i] then begin
    if lbQueueStatus.items[i] = 'ALL' then begin
      for j := lbQueueSelected.Items.Count -1 downto 0 do begin
        if lbQueueSelected.items[j] <> 'ALL' then
          lbQueueStatus.Items.Add(lbQueueSelected.items[j]);
        lbQueueSelected.Items.Delete(j);
        end;//for j
      lbQueueSelected.items.add('ALL');
      lbQueueStatus.Items.Delete(lbQueueStatus.Items.IndexOf('ALL'));
      lbQueueStatus.Enabled := false;
      bbtnOk.enabled := true;
      exit;
      end;  // if 'all'
    lbQueueSelected.Items.Add(lbQueueStatus.items[i]);
    lbQueueStatus.Items.Delete(i);
    end;// if selected
if lbQueueSelected.items.Count <> 0 then bbtnOk.enabled := true;
end;

procedure TfrmMagQueStatus.bRemoveStatusClick(Sender: TObject);
var i: integer;
begin
bAddStatus.Enabled := false;
bRemoveStatus.Enabled := false;
for i := lbQueueSelected.Items.Count - 1 downto 0 do
  if lbQueueSelected.Selected[i] then begin
    if lbQueueSelected.items[i] = 'ALL' then lbQueueStatus.Enabled := true;
    lbQueueStatus.Items.Add(lbQueueSelected.items[i]);
    lbQueueSelected.Items.Delete(i);
    end;
if lbQueueSelected.items.Count <1 then bbtnOk.enabled := False;
end;

procedure TfrmMagQueStatus.bbtnOKClick(Sender: TObject);
var
  i: integer;
  Screen : String;
begin
  with frmMagQueMan do begin
  sbPurge.Enabled := true;
  sbRetry.Enabled := true;
  sbSave.Enabled := true;
  pnlAction.Caption :='';
  Status := '';
  lbStatusScreen.Clear;
  if lbQueueSelected.items.Count > 0 then begin
    ToIens := trim(StringReplace(ToIens,char(39),char(32),[rfReplaceAll	]));
    if Queue = 'JBTOHD' then lblType.Caption := 'Type        Status'
    else lblType.Caption := '            Status';
    if lbQueueSelected.items.IndexOf('ALL') <> -1 then begin
      dmMain.fmlQueueRecord.Screen := 'I Y<'+ToIens +',$P(^(0),"^")="'+Queue+'",$P(^(0),U,12)='
       +IntToStr(magbtm1.place);
      lbStatusScreen.Items.add('ALL');
      end //'ALL'
    else begin
      dmMain.fmlQueueRecord.Screen := 'I Y<'+ToIens+',$P(^(0),"^")="'+Queue+'",$P(^(0),U,12)='+IntToStr(magbtm1.place);
      Screen := dmMain.fmlQueueRecord.Screen;
      for i := 0 to lbQueueSelected.Items.count -1 do begin
        lbStatusScreen.Items.add(lbQueueSelected.Items[i]);
        //if (pos('"D"',Screen) <> 0) then Screen := Screen + '!';
        Screen := Screen +',$D(^MAGQUEUE(2006.03,"D",'+IntToStr(magbtm1.place)+',"'+
          Queue + '","'+lbQueueSelected.Items[i]+'",Y))';
        end; //for i := 0 to lbQueueSelected.Items.count -1
      dmMain.fmlQueueRecord.Screen := Screen;
      end; // else not 'ALL'
    //pnlStatusSelect.Visible := false;
    if dmMain.fmlQueueRecord.ErrorList.Count > 0 then dmMain.fmlQueueRecord.DisplayErrors;
    FMListBox1.Clear;
    dmMain.fmlQueueRecord.Number := LimitListSize(10,120,Self);
    try
      dmMain.fmlQueueRecord.GetList(FMListBox1.Items);
    finally
      dmMain.RPCBroker1.RPCTimeLimit := magbtm1.intSaveRPCTimeLimit;
      end;
    FMListBoxUpdate(FMListBox1);
    for i := 0 to FMListBox1.Items.Count -1 do FMListBox1.Selected[i] := true;
    if dmMain.fmlQueueRecord.ErrorList.Count > 0 then
        showmessage('fmlQueueRecord.ErrorList');
        dmMain.fmlQueueRecord.DisplayErrors;
    if not dmMain.fmlQueueRecord.More then bbtnMore.Enabled:= false
    else begin
      bbtnMore.Enabled := true;
      MoreArray.Add(dmMain.fmlQueueRecord.FromIEN);
      end; //else
    lblpnlAction.Caption := 'Queue type: ' + Queue + '          Status: ';
    end; //if lbQueueSelected.items.Count > 0
    Application.ProcessMessages;
    ModalResult := mrOk;
   end;
//   Clear;
  end;

procedure TfrmMagQueStatus.bbtnCancelClick(Sender: TObject);

begin
  ModalResult:= mrCancel;
end;

procedure TfrmMagQueStatus.lbQueueSelectedClick(Sender: TObject);
var i : integer;
begin
  bAddStatus.Enabled := false;
  bRemoveStatus.Enabled := lbQueueSelected.SelCount > 0 ;
if lbQueueStatus.SelCount > 0 then
  for i := lbQueueStatus.Items.Count -1 downto 0 do
 lbQueueStatus.Selected[i] := false;
end;

procedure TfrmMagQueStatus.lbQueueStatusClick(Sender: TObject);
var i : integer;
begin
  bAddStatus.Enabled := lbQueueStatus.SelCount > 0;
  bRemoveStatus.Enabled := false;
if lbQueueStatus.SelCount > 0 then
  for i := lbQueueSelected.Items.Count -1 downto 0 do
 lbQueueSelected.Selected[i] := false;
end;
procedure TfrmMagQueStatus.Clear;

begin
    lbQueueStatus.Clear;
    lbQueueSelected.Clear;
end;
function TfrmMagQueStatus.LimitListSize(Percent,RecordSize:integer;Sender: TObject):string;
var
  MS: TMemoryStatus;
begin
  MS.dwLength := SizeOf(TMemoryStatus);
  GlobalMemoryStatus(MS);
  result :=  IntToStr((MS.dwAvailVirtual div Percent)div RecordSize);
end;
procedure TfrmMagQueStatus.bbtnHelpClick(Sender: TObject);
begin
  Application.CreateForm(TfrmMagBPHelp,frmMagBPHelp);
  frmMagBPHelp.WB.Navigate(ExtractFilePath(Application.ExeName)+
    'help\MAG_BPuserman.htm#_TOC72648942');

end;

procedure TfrmMagQueStatus.FormCreate(Sender: TObject);
begin
  frmMagQueStatus.Caption := format('%s - %s',[' '+magbtm1.SiteName+' ',' Manage Queues by status  ']);
end;

end.
