unit MagQueMan;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created:
   Site Name: Silver Spring, OIFO
   Developers: Richard Price
   Description: BP Queue Manager
   Note:
   }
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Diaccess, Fmcmpnts, Fmlookup, Trpcb, StdCtrls, Fmcntrls, Buttons, Menus,
  ExtCtrls,Printers,DiTypLib,  MFunStr, Grids, Gauges, OleCtrls;

type
  TfrmMagQueMan = class(TForm)
    FMLookUp1: TFMLookUp;
    FMListBox1: TFMListBox;
    bbtnMore: TBitBtn;
    MainMenu1: TMainMenu;
    pnlAction: TPanel;
    pnlHeader: TPanel;
    lblType: TLabel;
    bbtnPrevious: TBitBtn;
    sbSave: TBitBtn;
    sbPurge: TBitBtn;
    SaveDialog1: TSaveDialog;
    sbQueSet: TBitBtn;
    lblCQR: TLabel;
    FMLabel1: TFMLabel;
    FMValidator1: TFMValidator;
    FMFiler1: TFMFiler;
    FMValidator2: TFMValidator;
    lblQIEN: TLabel;
    sbRetry: TBitBtn;
    lbStatusScreen: TListBox;
    lblPnlAction: TLabel;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    function QueueTypeLookup(Sender: TObject):boolean;
    procedure bbtnPreviousClick(Sender: TObject);
//    procedure sbPrintClick(Sender: TObject);
    procedure sbSaveClick(Sender: TObject);
    procedure sbPurgeClick(Sender: TObject);
    procedure sbRetryClick(Sender: TObject);
    procedure FMListBoxUpdate(var ListBox: TFMListBox);
    procedure bbtnCancelClick(Sender: TObject);
    procedure bbtnMoreClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmMagQueMan: TfrmMagQueMan;
  ToIens,Queue,Status: string;
  MoreArray: TStringList;
  Deleted,intSaveRPCTimeLimit: integer;
  QueReset: boolean;
const TimeLimit = 3600;
implementation
uses Magbtm1, MagQueMS, UnitDM;
{$R *.DFM}

procedure TfrmMagQueMan.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  ModalResult := mrCancel;
end;
function TfrmMagQueMan.QueueTypeLookup(Sender: TObject):boolean;
var
 AddRecord: boolean;
     value: integer;
     fatal: boolean;
     rlist: TStringList;
begin
    fatal := false;
  with dmMain do begin
    result := true;
    bbtnMore.Enabled := false;
    bbtnPrevious.Enabled := false;
    QueReset := false;
    frmMagQueStatus.bAddStatus.Enabled := false;
    frmMagQueStatus.bRemoveStatus.Enabled := false;
    intSaveRPCTimeLimit := RPCBroker1.RPCTimeLimit;  {save off current time limit}
    RPCBroker1.RPCTimeLimit := TimeLimit;   {can take up to a minute to complete}
    MoreArray := TStringList.Create;
    FMLookup1.Title := format('%s - %s',[Magbtm1.SiteName,'Queue Type']);
    FMLookup1.AllowNew := false;
    fmlQueuePointer.Screen := 'I $P(^(0),U,4)='+ IntToStr(magbtm1.place);
    if FMLookup1.Execute(AddRecord) and (FMLookup1.RecordNumber <> '') then begin
      fmgQueuePointer.IENS := FMLookup1.RecordNumber;
      FMLabel1.IENS := FMLookup1.RecordNumber;
      RPCBroker1.RemoteProcedure := 'DDR GETS ENTRY DATA';
      try
        RList := TStringList.Create;
        with RPCBroker1.Param[0] do begin
          Value  := '.DDR';
          Ptype  := list;
          Mult['"FILE"']  := '2006.031';
          Mult['"IENS"']  := FMLookup1.RecordNumber+',';
          Mult['"FIELDS"'] := '1';
          Mult['"FLAGS"'] := 'I';
          end;
        RPCBroker1.LSTCALL(RList);
        if RList[0]='[Data]' then
        ToIens := piece(RList[1],U,4)
        else ToIens := '0';
      finally
        RList.Free;
      end;
      fmgQueuePointer.GetAndFill;
      ToIens := IntToStr(StrToInt(ToIens)+1);
      lblCQR.Caption := 'Current Queue Reference: '+FMLAbel1.FMCtrlInternal;
      Queue := fmgQueuePointer.GetField('.01').FMDBExternal;
      if not TryStrToInt(ToIens,value) then begin
         Showmessage('No Queues to reprocess!');
         exit;
         end;
      if Length(ToIens)< 1 then begin
        ToIens := fmgQueuePointer.GetField('1').FMDBInternal;
        ToIens := IntToStr(StrToInt(ToIens)+1);
        end;
      RPCBroker1.Param[0].VALUE := Queue;
      RPCBroker1.Param[0].PTYPE := LITERAL ;
      RPCBroker1.RemoteProcedure := 'MAGQB QSL';
      try
        RPCBroker1.LSTCALL(frmMagQueStatus.lbQueueStatus.Items);
      except
         On E: EBrokerError do begin
           MAGBTMF1.ErrorLog('MAGQB QSL','Queue Manager Queue pointerlookup failed',E.ClassName+' '+E.Message);
           SHOWMESSAGE('The Queue File is corrupt!');
           frmMagQueMan.Close;
           Exit;
           end;
      end;  //try
      frmMagQueStatus.lbQueueStatus.Items.Add('ALL');
      end {queue type selected}
    else begin
      Result := false;
      frmMagQueMan.Close;
      end;
    sbPurge.Enabled := false;
    sbRetry.Enabled := false;
    sbSave.Enabled := false;
    end;
end;

procedure TfrmMagQueMan.bbtnPreviousClick(Sender: TObject);
var prev: integer;

begin
   prev := MoreArray.IndexOf(dmMain.fmlQueueRecord.FromIEN) ;
   case prev of
     0: prev := 0;
    -1: if deleted > 0 then prev := deleted - 1
        else prev := 0;
      else dec(prev);
      end; //case prev of
   // while MoreArray[prev] = fmlQueueRecord.FromIEN do dec(prev);
   FMListBox1.Clear;
   if Prev = 0 then begin
     bbtnPrevious.Enabled := false;
     dmMain.fmlQueueRecord.GetList(FMListBox1.Items);
     if dmMain.fmlQueueRecord.More then bbtnMore.Enabled := true
     else bbtnMore.Enabled := false;
     FMListBoxUpdate(FMListBox1);
     exit
     end;
   dmMain.fmlQueueRecord.FromIEN := MoreArray[Prev-1] ;
   dmMain.fmlQueueRecord.GetMore(FMListBox1.Items);
   FMListBoxUpdate(FMListBox1);
   if MoreArray.IndexOf(dmMain.fmlQueueRecord.FromIEN)=0 then begin
      bbtnPrevious.enabled := false;
      bbtnMore.Enabled := true;
      end;
end;
procedure TfrmMagQueMan.FMListBoxUpdate(var ListBox: TFMListBox);
var i: integer;
    MultiSelect,ExtendedSelect: boolean;
begin
  MultiSelect := ListBox.MultiSelect;
  ExtendedSelect := ListBox.ExtendedSelect;
  ListBox.MultiSelect := false;
  ListBox.ExtendedSelect := false;
  for i := 0 to ListBox.Items.Count - 1 do begin
    ListBox.ItemIndex := i;
    ListBox.Items.Strings[i] := ListBox.GetSelectedRecord.ien +' '+
      ListBox.Items.Strings[i];
    end;
  ListBox.MultiSelect := MultiSelect;
  ListBox.ExtendedSelect:= ExtendedSelect;
  bbtnPrevious.enabled := false;
  ListBox.ItemIndex := 0;
  ListBox.Update;
end;

procedure TfrmMagQueMan.sbSaveClick(Sender: TObject);
begin
  SaveDialog1.InitialDir := ExtractFilePath(Application.ExeName);
  if SaveDialog1.Execute then begin
    if FileExists(SaveDialog1.FileName) then
      if MessageDlg(Format('OK to overwrite %s', [SaveDialog1.FileName]),
        mtConfirmation, mbYesNoCancel, 0) <> idYes then Exit;
    FMListBox1.Items.SaveToFile(SaveDialog1.FileName);
    end;  // if execute
end;

procedure TfrmMagQueMan.sbPurgeClick(Sender: TObject);
var
  i: integer;
begin
  with FMListBox1 do begin
    if items.count < 1 then begin
      ShowMessage('No Queue Records to Purge');
      exit;
      end;
    if SelCount < 1 then begin
      ShowMessage('No Queue Records to Selected');
      exit;
      end;
    for i := items.Count-1 downto 0 do begin
      ItemIndex := i;
      if not Selected[i] then continue;
      with dmMain.RPCBroker1 do begin
        Param[0].VALUE := GetSelectedRecord.IEN;
        Param[0].PTYPE := LITERAL;
        RemoteProcedure := 'MAGQB QUEDEL';
        Call;
        end;
      Selected[i] := false;
      Items.BeginUpdate;
      Items.Delete(i);
      Items.EndUpdate;
      Update;
      end;
    end; // with FMListBox1
    if MoreArray.Count > 0 then
    if MoreArray.IndexOf(dmMain.fmlQueueRecord.FromIEN) <> -1 then begin
      deleted := MoreArray.IndexOf(dmMain.fmlQueueRecord.FromIEN);
      MoreArray.Delete(MoreArray.IndexOf(dmMain.fmlQueueRecord.FromIEN));
      MoreArray.Sort;
      end;
end;

procedure TfrmMagQueMan.sbRetryClick(Sender: TObject);
var
  i: integer;
begin
  with FMListbox1 do begin
    if items.count < 1 then begin
      ShowMessage('No Queue Records to Requeue');
      exit;
      end;
    if SelCount < 1 then begin
      ShowMessage('No Queue Records to Selected');
      exit;
      end;
    for i := items.Count - 1 downto 0 do begin
      ItemIndex := i;
      if not Selected[i] then continue;
      with dmMain.RPCBroker1 do begin
        Param[0].VALUE := GetSelectedRecord.IEN;
        Param[0].PTYPE := LITERAL;
        RemoteProcedure := 'MAGQ REQ';
        Call;
        end;
      Items.BeginUpdate;
      Items.Delete(i);
      Items.EndUpdate;
      Update;
      end;
    end; // with FMListBox1
    if MoreArray.Count > 0 then
    if MoreArray.IndexOf(dmMain.fmlQueueRecord.FromIEN) <> -1 then begin
      deleted := MoreArray.IndexOf(dmMain.fmlQueueRecord.FromIEN);
      MoreArray.Delete(MoreArray.IndexOf(dmMain.fmlQueueRecord.FromIEN));
      MoreArray.Sort;
      end;
end;



procedure TfrmMagQueMan.bbtnCancelClick(Sender: TObject);
begin
  ModalResult := mrCancel ;
end;

procedure TfrmMagQueMan.bbtnMoreClick(Sender: TObject);
var i: integer;
begin
  with dmMain do begin
  if fmlQueueRecord.More then begin
    FMListBox1.Clear;
    fmlQueueRecord.GetMore(FMListBox1.Items);
    FMListBoxUpdate(FMListBox1);
    for i := 0 to FMListBox1.Items.Count -1 do FMListBox1.Selected[i] := true;
    if MoreArray.IndexOf(fmlQueueRecord.FromIEN) = -1 then
       MoreArray.Add(fmlQueueRecord.FromIEN);
    if fmlQueueRecord.More then begin
      bbtnMore.Enabled := true ;
      bbtnPrevious.Enabled := true;
      end
    else begin
      bbtnMore.Enabled := false;
      end;
    end;
  if fmlQueuePointer.ErrorList.Count > 0 then fmlQueuePointer.DisplayErrors;
  end;
end;

procedure TfrmMagQueMan.FormCreate(Sender: TObject);
begin
 dmMain.FMComponentInit(self);
 frmMagQueMan.Caption := format('%s - %s',[Magbtm1.SiteName,'BP Queue Manager ']);
end;

end.
