unit MagQueManSet;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created:
   Site Name: Silver Spring, OIFO
   Developers: Richard Price
   Description: BP Queue Manager Utility
   Note:
   }
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Diaccess, Fmcmpnts, Fmlookup, Trpcb, StdCtrls, Fmcntrls, Buttons, Menus,
  ExtCtrls,Printers,DiTypLib,  MFunStr, Grids, Gauges, OleCtrls;

type
  TfrmMagQueManSet = class(TForm)
    FMLookUp1: TFMLookUp;
    FMListBox1: TFMListBox;
    pnlHeader: TPanel;
    lblType: TLabel;
    lblCQR: TLabel;
    FMLabel1: TFMLabel;
    FMValidator1: TFMValidator;
    FMFiler1: TFMFiler;
    FMValidator2: TFMValidator;
    lblQIEN: TLabel;
    bbtnMore: TBitBtn;
    bbtnCancel: TBitBtn;
    bbtnQueSet: TBitBtn;
    bbtnPrevious: TBitBtn;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    function QueueTypeLookup(Sender: TObject):boolean;
    procedure sbPreviousClick(Sender: TObject);
    procedure sbQueSetClick(Sender: TObject);
    procedure FMListBoxUpdate(var ListBox: TFMListBox);
    procedure bbtnCancelClick(Sender: TObject);
    procedure sbMoreClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);

  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmMagQueManSet: TfrmMagQueManSet;
  ToIens,Queue,Status: string;
  MoreArray,LabelArray: TStringList;
  Deleted,intSaveRPCTimeLimit: integer;
  QueReset: boolean;
const TimeLimit = 3600;
implementation
uses Magbtm1, UnitDM;
{$R *.DFM}

procedure TfrmMagQueManSet.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  ModalResult := mrCancel;
  MoreArray.Free;
end;
function TfrmMagQueManSet.QueueTypeLookup(Sender: TObject):boolean;
var
 AddRecord: boolean;
         i: integer;
     fatal: boolean;
       err: TFMErrorObj;

begin
    fatal := false;
  with dmMain do begin
    result := true;
    bbtnMore.Enabled := false;
    bbtnPrevious.Enabled := false;
    QueReset := false;
    intSaveRPCTimeLimit := RPCBroker1.RPCTimeLimit;  {save off current time limit}
    RPCBroker1.RPCTimeLimit := TimeLimit;   {can take up to a minute to complete}
    Caption := format('%s - %s - %s',[magbtm1.SiteName,' BP Queue Manager ', Copy(CompName,0,3)]);
    Update;
    MoreArray := TStringList.Create;
    FMLookup1.AllowNew := false;
    fmlQueuePointer.Screen := 'I $P(^(0),U,4)='+ IntToStr(magbtm1.place);
//    FMLookup1.Title := Magbtm1.SiteName+' '+ FMLookup1.Title ;
    if FMLookup1.Execute(AddRecord) and (FMLookup1.RecordNumber <> '') then begin
      fmgQueuePointer.IENS := FMLookup1.RecordNumber;
      FMLabel1.IENS := FMLookup1.RecordNumber;
      fmgQueuePointer.GetAndFill;
      if fmgQueuePointer.ErrorList.Count > 0 then begin
        {loop through list of error objects}
        ToIens := '';
        for i:=0 to (fmgQueuePointer.ErrorList.Count-1) do begin
          err:= fmgQueuePointer.ErrorList.Items[i];
          if pos('is not a valid pointer',err.ErrorText.Text) <> 0 then begin
            ToIens := piece(err.ErrorText.Text,'The value ',2);
            ToIens := piece(ToIens,' is not',1);
            trim(ToIens);
            end;
          end;
        end
        else begin
          ToIens := fmgQueuePointer.GetField('1').FMDBInternal;
          ToIens := IntToStr(StrToInt(ToIens)+1);
          end;
        lblCQR.Caption := 'Current Queue Reference: '+ToIens;
        Queue := fmgQueuePointer.GetField('.01').FMDBExternal;
     end {queue type selected}
    else begin
      Result := false;
      end;
    end;
end;

procedure TfrmMagQueManSet.sbPreviousClick(Sender: TObject);
var prev: integer;
begin
   prev := MoreArray.IndexOf(dmMain.fmlQueueRecord.FromIEN) ;
   case prev of
     0: prev := 0;
      else dec(prev);
      end; //case prev of
   FMListBox1.Clear;
   if Prev = 0 then begin
     bbtnPrevious.Enabled := false;
     dmMain.fmlQueueRecord.GetList(FMListBox1.Items);
     FMListBoxUpdate(FMListBox1);
     if dmMain.fmlQueueRecord.More then bbtnMore.Enabled := true
     else bbtnMore.Enabled := false;
     exit;
     end;
   dmMain.fmlQueueRecord.FromIEN := MoreArray[Prev-1] ;
   dmMain.fmlQueueRecord.GetMore(FMListBox1.Items);
   FMListBoxUpdate(FMListBox1);
   if MoreArray.IndexOf(dmMain.fmlQueueRecord.FromIEN)=0 then begin
      bbtnPrevious.enabled := false;
      bbtnMore.Enabled := true;
      end;
end;
procedure TfrmMagQueManSet.FMListBoxUpdate(var ListBox: TFMListBox);
var i: integer;
begin
  ListBox.MultiSelect := false;
  ListBox.ExtendedSelect := false;
  for i := 0 to ListBox.Items.Count - 1 do begin
    ListBox.ItemIndex := i;
    ListBox.Items.Strings[i] := ListBox.GetSelectedRecord.ien +' '+
      ListBox.Items.Strings[i];
    end;
  ListBox.ItemIndex := 0;
  ListBox.Update;
end;


procedure TfrmMagQueManSet.sbQueSetClick(Sender: TObject);

begin
  with dmMain do begin
    if FMListBox1.ItemIndex <> -1 then begin
      FMLabel1.Caption := FMListBox1.GetSelectedRecord.IEN;
      if FMLabel1.Validate then begin
        FMFiler1.AddChgdControl(FMLabel1);
        FMFiler1.Update;
        fmgQueuePointer.GetAndFill;
        with dmMain.RPCBroker1 do begin
           Param[0].VALUE := IntToStr(magbtm1.place);
           Param[0].PTYPE := LITERAL;
           Param[1].VALUE := Queue;
           Param[1].PTYPE := LITERAL;
           RemoteProcedure := 'MAGQ QCNT';
           try
             Call;       //call
           except
             On E: EBrokerError do begin
             Application.terminate;
             end;
           end; // try
           end; //with broker
        ShowMessage('The current Queue pointer for '+Queue+' has been set to '+
          FMListBox1.GetSelectedRecord.IEN) ;
        frmMagQueManSet.Close;
        end // Validate
      else begin
        showmessage('Queue update not valid!');
        FMValidator1.DisplayErrors;
        end;
      end // <> -1
    else Showmessage('No "Queue Set" record selected!');
   end; // with dmMain
end;


procedure TfrmMagQueManSet.bbtnCancelClick(Sender: TObject);
begin
  ModalResult := mrCancel ;
end;

procedure TfrmMagQueManSet.sbMoreClick(Sender: TObject);
begin
  with dmMain do begin
  if fmlQueueRecord.More then begin
    FMListBox1.Clear;
    fmlQueueRecord.GetMore(FMListBox1.Items);
    FMListBoxUpdate(FMListBox1);
    if MoreArray.IndexOf(fmlQueueRecord.FromIEN) = -1 then
       MoreArray.Add(fmlQueueRecord.FromIEN);
    if fmlQueueRecord.More then begin
      bbtnMore.Enabled := true ;
      bbtnPrevious.Enabled := true;
      end
    else begin
      bbtnMore.Enabled := false;
      end;
    end;
  if fmlQueuePointer.ErrorList.Count > 0 then fmlQueuePointer.DisplayErrors;
  end;

end;

procedure TfrmMagQueManSet.FormCreate(Sender: TObject);
begin
  dmMain.FMComponentInit(self);
  Caption :=format('%s - %s - %s',[' '+Magbtm1.SiteName,'BP Queue Management by Status', Queue]);  ;
end;

end.
