unit MagViewLog;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created:
   Site Name: Silver Spring, OIFO
   Developers: Richard Price
   Description: BP Show log utility
   Note:
   }
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Menus, StdCtrls, ComCtrls,Printers;

type
  TfrmMagViewLog = class(TForm)
    MainMenu1: TMainMenu;
    mnuPrint: TMenuItem;
    ListBox1: TListBox;
    FindDialog1: TFindDialog;
    PrinterSetupDialog1: TPrinterSetupDialog;
    mnuSearch: TMenuItem;
    mnuFindNext: TMenuItem;
    mnuFind1: TMenuItem;
    Close1: TMenuItem;
    procedure FormShow(Sender: TObject);
    procedure mnuPrintClick(Sender: TObject);
    procedure mnuSearchClick(Sender: TObject);
    procedure FindDialog1Find(Sender: TObject);
    procedure mnuFindNextClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Close1Click(Sender: TObject);
    procedure FormActivate(Sender: TObject);

  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmMagViewLog: TfrmMagViewLog;

implementation

{$R *.DFM}
uses Magbtm1;
procedure TfrmMagViewLog.FormShow(Sender: TObject);
begin
  ListBox1.Items.LoadFromFile(magbtmf1.OpenDialog1.FileName);
end;

procedure TfrmMagViewLog.mnuPrintClick(Sender: TObject);
var
  PageNum: string;
  i,Ctr,x,PHeight,LineSpace,cnt: integer;
begin
  if Listbox1.SelCount < 1 then begin
    Windows.MessageBox(frmMagViewLog.Handle,PChar('No Log Records Selected'),' ',MB_SYSTEMMODAL);
    exit;
    end;
  if PrinterSetUpDialog1.Execute then begin
    Printer.BeginDoc;
    PHeight := Printer.PageHeight;
    LineSpace := PHeight DIV 60;
    Ctr := 1;
    x := 1;
    cnt := 0;
    PageNum := IntToStr(Printer.PageNumber);
    i := 0;
    repeat
      if ListBox1.Selected[i] then begin
        Printer.Canvas.TextOut(0,x,ListBox1.Items[i]);
        x := x + LineSpace;
        inc(cnt);
        inc(ctr);
        If ctr > 59 then begin
          Printer.NewPage;
          x := 0;
          Ctr := 0;
          PageNum := IntToStr(Printer.PageNumber);
          end;
        end;
      inc(i);
    until (cnt = ListBox1.SelCount) or
       (i = Listbox1.Items.Count + 1);
    Printer.EndDoc;
    end;
end;

procedure TfrmMagViewLog.mnuSearchClick(Sender: TObject);
begin
//  FindDialog1.Position := Point(RichEdit1.Left + RichEdit1.Width, RichEdit1.Top);
  FindDialog1.Position := Point(ListBox1.Left, ListBox1.Top);
  FindDialog1.Execute;
end;

procedure TfrmMagViewLog.FindDialog1Find(Sender: TObject);
  function FindText(FText:string;StartPos,EndPos:integer): integer;
    var
      i: integer;
    begin
      result := -1 ;
      i := StartPos;
      with ListBox1 do begin
        if Selected[StartPos] then begin
          if frdown in FindDialog1.Options then begin
            if i < items.count - 1 then inc(i);
            end
          else if i > 0 then dec(i);
          end; // if Selected[StartPos]
        repeat
          if frMatchCase in FindDialog1.Options then begin
            if pos(FText,Items[i]) <> 0 then result := i
            else if frdown in FindDialog1.Options then inc(i) else dec(i);
            end
          else begin
            if pos(UpperCase(FText),UpperCase(Items[i])) <> 0 then result := i
            else if frdown in FindDialog1.Options then inc(i) else dec(i);
            end;
        until (result <> -1) or (i = EndPos);
        end; // with ListBox1
    end;
  var
    FoundAt: LongInt;
    StartPos, ToEnd: integer;
  begin
    with ListBox1 do begin
      if ItemIndex <> 0 then StartPos := ItemIndex
      else StartPos := 0;
      if frDown in FindDialog1.Options then ToEnd := Items.Count - 1
      else ToEnd := 0;
      FoundAt := FindText(FindDialog1.FindText, StartPos, ToEnd);
      if FoundAt <> -1 then begin
        Selected[ItemIndex] := false;
        ItemIndex := FoundAt;
        Selected[FoundAt] := true;
        ListBox1.Update;
        end
      else Windows.MessageBox(frmMagViewLog.Handle,PChar('Cannot find '+'"'+FindDialog1.FindText+'"'),' ',MB_SYSTEMMODAL);
      end;
  end;

procedure TfrmMagViewLog.mnuFindNextClick(Sender: TObject);
begin
FindDialog1Find(self);
end;

procedure TfrmMagViewLog.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
MAGBTMF1.mnuOpenLog.checked := false;
end;

procedure TfrmMagViewLog.Close1Click(Sender: TObject);
begin
self.close;
end;

procedure TfrmMagViewLog.FormActivate(Sender: TObject);
begin
  {$ifdef Win32}
          SetForegroundWindow(frmMagViewLog.Handle);
  {$else}
          BringWindowToTop(frmMagViewLog.Handle);
  {$endif}
  Application.NormalizeTopMosts;
  SetWindowPos(frmMagViewLog.Handle, HWND_TOPMOST, 0,0,0,0,
   SWP_NOACTIVATE+SWP_NOMOVE+SWP_NOSIZE);


end;

end.
