unit magRemoteMailMember;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created:
   Site Name: Silver Spring, OIFO
   Developers: Richard Price
   Description: Main BP Queue Processor
   Note:
   }
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, MFunstr,StdCtrls, Buttons, Fmcmpnts, Diaccess, Fmcntrls, ExtCtrls;

type
  TfrmRemoteMailMember = class(TForm)
    fmeRemoteMailMember: TFMEdit;
    FMValidator1: TFMValidator;
    FMFiler1: TFMFiler;
    FMHelp1: TFMHelp;
    btnApply: TBitBtn;
    bbtnOK: TBitBtn;
    bbtnCancel: TBitBtn;
    bbtnHelp: TBitBtn;
    FMGets1: TFMGets;
    pnlExample: TPanel;
    StaticText1: TStaticText;
    StaticText2: TStaticText;
    procedure bbtnOKClick(Sender: TObject);
    procedure bbtnCancelClick(Sender: TObject);
    procedure btnApplyClick(Sender: TObject);
    procedure fmeRemoteMailMemberChange(Sender: TObject);
    procedure F1KeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormActivate(Sender: TObject);
    procedure fmeRemoteMailMemberClick(Sender: TObject);
    procedure fmeRemoteMailMemberKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure bbtnHelpClick(Sender: TObject);

  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  frmRemoteMailMember: TfrmRemoteMailMember;
  frmIEN: string;

implementation
{$R *.dfm}
uses MagSiteParameters,unitDM,magbtm1, magWebHelp;

procedure TfrmRemoteMailMember.bbtnOKClick(Sender: TObject);
var valid :string;
begin
   //enforce uppercase - managing a mixed case field in an indexed field is painful
  if btnApply.Enabled then begin
    //check here for validity
     fmeRemoteMailMember.Text := UpperCase(fmeRemoteMailMember.Text);
     valid :=  dmMain.MagFieldValidate(fmeRemoteMailMember.FMFile,'+1,',fmeRemoteMailMember.FMField,'',fmeRemoteMailMember.text);
     if pos('-1',piece(Valid,'^',1)) <> 0 then begin
       Windows.MessageBox(frmRemoteMailMember.Handle,PChar(piece(Valid,'^',2)),'',MB_SYSTEMMODAL);
       frmSiteParameters.SiteParametersDisplayHelp(fmeRemoteMailMember);
       fmeRemoteMailMember.Text := '';
       exit;
       end;
    //Check here for redundancy
    if StrToInt(dmMain.Find1(fmeRemoteMailMember.FMFile,frmIEN,'X',
      fmeRemoteMailMember.Text,'B','',true)) < 1 then
         if StrToInt(dmMain.AddFMEntry(fmeRemoteMailMember.FMFile,fmeRemoteMailMember.FMField,'+1'+frmIEN,fmeRemoteMailMember.Text,'E')) > 0
          then  //'Changes filed!'
         else Windows.MessageBox(frmRemoteMailMember.Handle,PChar('Update failed'),' ',MB_SYSTEMMODAL)
    else Windows.MessageBox(frmRemoteMailMember.Handle,PChar('This remote is already on file!'),' ',MB_SYSTEMMODAL);
    end // if btnApply.Enabled
  else ModalResult := mrOK;
end;

procedure TfrmRemoteMailMember.bbtnCancelClick(Sender: TObject);
begin
  ModalResult := mrOK;
end;

procedure TfrmRemoteMailMember.btnApplyClick(Sender: TObject);
begin
  bbtnOKClick(btnApply);
end;

procedure TfrmRemoteMailMember.fmeRemoteMailMemberChange(Sender: TObject);
begin
    fmeRemoteMailMember.Text := UpperCase(fmeRemoteMailMember.Text);
    btnApply.Enabled := true;
end;
procedure TfrmRemoteMailMember.F1KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if key = VK_F1 then
     frmSiteParameters.F1KeyDown(Sender,Key,Shift);
end;


procedure TfrmRemoteMailMember.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  frmRemoteMailMember.ModalResult := mrCancel;
  Action := caFree;
end;

procedure TfrmRemoteMailMember.FormActivate(Sender: TObject);
begin
  {$ifdef Win32}
          SetForegroundWindow(frmRemoteMailMember.Handle);
  {$else}
          BringWindowToTop(frmRemoteMailMember.Handle);
  {$endif}
  Application.NormalizeTopMosts;
  SetWindowPos(frmRemoteMailMember.Handle, HWND_TOPMOST, 0,0,0,0,
   SWP_NOACTIVATE+SWP_NOMOVE+SWP_NOSIZE);

end;

procedure TfrmRemoteMailMember.fmeRemoteMailMemberClick(Sender: TObject);
var
 valid: string;
begin
  if not fmeRemoteMailMember.FMModified then exit;
  with sender as TFMEdit do begin
    fmeRemoteMailMember.Text := UpperCase(fmeRemoteMailMember.Text);
    valid :=  dmMain.MagFieldValidate(FMFile,'+1,',FMField,'',text);
    if pos('-1',piece(Valid,'^',1)) <> 0 then begin
      Windows.MessageBox(frmRemoteMailMember.Handle,PChar(piece(Valid,'^',2)),'',MB_SYSTEMMODAL);
      frmSiteParameters.SiteParametersDisplayHelp(Sender);
      Text := '';
      SetFocus;
      btnApply.Enabled := false;
      exit;
      end;
    //Check here for redundancy
  if StrToInt(dmMain.Find1(fmeRemoteMailMember.FMFile,frmIEN,'X',
    fmeRemoteMailMember.Text,'B','',true)) > 0 then begin
    Windows.MessageBox(frmRemoteMailMember.Handle,PChar('This remote is already on file!'),' ',MB_SYSTEMMODAL);
    fmeRemoteMailMember.Text := '';
    btnApply.Enabled := false;
    end
  else btnApply.Enabled := true;
  end; // with sender
end;

procedure TfrmRemoteMailMember.fmeRemoteMailMemberKeyUp(Sender: TObject;
  var Key: Word; Shift: TShiftState);
begin
  if key = VK_Return then fmeRemoteMailMemberClick(Sender);
end;

procedure TfrmRemoteMailMember.bbtnHelpClick(Sender: TObject);
begin
  Application.CreateForm(TfrmMagBPHelp,frmMagBPHelp);
  frmMagBPHelp.WB.Navigate(ExtractFilePath(Application.ExeName)+
 'help/MAG_BP_User_Manual.htm#_Toc269903259');

end;

end.
