unit ImagInterfaces;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created:  Version 3.0.8
   Site Name: Silver Spring, OIFO
   Developers: Garrett Kirin
   [==
   Description:
   The Observer Design pattern defnes a One to many relationship between objects
   so that when one object (subject) changes state, all its dependents (observers)
    are notified and updated automatically.

    The subject sends out notifications without having to know who (or what class)
    its' observers are.  This gives the freedom to add and remove observers
    at any time.
    Also the subject doesn't know the concrete class of it's observers.
    Thus the coupling is minimal.

       Defined Interfaces for VistA Imaging.
        MagObserver : Attaches itself to a MagSubject and gets notified by
                a call to it's Update_ procedure of any changes.
        MagSubject : Accepts any number of Observers.  Notifies all Observers
        attached to it, whenever a change of state occurs.
       ==]
   Note:
   }
(*
        ;; +------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +------------------------------------------------------------------+

*)
interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls;

type

   IMagObserver = interface(IDispatch)
    ['{1BD79681-3D3D-11D4-A521-00C04F79A76D}']
    {The Observer has this method.  To get notified of a state change. }
    Procedure Update_(newstate : string);
end;
    IMagSubject = interface(IDispatch)
    ['{1BD79680-3D3D-11D4-A521-00C04F79A76D}']
        {Observers can Attach_ themselves to a Subject}
    Procedure Attach_(Observer : ImagObserver);
        {Observers can Detach_ themselves from a Subject}
    Procedure Detach_(Observer : ImagObserver);
        {In Notify_ Subject calls the Update_ method of all observers.
        (Note : other objects, code, can call the Notify method, forcing an update
         of all Observers.}
    Procedure Notify_(newstate : string);

        {Unused at this time.}
        {Observers and Call GetState to get current State.}
    Procedure GetState_(var State : string);
        {Unused at this time.}
    Procedure SetState_(State : string);
end;

implementation

end.
 