unit MagRowColSize;
    {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created:    8/1/2002
   Site Name: Silver Spring, OIFO
   Developers: Garrett Kirin
   [==   unit MagRowColSize;

   Description: Image Viewer Utility window : Allows editing of the Viewer Properties.
   Viewer Properties that are displayed for editing: are properties of TMagLayout.
        Rows, Columns, Maximum Image Count and Lock Image Size

   Objects and Forms in the Appication create instances of TRowColSize.
   The Execute method takes a reference to an instance of TMag4Viewer as a parameter
   This reference gives TRowColSize the ability to enable an Apply method.
   The user can 'Apply' and view the changes before exiting the TRowColSize
   Dialog window.

   ==]
   Note:
   }
(*
        ;; +------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +------------------------------------------------------------------+

*)
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, Spin, cMag4Viewer, ComCtrls, ToolWin, Menus, ExtCtrls;

type
  TRowColSize = class(TForm)
    Bevel1: TBevel;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    btnApply: TBitBtn;
    btnOK: TBitBtn;
    cbAbstracts: TCheckBox;
    cbAutoRedraw: TCheckBox;
    cbclearbeforeAdd: TCheckBox;
    cbdftLockSize: TCheckBox;
    cbDyncol: TCheckBox;
    cbDynrow: TCheckBox;
    cbFixedSize: TCheckBox;
    edtdftColumn: TEdit;
    edtdftMaxCount: TEdit;
    edtdftRow: TEdit;
    Label1: Tlabel;
    Label2: Tlabel;
    lbColumns: Tlabel;
    lbMaxCount: Tlabel;
    lbRows: Tlabel;
    MainMenu1: TMainMenu;
    N1x11: TMenuItem;
    N1x31: TMenuItem;
    N1x41: TMenuItem;
    N2x11: TMenuItem;
    N2x12: TMenuItem;
    N2x21: TMenuItem;
    N2x31: TMenuItem;
    N2x41: TMenuItem;
    N3x11: TMenuItem;
    N3x21: TMenuItem;
    N3x31: TMenuItem;
    N3x41: TMenuItem;
    N4x11: TMenuItem;
    N4x21: TMenuItem;
    N4x31: TMenuItem;
    N4x41: TMenuItem;
    RowCol1: TMenuItem;
    seColumns: TSpinEdit;
    seMaxCount: TSpinEdit;
    seRows: TSpinEdit;
    procedure BitBtn2Click(Sender: TObject);
    procedure btnApplyClick(Sender: TObject);
    procedure btnOKClick(Sender: TObject);
    procedure cbDyncolClick(Sender: TObject);
    procedure cbDynrowClick(Sender: TObject);
    procedure cbFixedSizeClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure N1x11Click(Sender: TObject);
    procedure N1x31Click(Sender: TObject);
    procedure N1x41Click(Sender: TObject);
    procedure N2x11Click(Sender: TObject);
    procedure N2x12Click(Sender: TObject);
    procedure N2x21Click(Sender: TObject);
    procedure N2x31Click(Sender: TObject);
    procedure N2x41Click(Sender: TObject);
    procedure N3x11Click(Sender: TObject);
    procedure N3x21Click(Sender: TObject);
    procedure N3x31Click(Sender: TObject);
    procedure N3x41Click(Sender: TObject);
    procedure N4x11Click(Sender: TObject);
    procedure N4x21Click(Sender: TObject);
    procedure N4x31Click(Sender: TObject);
    procedure N4x41Click(Sender: TObject);
    procedure seColumnsChange(Sender: TObject);
    procedure seMaxCountChange(Sender: TObject);
    procedure seRowsChange(Sender: TObject);
  private
    {   apply changes to the instance of TMag4Viewer }
    procedure ApplySettingsToViewer;
    procedure SetMaxMin;
    procedure SetRowCol(row, col: integer);
    {  determines if any changes have been made }
    procedure WinModified(tf: boolean);
    {  Get and Display MagViewLink Properties for the user to modify}
    procedure LinkToMagView(var MagViewLink: TMag4Viewer);
  public
    { called by any object, Viewer is a valid reference to an Instance of TMag4Viewer}
    procedure Execute(Viewer : TMag4Viewer);
  end;


var
  RowColSize: TRowColSize;
  CurViewer: TMag4Viewer;
implementation

{$R *.DFM}

{
********************************* TRowColSize **********************************
}
procedure TRowColSize.ApplySettingsToViewer;
begin
  WinModified(false);
  if CurViewer.LockSize then CurViewer.LockSize := false;
  application.processmessages;
  
  CurViewer.AbsWindow := cbAbstracts.checked;
  application.processmessages;
  if CurViewer.MaxCount <> seMaxCount.value then
    CurViewer.MaxCount := seMaxCount.value;
  application.processmessages;
  CurViewer.SetrowColcount(seRows.value, seColumns.value);
  application.processmessages;
  CurViewer.AutoRedraw := cbAutoRedraw.checked;
  CurViewer.clearbeforeadddrop := cbclearbeforeadd.checked;
  application.processmessages;
  if CurViewer.LockSize <> cbFixedSize.checked
    then CurViewer.LockSize := cbFixedSize.checked;
  application.processmessages;
end;

procedure TRowColSize.BitBtn2Click(Sender: TObject);
begin
  serows.Value := strtoint(edtdftRow.text);
  seColumns.Value := strtoint(edtdftColumn.text);
  setMaxMin;
  update;
  seMaxCount.Value := strtoint(edtdftMaxCount.text);
  cbFixedSize.checked := cbdftLockSize.checked;
  ApplySettingsToViewer;
end;

procedure TRowColSize.btnApplyClick(Sender: TObject);
begin
  ApplySettingsToViewer;
end;

procedure TRowColSize.btnOKClick(Sender: TObject);
begin
  ApplySettingsToViewer;
  modalresult := mrok;
end;

procedure TRowColSize.cbDyncolClick(Sender: TObject);
begin
  seColumns.Enabled := not cbDynCol.checked
end;

procedure TRowColSize.cbDynrowClick(Sender: TObject);
begin
  seRows.Enabled := not cbDynRow.checked
end;

procedure TRowColSize.cbFixedSizeClick(Sender: TObject);
begin
  WinModified(true);
end;

procedure TRowColSize.Execute(Viewer : TMag4Viewer);
var
  layout: Tmaglayout;
begin
  with TRowColSize.create(nil) do
    begin
      LinkToMagView(Viewer);
      try
      ShowModal;
      application.processmessages;
      finally
      release;
      end;
   end;
end;

procedure TRowColSize.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  WinModified(false);
end;

procedure TRowColSize.FormCreate(Sender: TObject);
begin
  CurViewer := nil;
end;

procedure TRowColSize.LinkToMagView(var MagViewLink: TMag4Viewer);
var
  layout: Tmaglayout;
begin
  CurViewer := MagViewLink;
  seRows.value := MagViewLink.RowCount;
  SeColumns.value := MagViewLink.ColumnCount;
  seMaxCount.value := MagViewLink.Maxcount;
  cbAbstracts.checked := MagViewLink.AbsWindow;
  cbFixedSize.checked := MagViewLink.LockSize;
  cbAutoRedraw.checked := MagViewLink.AutoRedraw;
  cbclearbeforeadd.checked := MagViewLink.clearbeforeadddrop;
  { layout isn't used yet.  It's for a future versioin. }
  layout := MagViewLink.DefaultLayout;
  edtdftRow.Text := inttostr(layout.Rows);
  edtdftColumn.Text := inttostr(layout.Columns);
  edtdftMaxCount.Text := inttostr(layout.MaxNumberOfImages);
  cbdftLockSize.checked := layout.LockImageState;
  if CurViewer.Caption <> '' then caption := CurViewer.Caption + ' settings';
      //lbDefaultLayout.Caption := MagViewLink.GetDefaultLayoutDesc;
    //p8t25
  (* if cbAbstracts.checked then
     begin
       seColumns.enabled := false;
       lbColumns.Enabled := false;
     end;
     *)
  winmodified(false);
end;

procedure TRowColSize.N1x11Click(Sender: TObject);
begin
  SetRowCol(1, 1);
end;

procedure TRowColSize.N1x31Click(Sender: TObject);
begin
  SetRowCol(1, 3);
end;

procedure TRowColSize.N1x41Click(Sender: TObject);
begin
  SetRowCol(1, 4);
end;

procedure TRowColSize.N2x11Click(Sender: TObject);
begin
  SetRowCol(2, 1);
end;

procedure TRowColSize.N2x12Click(Sender: TObject);
begin
  SetRowCol(1, 2);
end;

procedure TRowColSize.N2x21Click(Sender: TObject);
begin
  SetRowCol(2, 2);
end;

procedure TRowColSize.N2x31Click(Sender: TObject);
begin
  SetRowCol(2, 3);
end;

procedure TRowColSize.N2x41Click(Sender: TObject);
begin
  SetRowCol(2, 4);
end;

procedure TRowColSize.N3x11Click(Sender: TObject);
begin
  SetRowCol(3, 1);
end;

procedure TRowColSize.N3x21Click(Sender: TObject);
begin
  SetRowCol(3, 2);
end;

procedure TRowColSize.N3x31Click(Sender: TObject);
begin
  SetRowCol(3, 3);
end;

procedure TRowColSize.N3x41Click(Sender: TObject);
begin
  SetRowCol(3, 4);
end;

procedure TRowColSize.N4x11Click(Sender: TObject);
begin
  SetRowCol(4, 1);
end;

procedure TRowColSize.N4x21Click(Sender: TObject);
begin
  SetRowCol(4, 2);
end;

procedure TRowColSize.N4x31Click(Sender: TObject);
begin
  SetRowCol(4, 3);
end;

procedure TRowColSize.N4x41Click(Sender: TObject);
begin
  SetRowCol(4, 4);
end;

procedure TRowColSize.seColumnsChange(Sender: TObject);
begin
  winmodified(true);
end;

procedure TRowColSize.seMaxCountChange(Sender: TObject);
begin
  winmodified(true);
end;

procedure TRowColSize.seRowsChange(Sender: TObject);
begin
  winmodified(true);
end;

procedure TRowColSize.SetMaxMin;
begin
  if ((semaxcount.value) < (seRows.Value * secolumns.Value))
    then seMaxCount.value := (seRows.Value * secolumns.Value);
end;

procedure TRowColSize.SetRowCol(row, col: integer);
begin
  seRows.value := row;
  seColumns.value := col;
  if seMaxCount.Value < (row * col) then seMaxCount.value := (row * col);
  WinModified(true);
end;

procedure TRowColSize.WinModified(tf: boolean);
begin
  btnApply.enabled := tf;
  btnOK.enabled := tf;
end;

end.


