unit MaggMsgu;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created:
   Site Name: Silver Spring, OIFO
   Developers: Garrett Kirin
   [==      unit MaggMsgu;
   Description: Imaging Message History Window, and Advanced Image information
      window. (for System managers)
   Long term design is to have a central message dispatcher object.  Which will do
   various funtions with the message text.  Main funtion is to maintain a message
   history (message log).  
   This Utility window keeps a User defined number of the last number messages
   that were displayed in the application.  It is mainly for debugging purposes.
   User with the MAG SYSTEM key will see more messages that average user.  The
   extra messages usually contain DataBase, Network, OS specific information.

        Other controls in the application can call methods to display messages to
   a specific TPanel Object and have the message logged at the same time.

   Future  : Method to save the message history to a log file.
             Method to save to VistA (IMAGING WINDOWS SESSION file)
             Method to save specific coded messages to VistA Log File
             IMAGE ACCESS LOG, IMAGING WINDOWS SESSION Files
             Method to 'Send' history to Customer support/Selected mail recipient
   ==]

   Note:}
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)


interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, ComCtrls, ToolWin, FileCtrl, Menus, umagclasses;

type
  TMaggMsgf = class(TForm)
    MsgMemo: TMemo;
    ToolBar1: TToolBar;
    FontDialog1: TFontDialog;
    sysmemo: TMemo;
    tbWrap: TCheckBox;
    ToolButton3: TToolButton;
    TBSYSMEMO: TCheckBox;
    ToolButton1: TToolButton;
    tbStayOnTop: TToolButton;
    FileListBox1: TFileListBox;
    ToolButton2: TToolButton;
    bcolor: TToolButton;
    cd1: TColorDialog;
    PopupMenu1: TPopupMenu;
    mMsgcount: TMenuItem;
    SaveMessagehistorylisttofile1: TMenuItem;
    SaveDialog1: TSaveDialog;
    SaveSystemMessagetoFile1: TMenuItem;
    procedure tbWrapClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure tbSysMemoClick(Sender: TObject);
    procedure ToolButton1Click(Sender: TObject);
    procedure tbStayOnTopClick(Sender: TObject);
    procedure ToolButton2Click(Sender: TObject);
    procedure bcolorClick(Sender: TObject);
    procedure mMsgcountClick(Sender: TObject);
    procedure SaveMessagehistorylisttofile1Click(Sender: TObject);
    procedure PopupMenu1Popup(Sender: TObject);
    procedure FormActivate(Sender: TObject);
  private
    FMsgCount: integer;
    FSysMsgCount: integer; { Private declarations }
    function CurTime: string;
  public
    {   Set to TRUE if user has MAG SYSTEM key}
    syskey: boolean;
    {   Display a message to a TPanel object and log it in the history
        c : is a code sting containing any combinaton of the defined codes.
                The codes determine how to display the message.
          's' - system message, add to sysmemo.
          'd' - display in a dialog window.
          ''  - add to msgmemo, (and sysmemo)
          'e' - Error message, display 'Error' text with message and dialog
          'QA' - Integrity Error.  Display 'QUESTIONABLE INTEGRITY' in message
          }
    procedure MagMsg(c, s: string;  pmsg: Tpanel = nil);
    {   Display a Stringlist, and Log in in the history}
    procedure MagMsgs(c : string; t : tstringlist);
    {   if syskey=TRUE the system memo is visible.}
    procedure EnableSysMemo(value: boolean);
    {TODO:  get this out of here, move to unit with related functions }
    {   Erase all files in the \cache directory.  Usually called when application
        is being destroyed }
    procedure EraseCacheFiles;
    {  Specific message for debugging, stores a free text message and the Image IEN
       to the History file.}
    procedure IENCheck(whatImDoing: string; ien: string);
    {  Specific message for debugging, stores all Image IENS from the Tlist object
       to the History file.}
    procedure IENsToMsgWindow(t: Tlist);
  end;

var
  MaggMsgf: TMaggMsgf;

implementation
//uses magbroker;
{$R *.DFM}

procedure TMaggMsgf.IENsToMsgWindow(t: Tlist);
var I: integer;
  Iobj : TImageData;
  s: string;
begin
  s := '';
  for I := 0 to t.count - 1 do
  begin
    Iobj := t[i];
    s := s + ' ' + inttostr(i) + '-' + Iobj.mag0;
    if (((i + 1) mod 5) = 0) then
    begin
      magmsg('s', s, nil);
      s := '';
    end;
  end;
  if (s <> '') then magmsg('s', s, nil);
end;

procedure TMaggMsgf.IENCheck(whatImDoing: string; ien: string);
begin
  try
    sysmemo.lines.add(curtime + whatimdoing + '  ' + ien);
  except
    messagedlg('--- Invalid IEN' + #13 + 'Queing Images to Copy from JukeBox', mtconfirmation, [mbok], 0);
    msgmemo.lines.add(curtime + whatimdoing + '---  Error with IEN');
    sysmemo.lines.add(curtime + whatimdoing + '---  Error with IEN');
    sysmemo.lines.add(curtime + 'IEN is :' + ien);
   //sysmemo.lines.SaveToFile(apppath + '\ErrorLog.txt');
   // RPMagLogErrorText(sysmemo.lines, 30);
  end;

end;

procedure TMaggMsgf.MagMsg(c, s: string;  pmsg: Tpanel = nil);
var t: string;
  i: integer;
begin
  c := uppercase(c);
  if (pos('S', c) = 0) and (pmsg <> nil) then pmsg.font.color := clblack;
  if (pos('QA',c) > 0)
    then s := 'QUESTIONABLE INTEGRITY'+#13+#13
                 +s
                 +#13+#13+'Report this problem to Imaging Support.' ;
  if pos('D', c) > 0 then
  begin
    if pos('E', c) > 0 then messagedlg(s, mtError, [mbok], 0)
    else messagedlg(s, mtconfirmation, [mbok], 0);
  end;

  if pMsg.caption = s then exit; { gek 4/1/97 stop multiple lines of 'processing...'}
 { if this is a system message and user doesn't have a system key, we quit}
  if syskey then sysmemo.bringtofront;
  if pos('S', c) > 0 then
  begin

   {all are displayed except for 'S' (System)(Silent) types}
    sysmemo.lines.add(CurTime + ' * ' + s);
    exit;
  end;

  if pmsg <> nil then if pos('E', c) > 0 then pMsg.font.color := clred;

  while pos(#13, s) > 0 do
  begin
    i := pos(#13, s);
    t := s;
    t := copy(s, 1, i - 1) + ' ' + copy(s, i + 1, 240);
    s := t;
  end;

  if pmsg <> nil then
  begin
    pMsg.caption := s;
    pMsg.update;
  end;
  {   All messages are logged to msgwindow }
  { ..the next three lines have been changed in 3.0 to handle
      the problem of the message window not being open.  Messages are queued
      until the message window is created, then all sent.}
  msgmemo.lines.add(curtime + s);
  sysmemo.lines.add(curtime + s);
  while msgmemo.lines.count > FMsgCount do msgmemo.lines.delete(0);
  while sysmemo.lines.count > FSysMsgCount do sysmemo.lines.delete(0);

end;

function TMaggMsgf.CurTime: string;
begin
  result := FormatDateTime('hh:mm:ss', now) + '  ';
end;

procedure TMaggMsgf.tbWrapClick(Sender: TObject);
begin
  if tbWrap.checked then
  begin
    MsgMemo.wordwrap := True;
    MsgMemo.scrollbars := ssVertical;
    SysMemo.wordwrap := True;
    SysMemo.scrollbars := ssVertical;
  end
  else
  begin
    MsgMemo.wordwrap := False;
    MsgMemo.scrollbars := ssBoth;
    SysMemo.wordwrap := False;
    SysMemo.scrollbars := ssBoth;
  end;
end;

procedure TMaggMsgf.FormCreate(Sender: TObject);
begin
  syskey := false;
  FMsgCount := 200;
  FSysMsgCount := 300;
end;

procedure TMaggMsgf.EnableSysMemo(value: boolean);
begin
  syskey := value;
  if syskey then
  begin
    sysmemo.bringtofront;
    tbSysmemo.visible := true;
    tbsysmemo.CHECKED := true;
    bcolor.visible := true;
  end
  else
  begin
    sysmemo.sendtoback;
    tbsysmemo.visible := FALSE;
    bcolor.visible := false;
  end;
  Update;
end;

procedure TMaggMsgf.tbSysMemoClick(Sender: TObject);
begin
  if tbSysMemo.CHECKED then sysmemo.bringtofront
  else sysmemo.sendtoback;
end;

procedure TMaggMsgf.ToolButton1Click(Sender: TObject);
begin
  fontdialog1.Font := msgmemo.font;
  if fontdialog1.execute then
  begin
    msgmemo.font := fontdialog1.font;
    sysmemo.font := fontdialog1.font;
  end;

end;

procedure TMaggMsgf.tbStayOnTopClick(Sender: TObject);
begin
  if tbStayOnTop.down then maggmsgf.FormStyle := fsStayOnTop
  else maggmsgf.FormStyle := fsNormal;
end;

procedure Tmaggmsgf.EraseCacheFiles;
var i: integer;
  xfile: string;
begin
{TODO:  We get error if Video File is opened}
{file won't be erased if the mPlay32.exe is still open with a file.}
  for i := filelistbox1.Items.Count - 1 downto 0 do
  begin
    xfile := filelistbox1.Items[i];
    deletefile(xfile);
  end;
  application.processmessages;
end;

procedure TMaggMsgf.ToolButton2Click(Sender: TObject);
begin
  MsgMemo.clear;
  sysmemo.clear;
end;

procedure TMaggMsgf.bcolorClick(Sender: TObject);
begin
  if cd1.execute then sysmemo.color := cd1.color;
end;

procedure TMaggMsgf.mMsgcountClick(Sender: TObject);
begin
  try
    FMsgcount := strtoint(InputBox('Number of messages to keep in memory', 'Number of messages to keep in memory', inttostr(Fmsgcount)));
    FSysMsgCount := FMsgCount + 200;
  finally
  end;
end;

procedure TMaggMsgf.SaveMessagehistorylisttofile1Click(Sender: TObject);
begin
  if SaveDialog1.execute then SysMemo.Lines.SaveToFile(savedialog1.filename);

end;

procedure TMaggMsgf.PopupMenu1Popup(Sender: TObject);
begin
  mmsgcount.caption := 'Retain the last ' + inttostr(Fmsgcount) + ' messages...';
end;

procedure TMaggMsgf.MagMsgs(c : string; t: tstringlist);
var i : Integer;
begin
for i := 0 to t.count-1 do
  begin
    magmsg(c,t[i]);
  end;
end;

procedure TMaggMsgf.FormActivate(Sender: TObject);
begin
  sysmemo.ScrollBy(9,9);
end;

end.
