unit Maggplku;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created: Version 2.0
   Site Name: Silver Spring, OIFO
   Developers: Garrett Kirin
   [==
   Description:    unit Maggplku;
        This form handles the Patient Lookup functionality.  It is created and
        called from the TMag4Pat object.
    - RPC calls to get and display a list of patients from the DB that match
      the users imput.
    - allows user to select one patient only.
    - Patient information is querried from the database for selected patient.
    - The RPC's for Patient Sensitivity are called.
    - Patient Sensitivity warnings are displayed.
    - Sensitive Patient Logging is done when needed.
    - Means Test warnings are displayed if appropriate.
    ==]
   Note: }
(*
        ;; +---------------------------------------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +---------------------------------------------------------------------------------------------------+

*)

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, { maggut1, } xwbut1, Buttons, ExtCtrls, {MagBroker,}
  Trpcb, ComCtrls {, magpositions}, cMagDBBroker, uMagUtils;

type
  Tmaggplkf = class(TForm)
    lPat: TListBox;
    FontDialog1: TFontDialog;
    Panel1: TPanel;
    lCsmsg: TLabel;
    Panel3: TPanel;
    Label1: TLabel;
    ePatient: TEdit;
    Panel2: TPanel;
    bbok: TBitBtn;
    bbCancel: TBitBtn;
    bbHelp: TBitBtn;
    bbFont: TBitBtn;
    StatusBar1: TStatusBar;
    procedure ePatientKeyDown(Sender: TObject; var Key: Word;   Shift: TShiftState);
    procedure lPatClick(Sender: TObject);
    procedure bbokClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure bbFontClick(Sender: TObject);
    procedure lPatDblClick(Sender: TObject);
    procedure bbCancelClick(Sender: TObject);
    procedure lPatKeyDown(Sender: TObject; var Key: Word;  Shift: TShiftState);
    procedure FormShow(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure bbHelpClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure lPatEnter(Sender: TObject);
    procedure ePatientEnter(Sender: TObject);
    procedure bbokEnter(Sender: TObject);
    procedure bbCancelEnter(Sender: TObject);
    procedure bbHelpEnter(Sender: TObject);
    procedure bbFontEnter(Sender: TObject);
    procedure ePatientDblClick(Sender: TObject);

  private
    tdfn: tstringlist;
    procedure WMGetMinMaxInfo(var message: TWMGetMinMaxInfo); message WM_GETMINMAXINFO;
    procedure WinMsg(s: string);
    procedure SaveSelectedDFN;
    procedure EntryIsSelected;
    procedure DoPatLookup;


  public
        {       DataBase connection. }
    FDBBroker: TMagDBBroker;
        {       Old version set this public value when a patient was selected
                Newer design 'Select' method.  returns DFN on success.}
    FSelectedDFN: string;
        {       Not Implemented}
    FScreen: string;
        {       Now, this is not changed.  It is always true.
                If TRUE, selection window will not be shown when only one
                patient matches user input.
                If FALSE, selection window will always be shown.}
    FExitOnOneHit: boolean;
        {       makes RPC to search for 'patstr'.  Shows selection window
                if there are multiple matches. This method performs
                all Sensitivity and Means Test checking.}
    function Patlookup(patstr: string; var xmsg: string): boolean;
        {       same as PatLookup, but it returns DFN in Variable Param
                Note: this is called by TMag4Pat object. This method performs
                all Sensitivity and Means Test checking.}
    function Select(input: string; var selectmsg: string; var DFNx: string): boolean;
        {       Makes RPC to see if user can access this patient.}
    function Accessallowed(DFNX: string; var xmsg: string): boolean;
        {       Clears all patient information from this window}
    procedure ClearAll;
  end;

var
  maggplkf: Tmaggplkf;

implementation

uses fMagPatAccess {Ptaccess ){, MaggMsgu};

{$R *.DFM}

procedure Tmaggplkf.ePatientKeyDown(Sender: TObject; var Key: Word;  Shift: TShiftState);
begin
  if ((KEY <> VK_RETURN) or (ePatient.text = '')) then EXIT;
  DoPatLookup;
end;

procedure Tmaggplkf.DoPatLookup;
var xmsg : string;
begin
  xmsg := '';
  if not Patlookup(epatient.text, xmsg) then
  begin
    winmsg(xmsg);
    exit;
  end;
  if (lpat.items.count = 1) and (FExitOnOneHit) then modalresult := mrok;
  if (lpat.items.count > 0) then
  begin
    lpat.setfocus;
    bbOK.Enabled := true;
  end;
end;
        {       makes RPC to search for 'patstr'.  Shows selection window
                if there are multiple matches. This method performs
                all Sensitivity and Means Test checking.}
function Tmaggplkf.Patlookup(patstr: string; var xmsg: string): boolean;
var
  i: longint;
  str: string;
  t: Tstringlist;
begin
  result := false;
  WinMsg('Searching for "' + patstr + '"');

  {  str := patient file + number to return + user input ; }
  str := '100^' + UPPERCASE(patstr) + '^';
  {  if certain fields are wanted, make a string of ';' delimited field numbers and make
      that the third piece.   4th piece is unused, 5th..n pieces are the 'Screen' if any.
      if (screen.caption <> '') then str := str +'^^'+screen.caption; {}
  ePatient.text := patstr;
  ePatient.update;
  t := tstringlist.create;
  tdfn.clear;
  lpat.clear;
  bbok.enabled := false;
  try
    if not FDBBroker.RPMagPatLookup(STR, t, xmsg) then
    begin
      WinMsg(xmsg);
      exit;
    end;
    result := true;
    WinMsg('');
    for I := 0 to t.count - 1 do
    begin
      lpat.items.add(magpiece(t[i], '^', 1));
      tdfn.Add(magpiece(t[i], '^', 2));
    end;
    lPat.itemindex := 0; {templistbox.sorted := true;}
    SaveSelectedDFN;
  finally
    t.free;
  end;
end;

procedure Tmaggplkf.lPatClick(Sender: TObject);
begin
  if (lpat.Items.Count = 0) and (epatient.Text <> '') then
    begin
      DoPatLookup;
      exit;
    end;

  bbok.enabled := lpat.ItemIndex <> -1;
  SaveSelectedDFN;
end;

procedure Tmaggplkf.SaveSelectedDFN;
begin
  if lpat.ItemIndex = -1 then FselectedDFN := '0'
  else
  FSelectedDFN := MagStripSpaces(tdfn[lpat.itemindex]);
end;

procedure Tmaggplkf.bbokClick(Sender: TObject);
begin
  EntryIsSelected;
end;

procedure TMaggplkf.EntryIsSelected;
var t: tstringlist;
begin
  t := tstringlist.create;
  try
    if lPat.itemindex = -1
      then WinMsg('You must Select a list entry !!')
    else
    begin
      if FDBBroker.RPMaggPatBS5Chk(FSelectedDFN, t)
        then modalresult := mrOK
      else
        if frmPatAccess.execute(self, t, 5)
          then modalresult := mrOK
        else winmsg('User has Canceled Selection.');
    end;
  finally
    t.free;
  end;
end;

procedure Tmaggplkf.FormClose(Sender: TObject; var Action: TCloseAction);
begin
{HERE SHOULD BE CLEANUP}
  ClearAll;
  ePatient.setfocus;
end;

        {       Clears all patient information from this window}
procedure TMaggplkf.ClearAll;
begin
  winmsg('');
  ePatient.clear;
  lPat.clear;
  caption := 'Patient Lookup';
  lCsMsg.caption := ' type Patient Name (partial is ok), or full SSN, or Last Initial and Last 4 of SSN.   Press Enter.'
end;

procedure TMaggplkf.WinMsg(s: string);
begin
  statusbar1.panels[0].text := s;
  Statusbar1.update;
end;

procedure Tmaggplkf.bbFontClick(Sender: TObject);
begin
  if fontdialog1.execute then
  begin lPat.font := fontdialog1.font;
  end;
end;

procedure Tmaggplkf.lPatDblClick(Sender: TObject);
begin
  EntryIsSelected;
end;

procedure Tmaggplkf.bbCancelClick(Sender: TObject);
begin
  modalresult := mrCancel;
end;

procedure Tmaggplkf.lPatKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (key = vk_return) then EntryIsSelected;
end;

procedure Tmaggplkf.FormShow(Sender: TObject);
begin
  winmsg('');
  bbok.Enabled := lpat.items.count > 0;
  if lpat.items.count > 0 then lpat.setfocus
  else epatient.setfocus;
end;

procedure Tmaggplkf.FormCreate(Sender: TObject);
begin
{ form position is Screen Center}
  tdfn := tstringlist.create;
  {default to this.  changing this value isn't implemented}
  FExitOnOneHit := true;
end;

procedure Tmaggplkf.FormDestroy(Sender: TObject);
begin
{ form position is Screen Center}
  tdfn.free;
end;

procedure Tmaggplkf.bbHelpClick(Sender: TObject);
begin
  APPLICATION.HELPCONTEXT(153);
end;

{procedure WMGetMinMaxInfo( var message : TWMGetMinMaxInfo); message WM_GETMINMAXINFO;}

procedure Tmaggplkf.WMGetMinMaxInfo(var message: TWMGetMinMaxInfo);
var hy, wx: integer;
begin
  hy := trunc(245 * (pixelsperinch / 96));
  wx := trunc(525 * (pixelsperinch / 96));
  with message.minmaxinfo^ do begin
    ptmintracksize.x := wx;
    ptmintracksize.y := hy;
  end;
  message.result := 0;
  inherited;

end;

        {       same as PatLookup, but it returns DFN in Variable Param
                Note: this is called by TMag4Pat object. This method performs
                all Sensitivity and Means Test checking.}
function Tmaggplkf.Select(input: string; var selectmsg: string; var DFNx: string): boolean;
begin
  result := false;
  DFNx := '';
        {       If no text to search for, open the Patient Lookup window.}
  if input = '' then
  begin
    Maggplkf.showmodal;
    if maggplkf.ModalResult = mrCancel
      then
    begin
      selectmsg := 'Patient Lookup : ' + input + '  Canceled';
      exit;
    end;
  end
  else
  begin
    if not patlookup(input, selectmsg) then
    begin
      // maggmsgf.MagMsg('de', selectmsg, nilpanel);
      exit;
    end;
        { if count is 1 then we got exact hit, so skip the modal window.}
    if maggplkf.lpat.items.count > 1 then
    begin
      bbok.enabled := true;
      Maggplkf.showmodal;
      bbok.enabled := false;
      if maggplkf.ModalResult = mrCancel
        then
      begin
        selectmsg := 'Patient Lookup : ' + input + '  Canceled';
        exit;
      end;
    end;
  end;
{ modal result is OK ( user must have selected a patient for modal window to return OK )
  or only one hit on lookup.  In this case 0 entry is selected and FSelectedDFN has been set. {}

{ Check the BS5 cross reference. To make sure this is the desired patient.
  This has been done on the Mouse Click or <Enter> in the list box, calling  EntryIsSelected Procedure {}

{ Now we'll test for Accesss allowed to sensitive patient, and display the Means test required
  message if needed. {}

  if AccessAllowed(FselectedDFN, selectmsg) then
  begin
    result := true;
    DFNx := FselectedDFN;
  end;

end;

        {       Makes RPC to see if user can access this patient.}
function Tmaggplkf.Accessallowed(DFNx: string; var xmsg: string): boolean;
var
  t: tstringlist;
  SensitiveCode: integer;
begin
  result := false;
  t := tstringlist.create;
  try
    try
      FDBBroker.RPDGSensitiveRecordAccess(DFNx, sensitivecode, t);
(*         DG SENSITIVE RECORD ACCESS
           Output value line 1 of RESULT will be made less granular

    Possible sensitivecode's returned  =  action required

           -1 = RPC/API failed
           0 = No display or action required
           1 = Display warning message
           2 = Display warning message - require OK to continue
           3 = Display warning message - do not continue
           If the output value is 1 (display warning message) entry in DG SECURITY LOG file is automatically made; GUI application does not need to take action to log access   *)

(*         DG SENSITIVITY RECORD BULLETIN
           Input parameter ACTION (send bulletin, set log, or both) will be made optional with 'both' being the default value
           Input parameter DG1 (inpatient/outpatient status) will be removed     *)

      case SensitiveCode of
        -1: begin
            result := false;
            xmsg := t[0];
          end;
        0: begin
            result := true;
            xmsg := 'Access is Allowed';
          end;
        1: begin
            result := true;
                {       frmPatAccess just displays the warning message}
            frmPatAccess.execute(self, t, 1);
            xmsg := 'Access to this restricted patient has been logged.';
          end;

        2: begin
            if frmPatAccess.execute(self, t, 2)
              then
            begin
              result := true;
              xmsg := 'Access to this restricted patient has been logged.';
              FDBBroker.RPDGSensitiveRecordBulletin(DFNx);
            end
            else
            begin
              result := false;
              xmsg := 'User has Canceled access to the patient.'
            end;
          end;

        3: begin
            result := false;
            frmPatAccess.execute(self, t, 3);
            xmsg := 'Access to this restricted patient is Not Allowed';
          end;

      end; {CASE }

      if result then
        {    if access is allowed, we'll check if a Means Test is required.}
      begin
        winmsg(xmsg);
        FDBBroker.RPDGChkPatDivMeansTest(DFNx, sensitivecode, t);
        if (sensitivecode = 1) then frmPatAccess.execute(self, t, 4);
      end;

    except
      on E: EXCEPTION do
      begin
        result := false;
        if (POS('<SUBSCRIPT>', E.MESSAGE) > 0) then xmsg := '' //Demotest 'The Remote Procedure Call ' + FDBBroker.broker.remoteprocedure + ' doesn''t exist on VISTA.  Please Call IRM'
        else xmsg := E.message;
      end;
    end;
  finally
    t.free;
    cursor := crDefault;
  end;
end;

procedure Tmaggplkf.lPatEnter(Sender: TObject);
begin
  if lpat.items.count > 0 then WinMsg('<click> or <arrow-key> to highlite an entry.  <Dbl-Click> or <Enter> to select.')
  else begin
    WinMsg('');
    bbok.enabled := false;
  end;
  if (lpat.itemindex > -1) then bbok.enabled := TRUE;
end;

procedure Tmaggplkf.ePatientEnter(Sender: TObject);
begin
  WinMsg('type Patient Name (partial is ok), or full SSN, or Last Initial Last 4 of SSN.   Press <Enter>.');
end;

procedure Tmaggplkf.bbokEnter(Sender: TObject);
begin
  WinMsg('Press Enter to select highlited entry.');
end;

procedure Tmaggplkf.bbCancelEnter(Sender: TObject);
begin
  WinMsg('Press Enter to Cancel Patient Lookup.');
end;

procedure Tmaggplkf.bbHelpEnter(Sender: TObject);
begin
  WinMsg('Press Enter for Help on Patient Lookup.');
end;

procedure Tmaggplkf.bbFontEnter(Sender: TObject);
begin
  WinMsg('Press Enter to select a new Font.');
end;

procedure Tmaggplkf.ePatientDblClick(Sender: TObject);
begin
DoPatLookup;
end;

end.
