unit Maggrpcu;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created:   Version 2.5
   Site Name: Silver Spring, OIFO
   Developers: Garrett Kirin
   [==    unit Maggrpcu;
   Description: Imaging Report window.  Displays reports from associated procedures.
      exams, Notes etc.
    -  created by other classes when a Report window is needed.
    -  Has method to Print the report.
    -  Has option to open another report window.  user can compare reports.

    All forms of Type : Tmaggrpcf are added to the Dynamic report menu on
    the main window.   User can open multiple and easily switch between them.
      ==]

   Note:
   }
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)


interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, Buttons, Menus, Printers,
  //VA
  magpositions, umagutils;

type
  Tmaggrpcf = class(TForm)
    pDesc: TPanel;
    Memo1: TMemo;
    FontDialog1: TFontDialog;
    PrinterSetupDialog1: TPrinterSetupDialog;
    PrintDialog1: TPrintDialog;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Exit1: TMenuItem;
    N1: TMenuItem;
    PrintSetup1: TMenuItem;
    Print1: TMenuItem;
    N2: TMenuItem;
    N3: TMenuItem;
    New1: TMenuItem;
    N4: TMenuItem;
    Help1: TMenuItem;
    ImageReports1: TMenuItem;
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SpeedButton1Click(Sender: TObject);
    procedure N2Click(Sender: TObject);
    procedure Print1Click(Sender: TObject);
    procedure PrintSetup1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure New1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ImageReports1Click(Sender: TObject);
  private
    {  maintain minimum and Maximum values for window size}
    procedure WMGetMinMaxInfo(var message: TWMGetMinMaxInfo); message WM_GETMINMAXINFO;
  public
    {  open a new report window.  User can compare reports, open multiple }
    procedure NewReportWindow;
  end;

var
  maggrpcf: Tmaggrpcf;
  PrintText: text;
implementation

{$R *.DFM}

procedure Tmaggrpcf.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if (uppercase(name) = 'MAGGRPCF') then action := caHide
  else action := cafree;
end;

procedure Tmaggrpcf.WMGetMinMaxInfo(var message: TWMGetMinMaxInfo);
var hy, wx: integer;
begin
  hy := trunc(100 * (pixelsperinch / 96));
  wx := trunc(200 * (pixelsperinch / 96));
  with message.minmaxinfo^ do begin
    ptmintracksize.x := wx;
    ptmintracksize.y := hy;
  end;
  message.result := 0;
  inherited;

end;

procedure Tmaggrpcf.SpeedButton1Click(Sender: TObject);
begin
  if fontdialog1.execute
    then memo1.font := fontdialog1.font;
end;

procedure Tmaggrpcf.N2Click(Sender: TObject);
begin
fontdialog1.font := memo1.font;
  if fontdialog1.execute
    then memo1.font := fontdialog1.font;
end;

procedure Tmaggrpcf.Print1Click(Sender: TObject);
var
  Line: Integer; {declare an integer variable for the number of lines of text}
begin
  if PrintDialog1.Execute then
  begin
    AssignPrn(PrintText); {assign the global variable PrintText to the printer}
    Rewrite(PrintText); {create and open the output file}
    {   assign the current Font setting to the Printer object's canvas}
    Printer.Canvas.Font := Memo1.Font;
    {   write the contents of the Memo field to the   printer object}
    for Line := 0 to Memo1.Lines.Count - 1 do Writeln(PrintText, Memo1.Lines[Line]);
    System.Close(PrintText);
  end;
end;

procedure Tmaggrpcf.PrintSetup1Click(Sender: TObject);
begin
  PrinterSetupDialog1.execute;
end;

procedure Tmaggrpcf.Exit1Click(Sender: TObject);
begin
  close;
end;

procedure Tmaggrpcf.New1Click(Sender: TObject);
begin
  NewReportWindow;
end;

procedure Tmaggrpcf.NewReportWindow;
var maggrpcf1: Tmaggrpcf;
begin
  if memo1.Lines.Count = 0 then exit;
  Application.CreateForm(Tmaggrpcf, Maggrpcf1);

  Maggrpcf1.memo1.lines.assign(maggrpcf.memo1.lines);
  Maggrpcf1.new1.Visible := false;
  maggrpcf1.pdesc.caption := maggrpcf.pDesc.Caption;
  maggrpcf1.caption := maggrpcf.Caption;
  Maggrpcf1.memo1.FONT := MAGGRPCF.MEMO1.FONT;
  maggrpcf.memo1.lines.clear;
  maggrpcf.pdesc.Caption := '';
  maggrpcf.caption := 'Report : ';
  MAGGRPCF1.SetBounds(LEFT, TOP, WIDTH, Height);
  formtonormalsize(maggrpcf1);
  MAGGRPCF.SetBounds(LEFT + 30, TOP + 30, WIDTH, Height);
  maggrpcf.BringToFront;
end;

procedure Tmaggrpcf.FormCreate(Sender: TObject);
begin
  GetFormPosition(self as Tform);
end;

procedure Tmaggrpcf.FormDestroy(Sender: TObject);
begin
  SaveFormPosition(self as Tform);
end;

procedure Tmaggrpcf.ImageReports1Click(Sender: TObject);
begin
  Application.helpcontext(helpcontext);
end;

end.
