unit cMag4VGear;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created:
   Site Name: Silver Spring, OIFO
   Developers: Garrett Kirin
   [==     unit cMag4VGear;
   Description:
        TMag4VGear is a Wrapper around Accusoft's TGear control.  The Application
        has access to only those properties that the wrapper surfaces from the
        TGear class.
        In the Future:   TMag4VGear will implement the IMagImage Interface
        (yet to be created).  The application will be able to operate with
        any object that implements the interface, not just with TMag4VGear objects.
        It will be easy to substitute other Wrapper Objects that implement the Interface.
        We will be able to treat unrelated objects Polymorphically.
        We we won't be limited to using Accusoft controls for all our Imaging needs.

        TFrame is the ancestor of Tmag4Vgear.  By using a Frame we can make a
        Class that includes other controls and objects easier that doing it by hand.
        Being a TFrame, we have to surface certain properties of the included
        controls as published.  Example  TImageMouseUp event has to be declared
        as a published property, we connect it to the mouse up event of the TGear
        control.  But without surfacing it, the user of the control would not be
        able to connect an event handler to it.

         ==]
   Note:
   }
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)
//{$DEFINE PROTOTYPE}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  AxCtrls, OleCtrls, GEARLib_TLB, ExtCtrls, StdCtrls
  , GearDef, uMagClasses, {cMagVutils,} Buttons, umagdefinitions, {umagutils,}
  cmagimagelist, MEDXLib_TLB;

type
  TMag4Vgear = class;                           // ';' means forward declaration
        {       The Image Click event }
  TImageClickEvent = procedure(sender: TObject; Gearclicked: TMag4VGear) of object;
        {       The Image Mouse Down event }
  TImageMouseDownEvent = procedure(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer) of object;
        {       The Image Mouse Up event }
  TImageMouseUpEvent = procedure(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer) of object;
        {       The Image Key Down Event  }
  TImageKeyDownEvent  = procedure(Sender: TObject; var Key: Word;  Shift: TShiftState) of object;

  TMag4VGear = class(TFrame)
    pnlImage: TPanel;
    Gear1: TGear;
    pnlHeader: TPanel;
    btn1to1: TSpeedButton;
    btnDownArrow: TSpeedButton;
    pnlDesc: TPanel;
    lblImage: TLabel;
    lblDescIndex: TLabel;
    lblDescPage: TLabel;
    Med1: TMed;

    procedure lblImageMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure lblImageMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure lblImageDblClick(Sender: TObject);

    procedure FrameMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure FrameMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FrameStartDrag(Sender: TObject; var DragObject: TDragObject);
    procedure FrameEndDrag(Sender, Target: TObject; X, Y: Integer);

    procedure Gear1MouseDown(Sender: TObject; Button: TMouseButton;   Shift: TShiftState; X, Y: Integer);
    procedure Gear1Click(Sender: TObject);
    procedure Gear1DblClick(Sender: TObject);
    procedure Gear1MouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure Gear1MouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure Gear1KeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);

    procedure btn1to1Click(Sender: TObject);
    procedure btnDownArrowClick(Sender: TObject);


  private

    FImageClick:        TImageClickEvent;
    FImageMouseDown:    TImageMouseDownEvent;
    FImageMouseUp:      TImageMouseUpEvent;
    FImageKeyDown :     TImageKeyDownEvent;

    FMouseAction:       TMagImageMouse;
    //MagVUtils1:         TMagVUtils;
    FGroupImageList:    TMagImageList;

    FAbstractImage:     Boolean;
    FModINV:            boolean;
    FModFLIP:           boolean;
    FImageModified:     boolean;
    FIsDragAble:        boolean;
    FIsSizeAble :       boolean;
    FSelected:          Boolean;
    FAutoRedraw:        boolean;
    FPanWindow:         boolean;
    FZoomWindow:        Boolean;
    FresizeX:           boolean;
    FresizeY:           boolean;
    FShowImageOnly:     boolean;

    Fbrightnesspassed:  integer;
    FContrastPassed:    integer;
    Fdragx :            integer;
    Fdragy :            integer;
    FStartX:            integer;
    FStartY:            integer;
    FIndex:             integer;
    FFontSize:          integer;

    FModROT:            byte;

    procedure SetZoomWindow(Value: boolean);
    procedure SetPanWindow(Value: boolean);
    procedure SetSelected(Value: boolean);
    procedure SetAbstractImage(const Value: boolean);
    function GetPage: integer;
    function GetPageCount: integer;
    procedure SetPage(const Value: integer);
    procedure SetPageCount(const Value: integer);
    function GetImageDescription: string;
    procedure SetImageDescription(const Value: string);
    function IsImageAGroup: boolean;
    procedure SetShowImageOnly(const Value: boolean);
    function IsImageInGroup: boolean;
    procedure ImageDescriptionUpdate;
    procedure SetDescFont(fName: string; fSize, fHeight: integer);
    procedure GetNextGroupImage;
    procedure DeSkewImage;


  protected

  public
        {       This is a pointer to the parent control.  In some cases, this
                control needs to know if the parent is a TMag4Viewer.}
    FParentViewer: TWinControl;
        {       Image Data object.  Has data from VistA}
    PI_ptrData: TImageData;
        {       Prototype: not implemented.}
    PI_V4Data : TmagDicomData;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
        {  Future: with new version of Accusoft}
    procedure WinLevValue(WinValue,LevValue: integer);
        {  smooth edges of image, used on 1 but tifs for easier, clear reading}
    procedure SmoothImage;  //p8t43
        {  Reloads the Image from disk.}
    procedure RefreshImage;
        {  Redraws the Image, does not do a Reload}
    procedure ReDrawImage;
        {  undo all manipilations, and Fit to Window}
    procedure ResetImage;
        {  Load from the TImageData object
           Abs: if TRUE load the Abstract, Else it is a Full Res Image.}
    procedure LoadTheImage(Iobj: TImageData; Abs: boolean = false); overload;
        {  Loads the imagefile}
    procedure LoadTheImage(imagefile: string); overload;
    procedure FitToWidth;
    procedure FitToHeight;
    procedure FitToWindow;
        {  Fit 1 image pixel to 1 screen pixel.  Actual Size.}
    procedure Fit1to1;
    procedure FlipVert;
    procedure FlipHoriz;
        {  deg is 90,180 or 270}
    procedure Rotate(deg: integer);
    procedure Inverse;
        {  Preform both operations, DeSkew to straighten and Smooth to make
           edges clearer.  Gives an Image that is easier to view, less strain on eyes}
    procedure DeSkewAndSmooth;
        {  mouse will act as magnifier }
    procedure MouseMagnify;
        {  mouse will pan the image }
    procedure MousePan;
        {  mouse will select area.  That area will be zoomed to (as close as possible) }
    procedure MouseZoomRect;
        {  mouse pointer }
    procedure MouseReSet;

    procedure ShowHint(value : boolean);
    procedure ZoomValue(value: integer);
    procedure ContrastValue(Value: integer);
    procedure BrightnessValue(Value: integer);
        {  performs a DelDoc, to clear the Image from TGear.{}
    procedure ClearImage;
        {   returns a TMagImageState Object}
    function GetState: TMagImageState;
    procedure SetFontSize(value: integer);
        {  defines where the Pan Window for this image will be displayed.}
    procedure PanWindowSettings(H, W, X, Y: integer);
        {  for Multipage images }
        {  Future : These 'paging' methods will also work for Image Groups.}
    procedure PageFirst;
    procedure PagePrev;
    procedure PageNext;
    procedure PageLast;
  published

    (* 7/9/01  This shouldn't be published.  It's internal, when the image is
         clicked, we do stuff, and then see if OnImageClick is assigned ( ImageClick) *)
    //new   property PopUpmenu;
        {  show/hide the image description}
    property ShowImageOnly: boolean read FShowImageOnly write SetShowImageOnly;
        {  Information for Abstract images is displayed different that full size
            and when Loading Image from TImageData object, differnet images are loaded.}
    property AbstractImage: boolean read FAbstractImage write SetAbstractImage;
        {  Open/Hide a Zoom window for the Image.}
    property ZoomWindow: Boolean read FZoomWindow write SetZoomWindow;
        {  Open/Hide a Pan window for the Image.}
    property PanWindow: Boolean read FPanWindow write SetPanWindow;
        {  Future : Prototype Drag and Drop }
        {  Enable/DisAble dragging the control}
    property IsDragAble: boolean read FIsDragAble write FIsDragAble;
        {  Future : Allow resizing images. (used sparingly in ReSizeAbstracts Function}
        {  Enable/Disable Resizing the control.}
    property IsSizeAble: boolean read FIsSizeAble write FIsSizeAble;
        {  Not Implemented:  -- actually it is set to true, not changeable }
    property AutoRedraw: boolean read FAutoRedraw write FAutoRedraw;
        {  Querried by viewer, to determine if operation should be exeuted.}
    property Selected: boolean read FSelected write SetSelected;
        {   EventHandlers }
    property OnImageClick: TImageClickEvent read FImageclick write FImageClick;
    property OnImageMouseDown: TImageMouseDownEvent read FImageMouseDown write FImageMouseDown;
    property OnImageMouseUp: TImageMouseUpEvent read FImageMouseUp write FImageMouseUp;
    property OnImageKeyDown: TImageKeyDownEvent read FImageKeyDown write FImageKeyDown;
        {  This is set by the Viewer.  It is the Index of the related TImageProxy
            in the Viewers FProxyList of all images in the viewer.}
    property ListIndex: Integer read Findex write Findex;
        {  ImagePageCount is a ReadOnly property.  i.e. SetImagePageCount does nothing}
        {  returns number of pages in this image.}
    property PageCount: integer read GetPageCount write SetPageCount;
        {  Get/Set the current page to view.  If page > total pages, then view
           last page.}
    property Page: integer read GetPage write SetPage;
    //  {  GroupImageList, gives each group image a pointer to the group list.}
       {TODO: it should be readonly}
        {  Future.  Not Implemented}
    property GroupImageList: Tmagimagelist read FGroupImageList write FGroupImageList;
        {  The description display Above (for Full Res) Below (for Abstracts) the image.}
    property ImageDescription: string read GetImageDescription write SetImageDescription;
    {  Font size for Image Description. }
    property FontSize: integer read FFontSize write SetFontSize;
  end;

procedure Register;

implementation
{TODO: this needs to be taken out,  TMag4VGear shouldn't be dependent on TMag4Viewer}
uses cmag4viewer;
{$R *.DFM}

{ TMag4VGear }



constructor TMag4VGear.Create(AOwner: TComponent);

begin
  inherited;
Med1.DestGearControlInterface := Gear1.ControlInterface;
Gear1.extfile := 'IGMed32S.DLL';
Gear1.extcode := 'VAMEDICALCTR';
Gear1.extpass := 'X32ESTCC8WDLVQ';

  //MagVUtils1 := TMagVUtils.create(self);
  ListIndex := -1;
  lblImage.Align := alclient;
  lblDescPage.Caption := '';
  lblDescIndex.caption := '';
  Gear1.align := alclient;
  if (AOwner is TMag4Viewer) then FParentViewer := (Aowner as TMag4Viewer);
  (*
  Settings:	IG_ASPECT_NONE          (No adjustment)
                IG_ASPECT_DEFAULT       (Dont increase width or height of device rect)
                IG_ASPECT_HORIZONTAL    (Preserve width of device rect)
                IG_ASPECT_VERTICAL      (Preserve height of device rect)
                IG_ASPECT_MAXDIMENSION  (Horizontal and vertical aspects are calculated 
                                                        the larger of the two is used.)
                IG_ASPECT_MINDIMENSION  (Best fit)
  *)
  Gear1.AspectRatio := IG_ASPECT_MINDIMENSION;
  Gear1.AutoRedraw := true;
  name := '';
  FModINV := false;
  FModFLIP := false;
  FModROT := 0;
  FImageModified := false;
  Fbrightnesspassed := 100;
  FContrastPassed := 100;
  MouseReset;
  PI_V4Data := TmagDicomData.create;
  PI_V4Data.ClearData;

end;

destructor TMag4VGear.Destroy;
begin
  inherited;
  //
   { DONE -oGarrett -cConversion : Destroy the GEAR1 component here }
  // CAUSES the Display to Blink.  Also ... will need to recreate anyway.
  //  change to a Pool of Image Gear Components. Which are contiually reused.
  //  and all are destroyed when Application terminates.  See Destroy_p
end;


procedure Register;
begin
  RegisterComponents('Imaging', [TMag4VGear]);
end;


procedure TMag4VGear.LoadTheImage(imagefile: string);
VAR i : INTEGER;
begin
        {       MagOpenSecurity is made before this call, by Mag4Viewer or
                other callers.}
  if (UPPERCASE(extractfileext(imagefile)) = '.TIF')
    then
      begin
        Gear1.LoadDocument := imagefile;
        DeSkewImage;
      end
  else Gear1.LoadImage := imagefile;
  gear1.Redraw := true;
  ImageDescriptionUpdate;
end;

procedure TMag4VGear.LoadTheImage(Iobj: TImageData; Abs: boolean = false);
var s: string;
begin
  if abs then s := Iobj.AFile
  else s := Iobj.FFile;
  if Iobj.ImgType = 15
    then
      begin
        Gear1.LoadDocument := s;
        DeSkewImage;
      end
  else Gear1.LoadImage := s;
  Gear1.Redraw := true;
  ImageDescriptionUpdate;
{ TODO :
ERROR Checking when loading an Image.  look at Capture, we had
SaveGear, LoadGear,  if ErrorInGear procedures and functions.
Gear1.DebugLevel := 2  See Capture, it implements this }
end;



procedure TMag4VGear.BrightnessValue(Value: integer);
begin
  Fbrightnesspassed := value;
  Gear1.displaybrightness := (trunc(value / 10));
end;

procedure TMag4VGear.ClearImage;
begin
  Gear1.DelImage := true;
  Gear1.delDoc := true;
  Gear1.Update;
  lblimage.Caption := '';
  lblimage.Hint := '';
  lblDescpage.Caption := '';
  lbldescindex.Caption := '';
end;

procedure TMag4VGear.ContrastValue(Value: integer);
begin
  Fcontrastpassed := value;
  Gear1.displaycontrast := (value / 100);
end;

{ values for nFitMethod parameter of IG_display_fit_method
const IG_DISPLAY_FIT_TO_WINDOW=0  ;
const IG_DISPLAY_FIT_TO_WIDTH=1;
const IG_DISPLAY_FIT_TO_HEIGHT=2  ;
const IG_DISPLAY_FIT_1_TO_1=3;       }

procedure TMag4VGear.FitToWindow;
begin
  Gear1.fitmethod := IG_DISPLAY_FIT_TO_WINDOW;
end;

procedure TMag4VGear.FitToHeight;
begin
  Gear1.fitmethod := IG_DISPLAY_FIT_TO_HEIGHT;
end;

procedure TMag4VGear.FitToWidth;
begin
  Gear1.fitmethod := IG_DISPLAY_FIT_TO_WIDTH;
end;

procedure TMag4VGear.Fit1to1;
begin
  Gear1.fitmethod := IG_DISPLAY_FIT_1_TO_1;
end;


procedure TMag4VGear.FlipHoriz;
begin
        {       a Value is kept for FModFlip to be used when Resetting the image
                we have to modify the value for FModRot (rotate value) also
                so that the reset function works.}
  FModROT := (FModROT mod 4);
  case FModROT of
    0: FModROT := 1;
    1: FModROT := 0;
    2: FModROT := 3;
    3: FModROT := 2;
  end;
  Fimagemodified := true;
  Gear1.Flip := IG_FLIP_HORIZONTAL;
  FModFLIP := not (FModFLIP);
end;

procedure TMag4VGear.FlipVert;
        {       a Value is kept for FModFlip to be used when Resetting the image
                we have to modify the value for FModRot (rotate value) also
                so that the reset function works.}
begin
  FModROT := (FModROT mod 4);
  case FModROT of
    0: FModROT := 3;
    1: FModROT := 2;
    2: FModROT := 1;
    3: FModROT := 0;
  end;
  Fimagemodified := true;
  Gear1.Flip := IG_FLIP_VERTICAL;
  FModFLIP := not (FModFLIP);
end;

procedure TMag4VGear.Inverse;
begin
  Fimagemodified := true;
  Gear1.invert := true;
  FModINV := not (FModINV);
end;

        {       Do both functions.}
procedure TMag4VGear.DeSkewAndSmooth;
begin
DeSkewImage;
SmoothImage;
end;

procedure TMag4Vgear.DeSkewImage;
var i : integer;
pg : integer;
begin
{     if gear1.ImageBitsPerPixel > 1 then Deskew the 1 bit Image}
Gear1.SettingMode := MODE_ROTATE;
Gear1.SettingValue := IG_ROTATE_EXPAND;
Gear1.AutoDeskew := 1;
pg := Gear1.Page;
if (Gear1.PageCount > 1) then
  begin
    for i := 1 to Gear1.PageCount do
      begin
       Gear1.Page := 1;
       Gear1.SettingMode := MODE_ROTATE;
       Gear1.SettingValue := IG_ROTATE_EXPAND;
       Gear1.AutoDeskew := 1;
      end;
   Gear1.Page := pg;
  end;
end;

procedure TMag4VGear.SmoothImage;
begin

{IG_DSPL_ANTIALIAS_SCALE_TO_GRAY
The scale to gray algorithm is used and the output image becomes 4
bits per pixel.}

{
If AliasMode is set to IG_DSPL_ANTIALIAS_SCALE_TO_GRAY,
AliasThreshold determines the number of black and white pixels in
the destination gray pixel value. A value of 100 directs ImageGear to
take 100% of white pixels. A value of 0 directs ImageGear to take
100% of black pixels. The default value, which is 50, directs
ImageGear to take 50% white and 50% black pixels.
}
gear1.AliasQuality := 100;

GEAR1.AliasMode :=  IG_DISPLAY_ALIAS_SCALE_TO_GRAY ;
end;


procedure TMag4VGear.MousePan;
begin
  Gear1.selectevent := 0;
  Gear1.magnifywindow := 0;
  Gear1.HandPanning := true;
  Gear1.Cursor := magcrPan; //crHandPoint;
  FMouseAction := mactPan;
end;

procedure TMag4VGear.MouseMagnify;
begin
  Gear1.GUIWindow := GUIMAG;
  Gear1.GUIXpos := 100;
  Gear1.Guiypos := 100;
  Gear1.GUIHeight := 200;
  Gear1.Guiwidth := 500;
  Gear1.selectevent := 0;
  Gear1.magnifywindow := 1;
  Gear1.Cursor := magcrMagnify; //crSizeAll;
  FMouseAction := mactMagnify;
end;

procedure TMag4VGear.RefreshImage;
var onefile : string;
begin
  // Abstract viewer allows this call : To Refresh(reload) images.
{ TODO :   This needs changed to reload the image from network, or cache.
                                        and Where is  Open MagSecurity ? }
{       If the image was cached, the LoadImage or LoadDocument will be
        the path of the local image.}
{       All images are being cached, so no problem there with OpenSecureFile.}
{       Abstracts   --- OpenSecureFile ?  ? }
 Gear1.ClearImage := true;
 if Gear1.LoadDocument = ''
   then onefile := Gear1.LoadImage
   else onefile := Gear1.LoadDocument;
 LoadTheImage(onefile);
end;


procedure TMag4VGear.Rotate(deg: integer);
var xdeg: integer;
begin
  try
    xdeg := (deg div 90);
    Gear1.update;
    Gear1.AspectRatio := IG_ASPECT_DEFAULT;
    application.processmessages;
    Gear1.Rotate := DEG;
    FModROT := FModROT + xdeg;
    Fimagemodified := true;
  finally
    //if ErrorInGear(Gear1, 'Rotating Image') then winmsg('', 'Accusoft error during Rotate.');
  end;
end;

        {       Set the Images 'Selected' State, and change display of image to
                show it is selected}
procedure TMag4VGear.SetSelected(Value: boolean);
begin
  if not (Fselected = value) then
    begin
      Fselected := value;
      if Fselected then
        begin
          color := clblue;
           // future
          {$IFDEF PROTOTYPE}
              pnldesc.Color := clHighlight;
              pnldesc.Font.Color := clwhite;
              //lblImage.Color := clHighLight;
             // lblImage.Font.Color := clwhite;
         {$ENDIF}
        end
      // Show  border around full image, not AbstractImage
      else
        begin
         //Future:
          {$IFDEF PROTOTYPE}
             pnldesc.color := clBtnFace;
             // pnldesc.ParentFont := true;
              pnldesc.Font.Color := clBlack;
              //lblimage.color := clBtnFace;
              //lblimage.ParentFont := true;
          {$ENDIF}
         if FAbstractImage then Color := clbtnface
          else Color := clGrayText;
        end;
    end;
end;

procedure TMag4VGear.SetPanWindow(Value: boolean);
begin
  FPanWindow := value;
  if FPanWindow then
    begin
      Gear1.GUIWindow := GUIPAN;
      Gear1.PanWindow := true;
    end
  else
    begin
      Gear1.PanWindow := false;
    end;
end;

procedure TMag4VGear.SetZoomWindow(Value: boolean);
begin
  FZoomWindow := value;
  if FZoomWindow then
    begin
      Gear1.GUIWindow := GUIMAG;
        // testing - didn't work.
       (* Gear1.GUIHeight := 200;
        Gear1.GUIWIDTH := 400;
        Gear1.GUIXPOS := 10;
        Gear1.GUIYPOS := 10; *)
      Gear1.MagnifyWindow := 2;
    end
  else
    begin
      Gear1.MagnifyWindow := 0;
    end;
end;

procedure TMag4VGear.MouseZoomRect;
begin
  Gear1.selectevent := 2;
  Gear1.magnifywindow := 0;
  Gear1.Cursor := magcrCrossHair; //crCross;
  FMouseAction := mactZoomRect;
end;

procedure TMag4VGear.ZoomValue(value: integer);
begin
  Gear1.ZoomLevel := value;
end;

procedure TMag4VGear.ReDrawImage;
begin
// testing p8t48
if FAbstractImage then Gear1.FitMethod := IG_DISPLAY_FIT_TO_WINDOW;
  Gear1.Redraw := true;
end;



procedure TMag4VGear.Gear1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
//10-6 outfortesting   if (fdragable and (Shift = [ssLeft])) then BeginDrag(false, 20)    ;
if FIsDragable or FIsSizeAble then
 begin
   Fdragx := x;
   Fdragy := y;
   end;
    if assigned(OnImageMouseDown)
      then OnImageMouseDown(Self, Button, Shift, X, Y);
    //new  was working. by itself, having OnImageMouseDown and BegirDrag is confusing.
    //BeginDrag(false,10);
end;

procedure TMag4VGear.Gear1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
Fdragy := 0 ;
Fdragx := 0;
if assigned(OnImageMouseUP)
   then OnImageMouseUp(Self, Button, Shift, X, Y);
 //if shift = [ssleft]
// then  if assigned(OnImageclick) then OnImageClick(self, self); {TheImage as TOBject}
end;

procedure TMag4VGear.lblImageMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  {   FIsSizable is always False for Patch 8.  This is for future patch.}
  if FIsSizeAble then
    if (ssleft in shift) then
      begin
        left := left + x - FstartX;
        top := top + y - FStartY;
      end;
end;

procedure TMag4VGear.FrameMouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
var movex, movey: boolean;
begin
  {  FIsSizable is always False for Patch 8.  This is for future patch.}
  {   .....except .. We use this for the Resize Abstract function.  Allowing
                        the user to drag the sides of the image to a new size}
  if not FIsSizeable then exit;
  if  (shift = [ssleft]) then  //10/15/03 test to stop the disappearing act.
  begin
  { FresizeX and FresizeY are true if Dragging has started.  So we just
    change the width and height }
    if FresizeX or FresizeY then
      begin
        if FresizeX then width := x;
        if FresizeY then height := y;
        exit;  // p8t45
      end;
    end;
  movex := (X > (width - 5)) ;
  movey := (y > (height - 5));

  {  if cursor isn't near the right side or bottom, then exit}
  if not (movex or movey) then
    begin
      cursor := crdefault;
      exit;
    end;
  {  change the cursor to appropriate indicator}
  if (movex and movey)
    then cursor := crSizeNWSE
    else
      if movex
        then cursor := crsizeWE
        else if movey then cursor := crsizeNS;

  {  Only if Left mouse is pressed, we will start resizing}
  if (ssleft in shift) then
    begin
      {   Flag which direction we are resizing (or both)}
      if movex then FResizeX := true;
      if movey then FResizeY := true;
    end;

end;

procedure TMag4VGear.FrameMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  { if any dragging was taking place. Set the flags to false}
  FresizeX := false;
  FresizeY := false;
end;

procedure TMag4VGear.Gear1Click(Sender: TObject);
begin
//showmessage('on Gear1Click');    //testing - this doesn't happen if DragAble.
//10-6 outfortesting  if assigned(OnImageclick) then OnImageClick(self, self); {TheImage as TOBject}
end;

procedure TMag4VGear.lblImageMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  {     if we are allowing resizing, we also allow moving the image, if the
        user is dragging the Caption }
  if not FIsSizeable then
    if assigned(OnImageMouseDown)
      then OnImageMouseDown(Self, Button, Shift, X, Y);
// pre p8t21
  FStartX := x;
  FStartY := y;
end;

function TMag4VGear.IsImageAGroup: boolean;
begin
  if assigned(PI_PtrData) then
    begin
      if PI_PtrData.ImgType = 0 then result := false
      else result := (PI_PtrData.ImgType = 11); //((PI_PtrData.ImgType = 11) or (PI_ptrData.ImgType = 200));
      //p8t34 Groupof1
      //else result := (PI_PtrData.ImgType = 11) and (PI_PtrData.GroupCount > 1);
    end
  else result := false;
end;

        {Future. need to enable paging between images of a Group, like we
          do with Multipage images }
function TMag4VGear.IsImageInGroup: boolean;
begin
  result := false;
  exit;
{TODO: Need to have more properties of the ImageData, telling Group Parent.
        Then we can finish this function.}
  if assigned(PI_PtrData) then
    begin

      //if PI_PtrData.ImgType = 0 then result := false
      //else result := (PI_PtrData.ImgType = 11); //((PI_PtrData.ImgType = 11) or (PI_ptrData.ImgType = 200));
    end
  else result := false;
end;



procedure TMag4VGear.SetAbstractImage(const Value: boolean);
begin
  FAbstractImage := Value;
  if FAbstractImage then
    begin
      if IsImageAGroup then
        begin
          {$IFDEF PROTOTYPE}
             //Future: Stop using Color as the means of showing this is a group
              pnlDesc.BevelInner := bvLowered;
              pnlDesc.BevelOuter := bvRaised;
              pnlImage.BorderStyle := bsSingle;
            {$ELSE}
            pnldesc.Color := clblack;
            pnldesc.Font.Color := clwhite;
            {$ENDIF}
         end
         else  {is an Abs, but not a group}
         begin
          {$IFDEF PROTOTYPE}
             //Future: Stop using Color as the means of showing this is a group
              pnlDesc.BevelInner := bvNone;
              pnlDesc.BevelOuter := bvNone;
              pnlImage.BorderStyle := bsSingle;
            {$ELSE}
            pnldesc.Color := clBtnFace;
            pnldesc.Font.Color := clblack;


              {$ENDIF}
        end;
        Color := clbtnface;
        Gear1.BackColor := clBtnFace;
        Gear1.ScrollBars := false;
        Gear1.FitMethod := IG_DISPLAY_FIT_TO_WINDOW;
        pnldesc.Align := albottom;
        end
  else {NOT AN ABSTRACT}
    begin
             {$IFDEF PROTOTYPE}
        //Future : Stop using Color as the means of showing this is a group
              pnldesc.bevelinner := bvNone;
              pnlDesc.bevelOuter := bvNone;
              pnlImage.BorderStyle := bsSingle;
            {$ELSE}
              pnlDesc.Color := clBtnFace;
              pnlDesc.font.Color := clBlack;
            {$ENDIF}
      Color := clGrayText;
      Gear1.BackColor := clWhite;
      pnldesc.align := altop;
      lblimage.WordWrap := false;
    end;
end;



function TMag4VGear.GetPage: integer;
begin
  result := Gear1.Page;
end;

function TMag4VGear.GetPageCount: integer;
begin
  result := Gear1.PageCount;
end;

procedure TMag4VGear.SetPage(const Value: integer);
begin
  if (value > 0) and (value <= Gear1.Pagecount) then
    begin
     {P48T1 - NextLine out: Stop disappearing scrollbars. Why was it here? }
     //Gear1.ScrollBars := false;
     {P48T1 - NextLine in:
      For a single page document, setting page := 1 would force a fit to window}
    if (value <> Gear1.page) then
     Gear1.Page := value;
    end;
end;

procedure TMag4VGear.SetPageCount(const Value: integer);
begin
  // Read Only,  do nothing.
end;

function TMag4VGear.GetImageDescription: string;
begin
  result := lblimage.caption;
end;


procedure TMag4VGear.ImageDescriptionUpdate;
begin
lblDescPage.Caption := '';
  if FAbstractImage and (listindex > -1) then lblDescIndex.caption := '#' + inttostr(listindex + 1) + ' '
  else if ((not FAbstractImage) and (pagecount > 1))
         then lblDescPage.Caption := 'pg ' + inttostr(page) + ' of ' + inttostr(pagecount);
end;

procedure TMag4VGear.SetImageDescription(const Value: string);
begin
  lblimage.Caption := value;
  lblImage.Hint := value;
  if FAbstractImage and (listindex > -1) then ImageDescriptionUpdate
  else if ((not FAbstractImage) and (pagecount > 1))
    then ImageDescriptionUpdate;
end;

procedure TMag4VGear.ResetImage;
begin
{TODO: If we are Caching images.  it would be easier codewise to just ReLoad the
   image.  No chance of making a mistake.  But... Remote images not cached would
   take a long time to Reset... ?.  It would need some testing. }
  if FModFLIP then FlipHoriz;
        {       FModROT counter, to correctly maintain the rotation of the image.
                NOTE : FModROT will never be negative.  {}
  if ((FModROT mod 4) > 0) then Gear1.Rotate := (90 * (4 - (FModROT mod 4)));
  if FModINV then
    begin
      Gear1.invert := true;
      FModINV := false;
    end;
  if FModFlip then FModFLIP := false;
  if FModRot <> 0 then FModROT := 0;
        {       from Capture App
                Gear1.DisplayBrightness := trunc(tbicBrightness.Position / 10);{}
  Gear1.DisplayBrightness := 10;
        {       from Capture App
                Gear1.DisplayContrast := tbicContrast.Position / 100;{}
  Gear1.DisplayContrast := 1;
  Fimagemodified := false;
  Gear1.fitmethod := IG_DISPLAY_FIT_TO_WINDOW;
  Gear1.Redraw := true;
  MouseReset;
end;

procedure TMag4VGear.SetShowImageOnly(const Value: boolean);
begin
  FShowImageOnly := Value;
  pnldesc.Visible := not FShowImageOnly;
end;

function TMag4VGear.GetState: TMagImageState;
begin
  result := TmagImageState.create;
  result.brightnessvalue := Fbrightnesspassed;
  result.contrastvalue := FContrastPassed;
  result.zoomvalue := Gear1.ZoomLevel;
  result.Page := Gear1.Page;
  result.PageCount := Gear1.PageCount;
        {       TMagImageMouse = set of (mcrPan, mcrDrag, mcrMagnify,
                                                mcrSelect, mcrPointer);{}
  result.MouseAction := FMouseAction;
        {  for Dicom images.  Not Used in application.  Only in Testing.}
  result.ImageCount := 1;
  result.imagenumber := 1;
  result.WinMaxValue := 255; //self.PI_V4Data.win; ??
  result.WinMinValue := 0;
  result.WinValue := 125;
  result.levmaxvalue := 255;
  result.levminvalue := 0;
  result.levvalue := 125;
  result.isWinLevEnabled := false;

end;

procedure TMag4VGear.PanWindowSettings(H, W, X, Y: integer);
begin
  Gear1.GUIWINDOW := GUIPAN;
  Gear1.GUIHeight := H;
  Gear1.GUIWIDTH := W;
  Gear1.GUIXPOS := x;
  Gear1.GUIYPOS := y;
  Gear1.PanWindow := true;
  FmouseAction := mactPan;
end;

procedure TMag4VGear.SetFontSize(value: integer);
var size: integer;
begin
  {  when font size changes, we change the Height of the Tpanel that shows desc
        SetDescFont does the height adjustment of pnlDesc}
  size := value;
  case size of
    6:
      begin
        SetDescFont('Small Fonts', size, 25);
      end;
    7:
      begin
        SetDescFont('Small Fonts', size, 30);
      end;
    8, 9:
      begin
        SetDescFont('Arial', size, 35);
      end;
    10:
      begin
        SetDescFont('Arial', size, 40);
      end;
    11..99:
      begin
        SetDescFont('Arial', size, 45);
      end;
  end;
  if not FAbstractImage then pnldesc.height := pnldesc.Height div 2;
end;

        {SetDescFont does the height adjustment of pnlDesc}
procedure TMag4VGear.SetDescFont(fName: string; fSize, fHeight: integer);
begin
  pnldesc.Font.Name := fName;
  pnldesc.Font.Size := fsize;
  pnldesc.height := fheight;
end;

procedure TMag4VGear.MouseReset;
begin
  Gear1.selectevent := 0;
  Gear1.magnifywindow := 0;
  Gear1.HandPanning := false;
  Gear1.Cursor := crDefault;
  FMouseAction := mactPointer;
end;

procedure TMag4VGear.Gear1DblClick(Sender: TObject);
begin
{       send message to  viewer to maximize this image.
                        MaximizeImage; P8t44 in testing;   {}
{       FParentViewer is a pointer to TMag4Viewer, and set by the Viewer
        This allows the TMag4VGear object to be used outside the viewer.}
  if (FParentViewer is Tmag4Viewer) then
    begin
      if Tmag4Viewer(FParentViewer).abswindow then exit;
      (FParentViewer as TMag4Viewer).MaxImageToggle(Pi_PtrData);
    end;
end;

procedure TMag4VGear.lblImageDblClick(Sender: TObject);
begin
        {       send message to  viewer to maximize this image.
        {       MaximizeImage;  {}
  if (FParentViewer is Tmag4Viewer) then
    begin
    IF Tmag4Viewer(FParentViewer).abswindow then exit;
      (FParentViewer as TMag4Viewer).MaxImageToggle(Pi_PtrData);
    end;
end;

        {       Not implemented, reserved for Future functionality}
procedure TMag4VGear.btn1to1Click(Sender: TObject);
begin
  Fit1to1;
end;

        {       Not implemented, reserved for Future functionality}
procedure TMag4VGear.btnDownArrowClick(Sender: TObject);
begin
  if IsImageAGroup then //GetNextGroupAbs;
end;

        {       For Multi-Page Images  (later , treat Groups like Multipage}
procedure TMag4VGear.PageFirst;
begin
  if IsImageInGroup
    then
    begin
      // GetFirstGroupImage(
    end
  else
    begin
      page := 1;
      ImageDescriptionUpdate;
    end;
end;

        {       For Multi-Page Images  (later , treat Groups like Multipage}
procedure TMag4VGear.PageLast;
begin
  if IsImageInGroup
    then
    begin
      //
    end
  else
    begin
      page := pagecount;
      ImageDescriptionUpdate;
    end;
end;

        {       For Multi-Page Images  (later , treat Groups like Multipage}
procedure TMag4VGear.PageNext;
begin
  if IsImageInGroup
    then
    begin
      GetNextGroupImage;
    end
  else
    begin
      page := Page + 1;
      ImageDescriptionUpdate;
    end;
end;

        {       For Multi-Page Images  (later , treat Groups like Multipage}
procedure TMag4VGear.PagePrev;
begin
  if IsImageInGroup
    then
    begin
      //
    end
  else
    begin
      page := page - 1;
      ImageDescriptionUpdate;
    end;
end;


        {       Not Implemented }
procedure TMag4VGear.GetNextGroupImage;
begin
//  self.PI_ptrData.GroupIEN;
//  self.PI_ptrData.GroupIndex;
end;


        {       Not Implemented }
procedure TMag4VGear.FrameStartDrag(Sender: TObject;var DragObject: TDragObject);
//var magdragobj : TmagdragImageObject;
begin
//10-06 infortesting
//showmessage('FrameStartDrag');

exit;
// the dragobject worked, but didn't send an EndDrag, so the OnEndDrag was never fired.
dragobject := TmagDragImageObject.Create;
   if FparentViewer <> nil then
     begin
     TmagDragImageObject(dragobject).DraggedImages := TMag4Viewer(FParentViewer).GetSelectedImageList;
     //DragObject := magdragobj;
     end;
end;

procedure TMag4VGear.Gear1MouseMove(Sender: TObject; Shift: TShiftState; X,
  Y: Integer);
begin
if (shift = [ssleft]) and (FIsDragAble) and (Fdragy > 0) and (Fdragx > 0) then
  begin
    if abs( y - Fdragy) > 5
      then begindrag(TRUE)
     else if abs( x - Fdragx) > 5 then begindrag(TRUE);
  end;
end;

        {       Not Implemented }
procedure TMag4VGear.FrameEndDrag(Sender, Target: TObject; X, Y: Integer);
begin
//showmessage('end drag FRame');

//tried but target isn't a mag4viewer, it's the scrlv (scrollbox)
//if target <> nil then
//  begin
//  Tmag4Viewer(target).imagestomagview(Tmag4Viewer(FParentViewer).GetSelectedImageList);
//  end;
fdragx := 0;
fdragy := 0;
end;

        {       Not Implemented }
procedure TMag4VGear.Gear1KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
if assigned(onImageKeyDown) then OnImageKeyDown(self,key,shift);
end;

        {       Not Implemented }
procedure TMag4VGear.WinLevValue(WinValue,LevValue: integer);
//var i : integer;
begin
//MagApplyCTPreset(self, LevValue, WinValue);
end;

procedure TMag4VGear.ShowHint(value: boolean);
begin
 lblImage.ShowHint := value;
end;

end.

