unit cMag4Viewer;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created:
   Site Name: Silver Spring, OIFO
   Developers: Garrett Kirin
   [==   unit cMag4Viewer;
   Description: Image Viewer.
        The Viewers function is just to manipulate Tmag4VGear objects.
        It handles the Creation, destruction,  Alignment, Resizing etc.
        of it's contained images.  The Viewer passes image operations like
        FitToWidth, Invert, Zoom etc to the selected Image Objects.
        By Itself it has no association with any database.   Any database
        operations are handled by the TMagUtilsDB component linked to the Viewer.

        TMag4VGear is a Wrapper around Accusoft's TGear control.

        Future:   The Viewer will implement the IMagImage Interface
        (yet to be created) and support the handling of any object that
         implements the IMagImage Interface.  We will be able to treat unrelated
         objects Polymorphically.  We we won't be limited to using
         Accusoft controls for all our Imaging needs.

         The Viewer makes use of the TImageProxy Class.  A TImageProxy is just a
         placeholder for a Tmag4VGear object.  In the method 'ImagesToMagView'
         ('ImagesToMagView() is the method to load images into a Viewer)
         a TImageProxy is created for each image and added to the FProxyList.
         ==]
   Note:
   }
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)
//{$DEFINE PROTOUNDO}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ComCtrls, ExtCtrls,
  //
  cMag4VGear, cMagSecurity, uMagClasses, cmagvutils,
  cimageproxy, ImagInterfaces, cmagimagelist, cmaglistview, Menus, ToolWin,
  cMagUtilsDB,  maggmsgu, umagdefinitions
  , {uMagV4utils,} {cevtscroll,} cMagEventScrollBox,
  umagappmgr,  fMagLoadingImageMessage ,geardef, ImgList ;

type
  {   Class to store the properties of a Viewer Layout style }
  TMagLayout = class(Tobject)
  public
    Rows: integer;
    Columns: integer;
    MaxNumberOfImages: integer;
    LockImageState: boolean;
    FitMethod : integer;
  end;
  {  Not Implemented.}
  TMagUnDoAction = (magundoAdd, magundoRemove);
  {  Not Implemented.}
  TMagUnDoItem = class(TObject)
  public
    magCaption: string;
    magUnDoGroup: integer;
    magIproxy: TImageProxy;
    magUnDoAction: TMagUnDoAction;
  end;
        {different styles of displaying images and data}
  TMagDisplayStyles = (magdsLine, magdsData, magdsDetails);
        {diff styles of Viewer, Virtual: is only one being used.}
  TMagViewerStyles = (magvsVirtual, magvsStaticPage, magvsDynamic);

  TMag4Viewer = class;   // the ';' says forward declaration}
   {TMagVListChange is for OnListChangeEvent}
  TMagVListChange = procedure(sender: Tobject) of object;
   { TMagVClick is for OnViewerImageClick}
  TMagVClick = procedure(sender: tobject) of object;
    {TMagViewerClickEvent  is for OnViewerClick}
  TMagViewerClickEvent = procedure(sender: TObject; Viewer: TMag4Viewer; MagImage: TMag4VGear) of object;
        {Returns pointers to Viewer Object, and ImageObject }
  //TMag4Image = TMag4VGear;

  TMag4Viewer = class(TFrame, IMagObserver)

    scrlv:              TMagEventScrollBox;
    lbScrlvSize:        TLabel;
    timerViewer:        TTimer;
    pnlTop:             TPanel;
    lbHideScroll:       TPanel;
    stsbarMsg:          TStatusBar;
    imlViewerBmps:      TImageList;

    tbViewer:           TToolBar;
    tbClear:            TToolButton;
    tbRowBigger:        TToolButton;
    tbRowSmaller:       TToolButton;
    tbRefresh:          TToolButton;
    tbPageFirst:        TToolButton;
    tbPagePrev:         TToolButton;
    tbPageNext:         TToolButton;
    tbPageLast:         TToolButton;
    tbColumnBigger:     TToolButton;
    tbColumnSmaller:    TToolButton;
    tbConfig:           TToolButton;
    tbtnAbsPageNext:    TToolButton;
    tbtnAbsPagePrev:    TToolButton;
    tbtnAbsSmaller:     TToolButton;
    tbtnAbsBigger:      TToolButton;

    popupViewer:        TPopupMenu;
    mnuToolbar:         TMenuItem;
    AlignonTop1:        TMenuItem;
    AligntoLeft1:       TMenuItem;
    N1:                 TMenuItem;
    N2:                 TMenuItem;
    mnuScrollVertical:  TMenuItem;
    mnuScrollHorz:      TMenuItem;

    procedure scrlVClick(Sender: TObject);
    procedure pnlTopStartDrag(Sender: TObject; var DragObject: TDragObject);
    procedure pnlTopStartDock(Sender: TObject; var DragObject: TDragDockObject);
    procedure pnlTopClick(Sender: TObject);
    procedure pnlTopEndDrag(Sender, Target: TObject; X, Y: Integer);
    procedure pnlTopMouseMove(Sender: TObject; Shift: TShiftState; X, Y: Integer);
    procedure pnlTopMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FrameEndDock(Sender, Target: TObject; X, Y: Integer);
    procedure pnlTopDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
    procedure pnlTopDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure tbClearClick(Sender: TObject);
    procedure tbRowSmallerClick(Sender: TObject);
    procedure tbRowBiggerClick(Sender: TObject);
    procedure tbRefreshClick(Sender: TObject);
    procedure tbPagePrevClick(Sender: TObject);
    procedure tbPageNextClick(Sender: TObject);
    procedure tbColumnBiggerClick(Sender: TObject);
    procedure tbColumnSmallerClick(Sender: TObject);
    procedure mnuToolbarClick(Sender: TObject);
    procedure tbConfigClick(Sender: TObject);
    procedure AlignonTop1Click(Sender: TObject);
    procedure AligntoLeft1Click(Sender: TObject);
    procedure tbPageLastClick(Sender: TObject);
    procedure tbPageFirstClick(Sender: TObject);
    procedure tbtnAbsPagePrevClick(Sender: TObject);
    procedure tbtnAbsPageNextClick(Sender: TObject);
    procedure tbtnAbsSmallerClick(Sender: TObject);
    procedure tbtnAbsBiggerClick(Sender: TObject);
    procedure popupViewerPopup(Sender: TObject);
    procedure scrlVDragOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
    procedure scrlVDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure ScrlvScrollHorz(Sender: TObject);
    procedure ScrlvScrollVert(Sender: TObject);
    procedure ScrlvEndScroll(Sender: TObject);
    procedure mnuScrollVerticalClick(Sender: TObject);
    procedure mnuScrollHorzClick(Sender: TObject);
    procedure timerViewerTimer(Sender: TObject);
  private

{---------- Fields of Imaging Types ---}
    //Cur4Image: TMag4Image;
    Cur4Image:          TMag4VGear ;
    FMagImageList:      TMagImageList;
    FMagUtilsDB:        TMagUtilsDB;
    FUtils:             TMagVUtils;
    FDisplayStyle:      TMagdisplayStyles;
    FViewerStyle:       TMagViewerStyles;
    FTMagVClick:        TMagVClick;
    FTMagVlistchange:   TMagVListChange;
    FMagSecurity:       TMag4Security;
    FViewerClickEvent:  TMagViewerClickEvent;
    FLoadingMsgWin:     TfrmLoadingImageMessage;
{--------------------------------------}
    FLastRemember :     boolean;
    FLastHScroll,
    FLastVScroll:       integer;
    FUnDoList:          Tlist;
    FProxyList:         Tlist;
    FGearPool:          Tlist;
    FLoadedList:        Tlist;

    FUseAutoReAlign:    Boolean;
    FShowHints:         boolean;
    FImageDropped:      boolean;
    FShowToolbar:       boolean;
    FignoreMax:         boolean;
    FIgnoreLockSize:    boolean;
    FLockSizeSaved:     boolean;
    FScrollVertical:    boolean;
    FMaximizeImage:     boolean;
    FLockSize:          boolean;
    FScrollable:        boolean;
    FIsSizeAble:        boolean;
    FIsDragAble:        boolean;
    FApplytoall:        boolean;
    FAutoRedraw:        boolean;
    FPanWindow:         boolean;
    FZoomWindow:        Boolean;
    FAbsWindow:         Boolean;
    FScrollUp:          boolean;
    FClearBeforeAdd:    boolean;

    FCurUnDoGroup:      integer;
    FRowcount:          integer;
    FRowCountSaved:     integer;
    FColcount:          integer;
    FColCountSaved:     integer;
    FHorizScrollSaved:  integer;
    FColsp:             integer;
    FRowsp:             integer;
    FMaxCount:          integer;
    FLockHeight:        Integer;
    FLockWidth:         integer;
    FnextL:             integer;
    FnextT:             integer;
    FbaseW:             integer;
    FbaseH:             integer;
    FcurRow:            integer;
    FcurCol:            integer;
    FFontSize:          integer;

    FCaption:           string;
    FviewerDesc:        string;

    FPopupMenuImage:    TPopupMenu;

    procedure DragItOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
    procedure SetMaxCount(Value: integer);
    procedure SetLockSize(Value: Boolean);
    procedure SetZoomWindow(Value: boolean);
    procedure SetPanWindow(Value: boolean);
    procedure SetMaximizeImage(Value: boolean);
    procedure DropItOn(Sender, Source: TObject; X, Y: Integer);
    procedure SelectViewerImage(Sender: TObject);
    procedure CalcRowColusingHW;
    procedure CalcHWusingRowCol;
    procedure CalcNextLeftTop;
    procedure ImageVClick(Sender: Tobject; VGear: TMag4VGear);
    procedure ImageVMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure ImageVMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure ImageVKeydown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure AddImageToList(PI: TImageData; insertat: integer = -1);
    procedure InitLTHW;
    procedure SetAbsWindow(const Value: Boolean);
    procedure RealignProxy;
    procedure SetProxyPosition(i: integer);
    procedure LoadProxyImages(proxyindex: integer = -1);
    procedure LoadProxyImage(index: integer; closesecurity: boolean = false);
    function FirstVisProxy: integer;
    procedure ProxyClicked(Sender: TObject);
    procedure SetRowSpacing(const Value: integer);
    procedure SetColumnSpacing(const Value: integer);
    procedure SetColumnCount(const Value: integer);
    procedure SetRowCount(const Value: integer);
    procedure SetMagImageList(const Value: TMagImageList);
    procedure AttachMyself;
    procedure DeleteProxy(proxyindex: integer);
    procedure UnSelectAll;
    procedure SetViewerStyle(const Value: TMagViewerStyles);
    procedure SetMagUtilsDB(const Value: TMagUtilsDB);
    function GetImagePage: integer;
    function GetImagePageCount: integer;
    procedure SetImagePage(const Value: integer);
    procedure SetImagePageCount(const Value: integer);
    procedure SelectAnImage(xProxyindex: integer);
    function IsDuplicate(Iobj: TImageData): integer;
    function create_P(proxy: TimageProxy): TMag4VGear;
    procedure Destroy_p(proxy: TImageProxy);
    function GetDescVisible: boolean;
    procedure SetDescVisible(const Value: boolean);
    procedure RefreshViewerDesc;
    procedure PanCloseAll;
    procedure SetShowToolbar(const Value: boolean);
    function GetImageFontSize: integer;
    procedure SetImageFontSize(const Value: integer);
    procedure UpdateToolButtons;
    procedure UnDoGroupingNew;
    function GetSelectedCount: integer;
    //procedure UnDoListAddItem(txt: string; proxy: TImageProxy; undoaction: TMagundoAction);
    procedure UnDoItemClear(item: integer);
    procedure SetScrollAreaSize(ct: integer; vertscroll: boolean);
    function IsVisibleProxy(proxy: TimageProxy): boolean;
    procedure AddToLoadedList(proxy: TimageProxy);
  protected

  public
    {TODO: make these fields into Public procedures}
    FLastFit : integer; // used with other FLast* variables.
    DefaultLayout: TmagLayout;
    StopLoadingImages: boolean;

    constructor Create(AOwner: TComponent); override;
    destructor destroy; override;
{ ** UnDo functionality is Not implemented.
           when it is, it will enable a list (history) of UnDo items, that will
           redisplay the viewer in previous states. Main function will be
           to UnDo a removed image or images.}
    procedure SetRemember(value : boolean);
    function UnDoActionGetText: string;
    procedure UnDoListClear;
    procedure UnDoActionExecute;

{  ** Various Image Count methods  }
        {  opens the Image Viewer Settings window and allows user to choose
           number of Rows, Columns, Maximum Image to Display and LockImageSize.}
    procedure EditViewerSettings;
        {  Returns the Actual Number of Images that are visible. Not just Row * Col.
            If rows = 3 and columns = 4 but only 5 images are loaded, this function
            returns 5.}
    function VisibleCountActual: integer;
        {  Returns the number of Rows (row) and Columns (col) of the current
            viewer layout.}
    procedure VisibleRowCol(var row, col: integer);
        {  Returns the possible visible count.  result := Row * Columns.}
    function VisibleCount: integer;
        {  DblClick on Image to Maximize it.  DblClick Again to UnDo Maximize and
            return to the Layout before the image was maximized.}
    procedure MaxImageToggle(Iobj: TImageData);

{ **  TMagLayout contains Row,Col,MaxImage,LockSize properties.}
        {  Default layout is the saved user preference.
           Future: we can have multiple saved user layouts and
           some distributed layouts for the user to select from.}
    procedure ApplyDefaultLayout;
        {  returns a Text string that describes the Layout.}
    function GetDefaultLayoutDesc: string;
        {  When applying user preferences, this call populates default TMagLayout object}
    procedure SetDefaultLayout(Rows, Cols, MaxImages: integer ;LockSize: boolean; FitMeth : integer = 0);

{ ** Methods to return lists, single Images}
        {  Returns the list of TImageData Objects of all images in Viewer}
    function GetImageObjectList: Tlist;
        {  Returns just a list of TImageData Objects of all Selected Images}
    function GetSelectedImageList: Tlist;
        { count of images in viewer. All, proxies and loaded images.}
    function GetImageCount: integer;
        { returns the TGear Wrapper of the currently selected image.}
    function GetCurrentImage: TMag4VGear;
        { returns the TImageData object of the currently selected image}
    function GetCurrentImageObject: TImageData;

{ ** DataBase calls, passed through to TMagUtilsDB object.  TMagUtilsDB
           has a connection to the DataBase.  The Viewer does not. }
        { Print and Copy will first force user to select reason and enter Electronic
          signature before action can be performed.}
    procedure ImagePrint;
    procedure ImageCopy;
        { Display the report associated with the Image }
    procedure ImageReport;

{ ** Methods that deal with the Scrolling of Images }
        { scroll to specific Image referenced from ProxyList index }
    procedure ScrollToImage(ImageIndex: integer); overload;
        { scroll to specific Image referenced by the TImageData object of the Image }
    procedure ScrollToImage(IObj: TImageData); overload;
        { scroll to Image (calls ScrollToImage) and Load Image if it is proxy, and
          set as Selected Image}
    procedure ScrollAndDisplayImage(proxyindex: integer);
        { returns TRUE if Scrollbar is at the End of Scroll Range.
          (used to enable/disable Viewer Paging buttons)   }
    function ViewerScrollAtEnd: boolean;
        { returns TRUE if Scrollbar is at the Begginning of Scroll Range.}
    function ViewerScrollAtStart: boolean;
        { scroll to Begginning of Scroll range}
    procedure PageFirstViewer;
        { scroll to End of Scroll range}
    procedure PagePrevViewer;
        { scroll to Next Page in Scroll range}
    procedure PageNextViewer;
        { scroll to Previous Page in Scroll range}
    procedure PageLastViewer;

{ ** Various calls to move, realign, show certain images }
        { resize all visible images to fit in Viewer }
    procedure TileAll;
        { resize and align images to fit r: rows and c: columns in the viewer}
    procedure SetRowColCount(r, c: integer);
        { remove selected images from list.  Also hides the image. does not
          realign the remaining images.}
    procedure RemoveFromList;
        { called from many methods.  resizes and aligns the image to fit current rows
          and current columns of images in the viewer.  This function is called.
          When window is resized  }
    procedure ReAlignImages(ignmax: boolean = false; ignlocksize: boolean = false);
        { force a resize of all images to h: height and w: width then ReAlign the
          Images.  This is called when abstracts are resized }
    procedure ReSizeAndAlign(h,w : integer);
        { Not implemented.  This will change the 'Selected' state of all Images.}
    procedure ToggleSelected;
        { Remove all images from viewer.  There's no UnDoing this until
          undo functions are implemented.}
    procedure ClearViewer;

        {  enable/disable the showing of Hints in the Viewer}
    procedure ShowHints(value: boolean);
        {  The TStrings parameter is a list of images (full path to image)
           the items in the list are displayed in the Viewer.  }
    procedure ViewDirectoryImages(t: tstrings);
        { Main call to view an Image/List of Images in the Viewer
          ObjList: is a list of TImageData objects.
          allowdup : if user wants to display the same image twice. }
    procedure ImagesToMagView(ObjList: Tlist; allowdup: boolean = false);
        {  not implemented }
    procedure SetViewerDesc(desc: string; descvis: boolean);
        {  Future Prototype : to synchronize with selected images in other
           viewers, magListViews, MagTreeViews. }
    procedure SyncWithIMage(Iobj: TimageData);

{ **  This in the Implementation of the IMagObserver interface.
          The Viewer attaches to an ImagSubject object (in our case it
          is a TMagImageList object).  When the ImagSubject object changes, it
          notifies it's ImagObservers by calling their UpDate_ method.
          ImagObservers then update them selves.
          An example is the Tmag4Viewer in the Abstract window.  It updates itself
          when user selects a new filtered list of images.}
    procedure UpDate_(SubjectState: string);


{ ** The following methods are operations on the Image, the Viewer simply
          passes them through to the Image Object.
          (In the Future: These methods will be methods of the ImagImage interface) }

    procedure FitToWindow;
    procedure Fit1to1;
    procedure FitToWidth;
    procedure FitToHeight;
    procedure FlipVert;
    procedure FlipHoriz;
        {  deg : the degree to rotate the image from its current view.
                 90, 180, 270}
    procedure Rotate(deg: integer);
        {  Undo all rotate, brightness, zoom.  Fit the Image in Window.}
    procedure ResetImages;
    procedure Inverse;
        {  DeSkew : to straighten.  Smooth: uses Accusoft scale to gray algorithm
           that makes the image easier to read.}
    procedure DeSkewAndSmooth;
        {  performs a ReLoad of all images.}
    procedure ReFreshImages();
        {  mouse will act as magnifier }
    procedure MouseMagnify;
        {  mouse pointer }
    procedure MouseReset;
        {  mouse will pan the image }
    procedure MousePan;
        {  mouse will select area.  That area will be zoomed to (as close as possible) }
    procedure MouseZoomRect;
        {  change the zoom value of selected images.}
    procedure ZoomValue(value: integer);
    procedure ContrastValue(Value: integer);
    procedure BrightnessValue(Value: integer);
    procedure WinLevValue(WinValue, LevelValue: integer);
        {  for Multipage images }
        {  Future : These 'paging' methods will also work for Image Groups.}
    procedure PageFirstImage;
    procedure PageLastImage;
    procedure PageNextImage;
    procedure PagePrevImage;

  published
    {  Display Styly, ViewerStyle are not implemented. }
    property DisplayStyle: TMagDisplayStyles read FDisplayStyle write FDisplayStyle;
    property ViewerStyle: TMagViewerStyles read FViewerStyle write SetViewerStyle;
    {  Events to attach callbacks.}
    property OnViewerImageClick: TMagVClick read FTMagVClick write FTMagVClick;
    property OnViewerClick: TMagViewerClickEvent read FViewerClickEvent write FViewerClickEvent;
        { anytime an image is added, or removed from the viewer }
    property OnListChange: TMagVListChange read FTMagVlistchange write FTMagVListchange;
    {  Popup menu when Right Click on the Image.}
    property PopupMenuImage: TPopupMenu read FPopupMenuImage write FPopUpMenuImage;
    {  Imaging Components needed for total VistA Imaging Display functionality}
    {        Viewer will work without MagSecurity or MagUtilsDB but functionality
              will be limited }
    property MagSecurity: TMag4Security read FMagSecurity write FMagSecurity;
    property MagImageList: TMagImageList read FMagImageList write SetMagImageList;
    property MagUtilsDB: TMagUtilsDB read FMagUtilsDB write SetMagUtilsDB;
    {  If this is an Abstract window.  Some functions operate differently, and
          The Tmag4VGear objects also display text, captions differently.
          Some functions are not available for Abstracts.}
    property AbsWindow: Boolean read FAbsWindow write SetAbsWindow;
    property UseAutoReAlign: Boolean read FUseAutoReAlign write FUseAutoReAlign;
    {  Not Implemented.}
    property Caption: string read FCaption write FCaption;
    {  Font size for TMag4VGear object.  Caption under/over the Image}
    property ImageFontSize: integer read GetImageFontSize write SetImageFontSize;
    {  Show/Hide the Viewer Toolbar (Mainly for abstracts) }
    property ShowToolbar: boolean read FShowToolbar write SetShowToolbar;
    {  Abstracts have LockSize = true. When window is enlarged, abstracts stay
       same size and we see more of them.}
    property LockSize: boolean read FLockSize write SetLocksize;
    {   Open a Zoom Window for the Current Image }
    property ZoomWindow: Boolean read FZoomWindow write SetZoomWindow;
    {   Open a Pan Window for the Current Image }
    property PanWindow: Boolean read FPanWindow write SetPanWindow;
    {  not Implemented.  This is part of Future ViewerStyle functions.}
    property Scrollable: boolean read FScrollable write FScrollable;
    {  Direction of the ScrollBar.  (Viewer has only one.) }
    property ScrollVertical: boolean read FScrollVertical write FScrollVertical;
    {  Future : Prototype Drag and Drop }
    property IsDragAble: boolean read FIsDragAble write FIsDragAble;
    {  Future : Allow resizing images. (used sparingly in ReSizeAbstracts Function}
    property IsSizeAble: boolean read FIsSizeAble write FIsSizeAble;
    { operations on Images. will be applied to all loaded images }
    property ApplyToAll: boolean read FApplytoall write FApplytoall;
    {  Not Implemented:  -- actually it is set to true, not changeable }
    property AutoRedraw: boolean read FAutoRedraw write FAutoRedraw;
    {  To Add images to the viewer, this needs to be False.
       If true, the Viewer is cleared before any Drop, or AddImage function call}
    property ClearBeforeAddDrop: boolean read FClearBeforeAdd write FClearBeforeAdd;
    {  RowCount,RowSpacing,ColumnCount,ColumnSpacing will set the Properties, but
       will not automatically force a ReAlignImages.}
    property RowCount: integer read FRowcount write SetRowCount;
    property RowSpacing: integer read FRowsp write SetRowSpacing;
    property ColumnSpacing: integer read FColsp write SetColumnSpacing;
    property ColumnCount: integer read FColcount write SetColumnCount;
    {  The current image (Selected Image) will be maximized.}
    property MaximizeImage: boolean read FMaximizeImage write SetMaximizeImage;
    { reset the maximum number of Image to load into viewer. Does not do ReAlign }
    property MaxCount: integer read FMaxcount write SetMaxCount;
    {  LockWidth and LockHeight are used if FLockSize = TRUE.  Images are not
       resized, but the number of visible image changes when window is resized.}
    property LockWidth: integer read FLockWidth write FLockWidth;
    property LockHeight: integer read FLockHeight write FLockHeight;
 //   property Items: TStrings read FItems write SetItems;
    {  Not Implemented.}
    property ShowDescription: boolean read GetDescVisible write SetDescVisible;
    {  ImagePageCount is a ReadOnly property.  i.e. SetImagePageCount does nothing}
    property ImagePageCount: integer read GetImagePageCount write SetImagePageCount;
    {  Get/Set the current page to view}
    property ImagePage: integer read GetImagePage write SetImagePage;
 //   procedure ApplyCurrentToAll;  {TODO: Implement ApplyCurrentToAll}
  end;
const MaximumUnDoActions: integer = 5;

  (*NOTES. the old way, we knew we had an Accusoft TGear Control, and called its methods
     directly. We don't want to procede that way, and be tied to an Accusoft Control.
     So for now we'll call the methods of a TMag4VGear object, (a wrapper around
     a TGear control)
          (Later convert to an InterfacedObject ( yet to be defined) *)

procedure Register;

implementation
uses magRowColSize  ;
{$R *.DFM}

{ TMag4Viewer }


constructor TMag4Viewer.Create(AOwner: TComponent);
begin
  inherited;
  name := '';          //name := '' is a workaround for Delphi bug with Frames.
    FLastRemember := false;
    FLastFit := IG_DISPLAY_FIT_TO_WIDTH;
    FLastHScroll := 0;
    FLastVScroll:= 0;
  FFontSize := 8;
  StopLoadingImages := false;
  DefaultLayout := TMagLayout.create;
  FProxyList := Tlist.create;
  FGearPool := Tlist.create;
  FLoadedList := Tlist.create;
  FUtils := TMagVUtils.Create(nil);
  FUnDoList := Tlist.create;
  scrlV.align := alclient;
  lbscrlvsize.Caption := '';
  scrlV.HorzScrollBar.Tracking := true;
  scrlV.VertScrollBar.Tracking := true;
  height := 100;
  width := 85;
  OnDragOver := DragItOver;
  OnDragDrop := DropItOn;
  FUseAutoRealign := true;
  if FAbsWindow then FLockSize := True;
  FLoadingMsgWin := TfrmLoadingImageMessage.create(self);
  FLoadingMsgWin.update;
            if FAbsWindow then FLoadingMsgWin.SetDelay(10)     // 1000
          else FLoadingMsgWin.SetDelay(1000);   //3000
end;

destructor TMag4Viewer.destroy;
begin
// free memory of stuff we created.
  while (FProxyList.count > 0) do DeleteProxy(0);
  FProxyList.free;
  FGearPool.free;
  FLoadedList.free;
  Futils.free;
  FUnDoList.free;
  if assigned(FMagImageList) then
    begin
      FMagImageList.Detach_(IMagObserver(self));
      FMagImageList := nil;
    end;
  inherited;
end;

procedure Register;
begin
  RegisterComponents('Imaging', [TMag4Viewer]);
end;

procedure TMag4Viewer.scrlVClick(Sender: TObject);
begin
  if assigned(Onclick) then self.OnClick(Sender);
  if assigned(OnViewerClick) then self.OnViewerClick(self, self, cur4Image);
end;

procedure TMag4Viewer.pnlTopStartDrag(Sender: TObject;
  var DragObject: TDragObject);
begin
  //self.BeginDrag(true,3);
end;

procedure TMag4Viewer.pnlTopStartDock(Sender: TObject;
  var DragObject: TDragDockObject);
begin
  //self.BeginDrag(true,3);
end;

procedure TMag4Viewer.pnlTopClick(Sender: TObject);
begin
  //
end;

procedure TMag4Viewer.pnlTopEndDrag(Sender, Target: TObject; X,
  Y: Integer);
begin
  (*  tr.Left := 100;
    tr.Top := 100;
    tr.Right := 300;
    tr.Bottom := 300;
    self.ManualFloat(tr);*)
end;

procedure TMag4Viewer.pnlTopMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
begin
  //if (ssleft IN shift) then self.begindrag(false,5);
end;

procedure TMag4Viewer.pnlTopMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  //  self.begindrag(false, 10);
end;

procedure TMag4Viewer.DragItOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  { TODO : Add test to see if this viewer is allowing Droping }
  accept := true;
end;

procedure TMag4Viewer.DropItOn(Sender, Source: TObject; X, Y: Integer);
var I: integer;
  PI: TImageData;
  TN, TNC: TTreeNode;
  TLI: TListItem;
  listct: integer;
begin

  if FClearBeforeAdd then ClearViewer;
  listct := FProxyList.count;
  (* //new
  if (source is   Tfilelistbox) then
     begin
    {  with (source as tfilelistbox) do
     begin
     if selcount > 1 then
        begin
        for I := 0 to items.count -1 do
            if selected[i] then
                           begin
                           xlist.add(items[i]);
                           LoadoneImage(xlist.Strings[xlist.count-1]);
                           end;
        end
        else
        begin
        xlist.add(items[itemindex]);
        LoadoneImage(xlist.Strings[xlist.count-1]);
        end;

    end;
    }
    {xlist.assign((source as tfilelistbox).items);}
   {  for i := 0 to xlist.count-1  do {(source as Tfilelistbox).items.count-1 do}
         begin
         LoadoneImage(xlist[i]); {(source as Tfilelistbox).items[i]);}
         end;
       }{ this is out, for drag drop }
     end;
  //end new *)

  if (source is TTreeView) then
    begin
      TN := (Source as TTreeView).selected;
      if TN.HasChildren then
        for i := 0 to TN.count - 1 do
          begin
            TNC := TN.item[i];
            PI := TNC.data;
            AddImageToList(PI);
          end
      else
        begin
          PI := TN.data;
          AddImageToList(PI);
        end;
    end; {if (source is TTreeView) then}
  {-----}
  { for below, we need a way to get PI without having to USE a form.}
  if (source is TMag4VGear) then
    begin
      PI := (source as TMag4VGear).PI_ptrdata;
      AddImageToList(PI);
    end;
  {---------}
  if (source is TListView) then
    begin
      if TListView(source).selcount > 1 then
        begin
          for i := 0 to TlistView(source).items.count - 1 do
            if TlistView(source).items[i].selected then
              begin
                TLI := TMagListView(source).items[i];
                PI := TMagListViewData(TLI.data).iobj;
                AddImageToList(PI);
              end;
        end
      else
        begin
          TLI := TMagListView(source).selected;
          PI := TMagListViewData(TLI.data).iobj;
          AddImageToList(PI);
        end;
    end; { if source is TListView }
  ReAlignImages;
  if listct < FProxyList.count then
    begin
      ScrollToImage(listct);
      ReAlignImages;
    end;
end; {DropItOn}


procedure TMag4Viewer.SetProxyPosition(i: integer);
var Iproxy: TImageProxy;
begin
  Iproxy := FProxyList[i];
  if FbaseH = 0 then CalcHWusingRowCol;
  if not FScrollVertical
    then Iproxy.setbounds(FnextL - scrlV.horzscrollbar.position, FnextT, FbaseW, FBaseH)
  else Iproxy.setbounds(FnextL, FnextT - scrlV.Vertscrollbar.position, FbaseW, FBaseH);
  if Iproxy.Image <> nil then
    begin
      Iproxy.Image.setbounds(Iproxy.left, Iproxy.top, Iproxy.width, Iproxy.height);
      if iproxy.visible then Iproxy.visible := false;
      Iproxy.Image.ReDrawImage;
      Iproxy.image.Update;
    end
  else if not Iproxy.visible then iproxy.visible := true;
  CalcNextLeftTop;
  update;
end;

procedure TMag4Viewer.ImagesToMagView(ObjList: Tlist; allowdup: boolean = false);
var
  xFirstProxyIndex: integer;
  Iobj: TImageData;
  i,  dupindex: integer;
  undotext: string;
begin
  if objlist.count = 0 then exit;
  if Objlist.Count > 1 then undotext := 'UnDo - Add Image(s)'
  else undotext := 'UnDo - Add Image';
  UnDoGroupingNew;
  try
    {  Add or Clear first, depending on setting. {}
    dupindex := -1;
    if FClearBeforeAdd then clearViewer;
    if ObjList.Count = 1 then
      begin
        dupindex := IsDuplicate(Objlist[0]);
        if dupindex > -1 then
          begin
            {  If image is already displayed, scroll to it.  Quit if not add duplicate}
            ScrollAndDisplayImage(dupindex);
            if not allowdup then exit;
          end;
      end;
     {  first added image will have index of xFirstProxyIndex.}
    xFirstProxyIndex := FProxyList.Count;

{TODO:  HERE TRY TO speed up the display of large lists, by loading and displaying
                                        some, then the others. }
    for i := 0 to ObjList.Count - 1 do
      begin
        Iobj := ObjList[i];
        AddImageToList(Iobj, dupindex);    // adds to proxy list
      end;
    {   here we need a description of images in the viewer. Images have been
         added to list, display first FMaxcount of newly added Images.}
    if assigned(OnListChange) then OnListChange(self);
    ReAlignImages;   // calls RealignProxy
                        //   RealingProxy has:
                        //   for... loop for all proxies - calling SetProxyPosition
                        //   calls loadProxyImages
                                //  LoadProxyImages has :
                                //  for...Loop for all proxies - calling LoadProxyImage.
    ScrollAndDisplayImage(xFirstProxyIndex);
    application.processmessages;
    RefreshViewerDesc;
  finally
    FImageDropped := false;
  end;
end;

procedure TMag4Viewer.RefreshViewerDesc;
begin
  pnltop.Caption := 'Displaying: ' + inttostr(VisibleCountActual)
    + ' of ' + inttostr(Fproxylist.Count) + FViewerDesc;
  if assigned(onlistchange) then onlistchange(self);
end;

function TMag4Viewer.IsDuplicate(Iobj: TImageData): integer;
var i: Integer;
begin
  { loop to find if this Iobj is already in the FProxyList{}
  result := -1;
  for i := 0 to FProxyList.count - 1 do
    begin
      if (TImageProxy(Fproxylist[i]).imagedata.Mag0 = Iobj.Mag0) then
        begin
          result := i;
          break;
        end;
    end;
end;

procedure TMag4Viewer.ScrollAndDisplayImage(proxyindex: integer);
begin
  ScrollToImage(proxyindex); // needthis?  we scroll below.
  if TImageProxy(Fproxylist[proxyindex]).Image = nil
    then LoadProxyImage(proxyindex, true);
  SelectAnImage(proxyindex);
  update;
  application.ProcessMessages;
  ScrollToImage(proxyindex);
end;

procedure TMag4Viewer.SelectAnImage(xProxyindex: integer);
var
  xProxy: TImageProxy;
begin
  UnSelectAll;
  xProxy := TImageProxy(FProxyList[xProxyindex]);
  if assigned(xProxy.Image) then
    begin
      xProxy.image.Selected := true;
      Cur4Image := xProxy.Image;
      if (not abswindow) then
        begin
          if assigned(OnViewerClick) then OnViewerClick(self, self, cur4image);
          if assigned(OnViewerImageClick) then OnViewerImageClick(cur4image);
        end;
    end;
  UpdateToolButtons;
end;

procedure TMag4Viewer.AddImageToList(PI: TImageData; insertat: integer = -1);
var Iobj: TImageData;
  Iproxy: TImageProxy;
begin
  { DONE :
 Decide if we're creating a new copy of the list of objects (Iobj and MagAssign)
  or using the existing list, and referencing the existing objects. }
  {Here we are Creating a new copy of TImageData objeccts{}
  Iobj := TImageData.create;
  Iobj.magAssign(PI);
  Iproxy := TImageProxy.create(self);
  // IProxy.transparent := true;  // testing
  Iproxy.parent := scrlv;
  Iproxy.color := scrlv.Color;
  Iproxy.imagedata := Iobj;
  if FAbsWindow then
    begin
      Iproxy.font.Name := 'Small Fonts';
      Iproxy.Font.Size := 7;
    end;
  if insertat > -1 then
    begin
      FProxyList.Insert(insertat, IProxy);
      Iproxy.loadindex := insertat;
    end
  else
    begin
      FProxyList.add(Iproxy);
      Iproxy.loadindex := FProxyList.Count - 1;
      insertat := FProxyList.count;
    end;
  Iproxy.Caption := '#' + inttostr(insertat) + ' ' + Iobj.ExpandedDescription;
  Iproxy.OnClick := ProxyClicked;
end;

procedure TMag4Viewer.ProxyClicked(sender: Tobject);
begin
  LoadProxyImage(FProxyList.indexof(sender), true);
end;


procedure TMag4Viewer.TileAll;
var i: Integer;
begin
  if FScrollVertical then scrlv.VertScrollBar.Position := 0
  else scrlv.HorzScrollBar.Position := 0;
  i := FProxylist.count;
  case i of
    1: SetRowColCount(1, 1);
    2: SetRowColCount(1, 2);
    3, 4: SetRowColCount(2, 2);
    5, 6: SetRowColCount(2, 3);
    7, 8, 9: SetRowColCount(3, 3);
    10, 11, 12: SetRowColCount(3, 4);
    13, 14, 15, 16: SetRowColCount(4, 4);
    17, 18, 19, 20: SetRowColCount(4, 5);
    21, 22, 23, 24, 25: SetRowColCount(5, 5);
    26..30: SetRowColCount(5, 6);
    31..36: SetRowColCount(6, 6);
    37..42: SetRowColCount(6, 7);
    43..49: SetRowColCount(7, 7);
    50..56: SetRowColCount(7, 8);
    57..64: SetRowColCount(8, 8);
  else SetRowColCount(10, 10);
  end; {case}
end;

procedure TMag4Viewer.ReAlignImages(ignmax: boolean = false; ignlocksize: boolean = false);
begin
  FIgnoreMax := ignmax;
  FIgnoreLockSize := ignlocksize;
  case FViewerStyle of
    magvsVirtual: RealignProxy;
    magvsDynamic: RealignProxy;
    magvsStaticPage: RealignProxy;
  end;
  refreshViewerDesc;
  UpdateToolButtons;
end;

procedure TMag4Viewer.ReSizeAndAlign(h,w : integer);
var locksize,ignorelocksize : boolean;
begin
 ignorelocksize := FIgnoreLockSize;
 locksize := Flocksize;
 try
   FlockHeight := h;
   FlockWidth := w;
   ReAlignImages;
 finally
   FignoreLockSize :=  ignorelocksize;
   Flocksize :=locksize ;
 end;
end;

procedure TMag4Viewer.UpdateToolButtons;
begin
  StopLoadingImages := true;
  if not FScrollVertical then
    begin
      tbtnAbsPagePrev.enabled := (scrlv.HorzScrollBar.Visible) and (scrlv.HorzScrollBar.Position > 0);
      tbtnAbsPageNext.enabled := (scrlv.HorzScrollBar.Visible) and ((scrlv.HorzScrollBar.Position + Scrlv.width) < scrlv.HorzScrollBar.Range);
    end
  else
    begin
      tbtnAbsPagePrev.enabled := (scrlv.VertScrollBar.Visible) and (scrlv.VertScrollBar.Position > 0);
      tbtnAbsPageNext.enabled := (scrlv.VertScrollBar.Visible) and ((scrlv.VertScrollBar.Position + scrlv.height) < scrlv.VertScrollBar.Range);
    end;
end;

procedure TMag4Viewer.RealignProxy(); //FViewerStyle = magvsVirtual (Proxy's)
var i: integer;
  xcursor : TCursor;
begin
        {       Based on Row and Col count, we get obH and obW, with obH,obW we
                then calc the Height Width needed to hold all, and resize the
                Scrollbar. Then put proxy for each obImage and display visible ones.{}
  if application.Terminated then exit;
  if FProxyList.count = 0 then exit;
  maggmsgf.MagMsg('s', '###Realign Proxy');
  if (FColCountsaved > 0) and (Fignoremax) then
    begin
      FIgnoreLockSize := true;
      FColCount := FColCountSaved;
      FColCountSaved := 0;
      FRowCount := FRowCountSaved;
      FRowCountSaved := 0;
      FLockSize := FLockSizeSaved;
      FignoreMax := false;
    end;
  InitLTHW;
  update;
  SetScrollAreaSize(FProxyList.count, FScrollVertical);
  xcursor := Screen.cursor;
  screen.Cursor := crHourGlass;
  for i := 0 to FProxyList.count - 1 do
    begin
      SetProxyPosition(i);
    end;
  screen.cursor := xcursor;  // p8t35  was changing back to default too soon.
                            // was intentional. to allow clicking on an abs as
                            // other loaded.   Was an unknown feature, and
                            // now could be causing access violations.
  scrlv.Visible := true;
  LoadProxyImages(FirstVisProxy);
  lbscrlvsize.SetBounds(0, 0, 12, 12);
end;

{We know FBaseH, FBaseW, FcolCount, FRowCount, and 'ct' count of images}

procedure TMag4Viewer.SetScrollAreaSize(ct: integer; vertscroll: boolean);
var h, w: integer;
begin
  if ct < 100 then exit;
  if vertscroll then // need height;
    begin
      w := scrlv.width;
      h := Trunc(((ct div Fcolcount) + 1) * (FBaseH + Frowsp));
    end
  else // need width;
    begin
      h := scrlv.height;
      w := Trunc(((ct div Frowcount) + 1) * (FBasew + Fcolsp));
    end;
  lbscrlvsize.SetBounds(w - 30, h - 30, 2, 2);
end;

function TMag4Viewer.FirstVisProxy: integer;
var i: Integer;
  muchvisible: integer;
begin
  if FScrollVertical
    then muchvisible := FBaseH - (FBaseH div 2) // if  1/8 is showing
  else muchvisible := FBaseW - (FBaseW div 2); // if  1/8 is showing

  for i := 0 to FProxyList.count - 1 do
    begin
      if FScrollVertical
        then
        begin
          if ((TImageProxy(FProxylist[i]).top + muchvisible) > 0) then break;
        end
      else if ((TImageProxy(FProxylist[i]).left + muchvisible) > 0) then break;
    end;
  result := i;
end;

// Is this proxy, in the visible area on the ScroolBox (scrlv)

function TMag4Viewer.IsVisibleProxy(proxy: TimageProxy): boolean;
var  precentvisible: integer;
begin
  if FScrollVertical
    then precentvisible := FBaseH - (FBaseH div 2) // if  1/8 is showing
  else precentvisible := FBaseW - (FBaseW div 2); // if  1/8 is showing
  if FScrollVertical
    then result := ((proxy.top + precentvisible) > 0)
    and ((proxy.top + precentvisible) < (scrlv.height))
  else result := ((proxy.left + precentvisible) > 0)
    and ((proxy.left + precentvisible) < (scrlv.Width));
end;


// Load the Images from the ProxyList starting with proxyindex

procedure TMag4Viewer.LoadProxyImages(proxyindex: integer = -1);
var i, j, visct: integer;
  proxy: TImageProxy;
  cttoload: integer;
  xmsg: string;
  curct : integer;  // p8t43  only show stop loading message window after 1st image.
begin
  if FProxyList.count = 0 then exit;
  if proxyindex = -1 then proxyindex := FirstVisProxy;
  cttoload := FMaxCount; // The maximum to load : FMaxCount
  j := proxyindex; //Start with this one j (probably computed by using FindFirstVisibleProxy
  if FScrollVertical
    then visct := (rowcount * columncount) + ColumnCount // This forces FMaxCount >= VisibleCount;
  else visct := (rowcount * columncount) + RowCount;
  if (visct > FMaxCount) then cttoload := visct; // we'll load more than MaxCount if more are visible.
  if (cttoload > (FProxylist.Count - j)) then cttoload := FProxyList.count - j; // but not more than are in list.
  { TODO : Put in the Cancel Loading Abstracts Window Here : }
  if cttoload = 0 then exit;
  StopLoadingImages := false;
  if (FProxyList.count > 0) and FAbsWindow then   //p8t42 added FAbsWindow.
    begin
    //TESTING OUT FOR P8T42
    (*  if frmLoadingImageMessage = nil then
        begin
          frmLoadingImagemessage := tfrmLoadingImagemessage.create(self);
          if FAbsWindow then frmLoadingImageMessage.SetDelay(1000)
          else frmLoadingImageMessage.SetDelay(3000);
        end;
      frmLoadingImageMessage.Init(cttoload, self);
    *)
     FLoadingMsgWin.init(cttoload);
     FLoadingMsgWin.RequestToStopLoading := FALSE;

    end;
  curct := 0;
  for i := j to j + cttoload - 1 do
    begin
      if (i > FProxyList.count - 1) then break;
      proxy := FProxyList[i];
      if proxy.Image <> nil then
        begin
          if (not proxy.Image.Visible) then proxy.Image.Visible := true;
          //Proxy.Image.RefreshImage; // t35 repaint;
          continue;
        end;
      inc(curct) ;
      if (curct > 1 ) then FLoadingMsgWin.UpdStatus(proxy.imagedata.ImgDes, i, proxyindex, proxyindex + cttoload, FProxyList.count);
      LoadProxyImage(i);
      application.processmessages;
      if FLoadingMsgWin.RequestToStopLoading then break;
    end;
  if assigned(magSecurity) then FMagSecurity.MagCloseSecurity(xmsg);
   { frmLoadingImageMessage will be used for all vieweres, not just abstracts,
      full res window viewers will have a longer delay, before it comes visible.{}
   FLoadingMsgWin.close;
end;

procedure TMag4Viewer.LoadProxyImage(index: integer; closesecurity: boolean = false);
{ All loading of images into the Gear components goes through here.
  The call vGear.LoadTheImage(oneimage) does the actualLoading}
var
  vGear: TMag4VGear;
  oneimage,errstr,notfoundstr: string;
  xmsg: string;
  proxy: TImageProxy;
begin
 (* SLOW EVERYTHING DOWN.  FOR TESTING.
  FOR I := 0 TO 220000 DO
    BEGIN
    PNLTOP.CAPTION := INTTOSTR(I);
    PNLTOP.Update;
    // if visible then it is slower.
    IF GetDescVisible  AND (I > 5000) THEN BREAK
    END;                 *)
try
  proxy := TImageProxy(FProxyList[index]);
  vGear := create_P(proxy); //possible change the create(scrlV)
  vGear.listindex := index;
  proxy.Image := vGear;

  if FAbswindow then oneimage := Futils.ResolveAbstract(Proxy.imagedata)
  else oneimage := Proxy.imagedata.FFile;
  if FbaseH = 0 then CalcHWusingRowCol;

  vGear.setbounds(proxy.Left, proxy.Top, proxy.Width, proxy.Height);
  proxy.visible := false;
  vGear.visible := true;
  vGear.ImageDescription := proxy.imagedata.ExpandedDescription(abswindow);
  update;
  Cur4Image := vGear;
  { TODO : rethink the properties of vGear, ( Mag4VGear) do we need them all now. ?
       ien as tag. caused us problems in some sites,  to long.

  {-------- Switch image path to Cache directory, if it exists.---------}
  if FileExists(Futils.AppPath + '\cache\'+ ExtractFileName(oneImage) )
    then
      begin
       oneimage := Futils.AppPath + '\cache\'+ ExtractFileName(oneImage);
       maggmsgf.MagMsg('s','Using Cached Image : '+ ExtractFileName(oneImage));
      end
    else
    begin { we get here because cached image doesn't exist}
      if FAbsWindow then
        begin
          errstr :=   '\bmp\AbsError.bmp';
          notfoundstr :=   '\bmp\NotExist.bmp';
        end
        else
        begin
          errstr := '\bmp\FullResFileOpenError.bmp';
          notfoundstr := '\BMP\FullResFileNotFound.BMP';
        end;

      if assigned(magsecurity) then
          begin
            if (not FMagSecurity.MagOpenSecurePath(oneimage, xmsg))
              then oneimage := Futils.AppPath + errstr
          end;
      if (not fileexists(oneimage)) then oneimage := FUtils.AppPath + notfoundstr;
    end;
  vGear.LoadTheImage(oneimage);
finally
  if (assigned(magsecurity) and closesecurity)
    then FMagSecurity.MagCloseSecurity(xmsg);
end;
end;

function TMag4Viewer.create_P(proxy: TimageProxy): TMag4VGear;
var visct, i: integer;
  tmpVGear : TMag4VGear;
begin
  I := 0;
  visct := Visiblecount;
  if FScrollVertical then Visct := visct + FColcount
  else visct := visct + FRowcount;
  {  Here we will put one in the Gear Pool if max and visible counts are exceeded.}
  if (Floadedlist.Count > (FMaxCount - 1)) and (Floadedlist.count > (visct - 1)) then
    begin
      { Don't want to destroy a Currently VISIBLE Proxy Image. {}
      { we might only need the top row, but this way, could cause the whole page
       to be incrementally updated, robbing a page to pay a page,
                                        lets just rob the unseen guy)}
      if FScrollUP then
        for I := 0 to FLoadedList.count - 1 do
          begin
            if not IsVisibleProxy(FloadedList[i]) then break;
          end;
      if not FScrollUP then
        for I := FLoadedList.count - 1 downto 0 do
          begin
            if not IsVisibleProxy(FloadedList[i]) then break;
          end;
      Destroy_p(TImageProxy(FloadedList[i]));
      Floadedlist.Delete(i);
    end;

  if FGearPool.Count = 0 then
    begin
      Result := TMag4VGear.create(self);
      {         smoothing image with DeSkew (later for TIFS) and
                and setting AliasMode, AliasQuality
                we set it here, after Create.  It will stay set.

                We are setting Accusoft properties :
                TGear.AliasMode := IG_DISPLAY_ALIAS_SCALE_TO_GRAY;
                TGear.AliasQuality := 100;                       {}
     Result.SmoothImage;
     Result.Gear1.AutoRedraw := TRUE;
     Result.OnImageClick := ImageVclick; ;
     Result.OnImageMouseDown := ImageVMouseDown;
     Result.OnImageMouseUp := ImageVMouseUP;
     Result.OnImageKeyDown := ImageVKeyDown;
     Result.Height := 5;
     Result.Width := 5;
    end
  else
    begin
      result := FGearPool[0];
      FGearPool.Delete(0);
    end;
  AddToLoadedList(proxy);
{   Set Default properties }
 with result do
   begin
     PI_ptrData := proxy.imagedata;
     AbstractImage :=FAbsWindow;
     Fontsize := FFontSize;
     parent := scrlV;
     IsDragAble := FIsDragable;
     IsSizeAble := FIsSizeAble;
     if FLastRemember then
     case FLastFit of
       IG_DISPLAY_FIT_TO_WINDOW : result.fitToWindow;
       IG_DISPLAY_FIT_TO_WIDTH :  begin
                                    result.FitToWidth;
                                    result.MousePan;
                                   end;
       IG_DISPLAY_FIT_TO_HEIGHT : begin
                                    result.FitToHeight;
                                    result.MousePan;
                                   end;
       IG_DISPLAY_FIT_1_TO_1 :  begin
                                    result.Fit1to1;
                                    result.MousePan;
                                   end;

     END;

   end;
end;

{       keep it sorted by Proxy (left or right) so we can switch a VGear with
        an invisible one. {}
procedure TMag4Viewer.AddToLoadedList(proxy: TimageProxy);
var i: integer;
  caninsert: boolean;
begin
  caninsert := false;
  if FLoadedList.count = 0 then i := 0
  else
    for i := 0 to Floadedlist.count - 1 do
      begin
        if FScrollVertical and (proxy.Top < TImageProxy(Floadedlist[i]).top) then
          begin
            caninsert := true;
            break;
          end;
        if not FScrollVertical and (proxy.Left < TImageProxy(Floadedlist[i]).left) then
          begin
            caninsert := true;
            break;
          end;
      end;
  if caninsert then FloadedList.Insert(i, proxy)
  else FLoadedList.add(proxy);
end;
//oot 7/16/02
(*
procedure TMag4Viewer.ReAlignDynamic;
var i: integer;
begin
InitLTHW;        //commented out
update;
//HideAllImages;
for i := 0 to I3ptrList.count - 1 do
begin
 if I = FMaxcount then break;
 DisplayOneImage(i);
 //  PI := AllPIList[i];
 //  loadoneimage(AllPIList.count-1,PI);
end;
end;
*)

//oot 7/16/02
(*
procedure TMag4Viewer.ReAlignStatic;
var i: Integer;
begin
InitLTHW;   //commented out
update;
//HideAllImages;
for i := 0 to I3ptrList.count - 1 do
begin
 if I = FMaxcount then break;
 DisplayOneImage(i);
 //  PI := AllPIList[i];
 //  loadoneimage(AllPIList.count-1,PI);
end;
end;

*)

procedure TMag4Viewer.SetLockSize(Value: boolean);
var Iproxy: TImageProxy;
begin
  FLockSize := value;
  if (FLockSize and (FLoadedList.count > 0)) then
        begin
          Iproxy := FLoadedList[0];
          FLockHeight := Iproxy.Image.Height;
          FLockWidth := IProxy.Image.Width;
        end;
end;

procedure TMag4Viewer.SetZoomWindow(Value: boolean);
begin
  if Cur4Image = nil then exit;
  Cur4Image.Zoomwindow := value;
end;

procedure TMag4Viewer.SetPanWindow(Value: boolean);
var x, y: integer;
  viewerpt, imagept: tpoint;
begin
  FPanWindow := value;
  PanCloseAll;
  {    below, we are only appling PanWindow to the Current Image{}
  if Cur4Image = nil then exit;
  viewerpt := Parent.ClientOrigin;
  imagept := Cur4Image.ClientOrigin;
  x := imagept.x - 100 + Cur4Image.Gear1.Width;
  y := imagept.y;
  if value then Cur4Image.PanWindowSettings(110, 85, x, y)
  else Cur4Image.panwindow := value;

exit;
{ Below was when we were doing ApplyToAll pan window.
     Not this easy, pan windows don't move with the images when they are
     scrolled, so easy to get all confused. {}

(*
  if FApplyToall then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then
          begin
            imagept := Iproxy.Image.ClientOrigin;
            x := imagept.x - 100 + Iproxy.Image.Width;
            y := Imagept.y;
            if value then Iproxy.Image.PanWindowSettings(110, 85, x, y)
            else Iproxy.Image.PanWindow := false;
          end;
      end
  else //if (SelectedCount > 0) then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil
          then if TMag4Vgear(Iproxy.Image).selected
          then
            begin
              imagept := Iproxy.Image.ClientOrigin;
              x := imagept.x - 100 + Iproxy.Image.Width;
              y := Imagept.y;
              if value then Iproxy.Image.PanWindowSettings(110, 85, x, y)
              else Iproxy.Image.PanWindow := false;
            end;

      end;
*)

// below was when we were only appling PanWindow to the Current Image
(*  if Cur4Image = nil then exit;
  viewerpt := Parent.ClientOrigin;
  imagept := Cur4Image.ClientOrigin;
  x := imagept.x -100 + Cur4Image.Gear1.Width;
  y := imagept.y -100 ;
  if value then  Cur4Image.PanWindowSettings(110,85,x,y)
           else  Cur4Image.panwindow := value;   *)

 (* this is from old versions
  Full1.GUIWINDOW := GUIPAN;
  Full1.GUIHeight := 100;
  Full1.GUIWIDTH := 100;
  Full1.GUIXPOS := x;
  Full1.GUIYPOS := y;
*)
end;


procedure TMag4Viewer.PanCloseAll;
var I: integer;
begin
  for i := 0 to FProxyList.Count - 1 do
    begin
      if TImageproxy(FProxyList[i]).Image <> nil
        then TImageproxy(FProxyList[i]).Image.PanWindow := false;
    end;
end;

procedure TMag4Viewer.MaxImageToggle(Iobj: TImageData);
begin
  if ((FcolCount = 1) and (Frowcount = 1)) and ((Fcolcountsaved > 0) and (Frowcountsaved > 0))
    then
    begin
      SetRowColCount(Frowcountsaved, Fcolcountsaved);
        //ScrollToImage(iobj);
      if FHorizScrollSaved > 0 then scrlv.HorzScrollBar.Position := FHorizScrollSaved;
    end
  else SetMaximizeImage(true);
end;


procedure TMag4Viewer.SetMaximizeImage(Value: boolean);
var tcol, trow, thorizscroll: integer;
begin
  if Cur4Image = nil then exit;
  FMaximizeImage := value;
  if not value then
    begin
      ReAlignImages;
      exit;
    end;
  tcol := FColCount;
  trow := FRowCount;
  thorizscroll := scrlv.HorzScrollBar.Position;
  SetRowColCount(1, 1);
  ReAlignImages;
  ScrolltoImage(TImageData(Cur4Image.PI_ptrData));
  FColCountSaved := tcol;
  FRowCountSaved := trow;
  FHorizScrollSaved := thorizscroll;
  FLockSizeSaved := FLockSize;
  FLockSize := false;
end;

procedure TMag4Viewer.SetMaxCount(Value: integer);
begin
  if (value < 1)
    then FMaxCount := 1
    else FMaxCount := value;
end;


procedure TMag4Viewer.DeleteProxy(proxyindex: integer);
var Iobj: TImageData;
  Iproxy: TImageProxy;
  vGear: TMag4vGear;
begin
  Iproxy := FProxyList[proxyindex];
  Iobj := Iproxy.imagedata;
  { DONE :  AccessViolations, if Iobj isn't a MagAssigned object,
               then we are deleting the Object out of magImageList ( objlist); }
  Iobj.free;
  VGear := Iproxy.Image;
  if (VGear <> nil) then
    begin
      Destroy_p(Iproxy);
    end;
 //    .... brings back memories of old problems DESTROYING with Gear1.
  application.processmessages;
  if (FloadedList.IndexOf(iproxy) > -1) then FLoadedList.Delete(FloadedList.IndexOf(iproxy));
  FProxylist.Delete(proxyindex);
  Iproxy.Free;
end;

procedure TMag4Viewer.Destroy_p(proxy: TImageProxy);
var vGear: TMag4VGear;
begin
  vGear := proxy.image;
  proxy.image := nil;
  if VGear = nil then exit;
  if VGear.PanWindow then VGear.PanWindow := false;
  VGear.ResetImage;
  VGear.Visible := false;
  VGear.ClearImage;
  vGear.Update;
  //   vGear.Free;             //  in for test xx8 p8t35
  FGearPool.Add(VGear);        // out for test xx8 p8t35
  VGear.PI_V4Data.ClearData;   // out for test xx8 p8t35
  if not proxy.visible then proxy.visible := true;
end;


procedure TMag4Viewer.ClearViewer;
var i: Integer;
                { undoItem: TmagUnDoItem;}
begin
  if FProxyList.Count = 0 then exit;
   UnDoGroupingNew;
   pnltop.caption := '';
   application.processmessages;
   Update;
   try
   scrlv.visible := false; // in for testing p8t34
    for I := FProxyList.count - 1 downto 0 do
      begin
      DeleteProxy(i);
        (*
        UnDoListAddItem('UnDo - Close All', TImageProxy(FProxyList[i]), magundoRemove); //ClearViewer;
        TImageProxy(FProxyList[i]).Image.Visible := false;
        FProxyList.Delete(i);
        *)
      end;
    FLoadedList.Clear;
  finally
{TODO:  This might need checked.  if the form is closed, maybe error.}
   scrlv.visible := true;
  end;
  Cur4Image := nil;
  if assigned(OnViewerImageClick) then OnViewerImageClick(cur4Image); // 01/21/03
end;

procedure TMag4Viewer.CalcRowColusingHW;
var w, h, scrollbarw: integer;
begin
  h := scrlv.Height;
  w := scrlv.width;
  scrollbarw := 20; {TODO: how to calculate the ScrollBar Thumb Width ? }
  if FScrollVertical then w := w - scrollbarw
  else h := h - scrollbarw;
  FColCount := trunc((w - FColsp) / (FbaseW + FColsp));
  if FColCount = 0 then FColCount := 1;
  FRowCount := Trunc((h - FRowsp) / (FbaseH + FRowsp));
  if (FRowCount = 0) then FRowCount := 1;
end;

{ Calculate the Height and Width of the Image Obj, based on FColCount,
  FRowCount and the Height and Width of the Scroll Box (parent of the Image Obj}

procedure TMag4Viewer.CalcHWusingRowCol;
begin
  if (FColcount = 0) then FColcount := 1;
  if (FRowcount = 0) then Frowcount := 1;
  if FScrollVertical then
    begin
      FbaseH := (scrlV.Height - (FRowsp * FrowCount)) div FRowCount;
      FbaseW := (scrlv.Width - (FColSp * FcolCount) - 20) div FColCount;
    end
  else
    begin
      FbaseH := (scrlV.Height - (FRowsp * FrowCount) - 20) div FRowCount;
      FbaseW := (scrlv.Width - (Fcolcount * Fcolsp)) div FColCount;
    end;
  if not FAbsWindow then
    begin
    //1/28/03   diff := scrlv.width - (fcolcount * (FBaseW + Fcolsp));
    //1/28/03   if diff > 0 then width := width + diff;
    end;
end;

{ Calcutate the next pixel location for an Image. Based on FcurRow,Col FbaseH,W etc}

procedure TMag4Viewer.CalcNextLeftTop;
begin
  if not FScrollVertical then
    begin
      if FcurRow = FRowCount then
        begin
          FcurRow := 1;
          FcurCol := FcurCol + 1;
        end
      else FcurRow := FcurRow + 1;
      FnextT := (FcurRow * (FbaseH + FRowsp) - (FbaseH));
      FnextL := (FcurCol * (FbaseW + FColsp) - (FbaseW));
    end
  else // so Must be Horizontal Scroll.
    begin
      if FcurCol = FColCount then
        begin
          FcurCol := 1;
          FcurRow := FcurRow + 1;
        end
      else FcurCol := FcurCol + 1;
     // next two line wern't changed.
      FnextT := (FcurRow * (FbaseH + FRowsp) - (FbaseH));
      FnextL := (FcurCol * (FbaseW + FColsp) - (FbaseW));
    end;
end;


procedure TMag4Viewer.InitLTHW;
begin
  FcurRow := 1;
  FcurCol := 1;

  FnextT := FRowsp;
  FnextL := FColsp;
  try
    if Flocksize and (not FIgnoreLockSize) then
      begin
        FbaseH := FLockHeight;
        FbaseW := FLockWidth;

        if FBaseH < 55 then FBaseH := 55;
        if FBaseW < 40 then FBasew := 40;
        //p8t25 if abswindow then FbaseW := trunc(FbaseH * 0.75);
        CalcRowColusingHW;
      end
    else //if not Flocksize then
      begin
        CalcHWusingRowCol;
        if FBaseH < 55 then FBaseH := 55;
        if FBaseW < 40 then FBasew := 40;
        //p8t25 if abswindow then FbaseW := trunc(FbaseH * 0.75);
      end;
  finally
    FignoreLockSize := false;
  end;
end;

procedure TMag4Viewer.SetRowColCount(r, c: integer);
begin
  FColCountSaved := 0;
  FRowCountSaved := 0;
  FRowcount := r;
  FColCount := c;
  FIgnoreLockSize := true;
  ReAlignImages(true, true);
  if FLockSize then
    begin
      SetLockSize(true);
    end;
end;

procedure TMag4Viewer.SelectViewerImage(sender: TObject);
begin
  begin
    TMag4VGear(sender).selected := not Tmag4VGear(sender).selected;
    if (TMag4VGear(sender) = Cur4Image) and (TMag4VGear(sender).selected = false)
      then
      begin
        Cur4Image := nil;
      end;
  end;
end;

procedure TMag4Viewer.ImageVMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var clpt: TPoint;
begin
  if FIsDragable and (Shift = [ssleft]) then
    begin
      if TMag4VGear(sender).selected then
        begin
          Cur4Image := TMag4VGear(Sender);
        end
      else
        begin
          UnSelectAll;
          SelectViewerImage(sender);
          Cur4Image := TMag4VGear(Sender);
        end;
       //(Sender as TMag4VGear).Begindrag(false,20);  // Begindrag problem
    exit;
    end;
  if button = mbleft then
    begin
      if (ssctrl in Shift) then SelectViewerImage(sender);
      if (Shift = [ssleft]) then
        begin
          ImageVClick(Sender, TMag4VGear(sender));
        end;
    end;
  if button = mbright then
    begin
      if (Shift = [ssright]) then
        begin
          if TMag4VGear(Sender).Selected then
            begin
              Cur4Image := TMag4VGear(Sender);
            end
          else
            begin
              UnSelectAll;
              SelectViewerImage(sender);
              Cur4Image := TMag4VGear(Sender);
            end;
        end;
      clpt.x := x;
      clpt.y := y;
      clpt := (Sender as TMag4VGear).ClientToScreen(clpt);
      if assigned(FPopupMenuImage) then
        begin
          FPopupMenuImage.PopupComponent := (TMag4Vgear(sender));
          if FAbswindow then FPopupMenuImage.popup(clpt.x + 3, clpt.y + 5)
          else FPopupMenuImage.popup(clpt.x + 3, clpt.y + 15);
        end;
    end;
end;

procedure Tmag4Viewer.ImageVKeydown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if key = VK_Return then
    begin
      ImageVClick(Sender, TMag4VGear(sender));
      exit;
    end;
  if key = 33 then //PageUp
    begin
      PagePrevViewer;
    end;
  if key = 34 then //PageUp
    begin
      PageNextViewer;
    end;
end;

procedure TMag4Viewer.ImageVMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var clpt: TPoint;
begin
  if FIsDragable and (Shift = [ssleft]) then
    begin
      ImageVClick(Sender, TMag4VGear(sender));
    end;
  exit;
  if FIsDragable then (Sender as TMag4VGear).Begindrag(false, 20);
  if button = mbleft then
    begin
      if (ssctrl in Shift) then SelectViewerImage(sender);
      if (Shift = [ssleft]) then
        begin
          ImageVClick(Sender, TMag4VGear(sender));
        end;
    end;
  if button = mbright then
    begin
      if (Shift = [ssright]) then
        begin
          if TMag4VGear(Sender).Selected then
            begin
              Cur4Image := TMag4VGear(Sender);
            end
          else
            begin
              UnSelectAll;
              SelectViewerImage(sender);
              Cur4Image := TMag4VGear(Sender);
            end;
        end;
      clpt.x := x;
      clpt.y := y;
      clpt := (Sender as TMag4VGear).ClientToScreen(clpt);
      if assigned(FPopupMenuImage) then
        begin
          FPopupMenuImage.PopupComponent := (TMag4Vgear(sender));
          if FAbswindow then FPopupMenuImage.popup(clpt.x + 3, clpt.y + 5)
          else FPopupMenuImage.popup(clpt.x + 3, clpt.y + 15);
        end;
    end;

end;

procedure TMag4Viewer.UnSelectAll;
var I: integer;
  proxy: TImageProxy;
begin
  for i := 0 to FLoadedList.count - 1 do
    begin
      Proxy := FLoadedList[i];
      if (Proxy.image <> nil) then
        if (proxy.image.selected) then proxy.image.selected := false;
    end;
end;

procedure TMag4Viewer.ImageVclick(Sender: Tobject; VGear: TMag4VGear);
begin
  Cur4Image := VGear;
  Cur4Image.BringToFront;
  Update;
  UnSelectAll;
  { TODO : This could be redundant, have to check flow after mouse down. }
  Cur4Image.selected := true;

  if FPanWindow then SetPanWindow(FPanWindow);
  {DONE:  next line works but self is null.  Got It : in the call do
                                 a TMagView(sender) and we can see sender}
  if assigned(OnViewerImageClick) then OnViewerImageClick(cur4Image);
  if assigned(OnViewerClick) then OnViewerClick(self, self, Cur4Image);
end;

procedure TMag4Viewer.tbPageLastClick(Sender: TObject);
begin
  PageLastViewer;
end;

procedure TMag4Viewer.tbPageFirstClick(Sender: TObject);
begin
  PageFirstViewer;
  SelectAnImage(firstVisProxy);
end;

procedure TMag4Viewer.tbPagePrevClick(Sender: TObject);
begin
  PagePrevViewer;
end;

procedure TMag4Viewer.tbPageNextClick(Sender: TObject);
begin
  PageNextViewer;
end;


procedure TMag4Viewer.RemoveFromList;
var i: integer;
  Iproxy: TImageProxy;
  undotext: string;
begin
  if GetSelectedCount = 0 then exit;
  if GetSelectedCount > 1 then undotext := 'UnDo - Remove Image(s)'
  else undotext := 'UnDo - Remove Image';
  UnDoGroupingNew; //RemoveFromList
  { not doing => if FApplytoAll then..... for RemoveFromList.
     Intent is to remove the selected, not all.
     if (SelectedCount > 0) then {}

  (* FUnDoList.add(TMagCaptionedList.create);
     TList(FUnDoList(FUnDoList.Count-1)). *)
  for i := FProxyList.Count - 1 downto 0 do
    begin
      Iproxy := FProxyList[i];
      if Iproxy.Image <> nil
        then if TMag4Vgear(Iproxy.Image).selected then
          begin
            case FViewerStyle of
              magvsStaticPage:
                begin
              //  do nothing in staticpage style
                end;
              magvsDynamic:
                begin
                DeleteProxy(i);
                 (*
                  UnDoListAddItem(UnDotext, TImageProxy(FProxyList[i]), magundoRemove); //ClearViewer;
                  TImageProxy(FProxyList[i]).Image.Visible := false;
                  FProxyList.Delete(i);
                  *)
                end;
              magvsVirtual:
                begin
              //do nothing in magvsVirtual style
              //  ?? HideProxy(i);
                end;
            end;
        // TMag4Vgear(Iproxy.Image).MouseZoom;
          end;
    end;

  Cur4Image := nil;

  if assigned(OnViewerImageClick) then OnViewerImageClick(cur4Image);
  if assigned(OnViewerClick) then OnViewerClick(self, self, cur4image);
  refreshViewerDesc;
end;



procedure TMag4Viewer.ScrollToImage(ImageIndex: integer);
begin
  maggmsgf.MagMsg('s', '###ScrollToImage index ' + inttostr(imageindex));
  scrlV.ScrollInView(TImageProxy(FProxyList[imageindex]));
end;


procedure TMag4Viewer.VisibleRowCol(var row, col: integer);
begin
  row := RowCount;
  Col := ColumnCount;
  exit;
(*    for now we'll return RowCount, and ColumnCount
      we will probably need the functionality below, if a user resizes a
      window, and doesn't ReAlignImages, they will *)
  (*
    if I3ptrList.count = 0 then
    begin
      row := 1;
      col := 1;
      exit;
    end;
    Limg := I3ptrList[0];
    tw := Limg.width;
    th := Limg.height;

    Col := trunc((width - FColsp) / (tw + FColsp));
    if (Col = 0) then Col := 1;
    Row := Trunc((height - FRowsp) / (th + FRowsp));
    if (Row = 0) then Row := 1;
  *)
end;

function TMag4Viewer.VisibleCount: integer;
var
  across, down: integer;
begin
  VisibleRowCol(down, across);
  result := across * down;
end;

function TMag4Viewer.VisibleCountActual: integer;
var
  halfw, i: integer;
begin
  halfw := FBaseW div 2;
  result := 0;
  for i := 0 to FProxylist.count - 1 do
    begin
      if ((TImageProxy(FProxyList[i]).left + halfw) > scrlv.Left)
        and ((TImageProxy(FProxyList[i]).left + halfw) < (scrlv.Width))
        then result := result + 1;
    {TODO: IF NOT FScrollVertical needs to be here }
    end;
end;

procedure TMag4Viewer.FrameEndDock(Sender, Target: TObject; X, Y: Integer);
begin
  // Still might consider this, but it happens each time a floating window is moved.
  //   The ReAlignImages does a realign also, ( maybe take out the realign features of the
  //   ReAlignImages function, make them two functions.  and this button could be a realign
  //    which does a ReAlignImages also.
  //ReAlignImages;
end;

procedure TMag4Viewer.SetAbsWindow(const Value: Boolean);
begin
  FAbsWindow := Value;
  if FAbsWindow then
    begin
      FViewerDesc := ' Abstracts.';
      tbColumnBigger.Visible := false;
      tbColumnSmaller.Visible := false;
      tbRowBigger.visible := false;
      tbRowSmaller.visible := false;
      tbPageLast.visible := false;
      tbpageFirst.visible := false;
      tbPageNext.visible := false;
      tbpagePrev.visible := false;
      tbClear.visible := false;
      //testing, keep it visible tbConfig.Visible := false;

      tbtnAbsPagePrev.visible := true;
      tbtnAbsPageNext.visible := true;
      tbtnAbsSmaller.visible := true;
      tbtnAbsBigger.visible := true;
    end
  else
    begin
      FViewerDesc := ' Images.';
      // 01/06/03 Patch 8   Ruth - simplify things, Don't show toolbar
      tbViewer.Visible := false;
    end;
end;


(*
procedure TMag4Viewer.ViewerInfo;
var s: string;
begin
  s := '';
  if scrlv.HorzScrollBar.Tracking then s := s + '  horiz track = true';
  if scrlv.VertScrollBar.Tracking then s := s + ' Vert track = true';
  if showhint then s := s + '  showhint = true';
  if cur4image = nil then s := s + '  cur4image = nil';
  if maxedImage = nil then s := s + '  maxedImage = nil';

  if FZoomWindow then s := s + ' FZoomWindow = true';
  if FPanWindow then s := s + '  FPanWindow = true';
  if showhint then s := s + '  showhint = true';
  if dragmode = dmautomatic then s := s + '  dragmode = dmAutomatic';
  //Cur4Image := nil;
 //maxedImage  := nil;
 //FZoomWindow := false;
 //FPanWindow := false;

// if FColsp < 5 then FColsp := 5;
// if FRowsp < 5 then FRowsp := 5;
// if FRowCount < 1 then FRowCount := 1;
// if FColCount < 1 then FColCount := 1;
// if FMaxCount < 1 then FMaxCount := 1;
  s := s + '  FColSp ' + inttostr(FColSp);
  s := s + '  FRowsp ' + inttostr(FRowsp);
  s := s + '  FRowCount ' + inttostr(FRowCount);
  s := s + '  FColCount ' + inttostr(FColCount);
  s := s + '  FMaxCount ' + inttostr(FMaxCount);

  showmessage(s);
end; *)

procedure TMag4Viewer.SetRowSpacing(const Value: integer);
begin
  FRowsp := Value;
  if FRowsp < 1
    then FRowsp := 1;
end;

procedure TMag4Viewer.SetColumnSpacing(const Value: integer);
begin
  FColsp := Value;
  if FColsp < 1
    then FColsp := 1;
end;

procedure TMag4Viewer.SetColumnCount(const Value: integer);
begin
  if value < 1
    then FColcount := 1
  else FColcount := Value;
end;

procedure TMag4Viewer.SetRowCount(const Value: integer);
begin
  if value < 1
    then FRowcount := 1
  else FRowcount := Value;
end;

procedure TMag4Viewer.UpDate_(SubjectState: string);
begin
  clearViewer;
  if (SubjectState = '') then exit; //IMagSubject is cleared.
  if copy(SubjectState, 1, 1) = '0' then exit; //workaround bug.
  if (SubjectState = '-1') then FMagImageList := nil; //IMagSubject is freed.
 // HERE to speed the loading of group abs, and abs.  Load so many, then load the rest
 {TODO:   Speed up the loading , displaying of abstracts , and images.}
  if FMagImageList <> nil then ImagesToMagView(FMagImageList.Objlist); //update
end;

procedure TMag4Viewer.SetMagImageList(const Value: TMagImageList);
begin
  if (FMagImageList <> nil) then FMagImageList.Detach_(IMagObserver(self));
  FMagImageList := Value;
  if value <> nil then AttachMyself();
end;

procedure TMag4Viewer.AttachMyself;
begin
  if assigned(FMagImageList) then
    begin
      FMagImageList.Attach_(IMagObserver(self));
    end;
end;

procedure TMag4Viewer.ViewDirectoryImages(t: tstrings);
var dirlist: tlist;
begin
  dirlist := FUtils.DirImagesToObjList(t);
  try
    ImagestoMagView(dirlist);
  finally
    dirlist.free;
  end;
end;

procedure TMag4Viewer.pnlTopDragOver(Sender, Source: TObject; X,
  Y: Integer; State: TDragState; var Accept: Boolean);
begin
  DragItOver(Sender, source, x, y, state, accept);
end;

procedure TMag4Viewer.pnlTopDragDrop(Sender, Source: TObject; X,  Y: Integer);
begin
  dropiton(sender, source, x, y);
end;

procedure TMag4Viewer.SetViewerStyle(const Value: TMagViewerStyles);
begin
  FViewerStyle := Value;
  case FViewerStyle of
    magvsStaticPage:
      begin
        lbHideScroll.visible := true;
        lbHideScroll.BringToFront;
      end;
    magvsDynamic:
      begin

      end;
    magvsVirtual:
      begin
        lbHideScroll.visible := false;
        // what if we were comming from a magvsDynamic.
        ReAlignImages; // will this do it.
      end;
  end;
end;

procedure TMag4Viewer.ToggleSelected;
var i: integer;
  Iproxy: TImageProxy;
begin
  for i := 0 to FProxyList.Count - 1 do
    begin
      Iproxy := FProxyList[i];
      if Iproxy.Image <> nil
        then TMag4Vgear(Iproxy.Image).selected := not TMag4Vgear(Iproxy.Image).selected;
    end;
end;

procedure TMag4Viewer.tbClearClick(Sender: TObject);
begin
  clearViewer;
end;

procedure TMag4Viewer.tbRefreshClick(Sender: TObject);
begin
  LoadProxyImages(FirstVisProxy);
  RefreshImages;
end;

procedure TMag4Viewer.tbRowSmallerClick(Sender: TObject);
begin
  SetRowColCount(FRowcount - 1, FColCount);
end;

procedure TMag4Viewer.tbRowBiggerClick(Sender: TObject);
begin
  SetRowColCount(FRowcount + 1, FColCount);
end;

procedure TMag4Viewer.tbColumnBiggerClick(Sender: TObject);
begin
  SetRowColCount(FRowcount, FColCount + 1);
end;

procedure TMag4Viewer.tbColumnSmallerClick(Sender: TObject);
begin
  SetRowColCount(FRowcount, FColCount - 1);
end;

procedure TMag4Viewer.tbConfigClick(Sender: TObject);
begin
  EditViewerSettings;
end;

procedure TMag4Viewer.EditViewerSettings;
begin
RowColSize.execute(self);
end;

procedure TMag4Viewer.ScrollToImage(IObj: TImageData);
var i: integer;
  proxy: TImageProxy;
begin
  for i := 0 to FProxyList.count - 1 do
    begin
      proxy := FProxyList[i];
      if proxy.imagedata.Mag0 = Iobj.Mag0 then
        begin
          scrlV.ScrollInView(proxy);
          break;
        end;
    end;
end;

function TMag4Viewer.GetCurrentImage: TMag4VGear;
begin
  result := Cur4Image;
end;

function TMag4Viewer.GetCurrentImageObject: TImageData;
begin
  if Cur4Image = nil then result := nil
  else result := Cur4Image.PI_ptrData;
end;

procedure TMag4Viewer.ImageCopy;
var stat: boolean;
  xmsg: string;
begin
  if not assigned(FMagUtilsDB) then
    begin
      messagedlg('DataBase Utilities are not linked (cmagUtilsDB)',mtWarning,[mbok],0);
      exit;
    end;
  if not assigned(cur4image) then
    begin
      messagedlg('There are no selected images.',mtconfirmation,[mbok],0);
      exit;
    end;
  FMagUtilsDB.ImageCopy(cur4Image, stat, xmsg);
end;

procedure TMag4Viewer.ImagePrint;
var stat: boolean;
  xmsg: string;
begin
  if not assigned(FMagUtilsDB) then
    begin
      messagedlg('DataBase Utilities are not linked (cmagUtilsDB)',mtWarning,[mbok],0);
      exit;
    end;
  if not assigned(cur4image) then
    begin
      messagedlg('There are no selected images.',mtconfirmation,[mbok],0);
      exit;
    end;

  FMagUtilsDB.ImagePrint(Cur4Image, stat, xmsg);
end;

procedure TMag4Viewer.ImageReport;
var stat: boolean;
  xmsg: string;
begin
  if not assigned(FMagUtilsDB) then
    begin
      messagedlg('DataBase Utilities are not linked (cmagUtilsDB)',mtWarning,[mbok],0);
      exit;
    end;
  if not assigned(cur4image) then
    begin
      messagedlg('There are no selected images.',mtconfirmation,[mbok],0);
      exit;
    end;
  FMagUtilsDB.ImageReport(Cur4Image.PI_ptrData, stat, xmsg);
end;

procedure TMag4Viewer.SetMagUtilsDB(const Value: TMagUtilsDB);
begin
  FMagUtilsDB := Value;
end;

procedure TMag4Viewer.mnuToolbarClick(Sender: TObject);
begin
  mnuToolbar.checked := not mnuToolbar.checked;
  tbViewer.visible := mnuToolbar.checked;
end;

procedure TMag4Viewer.AlignonTop1Click(Sender: TObject);
begin
  tbViewer.align := alTop;
end;

procedure TMag4Viewer.AligntoLeft1Click(Sender: TObject);
begin
  tbViewer.align := alLeft;
end;

function TMag4Viewer.GetImagePage: integer;
begin
  if cur4Image = nil then result := 0
  else result := cur4image.Page;
end;

function TMag4Viewer.GetImagePageCount: integer;
begin
  if cur4Image = nil then result := 0
  else result := cur4image.PageCount;
end;

procedure TMag4Viewer.SetImagePage(const Value: integer);
begin
  if cur4Image = nil then exit;
  if (value > GetImagePageCount) then Cur4Image.Page := GetImagePageCount
  else if (value < 1) then Cur4Image.Page := 1
  else Cur4Image.Page := value;
end;


procedure TMag4Viewer.SetImagePageCount(const Value: integer);
begin
  //
end;

function TMag4Viewer.GetImageCount: integer;
begin
  result := FProxyList.count;
end;

function TMag4Viewer.GetImageObjectList: Tlist;
var objectlist: Tlist;
var i: Integer;
begin
  objectlist := tlist.create;
  try
    for i := 0 to FProxylist.Count - 1 do
      begin
        objectlist.add(TImageProxy(Fproxylist[i]).imagedata);
      end;
    result := objectlist;
  finally
    // can't free it here, dummy
    //  objectlist.free;
  end;
end;


function TMag4Viewer.GetSelectedCount: integer;
var I: integer;
begin
  result := 0;
  for i := 0 to FProxylist.Count - 1 do
    if ((TMag4VGear(TImageProxy(FProxyList[i]).image) <> nil)
      and TMag4VGear(TImageProxy(Fproxylist[i]).image).selected)
      then result := result + 1;
end;


function TMag4Viewer.GetSelectedImageList: Tlist;
var selectlist: Tlist;
var i: Integer;
begin
  selectlist := tlist.create;
  try
    for i := 0 to FProxylist.Count - 1 do
      if (TMag4VGear(TImageProxy(FProxyList[i]).image) <> nil)
        then
        begin
          if TMag4VGear(TImageProxy(Fproxylist[i]).image).selected then selectlist.add(TImageProxy(Fproxylist[i]).imagedata);
    //TMag4Vgear(Iproxy.Image).selected
        end;
    result := selectlist;
  finally
    // can't free it here, dummy
    //  selectlist.free;
  end;
end;

procedure TMag4Viewer.SetViewerDesc(desc: string; descvis: boolean);
begin
  pnltop.Caption := desc;
  if pnltop.Visible <> descvis then pnltop.visible := descvis;
end;

function TMag4Viewer.GetDescVisible: boolean;
begin
  result := pnltop.Visible;
end;

procedure TMag4Viewer.SetDescVisible(const Value: boolean);
begin
  pnltop.visible := value;
end;

procedure TMag4Viewer.PageFirstViewer;
begin
  scrlV.HorzScrollBar.position := 0;
  scrlv.VertScrollBar.Position := 0;
  LoadProxyImages(FirstVisProxy);
//  SelectAnImage(firstVisProxy);
end;

procedure TMag4Viewer.PageLastViewer;
begin
  scrlV.HorzScrollBar.position := scrlV.HorzScrollBar.Range;
  scrlv.VertScrollBar.Position := scrlv.VertScrollBar.Range;
  LoadProxyImages(FirstVisProxy);
//  SelectAnImage(firstVisProxy);
end;

procedure TMag4Viewer.PageNextViewer;
var trow, tcol: integer;
begin
  VisibleRowCol(tRow, Tcol);
  if ((scrlV.HorzScrollBar.Range - Scrlv.width) > scrlV.HorzScrollBar.position) then
    begin
      scrlV.HorzScrollBar.position := scrlV.HorzScrollBar.position + (Tcol * (FbaseW + Fcolsp)); {width;}
    {HorzScrollBar.position := HorzScrollBar.position  + width;}
    end;
  if ((scrlV.VertScrollBar.Range - Scrlv.height) > scrlV.VertScrollBar.position) then
    begin
      scrlV.VertScrollBar.position := scrlV.VertScrollBar.position + scrlV.height; {width;}
    {HorzScrollBar.position := HorzScrollBar.position  + width;}
    end;
  LoadProxyImages(FirstVisProxy);
//  SelectAnImage(firstVisProxy);
end;

procedure TMag4Viewer.PagePrevViewer;
begin
  if (scrlV.HorzScrollBar.position <> 0) then
    begin
    //scrlV.HorzScrollBar.position := scrlV.HorzScrollBar.position + (Tcol * (FbaseW + Fcolsp)); {width;}
      scrlV.HorzScrollBar.position := scrlV.HorzScrollBar.position - scrlv.width;
    end;
  if (scrlV.VertScrollBar.position <> 0) then
    begin
    //scrlV.HorzScrollBar.position := scrlV.HorzScrollBar.position + (Tcol * (FbaseW + Fcolsp)); {width;}
      scrlV.VertScrollBar.position := scrlV.VertScrollBar.position - scrlv.height;
    end;
  LoadProxyImages(FirstVisProxy);
//  SelectAnImage(firstVisProxy);
end;

procedure TMag4Viewer.tbtnAbsPagePrevClick(Sender: TObject);
begin
  PagePrevViewer;
  LoadProxyImages(FirstVisProxy); // p8t34 try for speed,
    UpdateToolButtons;
  //    ReAlignImages;
//  ReFreshImages;
end;

procedure TMag4Viewer.tbtnAbsPageNextClick(Sender: TObject);
begin
  PageNextViewer;
  LoadProxyImages(FirstVisProxy); // p8t34 try for speed,
  UpdateToolButtons;
//    ReAlignImages;
//  ReFreshImages;
end;

procedure TMag4Viewer.tbtnAbsSmallerClick(Sender: TObject);
var h, w: integer;
begin
  if FAbsWindow then
    begin
      h := trunc(FBaseH * 0.9);
      w := trunc(FBaseW * 0.9);
      FLockHeight := h;
      FBaseH := h;
      FBaseW := w;
      FLockWidth := w;
      CalcRowColusingHW;
      {TODO: this needs changed to ReAlignImages}
      RealignProxy; // tbtnAbsSmallerClick
    end
  else SetRowColCount(FRowcount + 1, FColCount + 1);
  UpdateToolButtons;
end;

procedure TMag4Viewer.tbtnAbsBiggerClick(Sender: TObject);
var h, w: integer;
  okay: boolean;
begin
  if FAbsWindow then
    begin
      h := trunc(FBaseH * 1.1);
      w := trunc(FBaseW * 1.1);
      (* Washington testing: don't restrict users '+' or '-'
      if FScrollVertical
        then Okay := ((H < (scrlv.Height)) and (w < scrlv.Width - 20))
      else Okay := ((H < (scrlv.Height - 20)) and (w < scrlv.Width)); *)
      okay := true;

      if okay then
        begin
          FLockHeight := h;
          FBaseH := h;
          FBaseW := w;
          FLockWidth := w;
          CalcRowColusingHW;
      {TODO: this needs changed to ReAlignImages}
          RealignProxy; // tbtnAbsBiggerClick
        end;
    end
  else SetRowColCount(FRowcount - 1, FColCount - 1);
  UpdateToolButtons;
end;

procedure TMag4Viewer.SetDefaultLayout(Rows,Cols,MaxImages : integer; LockSize: boolean; FitMeth : integer = 0);
begin
  DefaultLayout.Rows := Rows;
  DefaultLayout.Columns := Cols;
  DefaultLayout.MaxNumberOfImages := MaxImages;
  DefaultLayout.LockImageState := LockSize;
  DefaultLayout.FitMethod := Fitmeth;
end;

procedure TMag4Viewer.ApplyDefaultLayout;
begin
   {ReAlignImages, with new rows and columns, and then LockSize if needed.}
  LockSize := false;

  RowCount := DefaultLayout.Rows;
  ColumnCount := DefaultLayout.Columns;
  MaxCount := DefaultLayout.MaxNumberOfImages;
    FLastFit := DefaultLayout.FitMethod;
  ReAlignImages;
  LockSize := DefaultLayout.LockImageState;
end;

function TMag4Viewer.GetDefaultLayoutDesc(): string;
begin
  result := inttostr(defaultlayout.Rows) + 'x' + inttostr(defaultlayout.Columns)
    + ' max: ' + inttostr(defaultlayout.MaxNumberOfImages);
  if defaultlayout.LockImageState then result := result + ' size is locked';
end;

procedure TMag4Viewer.SetShowToolbar(const Value: boolean);
begin
  FShowToolbar := Value;
  tbViewer.Visible := FShowToolbar;
end;



function TMag4Viewer.GetIMageFontSize: integer;
begin
  result := FFontSize;
end;

procedure TMag4Viewer.SetImageFontSize(const Value: integer);
var i: integer;
  Iproxy: TImageProxy;
begin
  FFontSize := value;
  for i := 0 to FProxyList.Count - 1 do
    begin
      Iproxy := FProxyList[i];
      if Iproxy.Image <> nil
        then TMag4Vgear(Iproxy.Image).FontSize := value;
    end;

end;

procedure TMag4Viewer.SyncWithIMage(Iobj: TimageData);
var pidx: integer;
begin
  pidx := IsDuplicate(Iobj);
  if pidx = -1 then
    begin
      UnSelectAll;
      Exit;
    end;
  ScrollAndDisplayImage(pidx);
end;

procedure TMag4Viewer.popupViewerPopup(Sender: TObject);
begin
  mnuToolbar.Checked := tbViewer.Visible;
//prototype ViewerScrollVert  mnuScrollVertical.Checked := FScrollVertical;
//prototype ViewerScrollVert  mnuScrollHoriz.checked := Not FScrollVertical;
//prototype ViewerScrollVert  - ALSO need to make the menu options visible. (toolbar menu)
end;

procedure TMag4Viewer.scrlVDragOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
    //future
  if (IsDragObject(source)) or (source is TMag4VGear) then
    accept := true;
end;

procedure TMag4Viewer.scrlVDragDrop(Sender, Source: TObject; X,
  Y: Integer);
var
  v4Viewer: Tmag4Viewer;
begin
 // showmessage('Sender : ' + (sender as TObject).ClassName + #13 + 'Source : ' +(source as TObject).ClassName);
  if IsDragObject(source) then
    begin
      FImageDropped := true;
      ImagesToMagView((source as TmagDragImageObject).DraggedImages);
  // onEndDrag isn't being fired.
  //Tmag4vgear(source).enddrag(true); this doesn't do it, and defeats the purpose.
      exit;
    end;
  if source is TMag4VGear then
    if Tmag4vgear(source).Fparentviewer <> nil then
      begin
        v4Viewer := TMag4viewer(Tmag4vgear(source).Fparentviewer);
        FImageDropped := true;
        imagestomagview(v4viewer.getselectedimagelist);
      end;

 //worked if source is TMag4VGear then
 //worked begin
 //worked t := tlist.create;
 //worked t.add((source as TMag4VGear).PI_ptrData);
 //worked FImageDropped := true;
 //worked ImagesToMagView(t);
 //worked t.free;
 //worked end;

end;


(*
procedure TMag4Viewer.LevelValue(Value: integer);
begin
//
end;
*)

procedure TMag4Viewer.ShowHints(value: boolean);
var i: integer;
  Iproxy: TImageProxy;
begin
  //if FApplytoAll then
  FShowHints := value;
  for i := 0 to FProxyList.Count - 1 do
    begin
      Iproxy := FProxyList[i];
      if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).ShowHint(value);
    end
end;

(*
procedure TMag4Viewer.ScrlvScrollHorz(Sender: TObject);
begin
// this is OnScrollHorz
//testing next line out p8t35
  if (Scrlv.HorzDelta <> 0) then FScrollUp := (Scrlv.HorzDelta > 0);
  timerViewer.enabled := false;
  StopLoadingImages := true;
  application.processmessages;
  UpdateToolButtons;   // updates the NextPrev page buttons.
  if GetDescVisible then RefreshViewerDesc;
  timerViewer.enabled := true;
  //ReAlignImages;
  //ReFreshImages;
end;

*)

procedure TMag4Viewer.ScrlvEndScroll(Sender: TObject);
begin
  StopLoadingImages := true;
  UpdateToolButtons;   // updates the NextPrev page buttons.
  if GetDescVisible then RefreshViewerDesc;
  if FUseAutoRealign then LoadProxyImages(FirstVisProxy);
end;


procedure TMag4Viewer.ScrlvScrollHorz(Sender: TObject);
begin
// this is OnScrollHorz
//testing next line out p8t35
  if (Scrlv.HorzDelta <> 0) then FScrollUp := (Scrlv.HorzDelta > 0);
 //-- timerViewer.enabled := false;
  StopLoadingImages := true;
  application.processmessages;
  UpdateToolButtons;   // updates the NextPrev page buttons.
  if GetDescVisible then RefreshViewerDesc;
//--  timerViewer.enabled := true;
  //ReAlignImages;
  //ReFreshImages;
end;

procedure TMag4Viewer.ScrlvScrollVert(Sender: TObject);
begin
// this is OnScrollVert
//testing next line out p8t35
  if (Scrlv.VertDelta <> 0) then FScrollUp := (Scrlv.VertDelta > 0);
//--  timerViewer.enabled := false;
  StopLoadingImages := true;
  application.processmessages;
  UpdateToolButtons;
  if GetDescVisible then RefreshViewerDesc;
//--  timerViewer.enabled := true;
  //ReAlignImages;
  //ReFreshImages;
end;

procedure TMag4Viewer.mnuScrollVerticalClick(Sender: TObject);
begin
  mnuSCrollVertical.Checked := not mnuSCrollVertical.Checked;
  ScrollVertical := mnuSCrollVertical.Checked;
  ReAlignImages;
  //p8t34 ReFreshImages;
end;

procedure TMag4Viewer.mnuScrollHorzClick(Sender: TObject);
begin
  mnuScrollHorz.Checked := not mnuScrollHorz.Checked;
  ScrollVertical := mnuSCrollVertical.Checked;
  ReAlignImages;
  //p8t34 ReFreshImages;
end;



procedure TMag4Viewer.timerViewerTimer(Sender: TObject);
begin
showmessage('Viewer Timer');
{ TimerViewerTimer is called when user scrolls with the
   MagEventScrool component.  (Robert's component) {}
   timerViewer.Enabled := false;
   application.processmessages;
// UseAutoRealign is a global variable, needs to be a property
//  of the Component.  Not Application.
 { TODO : Change UseAutoRealign into a property of the component }
if FUseAutoRealign then LoadProxyImages(FirstVisProxy);
end;



function TMag4Viewer.ViewerScrollAtEnd: boolean;
begin
if not FScrollVertical
then result := not ((scrlv.HorzScrollBar.Visible) and ((scrlv.HorzScrollBar.Position + Scrlv.width) < scrlv.HorzScrollBar.Range))
else result := not ((scrlv.VertScrollBar.Visible) and ((scrlv.VertScrollBar.Position + scrlv.height) < scrlv.VertScrollBar.Range));
end;

function TMag4Viewer.ViewerScrollAtStart: boolean;
begin
  if not FScrollVertical
  then result := not ((scrlv.HorzScrollBar.Visible) and (scrlv.HorzScrollBar.Position > 0))
  else result := not ((scrlv.VertScrollBar.Visible) and (scrlv.VertScrollBar.Position > 0));
end;

(*    *********   Operations passed to TMag4VGear  *********      *)
(*    *********                                    *********      *)

{TODO: all the For... looping in each method, needs replaced with a
   call to a method with procedure parameters.  That way we only need
   to write the For...loop once. }

procedure TMag4Viewer.FitToWindow;
var i: integer;
  Iproxy: TImageProxy;
begin
   { TODO :
Need to rethink this.  Could just do the currently opened (FLoadedList) images,
then when a new image is opened, apply state of current image to the new image
( if ApplyToAll is still true }
  FlastFit := IG_DISPLAY_FIT_TO_WINDOW;
  if FApplytoAll then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).FitToWindow
      end

  else //if (SelectedCount > 0) then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then if TMag4Vgear(Iproxy.Image).selected then TMag4Vgear(Iproxy.Image).FitToWindow
      end;
  // else if Cur4Image <> nil then Cur4Image.fittoheight;
end;

procedure TMag4Viewer.Fit1to1;
var i: integer;
  Iproxy: TImageProxy;
//  vGear: TMag4vGear;
begin
  FlastFit := IG_DISPLAY_FIT_1_TO_1;
  if FApplytoAll then
    // for i := 0 to I3ptrList.count - 1 do TMag4Vgear(I3PtrList[i]).fittoheight
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).Fit1to1
      end

  else //if (SelectedCount > 0) then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then if TMag4Vgear(Iproxy.Image).selected then TMag4Vgear(Iproxy.Image).Fit1to1
      end;
  // else if Cur4Image <> nil then Cur4Image.fittoheight;
end;

procedure TMag4Viewer.FitToWidth;
var i: integer;
  Iproxy: TImageProxy;
//  vGear: TMag4vGear;
begin
  FlastFit := IG_DISPLAY_FIT_TO_WIDTH;
  if FApplytoAll then
    // for i := 0 to I3ptrList.count - 1 do TMag4Vgear(I3PtrList[i]).FitToWidth
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).FitToWidth
      end

  else //if (SelectedCount > 0) then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then if TMag4Vgear(Iproxy.Image).selected then TMag4Vgear(Iproxy.Image).FitToWidth
      end;
  // else if Cur4Image <> nil then Cur4Image.fittoheight;
end;

procedure TMag4Viewer.FitToHeight;
var i: integer;
  Iproxy: TImageProxy;
//  vGear: TMag4vGear;
begin
  FlastFit := IG_DISPLAY_FIT_TO_HEIGHT;
  if FApplytoAll then
    // for i := 0 to I3ptrList.count - 1 do TMag4Vgear(I3PtrList[i]).fittoheight
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).fittoheight
      end

  else //if (SelectedCount > 0) then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then if TMag4Vgear(Iproxy.Image).selected then TMag4Vgear(Iproxy.Image).fittoheight
      end;
  // else if Cur4Image <> nil then Cur4Image.fittoheight;
end;

procedure TMag4Viewer.FlipVert;
var i: integer;
  Iproxy: TImageProxy;
//  vGear: TMag4vGear;
begin
  if FApplytoAll then
    // for i := 0 to I3ptrList.count - 1 do TMag4Vgear(I3PtrList[i]).fittoheight
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).FlipVert
      end
  else //if (SelectedCount > 0) then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil
          then if TMag4Vgear(Iproxy.Image).selected
          then TMag4Vgear(Iproxy.Image).FlipVert
      end;
end;

procedure TMag4Viewer.FlipHoriz;
var i: integer;
  Iproxy: TImageProxy;
//  vGear: TMag4vGear;
begin
  if FApplytoAll then
    // for i := 0 to I3ptrList.count - 1 do TMag4Vgear(I3PtrList[i]).fittoheight
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).FlipHoriz
      end
  else //if (SelectedCount > 0) then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil
          then if TMag4Vgear(Iproxy.Image).selected
          then TMag4Vgear(Iproxy.Image).FlipHoriz
      end;

end;

procedure TMag4Viewer.ResetImages;
var i: integer;
  Iproxy: TImageProxy;
//  vGear: TMag4vGear;
begin
  if FApplytoAll then
    // for i := 0 to I3ptrList.count - 1 do TMag4Vgear(I3PtrList[i]).fittoheight
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).ResetImage;
      end
  else //if (SelectedCount > 0) then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil
          then if TMag4Vgear(Iproxy.Image).selected
          then TMag4Vgear(Iproxy.Image).ResetImage;
      end;

end;

procedure TMag4Viewer.Rotate(deg: integer);
var i: integer;
  Iproxy: TImageProxy;
begin
  if FApplytoAll then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).rotate(deg)
      end
  else
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil
          then if TMag4Vgear(Iproxy.Image).selected
          then TMag4Vgear(Iproxy.Image).rotate(deg)
      end;
end;

procedure TMag4Viewer.Inverse;
var i: integer;
  Iproxy: TImageProxy;
//  vGear: TMag4vGear;
begin
  if FApplytoAll then
    // for i := 0 to I3ptrList.count - 1 do TMag4Vgear(I3PtrList[i]).fittoheight
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).inverse;
      end
  else //if (SelectedCount > 0) then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil
          then if TMag4Vgear(Iproxy.Image).selected
          then TMag4Vgear(Iproxy.Image).inverse;
      end;

end;

{-------}

procedure TMag4Viewer.MouseMagnify;
var i: integer;
  Iproxy: TImageProxy;
//  vGear: TMag4vGear;
begin
  if FApplytoAll then
    // for i := 0 to I3ptrList.count - 1 do TMag4Vgear(I3PtrList[i]).fittoheight
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).MouseMagnify;
      end
  else //if (SelectedCount > 0) then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil
          then if TMag4Vgear(Iproxy.Image).selected
          then TMag4Vgear(Iproxy.Image).MouseMagnify;
      end;
end;

procedure TMag4Viewer.MousePan;
var i: integer;
  Iproxy: TImageProxy;
begin
  if FApplytoAll then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).MousePan;
      end
  else
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil
          then if TMag4Vgear(Iproxy.Image).selected
          then TMag4Vgear(Iproxy.Image).MousePan;
      end;
end;

procedure TMag4Viewer.MouseZoomRect;
var i: integer;
  Iproxy: TImageProxy;
//  vGear: TMag4vGear;
begin
  if FApplytoAll then
    // for i := 0 to I3ptrList.count - 1 do TMag4Vgear(I3PtrList[i]).fittoheight
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).MouseZoomRect;
      end
  else //if (SelectedCount > 0) then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil
          then if TMag4Vgear(Iproxy.Image).selected
          then TMag4Vgear(Iproxy.Image).MouseZoomRect;
      end;
end;

procedure TMag4Viewer.ZoomValue(value: integer);
var i: integer;
  Iproxy: TImageProxy;
//  vGear: TMag4vGear;
begin
  if FApplytoAll then
    // for i := 0 to I3ptrList.count - 1 do TMag4Vgear(I3PtrList[i]).fittoheight
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).zoomvalue(value);
      end
  else //if (SelectedCount > 0) then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil
          then if TMag4Vgear(Iproxy.Image).selected
          then TMag4Vgear(Iproxy.Image).zoomvalue(value);
      end;
end;

procedure TMag4Viewer.ContrastValue(Value: integer);
var i: integer;
  Iproxy: TImageProxy;
//  vGear: TMag4vGear;
begin
  if FApplytoAll then
    // for i := 0 to I3ptrList.count - 1 do TMag4Vgear(I3PtrList[i]).fittoheight
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).contrastvalue(value);
      end
  else //if (SelectedCount > 0) then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil
          then if TMag4Vgear(Iproxy.Image).selected
          then TMag4Vgear(Iproxy.Image).contrastvalue(value);
      end;
end;

{ passthrough to Image Ojbect       }
procedure TMag4Viewer.BrightnessValue(Value: integer);
var i: integer;
  Iproxy: TImageProxy;
//  vGear: TMag4vGear;
begin
  if FApplytoAll then
    // for i := 0 to I3ptrList.count - 1 do TMag4Vgear(I3PtrList[i]).fittoheight
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).brightnessvalue(value);
      end
  else //if (SelectedCount > 0) then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil
          then if TMag4Vgear(Iproxy.Image).selected
          then TMag4Vgear(Iproxy.Image).brightnessvalue(value);
      end;
end;

(*
procedure TMag4Viewer.WindowValue(Value: integer);
var i: integer;
  Iproxy: TImageProxy;
//  vGear: TMag4vGear;
begin
  if FApplytoAll then
    // for i := 0 to I3ptrList.count - 1 do TMag4Vgear(I3PtrList[i]).fittoheight
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).windowvalue(value);
      end
  else //if (SelectedCount > 0) then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil
          then if TMag4Vgear(Iproxy.Image).selected
          then TMag4Vgear(Iproxy.Image).windowvalue(value);
      end;
end;
*)
procedure TMag4Viewer.WinLevValue(WinValue, LevelValue: integer);
var i: integer;
  Iproxy: TImageProxy;
//  vGear: TMag4vGear;
begin
  if FApplytoAll then
    // for i := 0 to I3ptrList.count - 1 do TMag4Vgear(I3PtrList[i]).fittoheight
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).WinLevValue(Winvalue, LevelValue);
      end
  else //if (SelectedCount > 0) then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil
          then if TMag4Vgear(Iproxy.Image).selected
          then TMag4Vgear(Iproxy.Image).WinLevValue(Winvalue, LevelValue);
      end;
end;

procedure TMag4Viewer.PageFirstImage;
var i: integer;
  Iproxy: TImageProxy;
begin
  if FApplytoAll then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).PageFirst;
      end
  else
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil
          then if TMag4Vgear(Iproxy.Image).selected
          then TMag4Vgear(Iproxy.Image).PageFirst;
      end;
end;

procedure TMag4Viewer.PageLastImage;
var i: integer;
  Iproxy: TImageProxy;
begin
  if FApplytoAll then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).PageLast;
      end
  else
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil
          then if TMag4Vgear(Iproxy.Image).selected
          then TMag4Vgear(Iproxy.Image).PageLast;
      end;
end;

procedure TMag4Viewer.PageNextImage;
var i: integer;
  Iproxy: TImageProxy;
begin
  if FApplytoAll then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).PageNext;
      end
  else
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil
          then if TMag4Vgear(Iproxy.Image).selected
          then TMag4Vgear(Iproxy.Image).PageNext;
      end;
end;

procedure TMag4Viewer.PagePrevImage;
var i: integer;
  Iproxy: TImageProxy;
begin
  if FApplytoAll then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).PagePrev;
      end
  else
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil
          then if TMag4Vgear(Iproxy.Image).selected
          then TMag4Vgear(Iproxy.Image).PagePrev;
      end;
end;

procedure TMag4Viewer.MouseReset;
var i: integer;
  Iproxy: TImageProxy;
begin
  if FApplytoAll then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).MouseReset;
      end
  else
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil
          then if TMag4Vgear(Iproxy.Image).selected
          then TMag4Vgear(Iproxy.Image).MouseReset;
      end;

end;

procedure TMag4Viewer.ReFreshImages;
var I : integer;
begin
  for i := 0 to FloadedList.count-1  do
    begin
      TImageProxy(FloadedList[i]).Image.RefreshImage;
    end;
end;

procedure TMag4Viewer.DeSkewAndSmooth;
var i: integer;
  Iproxy: TImageProxy;
//  vGear: TMag4vGear;
begin
  if FApplytoAll then
    // for i := 0 to I3ptrList.count - 1 do TMag4Vgear(I3PtrList[i]).fittoheight
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil then TMag4Vgear(Iproxy.Image).DeSkewAndSmooth;
      end
  else //if (SelectedCount > 0) then
    for i := 0 to FProxyList.Count - 1 do
      begin
        Iproxy := FProxyList[i];
        if Iproxy.Image <> nil
          then if TMag4Vgear(Iproxy.Image).selected
          then TMag4Vgear(Iproxy.Image).DeSkewAndSmooth;
      end;
end;

(*
procedure TMag4Viewer.ApplyCurrentToAll;
//var i: integer;
 // zl: integer;
begin
  { TODO -oGarrett -cConversion :
  Apply current to All needs work.
  TheImage.  is gone, we need to be able to get the values from TMag4VGear
  and Not the Gear Component.  We can no longer see it.  because we
  don't want to know which component is displaying the Image. }
   *)
  (* new
  zl := Cur4Image.TheImage.zoomlevel;
     for i := 0 to I3ptrList.count-1 do
            begin

            Iptr := I3ptrList[i];
            //Iptr^.image.invert := Cur4Image.invert;
            Iptr^.image4.TheImage.fitmethod := Cur4Image.TheImage.fitmethod;
            Iptr^.image4.TheImage.selectevent := Cur4Image.TheImage.selectevent;
            Iptr^.image4.TheImage.magnifywindow := Cur4Image.TheImage.magnifywindow    ;
            Iptr^.image4.TheImage.HandPanning := Cur4Image.TheImage.HandPanning    ;
            Iptr^.image4.TheImage.zoomlevel := zl    ;
            Iptr^.image4.TheImage.displaybrightness := Cur4Image.TheImage.displaybrightness  ;
            Iptr^.image4.TheImage.displaycontrast := Cur4Image.TheImage.displaycontrast    ;
            Iptr^.image4.TheImage.redraw := true;
            application.processmessages;
            end;

end;*)

  (*   ****************   *)

procedure TMag4Viewer.UnDoGroupingNew;
begin
  if FUndolist.Count = 0 then FCurUnDoGroup := 1
  else
    FCurUnDoGroup := TMagUnDoItem(FUndolist[FUndolist.count - 1]).magUnDoGroup + 1;
end;

procedure Tmag4Viewer.UnDoListClear;
var i: integer;
begin
  for i := FUndoList.count - 1 downto 0 do
    begin
      UnDoItemClear(i);
    end;
end;

procedure Tmag4Viewer.UnDoItemClear(item: integer);
//var undoItem: TMagUnDoItem;
begin
// undoItem := FUndolist[item];
  FUndoList.Delete(item);
{TODO: here (later when we use the UnDo Function we have to release memory
                                                      of the IProxies, and TGears}
end;

(*procedure Tmag4Viewer.UnDoListAddItem(txt: string; proxy: TImageProxy; undoaction: TMagundoAction);
var undoItem: TMagUnDoItem;
  //     vGear: TMag4VGear;
begin
  undoItem := TMagUnDoItem.create;
  undoItem.magCaption := txt;
  undoItem.magUnDoGroup := FCurUnDoGroup;
  undoItem.magIproxy := proxy;
       //vGear := undoitem.magIproxy.image;
       //undoitem.magIproxy.Image := nil;
       //Destroy_P(vGear);
  undoItem.magUnDoAction := undoAction;
  FUndoList.add(undoItem);
end;
*)
function TMag4Viewer.UnDoActionGetText: string;
begin
  if FUnDoList.Count = 0
    then result := ''
  else result := TMagUnDoItem(FUnDoList[FUnDoList.count - 1]).magCaption;
end;

procedure TMag4Viewer.UnDoActionExecute;
var i: integer;
  curgrping: integer;
  undoItem: TMagUnDoItem;
//  undoaction: TMagUnDoAction;
begin
  if FunDoList.Count = 0 then exit; // Shouldn't happen;
  curGrping := TMagUnDoItem(FundoList[Fundolist.count - 1]).magUnDoGroup;
  (*     undoaction := TMagUnDoItem(FundoList[Fundolist.count - 1]).magUndoAction; *)
   for i := FUndoList.count - 1 downto 0 do
    begin
      undoitem := FUndoList[i];
      if undoitem.magUnDoGroup <> curGrping then break;
      FUnDoList.Delete(i);
      case undoitem.magUnDoAction of
        magundoRemove:
          begin
            if FProxyList.Count < undoitem.magiproxy.loadindex
              then FProxyList.add(undoitem.magiproxy)
            else FProxylist.Insert(undoitem.magIproxy.loadindex, undoitem.magIproxy);
           end;
        magundoAdd:
          begin;
           end;
      end;
    end;
  ReAlignImages;
end;



procedure TMag4Viewer.SetRemember(value: boolean);
begin
FLastRemember := value;
end;

end.

