unit cMagDBBroker;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created:
   Site Name: Silver Spring, OIFO
   Developers: Garrett Kirin
  [==    unit cMagDBBroker;
   Description: Imaging Database connection unit: An Abstract ancestor Class.
    Designed to represent all calls to VistA DB.
     Other classes will surface (redeclare, override and implement) the Abstract
     methods and members to make DataBase Calls.
     The units, components etc., of the application will reference MagDBBroker, not
     the descendant class (TmagDBMVista, or TMagDBDemo)
     This design enables the application to function the same way
     regardless of which database is connected.  M, (Oracle), Demo TXT files
     As of Patch 8 : cMagDBMVista overrides all calls and connects to M.
                     cMagDBDemo overrides all calls and displays demo files.
     The Application doesn't know, or care, if it is running a demo or actual
     M DB Connection.  ==]
     (? transform into an Interface)
        Inheritance vs Polymorphism

   Note:
   }
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Trpcb, hash, RPCconf1, uMagUtils, uMagDefinitions, umagclasses,
  maggmsgu, fmxutils, magfileversion;

type
  TMagDBBroker = class
  private

  protected
    { Protected declarations }
  public

    //gek function GetSilentCodes(Vserver, Vport: string;  var SilentAccess: string; var SilentVerify: string): boolean; virtual ; abstract;
        {True if connected to Database, returns error message}
    function CheckDBConnection(var xmsg: string): boolean; virtual ; abstract;

      {  Returns status of DataBase Connection.}
    function IsConnected: boolean; virtual ; abstract;

      {  Connect or Disconnect from DataBase }
    procedure SetConnected(const Value: boolean); virtual ; abstract;

      {  Returns the Broker property: ListenerPort}
    function GetListenerPort: integer; virtual ; abstract;

      {  Returns the Broker property: Server}
    function GetServer: string; virtual ; abstract;

      {  Sets the Broker property: ListenerPort}
    procedure SetListenerPort(const Value: integer); virtual ; abstract;

      {  Returns the Broker property: Server}
    procedure SetServer(const Value: string); virtual ; abstract;

      {  Returns the TRPCBroker object associated with this object}
    function GetBroker: TRPCBroker; virtual ; abstract;

      {  Associates this TMagDBBroker object with a TRPCBroker object }
    procedure SetBroker(const Value: TRPCBroker); virtual ; abstract;

      {  Creates a TRPCBroker object and associates with this TMagDBBroker object}
    procedure CreateBroker; virtual ; abstract;

      {  Return a Date in FM Format
        xmsg         : Error string if result is false.
        dateinput    : Users input.
        dateoutput   : '^' delimited string.  piece 1 - external value.
                                              piece 2 - internal FM value.                                              {}
    function RPFileManDate(var xmsg: string; dateinput: string; var dateoutput: string; NoFuture : boolean): boolean; virtual ; abstract;

      {  future patch}
    procedure RPXMultiProcList(lit: TStrings; dfn: string);  virtual ; abstract;

      {  Checks the version of the Client against the version installed on the server.
 Server code determines if Client should be warned or aborted
 first piece of T[0] is 0,1,2.  T[1..n] are lines of the message to display
  0 : Display Warning and let continue
  1 : Okay, No warning, continue.
  2 : Version is too old, Display message then Application must abort}
    procedure RPMagVersionCheck(T: Tstrings; version: string); virtual ; abstract;

      {   --------- Clinical Procedures -----------}
      {  Get a list of CP Requests for the patient}
    procedure RPGetClinProcReq(dfn: string; var t: Tstrings); virtual ; abstract;

       {  Convert the Consult# fConsIEN into a TIUDA so we can associate the Image with a TIUDA.
 The result of this call could show that a Visit is needed:
  - If visit is needed Then display the visit selection window.
       - the Visit Selection window returns an existing visit or new visit in
          a vString structure (vString structure is used by TIU )
 Then this call is made again with the vString value from the visit seletion window.
 Complete : this is set when user selects what type of report it is.
  Three choices are displayed for user to select, if user selects the choice :
    'This report will not be receiving any additional images from another instrument'
      then this Consult will be considered 'Complete'.
        dfn : internal entry of patient entry in PATIENT File.
   fConsIEN : Internal entry number of Consult Request (VistA Consults file)
   vString  : data string used by TIU. Is a string of information relevant to TIU .
   Complete : '0' or '2'.  '2' means this Consult is complete.
      }
    procedure RPGetTIUDAfromClinProcReq(dfn, fConsIEN, vstring, Complete: string; var t: Tstrings); virtual ; abstract;

      {  Get a list of Visits for a patient.  The CP Request sometimes needs a visit to
 complete the association with an Image.}
    procedure RPGetVisitListForReq(dfn: string; var t: Tstrings); virtual ; abstract;

      {  Update Consults with the Complete Flag. Consults will update
  it's status to 'pr' partial results. It can then be signed.
 This Call is made after successfully capturing the Image.}
    procedure RPUpdateConsult(consult, tiuien, cmpflag: string; var status: string); virtual ; abstract;

      {  --------------------}

      {  Send Message to Mail Group when Patient Accesses an Offline image
 Site Manager has made an entry in the file ^MAGQUEUE(2006.033,0) = OFFLINE IMAGES
 In Delphi,  Offline images are determined by the property in TImageData
 if TImageData(IObj).fulllocation[1]) = 'O') }
    procedure RPMaggOffLineImageAccessed(IObj: TImageData); virtual ; abstract;


      {   ------------ Set Queues ----------}
      {  Set JBTOHD Queues for Abstract and Full.}
    procedure RPMaggQueImage(IObj: TImageData); virtual ; abstract;

      {  Set JBTOHD Queues for BIG file.}
    procedure RPMaggQueBigImage(IObj: TImageData); virtual ; abstract;

      {  Set JBTOHD Queues for all Images for patient.
     whichimage : [A|F|B]  A(bs) F(ull) B(ig) or combination }
    procedure RPMaggQuePatient(whichimages, DFN: string); virtual ; abstract;

      {  Set JBTOHD Queues for all Images in a Group.
     whichimage : [A|F|B]  A(bs) F(ull) B(ig) or combination }
    procedure RPMaggQueImageGroup(whichimages, MAGIEN: string); virtual ; abstract;

      {  --------------------}

      {  Call to log actions into IMAGING WINDOWS SESSIONS File from Delphi
  Actions -view and -capture are automatically logged into IMAGE ACCESS LOG File
   also.  Action -View will update LAST ACCESS DATE in IMAGE File also.}
    procedure RPMag3Logaction(actionstring: string); virtual ; abstract;

      {  Return a list of all images in an Image Group}
    procedure RPMaggGroupImages(IObj: TImageData; var t: tstringlist; noQAcheck: boolean = false); virtual ; abstract;

      {  Gets all images for a patient.  Groups are returned as one item.
  Single images are returned as one item.}
//prior to P8  procedure RPMaggPatImages(DFN: string; var t: tstringlist); virtual ; abstract;

      {  Gets MAX (count) of the latest images for a patient.  Each Image of a Group
is returned as a one list item, and Single images are returned as one list item.}
    procedure RPMaggPatEachImage(DFN, MAX: string; var t: tstringlist); virtual ; abstract;

      {  Call to Logoff of VistA.  Imaging Workstation and Session files are updated
 with logoff Date Time.}
    procedure RPMaggLogOff; virtual ; abstract;

      {  Call to send information to VistA about Current Session.
 Information is stored in IMAGING WINDOWS WORKSTATION and IMAGING WINDOWS
 SESSION Files.
          Apppath:      Full path to the application executable.
          DispAppName:  Name of Imaging Display executable.
          CapAppName:   Name of Imaging Capture executable.
          Compname:     The workstation Network Computer Name.
          Location:     Free text description of where the Workstation is located.
          LastUpdate:   Date/Time when MagSetup.exe (MagInstall.exe) was run.
          startmode:    [1|2]  1 = Started standalone,  2 = Started by CPRS. }
    procedure RPMaggWrksUpdate(Apppath, DispAppName, CapAppName, Compname, Location, LastUpdate: string; startmode: integer); virtual ; abstract;

      {  Call to return a list of SECURITY KEYS help by the user
 example result
t[0]="CAPTURE KEYS OFF"   t[0] = CAPTURE KEYS ON or CAPTURE KEYS OFF
t[1]="MAG CAPTURE"
t[2]="MAG DELETE"
t[3]="MAG SYSTEM"
t[4]="MAGCAP LAB"
t[5]="MAGDISP ADMIN"
t[6]="MAGDISP CLIN"}
    procedure RPMaggUserKeys(var t: Tstringlist); virtual ; abstract;

      {  Gets the value of the TIMEOUT field from the IMAGING SITE PARAMETERS file.
 Application must timeout if inactive for that number of minutes.}
    procedure RPMaggGetTimeout(app: string; var minutes: string); virtual ; abstract;

      {   ---------------------  TIU CALLS ----------------}
//TIU Calls.
      {  Gets the TIU Report by calling RPC              'TIU GET RECORD TEXT';}
    procedure RPGetNoteText(tiuda: string; t: tstrings); virtual ; abstract;

      {  Gets list of  TIU Note titles by calling RPC    'TIU GET PN TITLES';}
    procedure RPGetNoteTitles(t, tint: tstringlist); virtual ; abstract;


//  not creating  TIU Entries in P8
      {  Create a New TIU Note by calling RPC            'TIU CREATE RECORD';}
    function RPCreateRecord(var msg: string; DFN, notetitle, notedate: string; notetext: tstrings): boolean; virtual ; abstract;
//  not signing  TIU Entries in P8
      {  Sets a TIU Note Status to Signed, by  RPC          'TIU SIGN RECORD';}
    function RPSignRecord(tiuda, hashesign: string; var MSG: string): boolean; virtual ; abstract;

      {  Gets list of  TIU Discharge Summaries for patient by  RPC  'TIU SUMMARIES';}
    procedure RPGetDischargeSummaries(dfn: string; var t: tstringlist); virtual ; abstract;

      {  Gets list of  TIU Notes for patient by  RPC         'TIU DOCUMENTS BY CONTEXT';}
    procedure RPGetNotesByContext(dfn: string; var t: tstringlist; context: integer; author, count: string; docclass: integer; seq: string; showadd: integer; incund: integer); virtual ; abstract;

      {  Gets data about Note from TIUDA. TIUPTR =     TIUDA^Document Type ^Document Date^DFN}
    function RPGetTIUData(tiuda: string; var TIUPTR: string): boolean; virtual ; abstract;

      {  Get a list of All Images for a TIU Note (prompted by Windows message from CPRS}
    procedure RPGetCPRSTIUNotes(tiuda: string; t: tstringlist; var success: boolean; var RPmsg: string); virtual ; abstract;

      {  Get the TIU value for Document Class :CP          RPC : 'TIU IDENTIFY CLINPROC CLASS';}
    function RPTIUCPClass: integer; virtual ; abstract;

      {  Get the TIU value for Document Class :CONSULT          RPC : 'TIU IDENTIFY CONSULTS CLASS';}
    function RPTIUConsultsClass: integer; virtual ; abstract;

      {   ---------------------- }

      {  Return DateTime in VA FileMan format
  datestr : Date in string format
  DisDt   : Date in OutPut format
  IntDt   : Date in Internal Fileman Format}
    function RPGetFileManDateTime(datestr: string; var DisDt, IntDT: string; NoFuture : boolean): boolean; virtual ; abstract;

//prior to P8    procedure RPMagCategories(t: tstrings); virtual ; abstract;
      {  }
    function RPMagEkgOnline: integer; virtual ; abstract;
      {  }
    procedure RPMagGetNetLoc(var success: boolean; var RPmsg: string; var Shares: TStringList; NetLocType: string); virtual ; abstract;

      {  Call to stuff error information from Delphi app into the Session file.
  save the first COUNT items in T to the IMAGING WINDOWS SESSION file.}
    procedure RPMagLogErrorText(T: Tstrings; COUNT: INTEGER); virtual ; abstract;

      {         Get a list of Supported file Extensions from IMAGE FILE TYPES file.
        Each item in the result TStrings : t  is a '^' delimited string
        in the format:
           piece 1:     3 character File Extension
           piece 2:     Readable Description of image format
           piece 3:     1|0  is an Imaging Viewer available for this image fomat.
           piece 4:     name of the Canned Bitmap to use for abstract Or Null
           piece 5:     are abstracts created for images with this extension.}
    procedure RPGetFileExtensions(var t: tstrings); virtual ; abstract;

      {  Delete a Users private Filter from IMAGE LIST FILTERS File }
    procedure RPFilterDelete(var rstat: boolean; var rmsg: string; fltien: string); virtual ; abstract;

      {  Save Changes to an Image Filter in IMAGE LIST FILTERS File
  t : is the filter String.  The function FilterToString (defined in uMagDefinitions.pas)
   converts a Tfilter object into a string representation for calls to VistA}
    procedure RPFilterSave(var rstat: boolean; var rmsg: string; t: Tstrings); virtual ; abstract;

      {  Get a list of Filters, Private Public or Both
     if 'duz' = '' we get public.
     if 'duz' = Valid DUZ we get that users filters
     if 'duz' = Valid DUZ and 'getall' = TRUE, then return Public and Private.}
    procedure RPFilterListGet(var rstat: boolean; var rmsg: string; var tlist: tstrings; duz: string = ''; getall: boolean = false); virtual ; abstract;

      {  Get details of a filter from FM File   IMAGE LIST FILTERS
 filter: is a string.  the function StringToFilter (defined in uMagDefinitions.pas)
 will convert the string to a TImageFilter object
        i.e.  FilterObject := StringToFilter(filter);}
    procedure RPFilterDetailsGet(var rstat: boolean; var rmsg, filter: string; fltien: string; fltname: string = ''; duz: string = ''); virtual ; abstract;

      {  Return a list of CTPresets for the site from IMAGING SITE PARAMETERS File}
    procedure RPCTPresetsGet(var rstat: boolean; var rmsg: string; var value: string); virtual ; abstract;

      {  Save values for a list of CTPresets for the site in IMAGING SITE PARAMETERS File}
    procedure RPCTPresetsSave(var rstat: boolean; var rmsg: string; value: string); virtual ; abstract;

      {  Get a list of Health Summaries for the Site}
    procedure RPMaggHSList(var rstat: boolean; var rmsg: string; var tlist: tstringlist; value: string); virtual ; abstract;

      {  Get the Health Summary report}
    procedure RPMaggHS(var rstat: boolean; var rmsg: string; var tlist: tstringlist; value: string); virtual ; abstract;

      {  Get the Radiology Report from RARPT IEN}
    procedure RPMaggRadReport(var rstat: boolean; var rmsg: string; var tlist: tstringlist; rarpt: string); virtual ; abstract;

      {  Get list of Images for a selected Rad Exam}
    procedure RPMaggRadImage(var rstat: boolean; var rmsg: string; var tlist: tstringlist; radstring: string); virtual ; abstract;

      {  Return the Patient Profile Report for a patient}
    procedure RPMaggDGRPD(var rstat: boolean; var rmsg: string; var rlist: tstringlist; dfn: string); virtual ; abstract;

      {  Return User Information and site information
         rlist[0] = 1|0  ^ message   1 = Okay   0 = Error.
         rlist[1] : from NEW PERSON FILE
                = DUZ ^ Full User NameName  ^ Initials
              2,3,4  from IMAGING SITE PARAMETERS FILE.
         rlist[2] = Network UserName ^ Encrypted PassWord.
         rlist[3] = MUSE Site number. ( default = 1)
         rlist[4] =  IEN ^ SITE CODE ^ DUZ(2) partition varialbe^ Institution (.01)
                                                  ^ Consolidated ? 

         rlist[5] = +<CP Version>|0 ^ Version of CP installed on Server
         rlist[6] = Warning msg if we can't resolve which Site Parameter entry to use.
         rlist[7] = Warning message  <reserverd for future>
         }
    procedure RPMaggUser2(var rstat: boolean; var rmsg: string; var rlist: tstrings; wsid: string); virtual ; abstract;

      {  Get list of Images for a Rad Exam selected in CPRS, prompted by Window message}
    procedure RPMaggCPRSRadExam(var rstat: boolean; var rmsg: string; var rlist: tstrings; cprsstring: string); virtual ; abstract;

      {  Get a list of Radiology Exams for a Patient}
    procedure RPMaggRadExams(var rstat: boolean; var rmsg: string; var rlist: tstrings; DFN: string); virtual ; abstract;

      {  Get Image information for 1 image}
    procedure RPMaggImageInfo(var rstat: boolean; var rmsg: string; var rlist: tstrings; ien: string; noQAcheck : boolean = false); virtual ; abstract;

      {  Get list of Filtered images for a Patient}
    procedure RPMag4PatGetImages(var Fstat: boolean; var rpcmsg: string; DFN: string; var Flist: tstrings; filter: TImageFilter = nil); virtual ; abstract;

      {  Get list of all User Preferences from DB, (IMAGING USER PREFERENCES File) or 1 node
 of preferences. Set fields of the UserPreferences record. (upref)
 if parameter -code is not null, then only the node represented by -code will be
 retrieved from DB .
  When a form is open, a call is made to the relevant procedure to set the preferences
  of the window, and the variable upref (upref : record of userprefernces) is sent as paremeter.
  i.e. procedure UprefToFullView(var upref: userpreferences);}
    procedure RPMagGetUserPreferences(var Fstat: boolean; var rpcmsg: string; xlist: Tstringlist; code: string = ''); virtual ; abstract;

      {  Called with a  TStrings list of properties from Open forms, checked menu's etc.
 and saved to the IMAGING USER PREFERENCES File.
 The TStrings parameter xList could be All preferences, or only a few or one.
    Called from SaveUserSettings in frmMain.pas (Image Display main window)
    and SetUserCapPref in frmCapMain.pas (Image Capture main window)}
    procedure RPMagSetUserPreferences(var Fstat: boolean; var rpcmsg: string; xlist: Tstrings); virtual ; abstract;

      {  Old Get list of IndexTypes, not based on Class}
   // procedure RPMagIndexType(var Fstat: boolean; var Flist: tstringlist; MagDFN: string); virtual ; abstract;

      {  ImportAPI call. Queues data to be imported by BP
  The parameter DataArray is a list of  dataCode^dataValue as defined by the
  import API.
  i.e. DataArray(2)='AQD^ComputerName'
  The Call to DB could return error if Required fields are missing or
  data is invalid.  Usually returns Success and QueueNumber}
    procedure RPMag4RemoteImport(var Fstat: boolean; var Flist: tstringlist; DataArray: Tstringlist); virtual ; abstract;

      {  Get Patient Info based on DFN.  Called by TMag4Pat object.  the data returned is
 used to populate the properties of the TMag4Pat Object.}
    procedure RPMagPatInfo(var Fstat: boolean; var Fstring: string; MagDFN: string; isicn: boolean = false); virtual ; abstract;

      {  Prior to Patch 8}
//    procedure RPMag4PostProcessing(var Fstat: boolean; var Flist: tstringlist; Magien: string); virtual ; abstract;

      {  Called after successful Image save.  Any Post processing actions defined for the
 TYPE INDEX of the newly saved image will be executed. (Copy to HEC is the only
 one so far)}
    procedure RPMag4PostProcessActions(var Fstat: boolean; var Flist: tstringlist; Magien: string); virtual ; abstract;

      {  Generic Lookup call using FIND^DIC(, , ,  ,) }
    procedure RPMag3LookupAny(var Fstat: boolean; var fmsg: string; var Flist: tstringlist; InputString: string); virtual ; abstract;

      {  Future call.  probably move to cMagDBSysUtils}
    procedure RPMag4IAPIStats(var Fstat: boolean; var FList: tstrings; DtStart, DtEnd: string); virtual ; abstract;

      {  Call to Delete a single Image. User needs MAG DELETE Key
        Flist      : messages returned.  Displayed if Fstat = false.
        IEN        : Image IEN
        ForceDel   : [1|0]  1 = Delete Image even if user doesn't have MAG DELETE key
                            0 = Normal.  User needs the MAG DELETE KEY
        reason     : Reason for Deletion. User picked from list or entered free text.
        GrpDelOK   : Future.  To Allow deleting groups from GUI.
          {  }
    procedure RPMaggImageDelete(var Fstat: boolean; var rmsg: string; var Flist: tstrings; IEN, ForceDel: string; reason: string = ''; GrpDelOK: boolean = false); virtual ; abstract;

      {  Call to get a Report associated with the Image.  Only IEN is sent, M code determines
 the associated Lab,Rad,Med,Sur,CP,Note attacthed to the Image and returns it
 as a list. If a Long Description is defined for the Image, it is Inserted at the
 Top of the listing, before the Report.}
   // prior to 8 procedure RPMagGetImageReport(var Fstat: boolean; var Fmsg: string; Flist: tstrings; magien: string); virtual ; abstract;
      {   RPImageReport is called from cMagUtilsDB .  All calls from application to get
  a report from DB, go through cMagUtilsDB.}
    procedure RPImageReport(var Fstat: boolean; var Fmsg: string; Flist: tstrings; Iobj : TImageData; noQACheck : boolean = false); virtual ; abstract;

      {  Get a list of Images associated with a TIU Report.
 If multiple Groups or Groups and Images.  We put all images (abstracts) in one
 list and display them in the Group Abstract Viewer.}
    procedure RPMag3TIUImage(var Fstat: boolean; var Flist: tstringlist; MagIEN, TIUDA: string); virtual ; abstract;

      {  ImportAPI  Call.  When BP calls IAPI with a Queue Number,  this call is made by IAPI
 to get the DataArray from VistA.  IMPORT QUEUE File.}
    procedure RPMag4DataFromImportQueue(var Fstat: boolean; var Flist: tstringlist; QueueNum: string); virtual ; abstract;

      {  ImportAPI Call.  The IAPI creates a DataArray from it's properties and makes
 this DB call to assure the Data is valid before making the call to save the
 data to the IMAGE File.}
    procedure RPMag4ValidateData(var Fstat: boolean; var Flist: tstringlist; t: tstringlist; rettype: string); virtual ; abstract;

      {  ImportAPI Call.  After Validating the data, the IAPI makes this call to save the
 image to the IMAGE File.}
    procedure RPMag4AddImage(var Fstat: boolean; var Flist: tstringlist; t: tstringlist); virtual ; abstract;

      {  After a successful Image save. The IAPI and Capture Workstation make this call
 to set the Queues for Copy to the JukeBox Queues. And Optionally, to create an
  abstract.}
    procedure RPMagABSJB(var Fstat: boolean; var Flist: tstringlist; CreatAbsIEN, JBCopyIEN: string); virtual ; abstract;

      {  Call DB to get a list of Versions of Imaging that have been installed on the Server.
 The About box uses this to display that list.}
    procedure RPMaggInstall(var Fstat: boolean; var Flist: tstringlist); virtual ; abstract;

      {  Future.  Information about a supported file format based on it's extension.}
    function RPMag4GetFileFormatInfo(ext: string; var xmsg: string): boolean; virtual ; abstract;

      {  This call will validate a Users' Electronic Signature.}
    function RPVerifyEsig(esig: string; var xmsg: string): boolean; virtual ; abstract;

      {  Here we log an action in the IMAGING ACCESS LOG File ^MAG(2006.95
  parameter is '^' delimited string
   copy code^ duz ^ magien ^ actioin ^ dfn ^ 1
   i.e.  'A|B|C|D|E' ^^ MAGIEN ^ 'Copy/Download' ^ DFN ^ '1';
   DUZ is inserted as 2nd piece by the M routine.
   example : s = "C^^103660^Copy To Clipboard^1033^1"}
    procedure RPLogCopyAccess(s: string); virtual ; abstract;

      {  Get a list of Image Types from the IMAGE INDEX FOR TYPES File
  cls: a ',' seperated list of Classes.
  ignorestatus : flag to ignore the Status field of the TYPE entry.  1=Ignore   }
    procedure RPIndexGetType(lit: TStrings; cls: string; ignoreStatus: boolean = false); virtual ; abstract;

      {  Get a list of Image Proceure/Events from the IMAGE INDEX FOR PROCEDURE/EVENT File
  cls: a ',' seperated list of Classes.
  spec : a ',' seperated list of Specialties
  ignorestatus : flag to ignore the Status field of the Procedure entry.  1=Ignore    }
    procedure RPIndexGetEvent(lit: TStrings; cls: string = ''; spec: string = ''; ignoreStatus: boolean = false); virtual ; abstract;

      {  Get a list of Image Specialties/SubSpecialties from the IMAGE INDEX FOR SPECIALTY/SUBSPECIALTY File
  cls: a ',' seperated list of Classes.
  proc : a ',' seperated list of Procedure/Events
  ignorestatus : flag to ignore the Status field of the Specialty entry.  1=Ignore}
   procedure RPIndexGetSpecSubSpec(lit: TStrings; cls: string = ''; proc: string = ''; ignoreStatus: boolean = false); virtual ; abstract;


      {  Call to Do a Patient lookup using FIND^DIC
 STR is parameter in the format   FILE NUM ^ NUM TO RETURN ^ TEXT TO MATCH }
    function RPMagPatLookup(STR: string; t: tstrings; var xmsg: string): boolean; virtual ; abstract;


      {    ----------------   DG RPC Calls. ------------------  }
      {    Checks if More than one patient has Same Last Name and Last 4 of SSN
  It calls the Patient API  GUIBS5A^DPTLK6 to figure it out.
  Returns 1 in 1st string (t[0]) if more than 1 exist, Otherwise 0 }
    function RPMaggPatBS5Chk(DFN: string; var t: tstringlist): boolean; virtual ; abstract;

      {   Calls the RPC  'DG SENSITIVE RECORD ACCESS';
  Code :  is the action code returned.
           -1 = RPC/API failed
           0 = No display or action required
           1 = Display warning message
           2 = Display warning message - require OK to continue
           3 = Display warning message - do not continue
  t :  Warning message is returned from DG package. (if needed.)
           If the output value is 1 (display warning message) entry in
           DG SECURITY LOG file is automatically made; GUI application does not
            need to take action to log access}
    procedure RPDGSensitiveRecordAccess(DFN: string; var code: INTEGER; var t: TSTRINGLIST); virtual ; abstract;

      {   Calls the RPC   DG SENSITIVITY RECORD BULLETIN
  If the return of   RPDGSensitiveRecordAccess is 2 then display dialog.
     If user elects to continue.
     The RPC logs access to sensitive patient by user.
     and sends bulletin to Mail Group (Mail group determined by site)    }
    procedure RPDGSensitiveRecordBulletin(DFN: string); virtual ; abstract;

      {   Calls the RPC 'DG CHK PAT/DIV MEANS TEST';
  to check if means test is required for this patient and checks if means test
  display required for  user's division.
  Code is returned as 1 if Both true,   Otherwise 0
  If Both true, then display the text returned in t   }
    procedure RPDGChkPatDivMeansTest(DFN: string; var code: INTEGER; var t: TSTRINGLIST); virtual ; abstract;
      {    ----------------                 ------------------  }


      {  Get image information for one image.  result is a list in readable format.  Caller
 just displays the list.  no need to match '^'pieces to output fields.}
    procedure RPMag4GetImageInfo(magien: string; var Flist: Tstrings); virtual ; abstract;

      {  Call returns a list of all Photo ID's on file for a patient.
 The Images are returned in Rev Chronological Order, latest image
 first, oldest image last. Data in the list is in format to be converted to
 TImageData object}
    function RPMaggGetPhotoIDs(xdfn: string; t: tstringlist): BOOLEAN; virtual ; abstract;

      {  Import API Call : Report Status to calling application
 P8T32 10/29/03 The BP No Longer makes this call The Import Component
 (and OCX) call this RPC
t(0)= "0^message" or "1^message"
t(1)=TRKID,
t(2)=QNUM
t(3..N)=warnings
cb  : The TAG^RTN to call with Status Array
DoStatusCB : Flag (1|0) to supress the execution of the Status Callback
Note : Old Import API and BP that make this call, will function as before
because DOCB defaults to 1}
    procedure RPMag4StatusCallback(t: tstrings; cb: string; DoStatusCB: boolean = true); virtual ; abstract;

      {  DBSelect : This call uses Kernel API to display a list of Servers defined on the
 workstation for the User to select form.  If user selects one, it is returned in
 parameters vserver, vport }
          {   TODO : Need to return a message, 'Why connection didn't work, User cancel, Error, etc. }
          {   TODO : DBSelect needs : option of user sending the Server Port, and not being prompted for it . }
    function DBSelect(var vserver, vport: string; context: string = 'MAG WINDOWS'): boolean; virtual ; abstract;

      {  Use Kernel Broker to connect to the Server Port defined by the parameters
vserver, vport
if parameters accesscode and verifycode are defined, user will not be prompted
to enter them again. Silent Login}
    function DBConnect(vserver, vport: string; context: string = 'MAG WINDOWS'; accesscode: string = ''; verifycode: string = ''): boolean; virtual ; abstract;


  published
     {  TODO: the published properties were made public for the override to work
       from the other classes that inherit this one qad.
       research making components out of this, or child classes, to be able
       to drop onto form, rather that coding to create and destroy}
//   property Broker: TRPCBroker read GetBroker write SetBroker;
//   property Connected : boolean read IsConnected write SetConnected;
//   property listenerport : integer read GetListenerPort write SetListenerPort;
//   property server : string read GetServer write SetServer;
  end;


implementation




end.

