unit cMagDBSysUtils;
   {
   Package: MAG - VistA Imaging
 WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
 Date Created:
 Site Name: Silver Spring, OIFO
 Developers: Garrett Kirin
 [==
 Description:
   Imaging DataBase RPC Calls for System functions.
   (Calls made from Image Information Advanced window, MAGSYS.EXE and MAGWRKS.EXE)

   In the future there will be an  Abstract Class to inherit from, similiar to
   the abstract class cMagDBBroker and descendant classes cMagDBMVista and cMagDBDemo.
   ==]

 Note:  This function is not Released in Patch 8
  }
 (*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Trpcb, hash, RPCconf1, umagutils;

type
  TMagDBSysUtils = class(TComponent)
  private
    FBroker: TRPCBroker;
    function GetBroker: TRPCBroker;
    procedure SetBroker(const Value: TRPCBroker);
  protected
    { Protected declarations }
  public
    procedure RPMAG4SysWrksStats(t: tStringlist; CompName, days, dtfr, dtto: string);
    procedure RPMAG4SysUserStats(t: tStringlist; days, dtfr, dtto: string);
    procedure RPMAG4SysWrksData(t: tStringlist; CompName, days, dtfr, dtto: string);
    procedure RPMAG4SysSessByWrks(t: tStringlist; wrksien, days, dtfr, dtto: string);
    procedure RPMAG4SysSessByUser(t: tStringlist; duz, days, dtfr, dtto: string);
    procedure MaggSysSessionDisplay(var t: Tstringlist; sess: integer);
    function DBSelect(var vserver, vport: string): boolean;
    procedure CreateContext(context: string);
    procedure RPMaggSysGlbNode(magien: string; var t: tstringlist);
    procedure RPMaggSysParGlb(magien: string; var t: tstringlist);
    procedure RPMaggDevFldValues(magien, flags: string; var t: tstringlist);
    procedure RPMaggDevParFldValues(magien, flags: string; var t: tstringlist);
    procedure RPMaggSysAnyNode(GN1: string; var t: tstringlist);
    procedure RPMaggImageInfo(xmagien: string; var imagestring: string);
    function IsConnected: boolean;
  published
    property Broker: TRPCBroker read GetBroker write SetBroker;
  end;

procedure Register;

implementation

procedure TMagDBSysUtils.RPMAG4SysSessByWrks(t: tStringlist; wrksien, days, dtfr, dtto: string);
begin

    // if not broker.connected then exit;
  Fbroker.param[0].value := wrksien;
  Fbroker.param[0].ptype := literal;
  Fbroker.param[1].value := days;
  Fbroker.param[1].ptype := literal;
  Fbroker.param[2].value := dtfr;
  Fbroker.param[2].ptype := literal;
  Fbroker.param[3].value := dtto;
  Fbroker.param[3].ptype := literal;
  Fbroker.remoteprocedure := 'MAG4 SYS SESSIONS WRKS'; //'MAGG SYS WRKS SESSIONS';
  try
    Fbroker.lstcall(t);
  except
    on e: exception do t.Insert(0, '0^' + e.message);
  end;
end;

procedure TMagDBSysUtils.RPMAG4SysSessByUser(t: tStringlist; duz, days, dtfr, dtto: string);
begin

    // if not broker.connected then exit;
  Fbroker.param[0].value := duz;
  Fbroker.param[0].ptype := literal;
  Fbroker.param[1].value := days;
  Fbroker.param[1].ptype := literal;
  Fbroker.param[2].value := dtfr;
  Fbroker.param[2].ptype := literal;
  Fbroker.param[3].value := dtto;
  Fbroker.param[3].ptype := literal;
  Fbroker.remoteprocedure := 'MAG4 SYS SESSIONS USER'; //'MAGG SYS WRKS SESSIONS';
  try
    Fbroker.lstcall(t);
  except
    on e: exception do t.Insert(0, '0^' + e.message);
  end;
end;

procedure TMagDBSysUtils.RPMAG4SysUserStats(t: tStringlist; days, dtfr, dtto: string);
begin
  Fbroker.param[0].value := days;
  Fbroker.param[0].ptype := literal;
  Fbroker.param[1].value := dtfr;
  Fbroker.param[1].ptype := literal;
  Fbroker.param[2].value := dtto;
  Fbroker.param[2].ptype := literal;
  Fbroker.remoteprocedure := 'MAG4 SYS USER STATS';
  try
    Fbroker.lstcall(t);
  except
    on e: exception do t.Insert(0, '0^' + e.message);
  end;
end;


procedure TMagDBSysUtils.RPMAG4SysWrksData(t: tStringlist; CompName, days, dtfr, dtto: string);
begin
  Fbroker.param[0].value := CompName;
  Fbroker.param[0].ptype := literal;
  Fbroker.param[1].value := days;
  Fbroker.param[1].ptype := literal;
  Fbroker.param[2].value := dtfr;
  Fbroker.param[2].ptype := literal;
  Fbroker.param[3].value := dtto;
  Fbroker.param[3].ptype := literal;
  Fbroker.remoteprocedure := 'MAG4 SYS WRKS DATA';
  try
    Fbroker.lstcall(t);
  except
    on e: exception do t.Insert(0, '0^' + e.message);
  end;
end;

procedure TMagDBSysUtils.RPMAG4SysWrksStats(t: tStringlist; CompName, days, dtfr, dtto: string);
begin
  Fbroker.param[0].value := CompName;
  Fbroker.param[0].ptype := literal;
  Fbroker.param[1].value := days;
  Fbroker.param[1].ptype := literal;
  Fbroker.param[2].value := dtfr;
  Fbroker.param[2].ptype := literal;
  Fbroker.param[3].value := dtto;
  Fbroker.param[3].ptype := literal;
  Fbroker.remoteprocedure := 'MAG4 SYS WRKS STATS';
  try
    Fbroker.lstcall(t);
  except
    on e: exception do t.Insert(0, '0^' + e.message);
  end;
end;



procedure TMagDBSysUtils.MaggSysSessionDisplay(var t: Tstringlist; sess: integer);
begin
  fbroker.param[0].value := inttostr(sess);
  fbroker.param[0].ptype := literal;
  fbroker.remoteprocedure := 'MAGG SYS SESSION DISPLAY';
  fbroker.lstcall(t);
end;

function TMagDBSysUtils.DBSelect(var vserver, vport: string): boolean;
begin
  if (GetServerInfo(VServer, VPort) = mrOK) then
    begin
      Fbroker.server := VServer;
      Fbroker.listenerPort := strtoint(VPort);
      FBroker.connected := true;
      Result := FBroker.connected;
    end
  else
    result := false;
end;


procedure Register;
begin
  RegisterComponents('Imaging', [TMagDBSysUtils]);
end;

function TMagDBSysUtils.GetBroker: TRPCBroker;
begin
  Result := FBroker;
end;

procedure TMagDBSysUtils.SetBroker(const Value: TRPCBroker);
begin
  FBroker := Value;
end;

procedure TMagDBSysUtils.CreateContext(context: string);
begin
  Fbroker.CreateContext(context);
end;

procedure TMagDBSysUtils.RPMaggSysGlbNode(magien: string; var t: tstringlist);
begin
  FBroker.remoteprocedure := 'MAGG SYS GLOBAL NODE';
  FBroker.param[0].value := magien;
  FBroker.param[0].ptype := literal;
  FBroker.lstCall(t);
end;


procedure TMagDBSysUtils.RPMaggSysParGlb(magien: string; var t: tstringlist);
begin
  FBroker.remoteprocedure := 'MAGG SYS PARENT GLB';
  FBroker.param[0].value := magien;
  FBroker.param[0].ptype := literal;
  FBroker.lstCall(t);
end;


procedure TMagDBSysUtils.RPMaggDevFldValues(magien, flags: string; var t: tstringlist);
begin
  FBroker.remoteprocedure := 'MAGG DEV FIELD VALUES';
  FBroker.param[0].value := magien;
  FBroker.param[0].ptype := literal;
  FBroker.param[1].value := UPPERCASE(flags);
  FBroker.param[1].ptype := literal;
  FBroker.lstCall(t);
end;


procedure TMagDBSysUtils.RPMaggDevParFldValues(magien, flags: string; var t: tstringlist);
begin
  FBroker.remoteprocedure := 'MAGG DEV PARENT FIELD VALUES';
  FBroker.param[0].value := '2005';
  FBroker.param[0].ptype := literal;
  FBroker.param[1].value := magien;
  FBroker.param[1].ptype := literal;
  FBroker.param[2].value := UPPERCASE(flags);
  FBroker.param[2].ptype := literal;
  FBroker.lstCall(t);
end;


procedure TMagDBSysUtils.RPMaggSysAnyNode(GN1: string; var t: tstringlist);
begin
  FBroker.remoteprocedure := 'MAGG SYS ANY NODE';
  FBroker.param[0].value := GN1;
  FBroker.param[0].ptype := literal;
  FBroker.lstCall(t);
end;


{TODO: Make a New RPC ... MAG4 SYS IMAGE INFO to keep things straight.}
procedure TMagDBSysUtils.RPMaggImageInfo(xmagien: string; var imagestring: string);
begin
  FBroker.RemoteProcedure := 'MAGG IMAGE INFO';
  FBroker.Param[0].ptype := literal;
  FBroker.Param[0].value := xMagIEN;
  FBroker.call;
  imagestring := FBroker.results[0];
end;

function TMagDBSysUtils.IsConnected: boolean;
begin
  result := FBroker.connected;
end;

end.

