unit cMagImageList;
  {
   Package: MAG - VistA Imaging
 WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
 Date Created:
 Site Name: Silver Spring, OIFO
 Developers: Garrett Kirin
 [==    unit cMagImageList;
 Description: Image list functions and data.
  This component is central to any list of images in the application.
  cMagListView, cMagTreeView in the Image list window, or other windows.
  TmagViewer component in Abs, Group and Full.
  and Other components have Properties or fields that are of type : TMagImageList.
  and are associated with a cMagImageList object. Other Objects may attach
  themselves as Observers of TmagImageList.

 This list component has information specific to the list.
 i.e. Patient, Filter, List Description.
        Has a TStringList (baseList) of the data retrieved from the DataBase.
        and a cooresponding Tlist (objlist) of the baselist data converted to
         TImageData objects.

 TmagImageList implements the ImagObserver interface and Attaches itself to a
 TMag4Pat object (TMag4Pat implements the ImagSubject Interface) .
 TMagImageList implements the ImagSubject interface and accepts a list of Observers
 (Objects the support the ImagObserver interface can Attach themselves to
  TmagImageList as the Subject)
 the Update_ method of TMagImageList will be called by the TMag4Pat subject whenever
 the subject has a change of state.
 (State change of Tmag4Pat is New patient or Clear patient)
 In the Update_ method, TmagImageList Notifies all Observers of a change of state
  by calling the UpDate_ method of each Observer.
 TmagImageList Notifies all observers whenever the list changes.  As when the user
 selects a new Filtered list of Images.
  ==]
 Note:
  }
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,

//va
  ImagInterfaces, umagClasses, cMagPat, cMagDBBroker, umagDefinitions,
  cMagVutils, cMagUtils;

type
 {Implements the 2 Interfaces}
  TmagImageList = class(TComponent, IMagObserver, IMagSubject)
  private
    FObserversList: array of IMagObserver;
    FDFN: string;
    FImageFilter: TImageFilter;
    FMagPat: TMag4Pat;
    FMagDBBroker: TMagDBBroker;
    FVutils: TMagVutils;
    FUtils: TMagUtils;
    FbaseList: Tstrings;
    FObjList: Tlist;
    FIsGroupList: boolean;
    FGroupIEN: string;
    FListDesc: string;
    FlistName: string;
    procedure ConvertBaseListToObjectList;
    procedure AttachMyself();
    procedure SetFimageFilter(const Value: TImageFilter);
    procedure SetFlistdesc(const Value: string);
    procedure SetFlistName(const Value: string);
  public
    constructor Create(AOwner: TComponent); override;
    destructor destroy; override;
    {   This is the Tstringlist that is retrieved from VistA}
    property BaseList: Tstrings read FbaseList write FBaseList;
    {   This is items in the BaseList converted to TimageData objects.}
    property Objlist: Tlist read Fobjlist write Fobjlist;
    {   The TImageData object that was used when compiling the list.}
    property ImageFilter: TImageFilter read Fimagefilter write SetFimageFilter;

    {ImagSubject}
    {   Observers can Attach to a TMagImageList as a Subject with this call.}
    procedure Attach_(Observer: ImagObserver);
    {   Observers can Detach themselves from subjects}
    procedure Detach_(Observer: ImagObserver);
    {   Not Implemented.}
    procedure GetState_(var State: string);
    {   Not Implemented.}
    procedure SetState_(State: string);
    {   All Observers will be notified by a call to their Update_ method}
    procedure Notify_(SubjectState: string);


    procedure ClearAll();
    //Testing function. future.
    function GetAnImage(Iobj : TimageData; direction : integer) : TImageData;
    procedure SetMagPat(const Value: TMag4Pat);
    {ImagObserver}
    {   This is called by the ImagSubject when a change of state occurs.}
    procedure Update_(SubjectState: string); { Public declarations }
    {   A group list behaves differently than a filter images list.  So a
        seperate call to load it.}
    procedure LoadGroupList(t: Tstrings; groupien: string; DFN: string);
    {   Returns the Filters description if it is a filtered list.}
    function ListDetails(): string;
  published
    property MagPat: TMag4Pat read FMagPat write SetMagPat default nil;
    property MagDBBroker: TMagDBBroker read FMagDBBroker write FMagDBBroker;
    {   If a group list, it behaves differently than filtered list, when }
    property IsGroupList: boolean read FIsGroupList write FIsGroupList;
    property ListDesc: string read Flistdesc write SetFlistdesc;
    property ListName: string read FListName write SetFlistName;

  end;


procedure Register;

implementation
{ TMagImageList }

procedure TMagImageList.AttachMyself;
begin
  if assigned(FmagPat) then
    begin
      FMagPat.Attach_(IMagObserver(self));
    end;
end;

procedure TMagImageList.Attach_(Observer: ImagObserver);
begin
  //IMagSubject
  Setlength(FObserverslist, length(fobserverslist) + 1);
  FObserverslist[high(fObserverslist)] := observer;
end;

procedure TMagImageList.ClearAll;
begin
  //
end;

constructor TMagImageList.Create(AOwner: TComponent);
begin
  inherited;

  FbaseList := Tstringlist.create;
  FObjList := Tlist.create;
  FImageFilter := TImageFilter.Create();
  FImageFilter.Packages := [];
  FImageFilter.Classes := [];
  FVutils := TMagVutils.create(nil);
  FUtils := TMagUtils.create(nil);
  FDFN := '';
  FGroupIEN := '';
  FMagPat := nil;
end;

destructor TMagImageList.destroy;
begin
  FbaseList.free;
  FObjList.free;
  FImageFilter.free;
  FVutils.free;
  FUtils.free;
  if assigned(FmagPat) then
    begin
      FMagPat.Detach_(IMagObserver(self));
      FMagpat := nil;
    end;
  Notify_('-1');
  inherited;
end;

{   Detach_ :  an Observer is detaching itself from the subject TMagImageList}
procedure TMagImageList.Detach_(Observer: ImagObserver);
var i: integer;
begin
//IMagSubject
  for i := 0 to (high(Fobserverslist)) do
    begin
      if (Observer = FObserversList[i]) then
        begin
          FObserverslist[i] := nil;
        end;
    end;
end;

procedure TMagImageList.GetState_(var State: string);
begin
  //IMagSubject
end;

{.Notify_: Subject TMagImageList will call Update_ method of all Observers }
procedure TMagImageList.Notify_(SubjectState: string);
var i: integer;
  ivObserver: IMagObserver;
begin
  for i := 0 to (high(Fobserverslist)) do
    begin
      if FObserversList[i] <> nil then
        begin
          IvObserver := (FObserversList[i]);
          Ivobserver.Update_(SubjectState);
        end;
    end;
end;


procedure TMagImageList.SetMagPat(const Value: TMag4Pat);
begin
  if (FmagPat <> nil) then FMagPat.Detach_(self);
  FMagPat := Value;
  if value = nil then exit;
  FDFN := FMagPat.M_DFN;
  AttachMyself();
end;

procedure TMagImageList.SetState_(State: string);
begin
  //IMagSubject
end;

{Update_ : Observer TmagImageList has been Notified by Subject of a change}
procedure TMagImageList.Update_(SubjectState: string);
var stat: boolean;
  statmsg: string;
begin
  FBaselist.Clear;
  FObjlist.Clear;
  FGroupIEN := '';
  Flistdesc := '';
  //FlistName := '';
  try
  if SubjectState = '' then
    begin
      notify_(SubjectState);
      exit;
    end;
  if SubjectState = '-1' then
    begin
      FDFN := '';
      FMagPat := nil;
      notify_('');
      exit;
    end;
  if FIsGroupList then exit;
  if not assigned(FMagDBBroker) then
    begin
      showmessage('Error: DB Broker UnAssigned in TMagImageList.');
      exit;
    end;
  screen.cursor := crHourGlass;
  FMagDBBroker.RPMag4PatGetImages(stat, statmsg, FMagPat.M_DFN, FBaseList, FImageFilter); // Patch 8
  screen.Cursor := crDefault;
  Flistdesc := statmsg;
  //FlistName := FImageFilter.Name;
  if FlistName = '' then FlistName := FlistDesc;
  if stat
    then
    begin
      ConvertBaseListToObjectList;
      notify_(FMagPat.M_DFN);
    end
  else
    begin
      if pos('No images for filter', statmsg) > 0
        then Notify_('0 Images. ' + statmsg)
      else
        begin
          messagedlg(statmsg, mterror, [mbok], 0);
          Notify_('');
        end;
    end;
finally
screen.cursor := crdefault;
end;

end;

{ConvertBaseListToObjectList: convert the Baselist to a list of TImageData Objects}
procedure TMagImageList.ConvertBaseListToObjectList;
var i: integer;
  IObj: TImageData;
begin
  for i := 1 to FbaseList.count - 1 do
    begin
      try
        Iobj := FVutils.StringToIMageObj('xx^' + Futils.magPiece(FbaseList[i], '|', 2));

        FObjlist.Add(Iobj);
        // Need to rething the GroupIEN, maybe the pointer to cMagImagelist
        //   in the Iobj  ? ?
        If (FgroupIEN <> '') then Iobj.GroupIEN := Strtoint(FGroupIEN);
        Iobj.GroupIndex := (FObjlist.count - 1);
        FBaselist[i] := Futils.magPiece(FbaseList[i], '|', 1) + '|' + inttostr(FObjlist.count - 1);
      except
      //
      end;
    end;
end;

procedure TmagImageList.LoadGroupList(t: Tstrings; groupien, DFN: string);
var i: Integer;
xCursor : TCursor;
begin
// here clear the old list, if needed,
xCursor := Screen.cursor;
  Notify_('');
Screen.cursor := xCursor;
  FGroupIEN := groupien;
{ TODO : Have to check if the DFN Parameter is the same as FMagPat.M_DFN
if not do ?  ? ? ? ? }
  FDFN := DFN;
  FBaseList.Clear;
  FObjList.Clear;
// convert the OLD Style BaseList to Object list
// the first OLD Style BaseList didn't have a column header list. so lets fake it.
  FBaseList.add('old^Style');
  for i := 0 to t.count - 1 do
    begin
//s := t[i];
      FBaseList.Add('conv^|' + copy(t[i], pos('^', t[i]) + 1, 999)); //
    end;
  ConvertBaseListToObjectList;
// Then Call Update;
{ TODO : Can't send DFN for now, don't have it., and don't have GroupIEN.
        But also don't need to.  The observers will querry the subject (if they
        need to)  when they get notified of a change of state. }
//Notify_(FDFN);
  Notify_('1');
end;



procedure Register;
begin
  RegisterComponents('Imaging', [TmagImageList]);
end;

procedure TmagImageList.SetFimageFilter(const Value: TImageFilter);
begin
  if value = nil then exit;
  if assigned(FimageFilter) then FimageFilter.Free;
  Fimagefilter := Value;
  FListName := FImageFilter.Name;
end;

procedure TmagImageList.SetFlistdesc(const Value: string);
begin
// do nothing from the property Setter.  i.e. Readonly  Flistdesc := Value;
end;

function TmagImageList.ListDetails: string;
begin
  if not assigned(FImageFilter) then result := 'List is not filtered.'
  else result := ImageFilter.Detailstring;
end;

procedure TmagImageList.SetFlistName(const Value: string);
begin
//  Read Only don't allow sets from Property value.  FListName := Value;
end;

//Testing function. future.
function TmagImageList.GetAnImage(Iobj: TimageData; direction: integer): TImageData;
var I : integer;
begin
result := Iobj;
case direction of
  -2 : result := Objlist[Objlist.count-1];
  0: result := Objlist[0];
 else
   begin
   for i := 0  to Objlist.Count -1 do
     begin
       if TImageData(Objlist.Items[i]).mag0 = Iobj.mag0 then
         case  direction of
          -1 :
            begin
            if i = 0 then result := nil  //Objlist[i];
            else result := objlist[i-1];
            end;
          +1:
            begin
             if i = objlist.count-1 then result := nil // objlist[objlist.count-1];
             else result := objlist[i+1];
            end;
          end;
     end;
   end;
end;

end;

end.

