unit cMagListView;
   {
   Package: MAG - VistA Imaging
 WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
 Date Created:
 Site Name: Silver Spring, OIFO
 Developers: Garrett Kirin
[==  unit cMagListView;
 Description: Imaging List View component.
 - Adds Imaging Specific methods to the Delphi ListView component with the
 procedure LoadListFromMagImageList(mil: TMagImageList);
 - Links each list item to the TImageData object in the associated TmagImageList component.
 - Adds function to fit all columns into the visible are of the component.
 - Adds column sorting and selecting utilities as built in features of the component.
        Creates Form TfrmColumnSelect in unit fmagColumnSelect to display a list of columns
        and allow user to check/uncheck the visible status
 - Sort codes can be defined in the Column headers
    S1,S2 are sort codes S1 : date/time, S2 : number.
 - Column width defaults can be defined in the Column Header
    W0 : default width of 0 (hidden)
==]
 Note:
  }
 (*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, umagutils, uMagClasses{, cMagImageList};
type
{ an object of TmagListViewData is created for each item in the list view.
  it holds pointers to the actual data in the TMagImageList object
  The Data property of each list item points to a object of this type.}
  TMagListViewData = class(Tobject)
  public
    data: string;          // index of this item in the TMagImageList object
    index: integer;        // index of this item in the TMagImageList object
    Iobj: TImageData;      // pointer to the TImageData object of this item
  end;

type
  TMagListView = class(TListView)
  private
    FColumnWidthDefaults: string;
    Fcurcolumnindex: integer;
    Fcolumnsorts: string;
    Finverse: boolean;
    { Column Captions to display in List View control. '^' delimited string.}
    procedure LoadColumns(s: string);
    { Integers are strings justified to sort correctly}
    procedure SortJustifystring(var lic: string);
    { Date time values converted to string equivalent to sort correctly}
    procedure SortDateTime(var lic: string);
    {  Specific to VistARad : Radiology Day/Case number}
    procedure SortDayCase(var lic: string);
    {  Create a new instance of object, to store additional data for list item}
    function NewListViewData(data: string): TMagListViewData;
    {  called internally to do the sorting of 2 items}
    procedure DoCompare(Sender: TObject; Item1, Item2: TListItem; Data: Integer; var Compare: Integer);
    {  uses the user preferences string of column widths  FColumnWidthDefaults: string;
       resizes the columns according to the '^' delimited string of user prefs.}
    procedure ApplyColumnWidthDefaults;

  protected
    procedure ColClick(Column: TListColumn); override;
    property OnCompare;

  public
    FColWidth: array of integer;
    FColVisible: array of boolean;
    constructor Create(Aowner: TComponent); override;
    destructor Destroy; override;
    {  called with the User preferences of '^' delimited column widths.}
    procedure SetColumnWidths(colwidths: string);
    {  returns '^' delimitd string of column widths.  Then saved as user pref}
    function GetColumnWidths: string; //comma delimited string of col widths
    {  Loads columns and data from Imaging object TmagImageList}
    procedure LoadListFromMagImageList(baselist: Tstrings; objlist : Tlist); //t46 (mil: TMagImageList);
    {  //prototype, not used.  Adds one list to another}
    procedure AddListToList(t: tstrings);
    {  utility to resize all columns so that they are all visible }
    procedure FitColumnsToForm;
    {  utility to resize columns to the longest value of a column entry.}
    procedure FitColumnsToText;
    {  Opens the Utility Dialog window TfrmColumnSelect.  Use can select columns
       to be visible/invisible  (invisible columns have Width = 0 }
    procedure SelectColumns;
    {  Load the control from any TStrings object. Each item is a '^' delimited
       string of column data  }
    procedure LoadListFromStrings(t: tstrings);
    {  //prototype.  Highlight the List item that is associated with IObj.
         used to synchronize with selected images in TMag4Viewer}
    procedure SyncWithImage(Iobj: TimageData);
    procedure ClearItems;
  published
   // change the defaults of certain properties.
    property ViewStyle default vsreport;
    property ReadOnly default true;
    property RowSelect default true;
    property HideSelection default false;
  end;

procedure Register;

implementation
uses fmagColumnSelect;

constructor TMagListView.Create(Aowner: TComponent);
begin
  inherited;
  OnCompare := DoCompare;
  ViewStyle := vsReport;
  ReadOnly := True; //
  RowSelect := true;
  HideSelection := false;
end;

destructor TMagListView.Destroy;
begin
  inherited;
{ TODO : This was causing error ,like the other LVUtils }
  (*try
  //clearitems;
  except
    on e: exception do
        //
  end; *)
end;

procedure Register;
begin
  RegisterComponents('Imaging', [TMagListView]);
end;

procedure TMagListView.FitColumnsToText;
var I: integer;
  li: Tlistitem;
begin
  li := nil;
  li := getnextitem(li, sdall, [isNone]);
  while li <> nil do
    begin
      if (FColWidth[0] < canvas.textwidth(li.Caption)) then FColWidth[0] := canvas.textwidth(li.caption);
      for I := 1 to columns.count - 1 do
        if (FColWidth[i] < canvas.textwidth(li.subitems[i - 1])) then FColWidth[i] := canvas.textwidth(li.subitems[i - 1]);
      li := getnextitem(li, sdall, [isnone]);
    end;
  for I := 0 to columns.count - 1 do
    begin
      if FColVisible[i] then columns[i].width := FColWidth[i] + 15;
    end;
  update;

end;

procedure TMagListView.FitColumnsToForm;
var I, viscolct, colwidth: integer;
begin
  viscolct := 0;
  //get count of visible columns
  for I := 0 to columns.count - 1 do if (columns[i].width > 0) then viscolct := viscolct + 1;
  colwidth := (width - 15) div viscolct;
  for I := 0 to columns.count - 1 do
    begin
      if FColVisible[i] then columns[i].width := colwidth;
    end;
  update;
end;


procedure TMagListView.SetColumnWidths(colwidths: string);
begin
  FcolumnWidthDefaults := colwidths;
  ApplyColumnWidthDefaults;
end;

procedure TMagListView.ApplyColumnWidthDefaults;
var i: Integer;
w : string;
begin
  try
    if (FcolumnWidthDefaults = '') or (maglength(FcolumnWidthDefaults, ',') = 0)
      then exit;
    for i := 1 to maglength(FcolumnWidthDefaults, ',') do
      if columns.count < i
        then exit
      else
        begin
         w := magpiece(FcolumnWidthDefaults, ',', i);
         if (w = '') then w := '0';
         FColVisible[i-1] := (w <> '0');
         columns[i - 1].Width := strtoint(w);
        end;
  except
//
  end;
end;

{Returns a comma delimited string of col widths}
function TMagListView.GetColumnWidths: string;
var I: integer;
begin
  result := '';
  for I := 0 to columns.count - 1 do
    result := result + inttostr(columns[i].Width) + ',';
  result := copy(result, 1, length(result) - 1);
end;

procedure TMagListView.AddListToList(t: tstrings);
// Not used.  still in development  01/18/2002
var i, j, j2: integer;
  Li: Tlistitem;
  columndata: string;
begin
  try
    j2 := maglength(magpiece(t[0], '|', 1), '^');
    for I := 0 to t.count - 1 do
      begin
        columndata := magpiece(t[i], '|', 1);
        Li := Items.Add;
        Li.Caption := magpiece(columndata, '^', 1);
      // below is where we were adding extra subitems to existing LI
        for J := 2 to j2 do
          begin
            Li.SubItems.Add(magpiece(columndata, '^', j));
          end;
        li.data := NewListViewData(magpiece(t[i], '|', 2));
      //5-23-02 LI.ImageIndex := -1;
      end;
  finally
  end;

end;

procedure TMagListView.ClearItems;
var i: integer;

begin
{DONE: WE MAY need to loop through Items and Kill any data structures pointed
    to, If we use the Data property to point to information.}
  visible := false;
  try
    for i := items.count - 1 downto 0 do
      begin
    // testing memory usage without this line, shows that
    //  this is freeing memory.
        TMagListViewData(items[i].data).free;
      end;
  //li := nil;
  //GetNextItem(li,sddown,isall);
    items.Clear;
  // ClearColumns;
  finally
    visible := true;
  end;
end;

procedure TMagListView.ColClick(Column: TListColumn);
var I: integer;
begin
  inherited;
  for i := 0 to Columns.Count - 1 do
    begin
      columns[i].AutoSize := false;
    end;
  FInverse := not FInverse;
  Fcurcolumnindex := column.index;
  //5-23-02 if FInverse then column.ImageIndex := 0
  //5-23-02  else column.ImageIndex := 1;
  customsort(nil, 0);
end;

procedure TMagListView.DoCompare(Sender: TObject; Item1, Item2: TListItem;
  Data: Integer; var Compare: Integer);
var
  lic1, lic2: string;
  sortcode: string;
begin
  inherited;
  if Fcurcolumnindex = 0 then
    begin
      lic1 := item1.caption;
      lic2 := item2.caption;
    end
  else
    begin
      lic1 := item1.SubItems[Fcurcolumnindex - 1];
      lic2 := item2.SubItems[Fcurcolumnindex - 1];
    end;
 //S1,S2 are sort codes S1 : date/time, S2 : number.  W* are default column widths W0 : default width of 0 (hidden)
  sortcode := magpiece(magpiece(fcolumnsorts, '^', Fcurcolumnindex + 1), '~', 2);
  if sortcode <> '' then
    begin
      if uppercase(sortcode) = 'S1' then begin
          SortDateTime(lic1);
          SortDateTime(lic2);
        end;
      if uppercase(sortcode) = 'S2' then begin
          SortJustifystring(lic1);
          SortJustifystring(lic2);
        end;
      if uppercase(sortcode) = 'S3' then begin
          SortDayCase(lic1);
          SortDayCase(lic2);
        end;
    end;

  { this is out now, John passes a sort code in the column header
  Case Fcurcolumnindex of
       0: begin                // ~S3
          SortDayCase(lic1);
          SortDayCase(lic2);
          end;
       6: begin                     // ~S1  (date time )
          SortDateTime(lic1);
          SortDateTime(lic2);
          end;
       8: begin                      // ~S2  ( Number )
          SortJustifystring(lic1);
          SortJustifystring(lic2);
          end;
       end;
      }
  { Try 08/07/01  CompareText Function instead of this
  if FInverse then COMPARE := -lstrcmp(PChar(LIC1), PChar(LIC2))
             else COMPARE := lstrcmp(PChar(LIC1), PChar(LIC2));
  }

  if FInverse then COMPARE := -CompareText(LIC1, LIC2)
  else COMPARE := CompareText(LIC1, LIC2);


end;

procedure TMagListView.LoadColumns(s: string);
var i, j: integer;
  Column: TListColumn;
  FSameColumns: boolean;
begin
  FSameColumns := true;
  j := maglength(s, '^');
  //Only clearing columns if the number of columns changed.
  // otherwise we assume the columns are the same
  {TODO: review this clearing of columns logic.}
  // we did delete all columns, and make new ones because we never know
  // what the columns will be, they could change with each Exam Listing.
  if columns.count <> j then FSameColumns := false
  else
    for I := 1 to j do
      begin
        if (Columns[i - 1].caption <> magpiece(magpiece(s, '^', i), '~', 1)) then FSameColumns := false;
      end;
  if FSameColumns then exit;
  columns.clear;
  for I := 1 to j do
    begin
      Column := columns.add;
     Column.AutoSize := false;
    // testing to show the Sort Code in the Caption
    (*if showrawheaders
       then Column.caption := magpiece(s,'^',i)
       else Column.caption := magpiece(magpiece(s,'^',i),'~',1);
      *)
      Column.caption := magpiece(magpiece(s, '^', i), '~', 1);
      // Try to default some columns to 0 width.
      if magpiece(magpiece(s, '^', i), '~', 3) = 'W0'
        then column.Width := 0   {TODO: this isn't working,  Column width goes to 96 ?}
        else Column.width := 100; //8/08/01 columnheaderwidth ;
    end;
  // set length of arrays
  setlength(FColWidth, columns.count);
  setlength(FColVisible, columns.count);
  // set minimum width equal to column header
  for i := 0 to columns.count - 1 do
    begin
      FColWidth[i] := canvas.textwidth(columns[i].Caption);
      columns[i].width := FColWidth[i] + 15;
    // initially all are visible
      FColVisible[i] := true;
    end;
// we will only get here if new columns and headers are created.
  ApplyColumnWidthDefaults;
end;

{ any Tstrings can be put into this listview control
  This method expects the first item (t[0]) are the column headers delimited
  by the '^', and all other items are '^' delimited data. }
procedure TMagListView.LoadListFromStrings(t: tstrings);
var i, j, j2: integer;
  Li: Tlistitem;
  columndata: string;
  curviewstyle: Tviewstyle;
begin
  visible := false;
  ClearItems;
  curviewstyle := viewstyle;
  viewstyle := vslist;
  try
    LoadColumns(t[0]);
    Fcolumnsorts := t[0];
    AllocBy := t.count + 3;
    j2 := maglength(magpiece(t[0], '|', 1), '^');
    for I := 1 to t.count - 1 do
      begin
        columndata := magpiece(t[i], '|', 1);
        Li := Items.Add;
        Li.Caption := magpiece(columndata, '^', 1);
      // below is where we were adding extra subitems to existing LI
        for J := 2 to j2 do
          begin
            Li.SubItems.Add(magpiece(columndata, '^', j));
          end;
        li.data := NewListViewData(magpiece(t[i], '|', 2));
      end;
  finally
    if ViewStyle <> curviewstyle then viewstyle := curviewstyle;
    visible := true;
  end;
end;

function TMagListView.NewListViewData(data: string): TMagListViewData;
begin
  result := TMagListViewData.create;
  result.data := data;
end;

{Specific to TmagImageList objects.  This will load the list}
procedure TMagListView.LoadListFromMagImageList(baselist: Tstrings; objlist : Tlist); //t46 (mil: TMagImageList);
var i, j, j2: integer;
  Li: Tlistitem;
  columndata: string;
  curviewstyle: Tviewstyle;
  maglistviewdata1: Tmaglistviewdata;
begin
  visible := false;
  Items.BeginUpdate;
  try
    ClearItems;
    curviewstyle := viewstyle;
    viewstyle := vslist;
    try
      //t46 From here to end, we replaced all mil.baselist to baselist
      //  and mil.objlist to objlist
      if baseList.Count = 0 then exit;
      LoadColumns(baseList[0]);
      Fcolumnsorts := baseList[0];

      AllocBy := baselist.count + 3;
      j2 := maglength(magpiece(baselist[0], '|', 1), '^');
      for I := 1 to baselist.count - 1 do
        begin
          columndata := magpiece(baselist[i], '|', 1);
          Li := Items.Add;
          Li.Caption := magpiece(columndata, '^', 1);

        // below is where we were adding extra subitems to existing LI
          for J := 2 to j2 do
            begin
              Li.SubItems.Add(magpiece(columndata, '^', j));
            end;
          { each item in the list view holds the internal data needed by
            methods of the control and other interested parties. }
          maglistviewdata1 := TMaglistviewdata.create;
          maglistviewdata1.data := magpiece(baselist[i], '|', 2);
          maglistviewdata1.index := strtoint(maglistviewdata1.data);
          maglistviewdata1.Iobj := objlist[maglistviewdata1.index];
          li.data := maglistviewdata1;
        end;
    finally
      if ViewStyle <> curviewstyle then viewstyle := curviewstyle;
      visible := true;
    end;
  finally
    Items.EndUpdate;
  end;

end;


procedure TMagListView.SortDateTime(var lic: string);
begin
  (* This was expected format in Johns; lists.
    We need to make it more generic. i.e. handle any date format
   //  02/05/1996@07:08
   lic := copy(lic,7,4)+copy(lic,1,2)+copy(lic,4,2)
             +copy(lic,12,2)+copy(lic,15,2)+copy(lic,18,2);
   *)
  try
    lic := formatdatetime('yyyymmddhhmmnn', strtodatetime(lic));
  except
    lic := lic;
  end;
end;

procedure TMagListView.SortDayCase(var lic: string);
var yr, mthdy, xcase: string;
  int1: integer;
begin
  int1 := strtoint(copy(lic, 5, 2));
  if (int1 < 5) then int1 := 2000 + int1
  else int1 := 1900 + int1;
  yr := inttostr(int1);
  mthdy := copy(lic, 1, 4);
  xcase := magpiece(lic, '-', 2);
  lic := yr + mthdy + xcase;
end;

procedure TMagListView.SortJustifystring(var lic: string);
begin
  lic := copy('0000000000', 1, 10 - length(lic)) + lic;
end;


procedure TMagListView.SelectColumns;
begin
// { TODO : Change this method to Execute(self) }
  with TfrmColumnSelect.create(self) do
    begin
      DisplayColumns(self);
      showmodal;
      Release;
    end;
end;

// prototype, highlite the list entry when the Abstract is selected
// in the Abstract window.
procedure TMagListView.SyncWithImage(Iobj: TimageData);
  var  li: Tlistitem;
begin
  li := nil;
  li := getnextitem(li, sdall, [isNone]);
  while li <> nil do
    begin
      if TMagListViewData(li.data).Iobj.Mag0 = Iobj.Mag0
        then
        begin
          if not li.Selected then li.Selected := true;
          li.MakeVisible(false);
          break;
        end;
      li := getnextitem(li, sdall, [isnone])
    end;

end;


end.

