unit cMagPat;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created:
   Site Name: Silver Spring, OIFO
   Developers: Garrett Kirin
   [==  unit cMagPat;
   Description:  Imaging Patient Class
     Implements the IMagSubject Interface that allows the component to operate oblivious of what
     Type of Object is attached to it.  It simply notifies IMagObservers.
     Has all functions for Patient Lookup, Sensitivity checks, and logging.

   In Patch 8, we consolidated all Patient functions and procedures into the
    TMag4Pat Class.
    An Object of type TMag4Pat has the following functionality:
    - The Patient Lookup window is created and opened.
      (only Tmag4pat uses, or knows about,  the Patient lookup window. Application
       no longer needs to know the details of Patient Lookup window. Maggplku.dfm)
    - Patient information is querried from the database for selected patient.
    - The RPC's for Patient Sensitivity are called.
    - Patient Sensitivity warnings are displayed.
    - Sensitive Patient Logging is done when needed.
    - Means Test warnings are displayed if appropriate.
    - When the Patient changes,  All Attached Observers are Notified
      by calling the Update_ method of each Observer.
        ==]
   Note:
   }
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)

interface
{ TODO : search for //oo Caption   This is probable place to Notify Observers }
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, uMagUtils, IMagInterfaces, cMagDBBroker, Maggplku;

type
  TMag4Pat = class(TComponent, IMagSubject)
  private
    FDBBroker: TMagDBBroker;
    FObserversList: array of IMagObserver;
    FPatName: string;
    FSSN: string;
    FDFN: string;
    FICN: string;
    FSite: string;
    FProdAcct: boolean;
    FDOB: string;
    FTotalImageCount: string;
    FDemog: string;
    FCachePhotoID: string;
    FSSNDisplay: string;
    FDOBDisplay: string;
    FNameDisplay: string;
    FFakePatientName: string;
    FServiceConnected: string;
    FType: string;
    FVeteran: string;
    //FRestricted   : boolean;
    //FEmployee     : boolean;
    //FMeansTestReq : boolean;
    FUseFakeName: boolean;
    FLastResult: string;
     //procedure UseFakeName(value :boolean);
    procedure SetFakePatientName(value: string);
    procedure SetUseFakeName(value: boolean);
    function GetPatientInformation(xDFN: string; var resultmsg: string; isicn : boolean = false): boolean;
    procedure SetPatientInfo(str: string);
   protected
    { Protected declarations }
   public
    constructor create(AOwner: TComponent); override;
    destructor destroy; override;

    {   object that implements the ImagObserver interface calls this method
        to attach itself to the ImagSubject.}
    procedure Attach_(Observer: ImagObserver);

    {   Observers can Detach and Attach any time they want}
    procedure Detach_(Observer: ImagObserver);

    {   TMag4Pat objects will call the Update_ method of all observers from
        this Notify_ method.  (The Notify_ method is public on purpose.  Other
        objects, components can call Notify_ to force an update of all observers.}
    procedure Notify_(SubjectState: string);

    {GetState and SetState aren't really used.  State is sent as parameter to
     observers. when patient changes,  (state = 1 )
                when patient is cleared  (state = '')
                or Mag4Pat object is being destroyed (state = -1 )
     The way this is designed, the Observer has to 'Know' the structure of the
     subject to get some information.  Observers query the published properties
     M_PatName, M_DFN etc.
     If we used a more detailed state object, the objserver wouldn't need to
     know the properties of cMag4Pat. Both ways are acceptable, used in different
     situations.

    We don't send the DFN as the value of State when a patient changes. We just send
    a '1'.  The Observer then has to Query the Subject for the information.}
    {GetState_ will returmn DFN as the var State : string}
    procedure GetState_(var State: string);

    {SetState_ assumes the State : string is the DFN, and uses it at such}
    procedure SetState_(State: string);
    procedure Clear;

    {   Copy one Tmag4Pat object to another.}
    procedure Assign(Source: TPersistent); override;

    procedure SetDemoPatName(value: string);

    {   overloaded to support older classes that don't send 'Var xmsg: string'
        SelectPatient will open the patient selection window with all patients
        that match the user input.
        If only one patient matches user input. SelectPatient window is not
        displayed.}
    function SelectPatient(input: string; var xmsg: string): boolean; overload;
    function SelectPatient(input: string = ''): boolean; overload;

    {   Called in 3 situations.
         - User has selected a Patient from the Patient History menu items.
         - CPRS has sent a Windows message, saying change patient
           (CCOW Change of patient context in the future
        - TMag4Pat object has validated a new patient selection, calls this method itself}
     function SwitchToPatient(dfn: string; var xmsg: string; doclear: boolean = true; isicn: boolean = false): boolean;
  published

    { Most of these properties are Read Only.  Ther aren't and 'write' operations
        on the property}
    {   This is needed to make any calls to the DataBase.}
    property M_DBBroker: TMagDBBroker read FDBbroker write FDBBroker;

    {   Full Name of Patient.}
    property M_PatName: string read FPatName;

    {   Total number of Images on file for this patient.}
    property M_TotalImageCount: string read FTotalImageCount;

    {   SSN of patient.  in  9N format.}
    property M_SSN: string read FSSN;

    {   DFN: internal entry number Patient in the PATIENT File.}
    property M_DFN: string read FDFN;

     {  ICN number of patient computed by : $$GETICN^MPIF001}
     property M_ICN: string read FICN;
     {  LOCAL STATION NUMBER. $p(3,"^") of $$SITE^VASITE}
     property M_Site: string read FSite;
     {  Is this a Production account :  by $$PROD^XUPROD}
     property M_ProdAcct: boolean read FProdAcct;


    {   Date of Birth of Patient.   Internal FileMan format.}
    property M_DOB: string read FDOB;

    {   Patient demographics.}
    property M_Demog: string read FDemog;

    //property M_CachePhotoID : String read FCachePhotoID; //write FCachePhotoID;

    {   Last result of a Patient Lookup.}
    property M_LastResult: string read FlastResult;

    {   SSN in the format 3N - 2N - 4N.  If using fake name it is '000-00-0000'}
    property M_SSNdisplay: string read FSSNDisplay;

    {   Date of Birth in readable format.}
    property M_DOBdisplay: string read FDOBDisplay;

    {   Normally the same as M_PatName property.  When Using a fake name for
         demo purposes.  This is changed to 'LightYear, Buzz'}
    property M_NameDisplay: string read FNameDisplay;

    //property M_Restricted : boolean read FRestricted;
    //property M_Employee    : boolean read FEmployee;
    //property M_MeansTestReq : boolean read FMeansTestReq;

    {   Set to 'LightYear, Buzz' by the system.}
    property M_FakePatientName: string read FFakePatientName write SetFakePatientName;

    {   If TRUE, the real patient name will be hidden, and fake patient name is used.}
    property M_UseFakeName: boolean read FUseFakeName write SetUseFakeName;

    {   Is patient service connected or not   Field # ___ from PATIENT File.}
    property M_ServiceConnected : string read FServiceConnected;

    {   Type of Patient.    Field # ___ from PATIENT File.}
    property M_Type: string read FType;

    {   Is patient a veteran or not   Field # ___ from PATIENT File.}
    property M_Veteran : string read FVeteran;
  end;

procedure Register;

implementation
uses uMagClasses;


{str is the result of the RPC Broker call to get patient information.
  here we set values for the Patient Properties.}
procedure TMag4pat.SetPatientInfo(str: string);
begin
    (* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     ANY CHANGE TO THIS FUNCTION NEEDS COORESPONGING CHANGE IN
                FUNCTION     ASSIGN
                        and
               FUNCTION      CLEAR
     !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*)
  //                 1        2      3     4     5     6     7     8        9                     10
  //format str:   status ^   DFN ^ name ^ sex ^ DOB ^ SSN ^ S/C ^ TYPE ^ Veteran(y/n)  ^ Patient Image Count
  //DPT field :                    .01   .02   .03   .09   .301   391    1901

  FDFN := magpiece(str, '^', 2);
  FPatName := magpiece(str, '^', 3);
  FDOB := magpiece(str, '^', 5);
  FSSN := magpiece(str, '^', 6);
  FServiceConnected:=magpiece(str, '^', 7);
  FType:=magpiece(str, '^', 8);
  FVeteran:=magpiece(str, '^', 9);
  FTotalImageCount := magpiece(str, '^', 10);
  FICN := magpiece(str,'^',11);
  FSite := magpiece(str,'^',12);
  FProdAcct := (magpiece(str,'^',13) = '1');
  FCachePhotoID := '';
  FSSNDisplay := copy(FSSN, 1, 3) + '-' + copy(FSSN, 4, 2) + '-' + copy(FSSN, 6, 4);
  FDOBDisplay := FDOB;
  FNameDisplay := FPatName;

  //FRestricted   := false;
  //FEmployee     := false;
  //FMeansTestReq := false ;

  FDemog := FDOBDisplay + '   ' + FSSNDisplay + '   ' + FServiceConnected + '   ' + FType;

  SetUseFakeName(FuseFakeName);
end;

{Used to assign the values of one cMag4Pat object into another cMag4Pat object.}
procedure TMag4Pat.Assign(Source: TPersistent);
begin
    (* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     ANY CHANGE TO THIS FUNCTION NEEDS COORESPONGING CHANGE IN
                FUNCTION     SetPatientInfo
     AND
                FUNCTION      CLEAR
     !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*)
  Clear;
  FTotalImageCount := (source as TMag4Pat).M_TotalImageCount;
  FDFN := (source as TMag4Pat).M_DFN;
  FICN := (source as TMag4Pat).M_ICN;
  FSite := (source as TMag4Pat).M_Site;
  FProdAcct := (source as TMag4Pat).M_ProdAcct;
  FPatName := (source as TMag4Pat).M_PatName;
  FDOB := (source as TMag4Pat).M_DOB;
  FSSN := (source as TMag4Pat).M_SSN;
  FServiceConnected := (source as TMag4Pat).M_ServiceConnected;
  FType := (source as TMag4Pat).M_Type ;
  FVeteran := (source as TMag4Pat).M_Veteran  ;
  FTotalImageCount := (source as TMag4Pat).M_TotalImageCount  ;
  //FCachePhotoID    := (source as TMag4Pat).M_CachePhotoID;
  FcachePhotoID := '';
  FSSNDisplay := (source as TMag4Pat).M_SSNDisplay;
  FDOBDisplay := (source as TMag4Pat).M_DOBDisplay;
  FNameDisplay := (source as TMag4Pat).M_NameDisplay;

  //FRestricted   := (source as TMag4Pat).M_Restricted;
  //FEmployee     := (source as TMag4Pat).M_Employee;
  //FMeansTestReq := (source as TMag4Pat).M_MeansTestReq;

  FDemog := (source as TMag4Pat).M_Demog;

  FUseFakeName := (source as TMag4Pat).M_UseFakeName;
  FFakePatientName := (source as TMag4Pat).M_FakePatientName;

  Notify_(FDFN);
end;


{Clear the patient object,  Notify all observers of a change of State.}
procedure TMag4Pat.Clear;
begin
    (* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     ANY CHANGE TO THIS FUNCTION NEEDS COORESPONGING CHANGE IN
                FUNCTION:     ASSIGN
                        AND
                FUNCTION:      SetPatientInfo
     !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!*)
  FDFN := '';
  FICN := '';
  FSite := '';
  FProdAcct := false;
  FPatName :=  '';
  FDOB :=  '';
  FSSN :=  '';
  FServiceConnected := '';
  FType := '';;
  FVeteran := '';
  FTotalImageCount := '';
  FCachePhotoID := '';
  FSSNDisplay := '';
  FDOBDisplay := '';
  FNameDisplay := '';
  FDemog := '';
        {       Notify all observers.  Send a clear patient event.}
  Notify_('');
end;

{When we set UseFakeName to TRUE, we change the FNameDisplay to use the FakePatient Name value
 and use fake values for the SSN,Date of Birth, Service Connected and Type of patient.}
procedure TMag4Pat.SetUseFakeName(value: boolean);
begin
  FUseFakeName := value;
  if value
    then
      begin
        FNameDisplay := FFakePatientName;
        FDemog := '01/01/1910' + '   ' + '000-00-0000' + '   ' + 'Yes' + '   ' + 'Hero';
      end
    else
      begin
       FNameDisplay := FPatName;
       FDemog := FDOBDisplay + '   ' + FSSNDisplay + '   ' + FServiceConnected + '   ' + FType;
       end;
end;

procedure TMag4Pat.SetFakePatientName(value: string);
begin
  FFakePatientName := value;
end;
{We also Create the Patient Lookup window when this object is created.}
constructor TMag4Pat.create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  maggplkf := TMaggplkf.create(nil);
  maggplkf.FDBBroker := M_DBBroker;
  self.M_FakePatientName := 'LightYear,Buzz';
end;

{Have to Noify_ the Observers, so they can set their pointers (pointer to the
 cMag4Pat object to nil}
destructor TMag4Pat.destroy;
begin
  maggplkf.Release;
  Notify_('-1');
  inherited;
end;


procedure TMag4Pat.SetDemoPatName(value: string);
begin
  FPatName := value;
  if not FUseFakeName then FNameDisplay := value;
end;

procedure Register;
begin
  RegisterComponents('Imaging', [TMag4Pat]);
end;

{Observers call this method.  They are then Notified of any change of State.
     Change of Patient  (State = 1 )
 or  Clear Patient (state = '')
 or  TMag4Pat object is being destroyed.  (state = -1) }
procedure TMag4Pat.Attach_(Observer: ImagObserver);
begin
  Setlength(FObserverslist, length(fobserverslist) + 1);
  FObserverslist[high(fObserverslist)] := observer;
end;

{An observer can detach from the Subject at any time.}
procedure TMag4Pat.Detach_(Observer: ImagObserver);
var i: integer;
begin
  for i := 0 to (high(Fobserverslist)) do
    begin
      if (Observer = FObserversList[i]) then
        begin
          FObserverslist[i] := nil;
        end;
    end;
end;

procedure TMag4Pat.GetState_(var State: string);
begin
  state := FDFN;
end;

procedure TMag4Pat.Notify_(SubjectState: string);
var i: integer;
  ivObserver: IMagObserver;
begin
  for i := 0 to (high(Fobserverslist)) do
    begin
      if FObserversList[i] <> nil then
        begin
          IvObserver := (FObserversList[i]);
          Ivobserver.Update_(SubjectState);
        end;
    end;
end;

procedure TMag4Pat.SetState_(State: string);
begin
  FDFN := state;
  { TODO : Look into this, could be Notifying Twice.}
  Notify_(state);
end;


function TMag4Pat.SelectPatient(input: string = ''): boolean;
var xmsg: string;
begin
  result := SelectPatient(input, xmsg);
  FLastResult := xmsg; //
end;

function TMag4Pat.SelectPatient(input: string; var xmsg: string): boolean;
var xdfn, vServer, vPort: string;
begin
        { If not logged in, we exit;}
  try
    result := false;
    if not assigned(FDBBroker) then
      begin
        xmsg := 'Error : DB Broker UnAssigned in TMag4Pat.';
        exit;
      end;
    if not FDBBroker.IsConnected then
      if not FDBBroker.DBSelect(vServer,vPort, 'MAG WINDOWS') then
        begin
          xmsg := 'No Connection to VISTA.';
          exit;
        end;
    {  To assure Patient lookup form, is connected to same database as Patient Component }
    maggplkf.FDBBroker := M_DBBroker;
    if not MaggPlkf.Select(input, xmsg, xDFN) then
      begin
      { TODO: -oGarrett :
        need a message log component. All components have a property of this
        component, and if assigned send messages to it.
        It tracks in VistA, makes Log File if needed. }//maggmsgf.MagMsg('', xmsg, pmsg);
        exit;
      end;
        {       A new patient has been selected, Clear current and switch to new.}
    Clear;
    result := SwitchToPatient(xdfn, xmsg, false);
  finally
        {       store the last result}
    FLastResult := xmsg;
  end;
end;

{Called in 3 situations.
 - User has selected a Patient from the Patient History menu items.
 - CPRS has sent a Windows message, saying change patient
   (CCOW Change of patient context in the future
 - TMag4Pat object has validated a new patient selection, calls this method itself}
function TMag4Pat.SwitchToPatient(dfn: string; var xmsg: string; doclear: boolean = true; isicn: boolean = false): boolean;
begin
        {       doClear is set to false, if this function is called internally, and patient
                has already been cleared.  }
  if doclear then clear;
  application.processmessages;
        {       If GetPatientInformation is successful, Notify all Observers.}
  result := GetPatientInformation(DFN, xmsg, isicn);
  if result then Notify_(dfn);
end;

function TMag4Pat.GetPatientInformation(xDFN: string; var resultmsg: string; isicn : boolean = false): boolean;
begin
  FDBBroker.RPMagPatInfo(result, resultMsg, xdfn,isicn);
  if result then SetPatientInfo(resultMsg);
end;


end.

