unit cMagPatPhoto;
  {
   Package: MAG - VistA Imaging
 WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
 Date Created:
 Site Name: Silver Spring, OIFO
 Developers: Garrett Kirin
 [==  unit cMagPatPhoto;
 Description:  Image Patient Photo Component.
 Inherits from TFrame.  We use TFrame as the ancestor
 for some imaging components when the component being developed needs multiple visual
 controls.  All the needed controls can be placed on the Frame and function as
 one component.

 TMagPatPhoto implements the ImagObserver interface with Tmag4Pat as the subject.
 When the patient changes, Tmag4pat Notifies TMagPatPhoto, and TMagPatPhoto updates
 the Patient Photo.
 TMagPatPhoto also has properties:
  MagDBBroker :  TMagDBBroker
        The Object can make RPC Call to get the Patient Photo from the Image File.
  MagSecurity :  TMag4Security
        The Object can open secure connections to the Imaging Servers.

  memPhoto : Tmemo
        The component can also display information beside the Patient's Photo.

  Display Styles - called from PopupMenu (Popup Menu items.  Not implemented.)
        Photo - Name : also Shows patient name
        Photo - Info : also shows the memPhoto control for informational display
        Photo Only   : Patient Photograph is all that is displayed. (Default Style)

        Photo on Top : Default.  If memPhoto is visible this property is querried.
        Photo to Left:  If memPhoto is visible this property is querried.


        ==]


 Note:
  }
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, AxCtrls, OleCtrls, GEARLib_TLB,Geardef,
  ImagInterfaces, cmagdbbroker, cMagPat, Menus,
  cmagutils, cMagSecurity , umagClasses, umagdefinitions,
  cMagVutils, fmxutils;

type
  TMagPatPhoto = class(TFrame, IMagObserver)
    {TODO: Change TGear to TMag4VGear}
    gearPat: TGear;
    memPhoto: TMemo;
    Splitter1: TSplitter;
    PopupMenu1: TPopupMenu;
    PhotoName1: TMenuItem;
    PhotoInfo1: TMenuItem;
    PhotoOnly1: TMenuItem;
    N1: TMenuItem;
    PhotoonTop1: TMenuItem;
    PhototoLeft1: TMenuItem;
    {Popup Menu items.  Not implemented.}
    procedure PhotoName1Click(Sender: TObject);
    procedure PhotoInfo1Click(Sender: TObject);
    procedure PhotoonTop1Click(Sender: TObject);
    procedure PhototoLeft1Click(Sender: TObject);
    procedure PhotoOnly1Click(Sender: TObject);
  private
    FDFN: string; { Private declarations }
    FMagPat: TMag4Pat;
    FMagDBBroker: TMagDBBroker;
    Fmut: TmagUtils;
    FMagSecurity: TMag4Security;
    FHideWhenNull: Boolean;
    procedure AttachMyself();
    procedure  InitGear();
    procedure SetMagPat(const Value: TMag4Pat);
    procedure ClearPat;
  public
    constructor Create(AOwner: TComponent); override;
    destructor destroy; override;
        {       This class implements the ImagObserver interface}
    procedure Update_(SubjectState: string); { Public declarations }
  published
        {       MagHideWhenNull not implemented.}
    property MagHideWhenNull: boolean read FHideWhenNull write FHideWhenNull;
        {       Reference to the Patient Object.}
    property MagPat: TMag4Pat read FMagPat write SetMagPat;
        {       Reference to the DataBase broker component.}
    property MagDBBroker: TMagDBBroker read FMagDBBroker write FMagDBBroker;
        {       Imaging security component.  This Class opens images.}
    property MagSecurity: TMag4Security read FMagSecurity write FMagSecurity;
  end;


procedure Register;

implementation

{$R *.DFM}

{ TFrame1 }

procedure TMagPatPhoto.AttachMyself;
begin
  if assigned(FmagPat) then
    FMagPat.Attach_(IMagObserver(self));
end;

constructor TMagPatPhoto.Create(AOwner: TComponent);
begin
  inherited;
  Fmut := TmagUtils.create(nil);
  InitGear;
end;

destructor TMagPatPhoto.destroy;
begin
  Fmut.free;
  if assigned(FmagPat) then
    begin
      FMagPat.Detach_(self);
      FMagPat := nil;
    end;
  inherited;
  //
end;

procedure TMagPatPhoto.InitGear;
begin
{ values for nFitMethod parameter of IG_display_fit_method
const IG_DISPLAY_FIT_TO_WINDOW=0  ;
const IG_DISPLAY_FIT_TO_WIDTH=1;
const IG_DISPLAY_FIT_TO_HEIGHT=2  ;
const IG_DISPLAY_FIT_1_TO_1=3;       }

{
Const IG_ASPECT_NONE = 0;
Const IG_ASPECT_DEFAULT = 1;
Const IG_ASPECT_HORIZONTAL = 2;
Const IG_ASPECT_VERTICAL = 3;
Const IG_ASPECT_MAXDIMENSION = 4;
Const IG_ASPECT_MINDIMENSION = 1;
}
  {Fit the whole photo into the control}
  GearPat.fitmethod := IG_DISPLAY_FIT_TO_WINDOW;
  GearPat.AspectRatio := IG_ASPECT_MINDIMENSION;
  GearPat.redraw := true;
end;
{  * TMagPatPhoto.SetMagPat *
  When set, this component also attaches itself as ImagObserver to TMag4Pat Subject}
procedure TMagPatPhoto.SetMagPat(const Value: TMag4Pat);
begin
  if (FmagPat <> nil) then FMagPat.Detach_(self);
  FMagPat := Value;
  if value = nil then exit;

  FDFN := FMagPat.M_DFN;
  AttachMyself();
end;


procedure TMagPatPhoto.ClearPat;
begin
  memPhoto.Lines.Clear;
  GearPat.DelImage := true;

end;
{Notified from Subject.  Update the Patient Photo.}
procedure TMagPatPhoto.Update_(SubjectState: string);
var t: Tstringlist;
  xmsg, image: string;
begin
  clearpat;
  if (SubjectState = '') then exit
  else if (SubjectState = '-1') then FMagPat := nil
  else
    begin
      {   TODO : Need to make use of MagSecurity., and change Gear1 to Mag4VGear; }
      t := Tstringlist.create;
      {    Call MagDBBroker to get patient Photo.}
      if not assigned(FMagDBBroker) then exit;
      FMagDBBroker.RPMaggGetPhotoIDs(FMagPat.M_DFN, t);
      image := Fmut.magPiece(t[0], '^', 3); // 3 is full, 4 is abstract;

   //'B2^618^\\isw-imggold\image1$\DM\00\06\DM000618.JPG^\\isw-imggold\image1$\DM\00\06\DM000618.ABS^PHOTO ID Apr 06, 2000^
   //                                     3000406.1701^18^PHOTO ID^04/06/2000^^M^A^^^1^.04^Unk^^^1023^GREEN,DEAN^ADMIN/CLIN^^^^'
      if image = '' then exit;
  {-------- Switch image path to Cache directory, if it exists.---------}
    if FileExists(Fmut.AppPath + '\cache\'+ ExtractFileName(Image) )
      then
        begin
          image := Fmut.AppPath + '\cache\'+ ExtractFileName(Image);
          Gearpat.LoadImage := image;
        end
      else
        begin
        if FMagSecurity.MagOpenSecurePath(image, xmsg) then
          begin
            try
            copyfile(image, Fmut.AppPath + '\Cache');
            image := Fmut.AppPath + '\cache\'+ ExtractFileName(Image);
            Gearpat.LoadImage := image;
            finally
            FMagSecurity.MagCloseSecurity(xmsg);
            end;
          end;
        end;

      end;
end;




procedure Register;
begin
  RegisterComponents('Imaging', [TMagPatPhoto]);
end;
{Not implememted.  If Patient Name control is visible.  User can Show/Hide the
 patient Photo if desired.}
procedure TMagPatPhoto.PhotoName1Click(Sender: TObject);
begin
  GearPat.Visible := true;
  memPhoto.visible := true;
end;

procedure TMagPatPhoto.PhotoInfo1Click(Sender: TObject);
begin
  GearPat.Visible := true;
  memPhoto.visible := true;
end;

procedure TMagPatPhoto.PhotoonTop1Click(Sender: TObject);
begin
  memPhoto.visible := true;
  GearPat.align := altop;
  GearPat.height := height div 2;
  Splitter1.align := altop;
  Splitter1.Top := GearPat.height + 10;
end;

procedure TMagPatPhoto.PhototoLeft1Click(Sender: TObject);
begin
  memPhoto.visible := true;
  GearPat.align := alleft;
  GearPat.width := width div 2;
  Splitter1.align := alleft;
  Splitter1.left := GearPat.width + 10;
end;

procedure TMagPatPhoto.PhotoOnly1Click(Sender: TObject);
begin
  memPhoto.visible := false;
end;

end.

