unit cMagUtils;
  {
   Package: MAG - VistA Imaging
 WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
 Date Created:     Version 2.0
 Site Name: Silver Spring, OIFO
 Developers: Garrett Kirin
 [==     unit cMagUtils;
 Description:     Imaging Utilities.
  Some functions and methods are used by many Forms (windows) and components of
 the application.  Utility components are designed to be a repository of such
 utility functions used throughout the application.
 Other Compontents that need the methods contained in the utililty components
 can declare a property of that type and have the utility object set at design time.

    This utility class has M functions that are recreated for Delphi use.  String
    functions like $P, $L.
    Also has INI functions to Get and Set an INI entry.
  ==]
 Note:
  }
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  inifiles, magguini;

type
  TMagUtils = class(TComponent)
  private

    { Private declarations }
  protected
    { Protected declarations }
  public

        {       Mumps $TR function: return a string that
                changes all 'frchar' to 'tochar' in string 'str'}
    function Transpose(str: string; frchar,tochar : char): string;

        {       If form is minimized, hidden, or behind other forms, use this}
    procedure FormToNormalSize(xForm: Tform);

        {       Application path.}
    function AppPath(): string;

        {       full path name to the blank.bmp file.  Used to Clear a TGear control.}
    function BlankImage(): string;

        {       Strips spaces ' ' from front and back}
    function Stripspaces(str: string): string;

        {       Mumps $P function: return the 'piece' piece delimited by 'delim'}
    function MagPiece(STR, delim: string; piece: INTEGER): string; // $Piece

        {       Mumps $P function: return the 'piece' piece delimited by 'delim'}
    function dP(STR, delim: string; piece: INTEGER): string; // $Piece

        {       Mumps $L function: return number of 'delim' delimited pieces}
    function MagLength(str, delim: string): INTEGER;  //$length

        {       Mumps $L function: return number of 'delim' delimited pieces}
    function dL(STR, delim: string): INTEGER; // $Length

        {       Gets the 'section:entry' value from INI file.
                INI file name is retrieved by the 'GetConfigFileName' method}
    function GetIniEntry(section, entry: string): string;

        {       Sets the 'section:entry' value in the INI File
                INI file name is retrieved by the 'GetConfigFileName' method}
    procedure SetIniEntry(section, entry, value: string);

  published
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Imaging', [TMagUtils]);
end;

{ TMagUtils }

procedure TMagUtils.FormToNormalSize(xForm: Tform);
begin
  if (xForm.WindowState = wsminimized) then xForm.windowstate := wsnormal;
  xform.update;
  if xform.Visible = false then xForm.visible := true;
  xform.bringtofront;
  xform.update;
  application.processmessages;
end;

function TMagUtils.GetIniEntry(section, entry: string): string;
var magini: Tinifile;
begin
  magini := Tinifile.create(GetConfigFileName);
  result := magini.readstring(section, entry, '');
  magini.free;
end;

procedure TMagUtils.SetIniEntry(section, entry, value: string);
var magini: Tinifile;
begin
  magini := Tinifile.create(GetConfigFileName);
  magini.writestring(section, entry, value);
  magini.free;
end;

Function TMagutils.Transpose(str: string; frchar,tochar : char): string;
var i : integer;
begin
result := '';
for i := 1 to maglength(str,frchar) do result := result + magpiece(str,frchar,i) + toChar  ;
result := copy(result,1,length(result)-1);
end;

function TMagUtils.AppPath(): string;
begin
  result := copy(extractfilepath(application.exename), 1, length(extractfilepath(application.exename)) - 1);
end;

function TMagUtils.BlankImage: string;
begin
  result := AppPath + '\bmp\blank.bmp';
end;

{ $Length (same as MagLength function)}
function TMagUtils.dL(str, delim: string): INTEGER;
var I, J: INTEGER;
  endStr: BOOLEAN;
begin
  endStr := FALSE;
  I := 0;
  while not endStr do
    begin
      I := I + 1;
      if (POS(delim, str) = 0) then endStr := TRUE;
      J := POS(delim, str); str := COPY(str, J + 1, LENGTH(str));
    end;
  result := I;
end;


{   $Piece (same as MagPiece function)}
function TMagUtils.dP(str, delim: string; piece: INTEGER): string;
var I, K: INTEGER;
  s: string;
begin
  I := Pos(delim, str);
  if (I = 0) and (piece = 1) then begin result := str; EXIT; end;
  for K := 1 to piece do
    begin
      I := POS(delim, str);
      if (I = 0) then I := LENGTH(str) + 1;
      S := COPY(str, 1, I - 1);
      str := COPY(str, I + 1, LENGTH(str));
    end;
  result := S;
end;

function TMagUtils.Stripspaces(str: string): string;
var done: boolean;
  s: string;
begin
  if str = '' then begin result := ''; exit; end; {fix for hang}
  done := false;
  repeat
    s := str[1];
    if (s = ' ') then str := copy(str, 2, length(str));
    if not (str[1] = ' ') then done := true;
    if str = '' then done := true; {fix for hang}
  until done;
  done := false;
  repeat
    s := str[length(str)]; if (s = ' ') then str := copy(str, 1, length(str) - 1);
    if not (str[length(str)] = ' ') then done := true;
    if str = '' then done := true; {fix for hang}
  until done;
  result := str;
end;

function TmagUtils.MagLength(str, delim: string): INTEGER;
var I, J: INTEGER;
  estr: BOOLEAN;
begin
  estr := FALSE;
  I := 0;
  while not ESTR do
  begin
    I := I + 1;
    if (POS(delim, str) = 0) then estr := TRUE;
    J := POS(delim, str); str := COPY(str, J + 1, LENGTH(str));
  end;
  MAGLENGTH := I;
end;

function TMagUtils.MagPiece(str, delim: string; piece: INTEGER): string;
var I, K: INTEGER;
  s: string;
begin
  I := Pos(delim, str);
  if (I = 0) and (piece = 1) then begin result := str; EXIT; end;
  for K := 1 to piece do
    begin
      I := POS(delim, str);
      if (I = 0) then I := LENGTH(str) + 1;
      S := COPY(str, 1, I - 1);
      str := COPY(str, I + 1, LENGTH(str));
    end;
  result := S;
end;

end.

