unit cMagUtilsDB;

{
   Package: MAG - VistA Imaging
 WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
 Date Created:    Version 3.0.8
 Site Name: Silver Spring, OIFO
 Developers: Garrett Kirin
 [==  unit cMagUtilsDB;
 Description: Imaging DataBase Utilities
 Some functions and methods are used by many Forms (windows) and components of
 the application.  Utility components are designed to be a repository of such
 utility functions used throughout the application.
 Other Compontents that need the methods contained in the utililty components
 can declare a property of that type and have the utility object set at design time.

 TMagUtilsDB:   Designed to be a DataBase Utility component. Methods in this
 component access the DataBase.  example : The TMag4Viewer component does not have
  an association with the DataBase but it has a property MagUtilsDB : TMagUtilsDB,
  and uses the methods of MagUtilsDB object to access the ImageReport function.

  ==]
 Note:
  }
(*
        ;; +---------------------------------------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +---------------------------------------------------------------------------------------------------+

*)
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  cMag4VGear, cMagDBBroker, printers, GearDef, fmagCopyAgreement, umagclasses,
  cMagUtils, StdCtrls, maggmsgu, maggrpcu, cmagpat, maggut4 {, maggut1}
  , fmagGetEsigDialog;

type
  TMagUtilsDB = class(TComponent)
  private
    FDBBroker: TMagDBBroker;
    FMagUtils1: TMagUtils;
    FMagPat: TMag4Pat;
    function DoesFormExist(s: String) : boolean;
    procedure printFooter;
    procedure printHeader(headerL, HeaderR: String);
    procedure PrintTopSpacing(lines, texth, horzM: integer; var curline: integer);
    function GetEsigDialog(var xmsg : string): boolean;

  protected
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    {       Displays the Reason Dialog Box, Returns True if reason is selected.}
    function GetReason(rcode: integer; var reason: string): boolean;
    {       Displays a dialog window to allow the user to enter a reason and Electronic signature.}
    function GetEsig(var xmsg: String) : boolean;
    {       Prompts user for printer, Prints the Image to selected printer}
    procedure ImagePrint(MagVGear: TObject; var stat: boolean; var xmsg: String);
    {       Copies Image to the Clipboard}
    procedure ImageCopy(MagVGear: TObject; var stat: boolean; var xmsg: String);
    {       Retrieves Image report from DataBase and displays it in instance of Form: Maggrpcf
                Magien : Image IEN
                if rptmemo <> nil, the report will be inserted there instead.}
    procedure ImageReport(magien: String; var stat: boolean; var xmsg: String; rptmemo: TMemo = nil; noQACheck : boolean = false); overload;
    {       Retrieves Image report from DataBase and displays it in instance of Form: Maggrpcf
                Iobj : has the Image IEN as a Property
                if rptmemo <> nil, the report will be inserted there instead.}
    procedure ImageReport(IObj: TImageData; var stat: boolean; var xmsg: String; rptmemo: TMemo = nil; noQACheck : boolean = false); overload;
    {       Prints a TEXT file to printer}
    procedure ImageTextFilePrint(IObj: TImageData; txt: Tstrings; xHeader: String);
    {       Copies a TEXT file to the clipboard.}
    procedure ImageTextFileCopy(IObj: TImageData; memo: TMemo; xHeader: String);
    {       Displays the Patient profile report from VistA in instance of Form: Maggrpcf}
    procedure OpenPatientProfile;
    {       Opens a Radiology report in instance of Form : Maggrpcf
                User has clicked on an entry in Radiology Exam List window
                str : string of radiology data that is returned by Radiology API when
                imaging builds the list of radiology reports.}
    procedure RadiologyExamReport(str: String);
    {       Opens the report in in instance of Form: Maggrpcf
                t: contains the report.
                if rptmemo <> nil, the report will be inserted there instead.}
    procedure ShowReport(t: Tstrings; reportdesc, reporttitle: String; rptmemo: TMemo = nil);
  published
    {       DataBase broker.  All roads to the DataBase go through here.}
    property MagBroker : TMagDBBroker read FDBBroker write FDBBroker;
    {       Patient Object}
    property MagPat : TMag4Pat read FMagPat write FMagPat;
  end;

procedure Register;

implementation

uses umagkeymgr;

procedure TMagUtilsDB.RadiologyExamReport(str: String);
var
  t: Tstringlist;
  daycase: string[30];
  raddesc: string;
  rptdate: string[12];
  RARPT: string;
  rstat: boolean;
  rmsg: string;
begin
  t := Tstringlist.Create;
  try
    // TEMP := tradlist[strtoint(s)];
    daycase := FMagUtils1.magpiece(str, '^', 2);
    raddesc := FMagUtils1.magpiece(str, '^', 3);
    rptdate := FMagUtils1.magpiece(str, '^', 4);
    RARPT := FMagUtils1.magpiece(str, '^', 16);
    //LogActions('RADEXAM', 'REPORT', daycase);
    maggmsgf.MagMsg('', daycase + '   ' + raddesc + '   ' + rptdate);
    FDBBroker.RPMaggRadReport(rstat, rmsg, t, RARPT);
    if not rstat then
    begin
      if FMagUtils1.magpiece(t[0], '^', 1)='-2' then
        maggmsgf.MagMsg('DEQA', rmsg)
      else
        maggmsgf.MagMsg('de', rmsg);
      maggmsgf.MagMsg('s', 'RARPT: ' + RARPT + 'daycase: ' + daycase);
      exit
    end;
    if FMagPat.M_UseFakeName then
    begin
      magReplaceStrings(FMagPat.M_PatName, FMagPat.M_NameDisplay, t);
      magReplaceStrings(FMagPat.M_SSN, '000000000', t);
      magReplaceStrings(FMagPat.M_SSNdisplay, '000-00-0000', t);
      magReplaceString(FMagPat.M_PatName, FMagPat.M_NameDisplay, raddesc);
      magReplaceString(FMagPat.M_SSN, '000000000', raddesc);
      magReplaceString(FMagPat.M_SSNdisplay, '000-00-0000', raddesc)
    end;
    t.Delete(0);
    maggmsgf.MagMsg('', 'REPORT ' + daycase + '   ' + raddesc + '.  Done.');
    ShowReport(t, daycase + '   ' + raddesc + '   ' + rptdate, 'Radiology Report' + ' - ' + FMagPat.M_NameDisplay)
  finally
    t.Free
  end;
end;


procedure TMagUtilsDB.OpenPatientProfile;
var
  rmsg: string;
  rlist: Tstringlist;
  rstat: boolean;
begin
  rlist := Tstringlist.Create;

  try
    if (FMagPat.M_DFN='') then
    begin
      maggmsgf.MagMsg('d', 'You must first select a Patient ');
      exit
    end;
    maggmsgf.MagMsg('', 'Building Patient Profile...');
    FDBBroker.RPMaggDGRPD(rstat, rmsg, rlist, FMagPat.M_DFN);
    if not rstat then
    begin
      maggmsgf.MagMsg('de', rmsg);
      exit
    end;
    if FMagPat.M_UseFakeName then
    begin
      magReplaceStrings(FMagPat.M_PatName, FMagPat.M_FakePatientName, rlist);
      magReplaceStrings(FMagPat.M_SSN, '000000000', rlist);
      magReplaceStrings(FMagPat.M_SSNdisplay, '000-00-0000', rlist)
    end;
    ShowReport(rlist, FMagPat.M_Demog, 'Patient Profile - ' + FMagPat.M_NameDisplay);
    maggmsgf.MagMsg('', FMagPat.M_NameDisplay + ', Patient Profile Complete.')
  finally
    rlist.Free
  end;
end;


procedure TMagUtilsDB.ImageReport(magien: String; var stat: boolean; var xmsg: String; rptmemo: TMemo = nil; noQACheck : boolean = false);
var
  IObj: TImageData;
begin
  IObj := TImageData.Create;
  IObj.Mag0 := magien;
  ImageReport(IObj, stat, xmsg, rptmemo);
  IObj.Free
end;


procedure TMagUtilsDB.ImageReport(IObj: TImageData; var stat: boolean; var xmsg: String; rptmemo: TMemo = nil; noQACheck : boolean = false);
//procedure BuildReport(ien, reportdesc, title: string; rptmemo : TMemo = nil);
var //t : tstringlist;
  pat: string;
  Flist: Tstrings;
  FFlist: Tstringlist;
begin
  //t := tstringlist.create;  need later when all report functionality is
  //                moved to this component
  Flist := Tstringlist.Create;
  FFlist := Tstringlist.Create;
  try
    if not FDBBroker.IsConnected then
    begin
      stat := FALSE;
      xmsg := 'No Connection to VistA.'
    end;
    FDBBroker.RPImageReport(stat, xmsg, Flist, IObj, noQAcheck);
    if not stat then
    begin
      if (rptmemo<>nil) then
      begin
        rptmemo.lines.Add(' ----- Report not received for Image # ' + IObj.Mag0 + ' ----- ');
        rptmemo.lines.Add(xmsg);
        maggmsgf.MagMsg('s', ' Report Failed for Image IEN: ' + IObj.Mag0)
      end
      else
        maggmsgf.MagMsg('d', xmsg);
      maggmsgf.MagMsg('s', '  Report Failed for Image IEN: ' + IObj.Mag0);
      exit
    end;
    pat := FMagUtils1.magpiece(Flist[0], '^', 3);
    if (FMagUtils1.magpiece(Flist[0], '^', 1)='-2') //Patch 5
    then
    begin
      maggmsgf.MagMsg('DEQA', FMagUtils1.magpiece(Flist[0], '^', 2));
      maggmsgf.MagMsg('s', 'Image IEN in Question: ' + IObj.Mag0)
    end
    else
    begin
      Flist.Delete(0);
      FFlist.assign(Flist);
      if FMagPat.M_UseFakeName then
      begin
        magReplaceStrings(FMagPat.M_PatName, FMagPat.M_NameDisplay, FFlist);
        magReplaceStrings(FMagPat.M_SSN, '000000000', FFlist);
        magReplaceStrings(FMagPat.M_SSNdisplay, '000-00-0000', FFlist);
        pat := FMagPat.M_NameDisplay
      end;
      ShowReport(FFlist, xmsg, 'Image Report' + ' - ' + pat, rptmemo)
    end;

  finally
    Flist.Free;
    FFlist.Free
  end;
end;


procedure TMagUtilsDB.ImagePrint(MagVGear: TObject; var stat: boolean; var xmsg: String);
var  vGear: Tmag4VGear;
  printmsg: string;
  Myprintdialog: TPrintDialog;
  reason: string;
  i,y : integer;
begin
  stat := false; //
  //if GetEsig(xmsg) then //TODO ESIG
  begin
    stat := true;
    vGear := Tmag4VGear(MagVGear);
    (* Add SSN Take out Image IEN
    printmsg := vGear.PI_ptrData.PtName + ' ' + vGear.PI_ptrData.ExpandedIDDescription(false);*)
    printmsg := vGear.PI_ptrData.PtName + ' ' + vGear.PI_ptrData.SSN + ' ' + vGear.PI_ptrData.ExpandedDescription(FALSE);
    Myprintdialog := TPrintDialog.Create(Self);
    screen.cursor := crHourGlass;
    try
      Myprintdialog.Options := [poPageNums];
      Myprintdialog.MinPage := 1;
      Myprintdialog.MaxPage := vGear.Gear1.pagecount;
      Myprintdialog.FromPage := 1; //p48T3 vGear.Gear1.page;
      Myprintdialog.ToPage := vGear.Gear1.pagecount; //p48t3 page;
      Myprintdialog.PrintRange := prPageNums;
      if Myprintdialog.EXECUTE then
      begin
        try
          try
            if Myprintdialog.PrintRange=prAllPages then
            begin
              Myprintdialog.FromPage := 1;
              Myprintdialog.ToPage := vGear.Gear1.pagecount
            end;
            vGear.Gear1.SettingMode := MODE_PRINTDRIVER;
            vGear.Gear1.SettingValue := 1;
            vGear.Gear1.Printsize := IG_PRINT_FULL_PAGE;
            printer.begindoc;
            for I := Myprintdialog.FromPage to Myprintdialog.ToPage do
            begin
              printer.Title := printmsg;
              printer.canvas.textout(10, 0, printmsg + '    --- page ' + IntToStr(I) + ' of ' + IntToStr(vGear.Gear1.pagecount) + '  ---');
              vGear.Gear1.page := I;
              vGear.Gear1.printimage := printer.Handle;
              if I<Myprintdialog.ToPage then
                printer.newpage
            end;
            stat := true;
            xmsg := 'Image Printed.';
//            if FDBBroker<>nil then
//              FDBBroker.RPLogCopyAccess(reason + '^^' + Tmag4VGear(MagVGear).PI_ptrData.Mag0 + '^' + 'Print Image' + '^' + Tmag4VGear(MagVGear).PI_ptrData.DFN + '^' + '1')
          except
            on E : Exception do
              begin
                stat := FALSE;
                xmsg := 'ERROR Printing: ' + E.Message
              end;
          end;
        finally
          printer.enddoc
        end;
      end;
    finally
      Myprintdialog.Free;
      screen.cursor := crdefault
    end;
  end;
end;


procedure TMagUtilsDB.ImageCopy(MagVGear: TObject; var stat: boolean; var xmsg: String);
var
  reason: string;
begin
  stat := FALSE;
  if GetEsig(xmsg) then // TODO ESIG
  begin
    stat := true;
    Tmag4VGear(MagVGear).Gear1.ClipboardCopy := true;
    if FDBBroker<>nil then
      FDBBroker.RPLogCopyAccess(reason + '^^'
        + Tmag4VGear(MagVGear).PI_ptrData.Mag0 + '^' + 'Copy To Clipboard' + '^'
        + Tmag4VGear(MagVGear).PI_ptrData.DFN + '^' + '1')
  end;
end;


procedure TMagUtilsDB.ImageTextFileCopy(IObj: TImageData; memo: TMemo; xHeader: String);
var
  xmsg,
  reason: string;
begin
//p48t5 if GetEsig(xmsg) then // TODO ESIG
  begin
    memo.SelectAll;
    memo.CopyToClipboard;
    memo.SelLength := 0;
//p48t5    if FDBBroker<>nil then
//p48t5      FDBBroker.RPLogCopyAccess(reason + '^^' + IObj.Mag0 + '^' + 'Copy To Clipboard' + '^' + IObj.DFN + '^' + '1')
  end;
end;

function TmagUtilsDB.GetEsigDialog(var xmsg : string) : boolean;
var ct : integer;
   esig : string;
begin
result := false;
ct := 3;
while ct > 0 do
  begin
    if frmGetEsigDialog.execute(esig) then
      begin
          if not FDBBroker.RPVerifyEsig(esig, xmsg)
            then ct := ct-1
            else
            begin
              result := true;
              ct := 0;
            end;
      end
      else
      begin
       xmsg := 'Electronic Signature Canceled.';
       ct := 0;
      end;

  end;
end;



function TMagUtilsDB.GetEsig(var xmsg: String) : boolean;
var
  esig: string;
begin
  if userhaskey('MAG ROI') or userhaskey('TMP MAG ESIG') THEN
    begin
      result := true;
      exit;
    end;
  Result := FALSE;
  if FDBBroker<>nil then
  begin
      if not GetEsigDialog(xmsg) then
        begin
          maggmsgf.MagMsg('d', xmsg, nil);
          exit;
        end
        else
        begin
          AddDelKey('TMP MAG ESIG',TRUE);
          result := true;
        end;
  end
  else {if FDBBroker not assigned, we'll just copy. Demo Mode, Viewing a Directory}
    Result := true //Tmag4VGear(MagVGear).Gear1.ClipboardCopy := True;
end;

function TMagUtilsDB.GetReason(rcode : integer; var reason: string) : boolean;
begin
  result :=  frmCopyAgreement.EXECUTE(reason);
  if not result then maggmsgf.MagMsg('','Copy Agreement Canceled.', nil);
end;


procedure Register;
begin
  RegisterComponents('Imaging', [TMagUtilsDB])
end;


constructor TMagUtilsDB.Create(AOwner: TComponent);
begin
  inherited;
  FMagUtils1 := TMagUtils.Create(nil)
end;


destructor TMagUtilsDB.Destroy;
begin
  inherited;
  FMagUtils1.Destroy //
end;


procedure TMagUtilsDB.ShowReport(t: Tstrings; reportdesc, reporttitle: String; rptmemo: TMemo = nil);
begin
  if (rptmemo=nil)
     and
     (not DoesFormExist('maggrpcf')) then
  begin
    Maggrpcf := tmaggrpcf.Create(Application)
    {TODO: need to fix this, need access to Upref's}
    //     upreftoreport(upref);
  end;
  // StripControlCharacters(t);
  if rptmemo=nil then
  begin
    Maggrpcf.pDesc.caption := reportdesc;
    Maggrpcf.caption := reporttitle + ' ' + reportdesc;
    FMagUtils1.FormToNormalSize(Maggrpcf);
    rptmemo := Maggrpcf.Memo1
  end;
  rptmemo.lines.clear;
  rptmemo.lines := t
end;


function TMagUtilsDB.DoesFormExist(s: String) : boolean;
{Called with the Tform.name value of a form, to see if it exists, if we have to create it or not}
var
  I: integer;
begin
  Result := FALSE;
  for I := screen.CustomFormCount - 1 downto 0 do
  begin
    if (UPPERCASE(screen.customforms[I].Name)=UPPERCASE(s)) then
    begin
      Result := true;
      break
    end;
  end;
end;


//////////////////////////
procedure TMagUtilsDB.printHeader(headerL, HeaderR: String);
//var xFont : Tfont;
begin
  //xFont := printer.canvas.Font;
  {change font to Bold, Italic : to be different than the report.}
  printer.canvas.Font.Style := [fsBold, fsitalic];
  printer.canvas.textout(10, 0, headerL);
  SetTextAlign(printer.canvas.Handle, TA_RIGHT);
  printer.canvas.textout(printer.PageWidth, 0, HeaderR);
  {set font to normal.}
  SetTextAlign(printer.canvas.Handle, TA_LEFT);
  printer.canvas.Font.Style := []
  //  printer.canvas.font := xfont;
end;


procedure TMagUtilsDB.printFooter;
var
  xFont: Tfont;
begin
  xFont := printer.canvas.Font;
  {change font to Bold, Italic : to be different than the report.}
  printer.canvas.Font.Style := [fsBold, fsitalic];
  SetTextAlign(printer.canvas.Handle, TA_RIGHT);
  printer.canvas.textout(printer.PageWidth, printer.PageHeight - printer.canvas.TextHeight('X'), 'Printed: ' + FormatDateTime('d mmmm yyyy', Date));
  SetTextAlign(printer.canvas.Handle, TA_LEFT);
  {set font to normal.}
  //printer.canvas.font.style := [];
  printer.canvas.Font := xFont
end;


procedure TMagUtilsDB.PrintTopSpacing(lines, texth, horzM: integer; var curline: integer);
var
  J: integer;
begin
  for J := 1 to lines do
  begin
    inc(curline);
    printer.canvas.textout(horzM, curline * texth, ' ')
  end;
end;


procedure TMagUtilsDB.ImageTextFilePrint(IObj: TImageData; txt: Tstrings; xHeader: String);
var
  I,
  texth,
  pageH,
  lineperpage: integer;
  horzM: integer;
  linect,
  pagect,
  pagetot: integer;
  xmsg,
  reason: string;
begin
//p48t5  if GetEsig(xmsg) then // TODO ESIG
  begin
    linect := 0;
    pagect := 1;
    try
      xHeader := IObj.ExpandedIDDescription;
      with printer do
      begin
        begindoc;
        with canvas do
        begin
          {Establish printer parameters}
          //Font := richedit1.Font;
          texth := TextHeight('P'); {Height of current font in pixels}
          pageH := PageHeight; {Page Height (printable) in pixels}
          lineperpage := pageH div texth; { lines per page }
          horzM := 20;
          pagetot := txt.Count div lineperpage;
          if (txt.Count mod lineperpage)>0 then
            pagetot := pagetot + 1;

          //            inc(linect);
          printHeader(xHeader, 'Page ' + IntToStr(pagect) + ' of ' + IntToStr(pagetot));
          PrintTopSpacing(3, texth, horzM, linect);

          //            for j := 1 to 3 do
          //              begin
          //                inc(linect); textout(horzM, linect * texth, ' ');
          //              end;
          for I := 0 to txt.Count - 1 do
          begin
            inc(linect);
            if (linect +
                3)>lineperpage then
            begin
              printFooter;
              newpage;
              linect := 1;
              inc(pagect);
              printHeader(xHeader, 'Page ' + IntToStr(pagect) + ' of ' + IntToStr(pagetot));
              PrintTopSpacing(3, texth, horzM, linect);
              //                  for j := 1 to 3 do
              //                      begin
              //                        inc(linect); textout(horzm, linect * texth, ' ');
              //                      end;
              continue
            end;
            textout(horzM, (texth * linect), txt[I])
          end;
          //X Font.Name:='Book Antiqua';
          {Set font size and style, Print Heading}
          //X Font.Size:=14;
          //X Font.Style:=[fsBold];
          {Print right justified column}
          //X SetTextAlign(printer.canvas.handle,TA_RIGHT);
          //X textout(Hmar+100*mmx,15*ls,FormatDateTime('d mmmm yyyy',Date));
          //X textout(Hmar+100*mmx,16*ls,'$6,550,000.00');
          //X setTextAlign(printer.canvas.handle,TA_LEFT);
          {Print decorative heading}
          //X with brush do
          //X   begin
          //X     style:=bsSolid;
          //X     color:=clGreen;
          //X   end;
          //X Rectangle(HMar,ls*20,Printer.PageWidth,ls*23);
          //X Font.Size:=18;
          //X Font.Color:=clWhite;
          //X Font.Style:=[fsBold];
          //X TextOut(hmar+10*mmx,ls*21,'White heading in a green rectangle');
          //X brush.color:=clwhite;
          //X font.color:=clblack;
          //X font.style:=[];
          {Print large footer}
          // Font.Size:=24;
          // Font.Style:=[fsBold];
          // ls:=textheight('X');
          //x  TextOut(HMar, Printer.PageHeight - TextHeight('X'), 'This is the end, my friend!');
          //X Font.Size := 12;
          {Set font size and style, get line height for body}
          //X SetFontSize(12);
          //X Font.Style:=[];                     {restore font style to regular (no style)}
          {Print body}
        end;
      end;
      printFooter
    finally
      printer.enddoc
    end;
//p48t5    if FDBBroker<>nil then
//p48t5      FDBBroker.RPLogCopyAccess(reason + '^^' + IObj.Mag0 + '^' + 'Print Image' + '^' + IObj.DFN + '^' + '1')
  end;
end;



end.

