unit cMagVUtils;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created:  Version 3.0.8
   Site Name: Silver Spring, OIFO
   Developers: Garrett Kirin
[==      unit cMagVUtils;
   Description:  Imaging Viewer utilities
    Some functions and methods are used by many Forms (windows) and components of
    the application.  Utility components are designed to be a repository of such
    utility functions used throughout the application.
    Other Compontents that need the methods contained in the utililty components
    can declare a property of that type and have the utility object set at design time.


Class : TMagVUtils
    This utility class has methods dealing with Image Information and properties.
    Mainly used by the Image Viewer (TMag4Viewer).
      ==]
   Note:
   }
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  uMagClasses, cMagUtils, fmxutils,
  Printers, GearDef, cMagDBBroker, uMagDefinitions;

type
  TMagVUtils = class(TComponent)
  private
    Futl: TMagUtils;
  protected
    { Protected declarations }
  public
    constructor Create(AOwner: Tcomponent); override;
    destructor destroy; override;

            {   string from Image List returned by RPC convert to TImageData Object.}
    function StringToIMageObj(str: string): TImageData;

            {   actual Abstract or Canned Bitmap }
    function ResolveAbstract(imgObj: TImageData; ShowJB: boolean = false): string;

            {   Images from an OpenFile Dialog into List of TImageData objects.}
    function DirImagesToObjList(t: tstrings): Tlist;

            {   gets the VistA Image field Object Type, from the file's Extension}
    function ResolveTypeFromExt(filename: string): integer;

            {   Image used to Clear TGear component}
    function BlankImage: string;

            {   full path where the application is running}
    function AppPath: string;
  published
    { Published declarations }
  end;

procedure Register;

implementation

uses cMag4Viewer, cMag4VGear, umagdisplaymgr, dmsingle;



function TMagVUtils.AppPath: string;
begin
  result := Futl.Apppath;
end;

function TMagVUtils.BlankImage: string;
begin
  result := Futl.BlankImage;
end;

constructor TMagVUtils.Create(AOwner: Tcomponent);
begin
  inherited;
  Futl := TMagUtils.create(nil);
end;

destructor TMagVUtils.destroy;
begin
  Futl.Free;
  inherited;
end;


function TMagVUtils.DirImagesToObjList(t: tstrings): Tlist;
var i: Integer;
  Iobj: TimageData;
begin
  result := tlist.create;
  try
    for i := 0 to t.count - 1 do
      begin
      { TODO : Create Abstracts, and Resolve Image Format for the images in the directory. }
        Iobj := TImageData.create;
        Iobj.AFile := t[i];
        Iobj.FFile := t[i];
        Iobj.Proc := t[i];
        Iobj.ImgDes := t[i];
        Iobj.ImgType := ResolveTypeFromExt(Iobj.FFile);
        Iobj.Mag0 := inttostr(i);
        result.Add(Iobj);
      end;
  finally
    //
  end;
end;


function TMagVUtils.ResolveAbstract(imgObj: TImageData; ShowJB: boolean = false): string;
var xmsg,s: string;
  apppath: string;
  i : integer;
begin
  apppath := Futl.AppPath;
  s := imgObj.AbsLocation;
  result := imgObj.AFile;
    { TODO : URGENT  avi canned bitmap.   Need flag to say that this image has canned bitmap.
              or no abstract, so we don't get these errors. }
    { TODO :  need to change the 'O' and 'W' to enumerated type.
              maglocMag, maglocJB, }
if (FILEEXISTS(Apppath + '\Cache\' + extractfilename(RESULT))) then
  BEGIN
   result := apppath + '\cache\' + extractfilename(RESULT);
   EXIT;
  END;

  if (UPPERCASE(imgObj.AbsLocation) = 'O')
    then
      begin
        Result := AppPath + '\BMP\JBOFFLN.ABS';
        exit;
      end;
  if (UPPERCASE(imgObj.AbsLocation) = 'W') then
    begin
    { Abstract is on JukeBox. 'W' means 'Worm Drive'}
      if (not ShowJB)
      {if not viewing JukeBox Abstracts then Display the BMP}
      then
        begin
          Result := AppPath + '\BMP\ABSJBOX.BMP' ;
          exit;
        end

      { we are viewing JB Abs, so Queue it also}
      else //old   queabs.add(ptrMagRecord^.Mag0);
    end;
  //P48T2 DBI
  if imgobj.PlaceIEN <> WrksPlaceIEN then
    begin
      if (not IniViewRemoteAbs) then  
        result := AppPath + '\BMP\ABSREMOTE.BMP'
        else
          // later magybe allow queue to local sserver .  ?
    end;
  { 12-PACS Group   10-PACS Resident    13-ECG     14-cine }
  case imgobj.ImgType of
    12: result := AppPath + '\BMP\ABSPACG.BMP';
    10: Result := AppPath + '\BMP\ABSPACI.BMP';
    13: Result := AppPath + '\bmp\absekg.bmp';
    14: Result := AppPath + '\bmp\abscine.bmp';
    //   21: Result := AppPath + '\bmp\MotionVideoAbs.bmp';
    21: Result := AppPath + '\bmp\magavi.bmp';
    101: Result := AppPath + '\bmp\maghtml.bmp';
    102: Result := AppPath + '\bmp\magdoc.bmp';
    103: Result := AppPath + '\bmp\magtext.bmp';
    104: Result := AppPath + '\bmp\magpdf.bmp';
    105: Result := AppPath + '\bmp\magrtf.bmp';
    106: Result := AppPath + '\bmp\magwav.bmp';
  end; {case}
// No connection to server from here

if FWrksAbsCacheON  and (not(FILEEXISTS(Apppath + '\Cache\' + extractfilename(RESULT)))) then
  BEGIN
  if not Dmod.MagFileSecurity.MagOpenSecurePath(result, xmsg) then  EXIT;
    copyfile(result, Apppath + '\Cache');
    result :=   Apppath + '\Cache\' + extractfilename(RESULT) ;
    Dmod.MagFileSecurity.MagCloseSecurity(xmsg);
  END;

end;


function TMagVUtils.ResolveTypeFromExt(filename: string): integer;
var ext: string;
begin
{TODO: need to make a DB Call for this, and should be moved to cMagUtilsDB.pas}
  ext := uppercase(copy(ExtractFileExt(filename), 2, 99));
  result := 1;
  //case imgobj.ImgType of
  if ext = 'AVI' then result := 21
  else if ext = 'MPG' then result := 21
  else if ext = 'MP3' then result := 21
  else if ext = 'MP4' then result := 21
  else if ext = 'MPEG' then result := 21
  else if ext = 'HTM' then result := 101
  else if ext = 'HTML' then result := 101
  else if ext = 'DOC' then result := 102
  else if ext = 'TXT' then result := 103
  else if ext = 'ASC' then result := 103
  else if ext = 'PDF' then result := 104
  else if ext = 'RTF' then result := 105
  else if ext = 'WAV' then result := 106;

end;

function TMagVUtils.StringToIMageObj(str: string): TImageData;
begin
  Result := TImageData.create;
  {  Now we bring the Patient Name in the string from vista
      Result.PtName := PatName;  see below}
  Result.Mag0 := Futl.magPiece(str, '^', 2);
  Result.FFile := Futl.magPiece(str, '^', 3);
  Result.AFile := Futl.magPiece(str, '^', 4);
  Result.ImgDes := Futl.magPiece(str, '^', 5);
  (* PI^.FMProcDate :=magpiece(t[i],'^',6);   *)
  Result.ImgType := strtoint(Futl.magPiece(str, '^', 7));
  Result.Proc := Futl.magPiece(str, '^', 8);
  Result.Date := Futl.magPiece(str, '^', 9);
  //the PARENT DATA FILE image pointer is in $p(10)
  //PI^.DemoRpt :=  magpiece(t[i],'^',10);
  Result.AbsLocation := Futl.magPiece(str, '^', 11);
  Result.FullLocation := Futl.magPiece(str, '^', 12);
  Result.DicomSequenceNumber := Futl.magPiece(str, '^', 13);
  Result.DicomImageNumber := Futl.magPiece(str, '^', 14);
  Result.GroupCount := strtoint(Futl.magPiece(str,'^',15));
  //	; $p(15^16 ) are SiteIEN and SiteCode Consolidation - DBI
  //    delphi 16^17
        // P48T3 DBI
   Result.PlaceIEN := Futl.magPiece(str,'^',16);
   Result.PlaceCode := Futl.magPiece(str,'^',17);
  Result.QAmsg := Futl.magPiece(str, '^', 18);
  Result.BigFile := Futl.magPiece(str, '^', 19);
  Result.DFN := Futl.magPiece(str, '^', 20);
  Result.PtName := Futl.magPiece(str, '^', 21);
  Result.MagClass := Futl.magPiece(str, '^', 22);

  (*   from uMagClasses
  TImageData = class(Tobject)
  private
    FMag0: string;
    FAFile: string;
    FFFile: string;
    FImgDes: string;
    FImgType: integer; { a byte is 0 .. 255 }
    FPtName: string;
    FDate: string;
    FProc: string;
    FAbsLocation: string;
    FFullLocation: string;
    FDicomImageNumber: string;
    FDicomSequenceNumber: string;
    Fbaseindex: integer;
    FQAMsg : string;
    FBigFile : string;
    FDFN : string;
    FMagClass: String;

    FGroupCount: integer;
    FGroupIEN: integer;
    FGroupIndex: integer;
      *)
end;


procedure Register;
begin
  RegisterComponents('Imaging', [TMagVUtils]);
end;


end.

