unit cmagLabelNoClear;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created:
   Site Name: Silver Spring, OIFO
   Developers: Garrett Kirin
[==  unit cmagLabelNoClear;
   Description: Capture Component to support 'Hold Value' properties of the
   configuration.

   On the Capture Main form, each data input control, (editbox, ComboBox) is
   associated with a label of type TmagLabelNoClear.
   When user changes the 'Hold Value' of an input control, the associated
   TmagLabelNoClear label's 'NoClear' property is changed.

   As a visual aid for the user, this control has an associated Glyph, when
   the 'NoClear' property is changed the associated Glyph is made visible/invisible

   When a capture configuration is saved the 'NoClear' properties of all controls
   of type TmagLabelNoClear are querried. The value (TRUE/FALSE) of the 'NoClear'
   property is saved with the configuration. If NoClear is TRUE the value of the
   associated input control is also saved with the configuration.

==]
   Note:
   }
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, extctrls;

type
  TmagLabelNoClear = class(TLabel)
  private
    FGlyphControl : TImage;
    FNoClear : boolean;
    FVisible: boolean;
    procedure SetNoClear(const Value: Boolean);
    procedure SetVisible(const Value: boolean);
  protected
    { Protected declarations }
  public

  published
  property  Visible : boolean read FVisible write SetVisible;

  property NoClear : Boolean read FNoClear write SetNoClear;
  property GlyphyControl : TImage read FGlyphControl write FGlyphControl;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Imaging', [TmagLabelNoClear]);
end;

procedure TmagLabelNoClear.SetNoClear(const Value: Boolean);
begin
  FNoClear := Value;
if FNoClear then
        begin
         //font.color := clWhite;
         if Assigned(FGlyphControl) and FVisible then FGlyphControl.Visible := true;
         end
         else
         begin
         //font.color := clBlack;
         if Assigned(FGlyphControl) then FGlyphControl.Visible := false;
         end;
end;

procedure TmagLabelNoClear.SetVisible(const Value: boolean);
begin
  FVisible := Value;
  if Fvisible Then show
                else hide;
  if not FVisible then if Assigned(FGlyphControl) then FGlyphControl.Visible := false;
  if FVisible then if Assigned(FGlyphControl) then FGlyphControl.Visible := FNoClear;
end;

end.
