unit cmagObserverLabel;
  {
   Package: MAG - VistA Imaging
 WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
 Date Created:
 Site Name: Silver Spring, OIFO
 Developers: Garrett Kirin
[==     unit cmagObserverLabel;
 Description: Label Component with IMagObserver Interface.
 Attaches to IMagSubject(TMag4Pat)
   and displays and maintains the Name (or FakeName) of the patient defined in Tmag4Pat.
   Whenever Tmag4Pat changes states ( changes patients) it notifies all attached
   observers.  TmagObserverlabel updates it's caption to the new patient name.
==]
 Note:
  }
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ImagInterfaces, cMagPat;

type
  TmagObserverLabel = class(TLabel, ImagObserver)
  private
    FMagPat: TMag4Pat;
    procedure AttachMyself;
  protected
    { Protected declarations }
  public
        {       Implementation of ImagObserver interface.  Subject calls this to notify
                observer of a change in state.}
    procedure Update_(SubjectState: string); { Public declarations }
        {       Set the reference to Patient object}
    procedure SetMagPat(const Value: TMag4Pat);
    constructor create(AOwner: Tcomponent); override;
    destructor destroy; override;
  published
        {       Set the reference to Patient object}
    property MagPat: TMag4Pat read FMagPat write SetMagPat; { Published declarations }
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Imaging', [TmagObserverLabel]);
end;

{ TmagObserverLabel }

constructor TmagObserverLabel.create(AOwner: Tcomponent);
begin
  inherited;
  //
end;

procedure TmagObserverLabel.SetMagPat(const Value: TMag4Pat);
begin
        {       Detach from current patient object}
  if (FmagPat <> nil) then FMagPat.Detach_(self);
  FMagPat := Value;
        {       Attach to New patient object}
  if value <> nil then AttachMyself();
end;


procedure TmagObserverLabel.AttachMyself;
begin
  if assigned(FmagPat) then
    begin
      FMagPat.Attach_(IMagObserver(self));
    end;
end;

procedure TmagObserverLabel.Update_(SubjectState: string);
   {    subjectState can be '', '-1'  or  '1' }
begin
  try
    caption := '';
    if (SubjectState <> '') then
        if (SubjectState = '-1')
        {       if Subject is being destroyed. Clear Pointer.}
        then FMagPat := nil
        {       Subject has value, display it.}
        else caption := FMagPat.M_NameDisplay;
  except
    on e: exception do
      showmessage('Exception in ObserverLabel Update_' + e.message);
  end;
end;


destructor TmagObserverLabel.destroy;
begin
  {    first see if reference to Patient ojbect is defined, because it may have been
        destroyed already }
  if assigned(FmagPat) then
    begin
      FMagPat.Detach_(IMagObserver(self));
      FMagpat := nil;
    end;
  inherited;
end;

end.

