unit fMagLoadingImageMessage;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created:  12/2000
   Site Name: Silver Spring, OIFO
   Developers: Garrett Kirin
   [==    unit fMagLoadingImageMessage;
   Description: frmMagLoadingImageMessage is a Utility form used by
   Tmag4Viewer to display information during the process of loading a list
   of images into the viewer.
   The window (frmMagLoadingImageMessage) allows the user to stop the loading
   of abstracts or images before the entire list is loaded.
   ==]
   Note:
   }
(*
        ;; +---------------------------------------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +---------------------------------------------------------------------------------------------------+

*)

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, Gauges, ExtCtrls, activex  ;

type
  TfrmLoadingImageMessage = class(TForm)
    bbCancel: TBitBtn;
    Label1: TLabel;
    lbImagename: TLabel;
    Label2: TLabel;
    Gauge1: TGauge;
    lbnumbers: TLabel;
    procedure bbCancelClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Timer1Timer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FTimerStarted :boolean;
    FDelayOver: boolean;
  public
    RequestToStopLoading : boolean;
        {       Delays the window from opening for milsec milliseconds.}
    procedure SetDelay(milsec : integer);
        {       initialize with Number of Images that are being loaded.}
    procedure init(count: integer);
        {       after each image, update the information displayed.
                s: filename
                cur : current # of image being loaded.
                firstnum : number of first image that was loaded.
                lastnum : number of last image that will be loaded.
                totalimages : total number of images that will be loaded.
                i.e. 'loading  5 to 12 of 24 images.'
                     ' # 6 Image description' }
    procedure UpdStatus(s: string; cur, firstnum, lastnum, totalimages: integer);
        {       update the progress bar.}
    procedure UpdProgress(i: integer);
  end;

var
  frmLoadingImageMessage: TfrmLoadingImageMessage;

implementation


{$R *.DFM}


procedure TfrmLoadingImageMessage.bbCancelClick(Sender: TObject);
begin
  RequestToStopLoading := true;
end;

procedure TfrmLoadingImageMessage.init(count: integer);
var tr : Trect;
begin
FdelayOver := false;
FTimerStarted := false;
  lbnumbers.caption := '';
  lbimagename.caption := '';
  lbnumbers.update;
  lbimagename.update;
  Gauge1.progress := 0;
  Gauge1.update;
if (left + width) > screen.Width then
   left := (screen.width - width);
if (top + height) > screen.height then
   top := (screen.height - height);
end;

procedure TfrmLoadingImageMessage.UpdProgress(i: integer);
begin
  Gauge1.progress := i;
  Gauge1.update;
end;

procedure TfrmLoadingImageMessage.UpdStatus(s: string; cur, firstnum, lastnum, totalimages: integer);
begin
  if not FTimerStarTed then
    begin
      FTimerStarted := true;
    end;
    lbnumbers.caption := inttostr(firstnum + 1) + '  to  ' + inttostr(lastnum) + '  of  ' + inttostr(totalimages);
    lbimagename.caption := inttostr(cur + 1) + '   ' + s;
    begin
    FormStyle := fsStayOnTop;
    visible := true;
  end;
end;

procedure TfrmLoadingImageMessage.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  FDelayOver := false;
  FTimerStarted := false;
end;

procedure TfrmLoadingImageMessage.Timer1Timer(Sender: TObject);
begin
  FDelayOver := true;
end;

procedure TfrmLoadingImageMessage.FormCreate(Sender: TObject);
begin
  CoInitialize(nil);
  FTimerStarted := false;
  RequestToStopLoading := false;
  FDelayOver := false;
end;

procedure TfrmLoadingImageMessage.SetDelay(milsec: integer);
begin
  FDelayOver := false;
end;

end.
