unit fMagPatAccess;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created: Version 2.5
   Site Name: Silver Spring, OIFO
   Developers: Garrett Kirin
   [==   unit fMagPatAccess;
   Description:
     This window is created by the Patient Lookup window (maggplku.dfm)
     it's function is :
    - Patient Sensitivity warnings are displayed.
    - Sensitive Patient Logging is done when needed.
    - Means Test warnings are displayed if appropriate.

    This window implements the execute function common to dialog windows.
        It creates itself, displays it's message, and destroys itself.
        This design releives the caller of Creating it, showmodal and then releasing it.

   Note: }
(*
        ;; +---------------------------------------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +---------------------------------------------------------------------------------------------------+

*)

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, ExtCtrls;

type
  TfrmPatAccess = class(TForm)
    Timer1: TTimer;
    ScrollBox1: TScrollBox;
    lbAlign: TLabel;
    Memo1: TMemo;
    lbmsg: TLabel;
    Panel1: TPanel;
    lbAlerttext: TLabel;
    Panel2: TPanel;
    bbOK: TBitBtn;
    Panel3: TPanel;
    bbCancel: TBitBtn;
    procedure Timer1Timer(Sender: TObject);
    procedure image1Click(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure bbOKClick(Sender: TObject);
    procedure bbCancelClick(Sender: TObject);
    procedure Memo1Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
  public
        {       msg : is the message to display
                code: tells this window which buttons to display
                User selects OK, result is TRUE.
                else result is FALSE.}
    function execute(Sender: Tobject; msg: tstringlist; code: integer): boolean; { Public declarations }
  end;

var
  frmPatAccess: TfrmPatAccess;

implementation

{$R *.DFM}
{This window implements the execute function common to dialog windows.
It creates itself, displays it's message, and destroys itself.
This design releives the caller of Creating it, showmodal and then releasing it.
}
function TfrmPatAccess.execute(Sender: Tobject; msg: tstringlist; code: integer): boolean;
var i: INTEGER;
  th, tw, sx: integer;
begin
  frmPatAccess := TfrmPatAccess.create(application.mainform);
  with frmPatAccess do
  begin
    update;
    application.processmessages;
    {code and msg : are returned by the DG RPC's (PIMMS Pkg) that perform the
      patient sensitivity check}
    case code of
      1: begin
          lbalerttext.caption := 'RESTRICTED PATIENT RECORD !!!';
          lbmsg.caption := 'Access to this patient has been logged. Click ''OK'' to continue';
          bbOK.ENABLED := TRUE;
          bbCancel.ENABLED := false;
        end;
      2: begin
          lbalerttext.caption := 'RESTRICTED PATIENT RECORD !!!';
          lbmsg.caption := 'Click ''OK'' to Access the Patient. (Access will be logged)';
          bbOK.ENABLED := TRUE;
          bbCancel.ENABLED := TRUE;
        end;
      3: begin
          lbalerttext.caption := 'RESTRICTED PATIENT RECORD !!!';
          lbmsg.caption := 'Access to this restricted patient is Not Allowed.';
          bbOK.ENABLED := false;
          bbCancel.ENABLED := TRUE;
        end;
         // Means Test Required
      4: begin
          lbalerttext.caption := 'MEANS TEST REQUIRED !!!';
          lbmsg.caption := 'A Means Test is Required for this patient.';
          bbOK.ENABLED := TRUE;
          bbCancel.ENABLED := false;
        end;
      5: begin
          lbalerttext.caption := 'SIMILIAR PATIENT RECORDS !!!';
          lbmsg.caption := 'Click ''OK'' to continue with selected patient.';
          bbOK.ENABLED := TRUE;
          bbCancel.ENABLED := TRUE;
        end;

    end;
    memo1.lines := msg;
    for i := 1 to (memo1.lines.count + 2) do MEMO1.LINES.INSERT(0, '  ');

    canvas.font := MEMO1.font;
    update;
    th := 0; tw := 0;
    for i := 0 to memo1.lines.count - 1 do
    begin
      sx := canvas.textwidth(memo1.lines[i]);
      if sx > tw then tw := sx;
      sx := canvas.textheight(memo1.lines[i]);
      if sx > th then th := sx;
    end;
    width := tw + 40;
    if width < 425 then width := 425;
    memo1.height := (th * (memo1.lines.count + 2));
    scrollbox1.height := memo1.height div 2;
    scrollbox1.vertscrollbar.range := memo1.height;
    clientheight := lbmsg.top + lbmsg.height + 60;
    update;
    application.processmessages;
    showmodal;
  end;
  if frmPatAccess.modalresult = mrok then result := true
  else result := false;
{TODO: This was giving Access violations.  Need to retest and make sure we
       are freeing memory. AccViol maybe caused by main form also creating
       the form in error}
//frmPatAccess.free;  ?
end;

procedure TfrmPatAccess.Timer1Timer(Sender: TObject);
begin
  scrollbox1.vertscrollbar.position := scrollbox1.vertscrollbar.position +
    scrollbox1.vertscrollbar.increment;
end;

procedure TfrmPatAccess.image1Click(Sender: TObject);
begin
  timer1.enabled := not timer1.enabled;
end;

procedure TfrmPatAccess.FormPaint(Sender: TObject);
begin
(*scrollbox1.left :=  (clientwidth-scrollbox1.width) div 2; *)
end;

procedure TfrmPatAccess.FormCreate(Sender: TObject);
begin
(*clientheight := lbAlign.top+lbalign.height;
  clientwidth := lbAlign.left+lbAlign.width; *)

end;

procedure TfrmPatAccess.bbOKClick(Sender: TObject);
begin
  timer1.enabled := FALSE;
end;

procedure TfrmPatAccess.bbCancelClick(Sender: TObject);
begin
  timer1.enabled := FALSE;
end;

procedure TfrmPatAccess.Memo1Click(Sender: TObject);
begin
  Timer1.enabled := not Timer1.enabled;
end;

procedure TfrmPatAccess.FormShow(Sender: TObject);
begin
  scrollbox1.VertScrollBar.Position := 0;
  timer1.interval := 15;
  timer1.enabled := true;
end;

procedure TfrmPatAccess.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  WIDTH := 900;
end;

end.
