unit uMagClasses;
  {
  	Package: MAG - VistA Imaging
	WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
	Date Created:    2002
	Site Name: Silver Spring, OIFO
	Developers: Garrett Kirin
[==  unit uMagClasses;
	Description:
           Image Classes
           TImageData: This object encapsulates information about the
           image entry from the IMAGE File (^MAG(2005,)).
           It is used extensively through out the application.
           Created when the TmagImageList object retrieves a list of images
           from the database, TMagImageList calls -
               function StringToIMageObj(str: string): TImageData;
           passing the '^' delimited string returned by the RPC, RPMaggImageInfo and receiving
           a TImageData object in return.  The object is then added to the ObjList :Tlist
           property of TmagImageList.
           M routine MAGGTII creates the string.


           TMagDICOMData:    Not Implemented.
             Prototype for Implementing full display ability
             of any type of image in MagViewer component. Needs input from
             Ruth. Ruth is developing a unit that any object can call, to
             use her current DisplayAnyImage code.  Also Waiting for newest
             version of AccuSoft.

           TmagDragImageObject:   Not Implemented.
              Prototype for dragging image lists between applications.
                not implemented.

                 ==]
	Note:
  }
(*
        ;; +------------------------------------------------------------------+
        ;; Property of the US Government.
        ;; No permission to copy or redistribute this software is given.
        ;; Use of unreleased versions of this software requires the user
        ;;  to execute a written test agreement with the VistA Imaging
        ;;  Development Office of the Department of Veterans Affairs,
        ;;  telephone (301) 734-0100.
        ;;
        ;; The Food and Drug Administration classifies this software as
        ;; a Class II medical device.  As such, it may not be changed
        ;; in any way.  Modifications to this software may result in an
        ;; adulterated medical device under 21CFR820, the use of which
        ;; is considered to be a violation of US Federal Statutes.
        ;; +------------------------------------------------------------------+

*)
interface

uses StdCtrls, SysUtils, Controls,
  WinTypes, WinProcs, Messages, Classes, Graphics,
  Forms, Dialogs, Grids, Menus, ExtCtrls, inifiles;

type
  {Prototype for dragging image lists between applications.
   not implemented.}
  TmagDragImageObject = class(TDragObject)
  public
   DraggedImages : Tlist;
   end;
  {Prototype for Implementing full display ability of any type of
   image in MagViewer component.
   Needs input from Ruth, as to Dicom data needed.
    Not Implemented.
   Waiting for newest version of AccuSoft.}
  TMagDICOMData = class(TObject)
   public
    PtName: string;  {P5}
    PtID: string[11]; {P5}
    Modality: string[10];
    PixelSpace1: real;
    PixelSpace2: real;
    PixelSpace3: real;
    Haunsfield: smallint;
    SliceThickness: real;
    Pt_Pos: string[3];
    Laterality: string[20];
    SliceLoc: real;
    Rescale_Int: real;
    Rescale_Slope: real;
    Window_Center: smallint; {from DICOM hdr}
    Window_Width: smallint;   {from DICOM hdr}
    Rows: smallint;
    Columns: smallint;
    Filename: string; {RED 2/7/02}
    PatOr1: string;
    PatOr2: string;
    ImageOr: array[1..6] of real;
    bitdepth: integer; {from Accusoft call, DICOM only}
    bits: integer; {from text file}   {P5}
    seriesno: real;
    imageno: string; {P5}
    contrast: string;
    originalbits: integer; {when image is loaded, this is the no. of bits}
    protocol: string;
    Warning: integer;
    BigFile: string; {P5}
    //oo1 gearcontrol: tgear; {RED 1/31/02}
    //oo1 medcontrol: tmed; {RED 1/31/02}
    panel: Tpanel; {RED 10/20/02}
    panelback: Tpanel;
    GList: TList; {RED 11/02/02}
    Imagect: Integer; {RED 11/24/02 - count of images in group in this display control}
    minpixel: longint; {from Init procedure}
    maxpixel: longint;
    winposition: longint;
    levposition: longint;
    //oo1 imgtype: integer; {RED 3/20/02}  {1 = DICOM; 2 = TGA 8 BITS; 3 = TGA >8 BITS; 4 = OTHER -- corresponds to init performed}
    imagebittype: integer; {RED 3/20/02}  {1 = DICOM; 2 = TGA 8 BITS; 3 = TGA >8 BITS; 4 = OTHER -- corresponds to init performed}
    ien: integer;
    ImageIndex: integer;
    magien: integer; {RED 12/7/02}
    Smallest_PixelVal: longint;{RED 7/25/02 from DICOM hdr}
    Largest_PixelVal: longint; {RED 7/25/02 from DICOM hdr}
    Pos_Ref_Indicator: string;
    dicomimage: boolean;
    GListState: boolean;
    blank: string;
    //oo1 procedure clearDICOMRecorda(winnum: integer; ClearList: boolean);
    procedure ClearData;
  end;

  TImageData = class(Tobject)
  private
    FMag0: string;                      //Image IEN                    Fld 001
    FAFile: string;                     //Full Path of Abstract
    FFFile: string;                     //Full Path of Image
    FImgDes: string;                    //SHORT DESCRIPTION            Fld 10
    FImgType: integer;                  //OBJECT TYPE                  Fld 3
    FPtName: string;                    //Patient Name
    FDate: string;                      //PROCEDURE EXAM DATE/TIME     Fld 15
    FProc: string;                      //PROCEDURE                    Fld 6
    FAbsLocation: string;               //M,W,O
    FFullLocation: string;              //M,W,A
    FDicomImageNumber: string;          //if one exists for image. Fld 2
                                        //  is a field in the OBJECT GROUP Mult
    FDicomSequenceNumber: string;       //if one exists for image. Fld 1
                                        //  is a field in the OBJECT GROUP Mult
    Fbaseindex: integer;                // used internally by Mag4Viewer.
    FQAMsg : string;                    // If QA check produces a warning
                                        //  it is put here
    FBigFile : string;                  //Full Path to BIG File.
    FDFN : string;                      //Patient DFN                  Fld 5
    FSSN : string;                      //Patient SSN
    FMagClass: String;                  //CLASS INDEX                  Fld 41
    FGroupCount: integer;               //Number of images in this group
    FGroupIEN: integer;                 //GROUP PARENT                 Fld 14
    FGroupIndex: integer;               //in the ObjList : Tlist maintained
                                        // by TMagimageList
    //DBI
    FPlaceCode : string;
    FPlaceIEN : string;



    procedure SetMag0(const Value: string);
    procedure SetAbsLocation(const Value: string);
    procedure SetAFile(const Value: string);
    procedure Setbaseindex(const Value: integer);
    procedure SetDate(const Value: string);
    procedure SetDicomImageNumber(const Value: string);
    procedure SetDicomSequenceNumber(const Value: string);
    procedure SetFFile(const Value: string);
    procedure SetFullLocation(const Value: string);
    procedure SetImgDes(const Value: string);
    procedure SetImgType(const Value: integer);
    procedure SetProc(const Value: string);
    procedure SetPtName(const Value: string);
    procedure SetBigFile(const Value: string);
    procedure SetQAMsg(const Value: string);
    procedure SetDFN(const Value: string);
    procedure SetMagClass(const Value: string);

                
  public
    {    Returns an Description to use by application
        [Dicom numbers], Image Description,  Procedure, Proc Exam Date/Time }
    function ExpandedDescription(lf : boolean = true) : string;
    {   Returns an Description to use by application
        Image IEN, [Dicom numbers], Image Description,  Procedure, Proc Exam Date/Time }
    function ExpandedIDDescription(lf : boolean = true) : string;
    {   Assigns all values from one TImageData to another. Makes a Copy}
    procedure magAssign(IData: TImageData);

    property Mag0: string read FMag0 write SetMag0;
    property AFile: string read FAFile write SetAFile;
    property FFile: string read FFFile write SetFFile;
    property ImgDes: string read FImgDes write SetImgDes;
    property ImgType: integer read FImgType write SetImgType; { a byte is 0 .. 255 }
    property PtName: string read FPtName write SetPtName;
    property Date: string read FDate write SetDate;
    property Proc: string read FProc write SetProc;
    property AbsLocation: string read FAbsLocation write SetAbsLocation;
    property FullLocation: string read FFullLocation write SetFullLocation;
    property DicomImageNumber: string read FDicomImageNumber write SetDicomImageNumber;
    property DicomSequenceNumber: string read FDicomSequenceNumber write SetDicomSequenceNumber;
    property baseindex: integer read Fbaseindex write Setbaseindex;
    property QAMsg : string read FQAMsg write SetQAMsg;
    property BigFile : string read FBigFile Write SetBigFile;
    property DFN : string read FDFN write SetDFN;
    property SSN : string read FSSN write FSSN;
    property MagClass : string read FMagClass write SetMagClass;
    property GroupIEN: integer read FGroupIEN write FGroupIEN;
    property GroupIndex : integer read FGroupIndex write FGroupIndex;
    property GroupCount : integer read FGroupCount write FGroupCount;
    property PlaceIEN : string read FPlaceIEN write FPlaceIEN;
    property PlaceCode : string read FPlaceCode write FPlaceCode;
  end;






const maglocUnknown: string = '';
const maglocOffLine: string = 'O'; //integer = 1;
const maglocJB: string = 'W'; //integer = 2;
const maglocMag: string = 'M'; //integer = 2;



(*   moved to cMagVUtils.pas
   procedure StringsToIMageRec(Temppat: TMag4Pat; t: Tstrings);*)
implementation
 uses  uMagUtils, ImagInterfaces, uMagDefinitions,   cMagPat ;
{ TImageData }

procedure TImageData.SetAbsLocation(const Value: string);
begin
  FAbsLocation := Value;
end;

procedure TImageData.SetAFile(const Value: string);
begin
  FAFile := Value;
end;

procedure TImageData.Setbaseindex(const Value: integer);
begin
  Fbaseindex := Value;
end;

procedure TImageData.SetDate(const Value: string);
begin
  FDate := Value;
end;

procedure TImageData.SetDicomImageNumber(const Value: string);
begin
  FDicomImageNumber := Value;
end;

procedure TImageData.SetDicomSequenceNumber(const Value: string);
begin
  FDicomSequenceNumber := Value;         //This is the Dicom Series Number
end;

procedure TImageData.SetFFile(const Value: string);
begin
  FFFile := Value;
end;

procedure TImageData.SetMag0(const Value: string);
begin
  FMag0 := Value;
end;

procedure TImageData.SetFullLocation(const Value: string);
begin
  FFullLocation := Value;
end;

procedure TImageData.SetImgDes(const Value: string);
begin
  FImgDes := Value;
end;

procedure TImageData.SetImgType(const Value: integer);
begin
  FImgType := Value;
end;

procedure TImageData.SetProc(const Value: string);
begin
  FProc := Value;
end;

procedure TImageData.SetPtName(const Value: string);
begin
  FPtName := Value;
end;


procedure TImageData.magAssign(IData: TImageData);
begin
  FMag0 := Idata.Mag0;
  FAFile := Idata.AFile;
  FFFile := Idata.FFFile;
  FImgDes := Idata.ImgDes;
  FImgType := Idata.ImgType;
  FPtName := Idata.PtName;
  FDate := Idata.Date;
  FProc := Idata.Proc;
  FAbsLocation := Idata.AbsLocation;
  FFullLocation := Idata.FullLocation;
  FDicomSequenceNumber := Idata.DicomSequenceNumber;   //This is the Dicom Series Number
  FDicomImageNumber := Idata.DicomImageNumber;
  Fbaseindex := Idata.baseindex;
  FQAMsg := Idata.QAMsg;
  FBigFile := Idata.BigFile;
  FDFN := Idata.DFN;
  FSSN := Idata.SSN;
  FMagClass := Idata.MagClass;

  FGroupCount := Idata.GroupCount;
  FGroupIEN := Idata.FGroupIEN;
  FGroupIndex := Idata.FGroupIndex;
  //P48T3 DBI
  FPlaceIEN := Idata.FPlaceIEN;
  FPlaceCode := Idata.FPlaceCode;

end;




procedure TImageData.SetBigFile(const Value: string);
begin
  FBigFile := Value;
end;

procedure TImageData.SetQAMsg(const Value: string);
begin
  FQAMsg := Value;
end;

function TImageData.ExpandedDescription(lf : boolean = true): string;
var dicomdata, lfs : string;
placedata : string; //P48T2 DBI    Added PlaceData to ImageDescription.
begin
dicomdata := '';
placedata := '';

if lf then lfs := #13
      else lfs := ' ';
if self.DicomSequenceNumber <> '' then dicomdata := '['+self.DicomSequenceNumber+','+self.DicomImageNumber+'] ';
if WrksConsolidated then If self.PlaceCode <> '' then placedata := '['+self.PlaceCode+']';
result := placedata + dicomdata + imgdes + lfs + proc + ' ' + date;
end;

function TImageData.ExpandedIDDescription(lf : boolean = true): string;
begin
result := ' ID# '+mag0+' ' +  ExpandedDescription(lf);
end  ;

procedure TImageData.SetDFN(const Value: string);
begin
  FDFN := Value;
end;

procedure TImageData.SetMagClass(const Value: string);
begin
  FMagClass := Value;
end;

procedure TMagDICOMData.ClearData;
begin
  PtName := '';
  PtID := '';
  Modality := '';
  PixelSpace1 := 0.0;
  PixelSpace2 := 0.0;
  PixelSpace3 := 0.0;
  Haunsfield := 0;
  SLiceThickness := 0.0;
  Pt_Pos := '';
  Laterality := '';
  SliceLoc := 0.0;
  Rescale_Int := 0;
  Rescale_Slope := 0;
  Window_Center := 0;
  Window_Width := 0;
  rows := 0;
  columns := 0;
  Filename := '';
  bitdepth := 0;
  seriesno := 0;
  imageno := '0';
  PatOr1 := '';
  PatOr2 := '';
  bits := 0;
  contrast := '';
  originalbits := 0;
  protocol := '';
  warning := 0;{P1}

  {bigfile not reset, already done}

//oo1 If clearlist then
//oo1 begin
  //oo1 gearcontrol := nil;
  //oo1 medcontrol := nil;
  panel := nil;
  GList := nil;
  GListState := false;
  Blank := '';
//oo1   end;

  minpixel := 0;
  maxpixel := 0;
  winposition := 0;
  levposition := 0;
  //oo1 imgtype
  imagebittype := 0;
  ien := 0;
  smallest_PixelVal := 0;
  largest_PixelVal := 0;
  dicomimage := false;
  //oo1
  (*for m := 1 to 6 do
    begin
    ADICOMRec[winnum].ImageOr[m] := -2.0;
    end;  *)
end;




end.
