unit uMagDefinitions;
   {
   Package: MAG - VistA Imaging
   WARNING: Per VHA Directive 10-93-142 this unit should not be modified.
   Date Created:      2002
   Site Name: Silver Spring, OIFO
   Developers: Garrett Kirin
   [==    unit uMagDefinitions;
   Description: Imaging Definitions for application constants, ordinal types,
   sets etc.   Also contains functions/ procedures that deal with the types,
   const, defined here.
                ==]
   Note:
   }
(*
   ;; +---------------------------------------------------------------------------------------------------+
   ;; Property of the US Government.
   ;; No permission to copy or redistribute this software is given.
   ;; Use of unreleased versions of this software requires the user
   ;;  to execute a written test agreement with the VistA Imaging
   ;;  Development Office of the Department of Veterans Affairs,
   ;;  telephone (301) 734-0100.
   ;;
   ;; The Food and Drug Administration classifies this software as
   ;; a medical device.  As such, it may not be changed
   ;; in any way.  Modifications to this software may result in an
   ;; adulterated medical device under 21CFR820, the use of which
   ;; is considered to be a violation of US Federal Statutes.
   ;; +---------------------------------------------------------------------------------------------------+
*)
//{$DEFINE PROTOTYPE}
interface
  uses umagutils, sysutils, classes;

  {  all classes that are viewable in Display for Admin personnal}
  const magdisclsAdmin: string = 'ADMIN,ADMIN/CLIN,CLIN/ADMIN';
  {  all classes that are viewable in Capture for Admin personnal}
  const magcapclsAdmin: string = 'ADMIN,ADMIN/CLIN';
  {  all classes that are viewable in Display for Clinical personnal}
  const magdisclsClin: string = 'CLIN,CLIN/ADMIN,ADMIN/CLIN';
  {  all classes that are viewable in Capture for Clinical personnal}
  const magcapclsClin: string = 'CLIN,CLIN/ADMIN';
  {  all Classes.}
  const magclsAll : string = 'CLIN,CLIN/ADMIN,ADMIN,ADMIN/CLIN';

  const magcrCrossHair = 50; { cursors for Mag4VGear }
  const magcrMagnify = 51;
  const magcrPan = 52;

type
  { packages to attach images }
  TmagPackages = set of (mpkgMED, mpkgSUR, mpkgRAD, mpkgLAB, mpkgNOTE, mpkgCP, mpkgCNSLT, mpkgNONE);
  { possible classes }
  TMagImageClasses = set of (mclsCLIN, mclsADMIN);
  { constants for mouse cursor for Mag4VGear}
  TMagImageMouse = (mactPan, mactDrag, mactMagnify, mactZoomRect, mactSelect, mactPointer);
  { Class to hold all properties of an Image Filter (from IMAGE LIST FILTERS File) }
  TImageFilter = class(Tobject)
  private
    FFltID : string;
    Fname : string;
    Fpkgs: TmagPackages;
    FClasses: TmagImageClasses;
    FTypes: string;
    FProcEvent: string;
    FSpecSubSpec: string;
    FfromDate: string;
    FtoDate: string;
    FMthRange: integer;
    FScope : string;

  public
    property FilterID : string read FFltID write FFltID;
    property Name : string read FName write Fname;
    property Packages: TmagPackages read FPkgs write FPkgs;
    property Classes: TMagImageClasses read FClasses write FClasses;
    property Types: string read FTypes write Ftypes;
    property ProcEvent: string read FProcEvent write FProcEvent;
    property SpecSubSpec: string read FSpecSubSpec write FSpecSubSpec;
    property FromDate: string read FfromDate write Ffromdate;
    property ToDate: string read FtoDate write FtoDate;
    property MonthRange : integer Read FMthRange write FMthRange;
    property Origin : String read FScope write Fscope;
    function Detailstring: string;
    function DetailedDesc : Tstrings;
  end;

  {  Mag4VGear objects can be querried for their 'state'  this object is passed
     as a result or a variable parameter in certain methods}
  TMagImageState = class(Tobject)
  private
    //
  public
    // rotatecount starts at 4, each 90 inc by 1,
    RotateCount: integer;
    // reversecount starts at 0, each invert, inc by 1, odd numbers tell
    // us that image is inverted from it's original.
    ReverseCount: integer;
    // 0 to 255 start at 100.
    // control uses :
    //     Full1.DisplayContrast :=  tbicContrast.Position / 100;
    contrastvalue: integer;
    // -900 to 1100 start at 100
    // control uses :
    //  Full1.DisplayBrightness := trunc(tbicBrightness.Position / 10);
    brightnessvalue: integer;
    //20 to 800 start at 100;
    // control uses :
    //          full1.zoomlevel := tbicZoom.Position;
    zoomvalue: integer;
    // these two, set the control, then set zoom value := 100
    fitheight: boolean;
    fitwidth: boolean;
    // This is the current Viewer Page being displayed.
    PageViewer: integer;
    //This is number of pages in the viewer
    PageViewerCount : integer;
    //This is the current displayed page;
    Page: integer;
    // total number of pages.
    PageCount : integer;
    // Image is member of Group, this is the Group index.
    ImageNumber: integer;
    // total number of Images in Group.
    ImageCount : integer;
    // Rad Images Window and Level values
    WinValue : integer;
    LevValue : integer;
    // Not sure but Win and Lev min and Max might be diff for diff images.
    WinMinValue : integer;
    WinMaxValue : integer;
    LevMinValue : integer;
    LevMaxValue : integer;
    IsWinLevEnabled : boolean;
    
    //
    MouseAction : TMagImageMouse;

  end;

{  Quick conversion utilities }
function FilterToString(filter : TImageFilter): string;
procedure FilterToFieldList(flt : TImageFilter; var t : Tstrings; duz: string = '');
function StringToFilter(value: string): TImageFilter;
function StringToClasses(str : string) : TmagImageClasses;
function StringToPackages(str: string) : TmagPackages;
function PackagesToString(pkg: TmagPackages): string;
function ClassesToString(cls: TmagImageClasses): string;
function ClassesToSet(cls: TmagImageClasses): string;

{ imitate some of the 'M' string functions
    used mainly in userprefrences to convert Integer and Boolean properties
    to string values before saving in VistA.  and vice versa}
function magStrToInt(value : string): integer;
function magStrToBool(value : string) : boolean;
function magIntToBool(value : integer): boolean;
function magBoolToStr(value : boolean) : string;
function magBoolToInt(value : boolean) : integer;
function magBoolToStrInt(value : boolean) : STRING;

var
     WrksPlaceCode,WrksPlaceIEN,WrksInst : string;      //p48t2 DBI
     WrksConsolidated : boolean;
     IniViewRemoteAbs : boolean;
implementation


function magStrToInt(value : string): integer;
begin
  try
  result := strtoint(value);
  except
   on exception do result := 0;
   end;
end;

function magIntToBool(value : integer): boolean;
begin
// Called with value from a set of codes that is 1 | 0
result := (value <> 0)
end;

function magStrToBool(value : string) : boolean;
begin
// Called with a 'TRUE' 'True' 'False' 'FALSE' '0' '1'
Result := ((uppercase(value) = 'TRUE') or (value = '1'));
end;

function magBoolToStr(value : boolean) : string;
begin
if value
  then result := 'True'
  else result := 'False';
end;

function magBoolToStrInt(value : boolean) : STRING;
begin
if value
  then result := '1'
  else result := '0';
end;

function magBoolToInt(value : boolean) : integer;
begin
if value
  then result := 1
  else result := 0;
end;




function PackagesToString(pkg: TmagPackages): string;
begin
  //MED,SUR,RAD,LAB,NOTE,CP,CNSLT,NONE
  result := '';
  if (mpkgMED in pkg) then result := result + 'MED,';
  if (mpkgSUR in pkg) then result := result + 'SUR,';
  if (mpkgRAD in pkg) then result := result + 'RAD,';
  if (mpkgLAB in pkg) then result := result + 'LAB,';
  if (mpkgNOTE in pkg) then result := result + 'NOTE,';
  if (mpkgCP in pkg) then result := result + 'CP,';
  if (mpkgCNSLT in pkg) then result := result + 'CNSLT,';
  if (mpkgNONE in pkg) then result := result + 'NONE,';
  result := copy(result, 1, length(result) - 1);
end;

function StringToPackages(str: string) : TmagPackages;
var pkg : TmagPackages;
begin
  pkg := [];
  result := [];
  //MED,SUR,RAD,LAB,NOTE,CP,CNSLT,NONE
  if (pos('MED',str) > 0) then result  := result + [mpkgMED];
  if (pos('SUR',str) > 0) then result  := result + [mpkgSUR];
  if (pos('RAD',str) > 0) then result  := result + [mpkgRAD];
  if (pos('LAB',str) > 0) then result  := result + [mpkgLAB];
  if (pos('NOTE',str) > 0) then result  := result + [mpkgNOTE];
  if (pos('CP',str) > 0) then result  := result + [mpkgCP];
  if (pos('CNSLT',str) > 0) then result  := result + [mpkgCNSLT];
  if (pos('NONE',str) > 0) then result  := result + [mpkgNONE];
end;


function ClassesToString(cls: TmagImageClasses): string;
begin
  result := '';
  if (mclsCLIN in cls) then result := result + magdisclsClin + ',' ;
  if (mclsADMIN in cls) then result := result + magdisclsADMIN + ',' ;
  result := copy(result, 1, length(result) - 1);
end;

function ClassesToSet(cls: TmagImageClasses): string;
begin
  //CLIN, ADMIN
  result := '';
 if (mclsCLIN in cls) then result := result + 'CLIN' + ',';
  if (mclsADMIN in cls) then result := result + 'ADMIN' + ',';
  result := copy(result, 1, length(result) - 1);
end;

function StringToClasses(str : string) : TmagImageClasses;
begin
  //CLIN, ADMIN   Could also be the only strings sent.  Then convert to
  //              proper mcls...
  result := [];
  if (pos(magclsAll,str) > 0) then
    begin
      result := result + [mclsClin,mclsAdmin];
      exit;
    end;
  if (pos(magdisclsClin,str) > 0) then result := result + [mclsClin];
  if (pos(magdisclsADMIN,str) > 0) then result := result + [mclsAdmin];
  if str = 'ADMIN' then result := result + [mclsAdmin];
  if str = 'CLIN' then  result := result + [mclsClin];
end;

{ TImageFilter }

function TImageFilter.DetailedDesc: Tstrings;
var s : string;
begin
// Not yet called by Application.
result := Tstringlist.create;
  Result.add('FILTER DETAILS');
  result.add('');
  result.add('Name :      '+Fname);
  result.add('');

  s := ClassesToString(Fclasses);
  if s='' then s := 'Any';
  result.add('Class :    '+s);
  result.add('');

  s := FScope;
  if s='' then s := 'Any';
  result.add('Origin :     '+s);
  result.add('');

  // we have a few if, rather that if then else for a reason
  //  we want to make sure the date properties are getting cleared when
  //  they should.  Only one of the IF's below, should be TRUE.
  //   If not, then we'll see it and know of a problem.
  if ((FfromDate <> '') or (FToDate <> '')) then
  begin
  result.add('Dates:    '+'from: '+FfromDate+'  thru  '+FToDate);
  end;
  if (FMthRange <> 0) then
  begin
  result.add('Dates:    for the last '+inttostr(abs(FMthRange))+' months.');
  end;
  if (FmthRange = 0) and (FfromDate ='') and (FtoDate = '') then
  result.add('Dates:    All Dates.');
  result.add('');

  s := PackagesToString(Fpkgs);
  if s='' then s := 'Any';
  result.add('Packages:    '+s);
  result.add('');

  result.add('Types: ');
  s := FTypes;
  if s='' then s := 'Any';
  result.add(s);
  result.add('');

  result.add('Specialty/SubSpecialty:');
  s := FSpecSubSpec;
  if s='' then s := 'Any';
  result.add(s);
  result.add('');

  result.add('Procedure/Event:');
  s := FProcEvent;
  if s='' then s := 'Any';
  result.add(s);
  result.add('');

end;

function TImageFilter.Detailstring: string;
begin
  result := '';
  result := PackagesToString(Fpkgs);
  result := result + ';' + ClassesToString(Fclasses);
  result := result + ';' + FTypes;
  result := result + ';' + FSpecSubSpec;
  result := result + ';' + FProcEvent;
  result := result + ';' + FfromDate;
  result := result + ' - ' + FToDate;
  //result := copy(result,1,length(result)-1);
end;


function StringToFilter(value : string) : TImageFilter;
//var vCls : string;
begin
(* ^MAG(2005.87,D0,0)= (#.01) NAME [1F] ^ (#1) PACKAGE [2F] ^ (#2) CLASS [3S] ^
                 ==>(#3) TYPE [4F] ^ (#4) EVENT [5F] ^ (#5) SPEC [6F] ^ (#6)
                 ==>FROM [7D] ^ (#7) UNTIL [8D] ^ (#8) RELATIVE RANGE [9N] ^
                 ==>(#9) Origin [10S] ^

 MAGRY(1)=FLTIEN_"^"_^MAG(2005.87,FLTIEN,0)
*)
result := TImageFilter.create;
with result do
  begin
    FFltID := magpiece(value,'^',1);
    FName := magpiece(value,'^',2);
    FPkgs := StringToPackages(magpiece(value,'^',3));
    FClasses := StringToClasses(magpiece(value,'^',4));
    FTypes := magpiece(value,'^',5);
    FProcEvent := magpiece(value,'^',6);
    FSpecSubSpec := magpiece(value,'^',7);
    FFromDate := magpiece(value,'^',8);
    FToDate := magpiece(value,'^',9);
    if (magpiece(value,'^',10) <> '')
      then FMthRange := strtoint(magpiece(value,'^',10))
      else FMthRange := 0;
    FScope := magpiece(value,'^',11);
  end;
end;

function FilterToString(filter : TImageFilter): string;
var u : string;
begin
u := '^';
with filter do
  result := FFltID+u+Fname+u+ PackagesToString(Fpkgs)+u+ ClassesToString(FClasses)+u+Ftypes
      +u+FProcEvent+u+FSpecSubSpec+ FFromDate+u+FToDate+u+inttostr(FMthRange)+u+FScope;
end;


procedure FilterToFieldList(flt : TImageFilter;  var t : Tstrings; duz : string = '');
//var i : integer;
begin
(* ^MAG(2005.87,D0,0)= (#.01) NAME [1F] ^ (#1) PACKAGE [2F] ^ (#2) CLASS [3S] ^
                 ==>(#3) TYPE [4F] ^ (#4) EVENT [5F] ^ (#5) SPEC [6F] ^ (#6)
                 ==>FROM [7D] ^ (#7) UNTIL [8D] ^ (#8) RELATIVE RANGE [9N] ^
                 ==>(#9) Origin [10S] ^

 MAGRY(1)=FLTIEN_"^"_^MAG(2005.87,FLTIEN,0)
*)
t.clear;
with flt do
  begin
    if FFltID  <> '' then t.add('IEN^'+FFltID);
    if FName  <> '' then t.add('.01^'+FName);
    if FPkgs  <> [] then t.add('1^'+Packagestostring(Fpkgs));
    if FClasses  <> [] then t.add('2^'+ClassesToSet(FClasses));
    if FTypes  <> '' then t.add('3^'+FTypes);
    if FProcEvent  <> '' then t.add('4^'+FProcEvent);
    if FSpecSubSpec  <> '' then t.add('5^'+FSpecSubSpec);
    if FFromDate  <> '' then t.add('6^'+FFromDate);
    if FToDate  <> '' then t.add('7^'+FToDate);
    //if FMthRange  <> '' then
    t.add('8^'+inttostr(FMthRange));
    if FScope  <> '' then t.add('9^'+FScope);
    if duz <> '' then t.add('USER^'+duz)
                 else t.add('21^PUBLIC');
  end;
end;

end.

